@@ -619,7 +619,7 @@ static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
 	if (summaryview) {
 		summary_lock(summaryview);				
 		main_window_cursor_wait(mainwin);		
-		gtk_clist_freeze(GTK_CLIST(summaryview->ctree));	
+		summary_freeze(summaryview);	
 		folder_item_update_freeze();				
 	}
 
@@ -627,7 +627,7 @@ static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
 
 	if (summaryview) {
 		folder_item_update_thaw();				
-		gtk_clist_thaw(GTK_CLIST(summaryview->ctree));		
+		summary_thaw(summaryview);		
 		main_window_cursor_normal(mainwin);	
 		summary_unlock(summaryview);				
 		summary_show(summaryview, summaryview->folder_item);
@@ -753,7 +753,9 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 			}
 			if ((child_info = fork_child(cmd, msg_str, children))) {
 				/* Pass msginfo to catch_status () */
-				child_info->msginfo_list = g_slist_append (NULL, msginfo);
+				if (!(action_type & (ACTION_PIPE_OUT | ACTION_INSERT)))
+					child_info->msginfo_list = 
+						g_slist_append (NULL, msginfo);
 				children_list = g_slist_append(children_list,
 							       child_info);
 			}
@@ -764,7 +766,9 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 				       user_str, user_hidden_str, sel_str);
 		if (cmd) {
 			if ((child_info = fork_child(cmd, msg_str, children))) {
-				child_info->msginfo_list = g_slist_copy (msg_list);
+				if (!(action_type & (ACTION_PIPE_OUT | ACTION_INSERT)))
+					child_info->msginfo_list = 
+						g_slist_copy (msg_list);
 				children_list = g_slist_append(children_list,
 								child_info);
 			}
@@ -1069,7 +1073,8 @@ static void hide_io_dialog_cb(GtkWidget *w, gpointer data)
 static gint io_dialog_key_pressed_cb(GtkWidget *widget, GdkEventKey *event,
 				     gpointer data)
 {
-	if (event && event->keyval == GDK_Escape)
+	if (event && (event->keyval == GDK_Escape ||
+		      event->keyval == GDK_Return))
 		hide_io_dialog_cb(widget, data);
 	return TRUE;
 }
@@ -1133,6 +1138,7 @@ static void update_io_dialog(Children *children)
 		gchar *text;
 		
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(children->progress_bar),
+						  (children->initial_nb == 0) ? 0 :
 					      (gdouble) (children->initial_nb - children->nb) /
 					      (gdouble) children->initial_nb);
 		text = g_strdup_printf("%s %d/%d", _("Completed"), 
@@ -1338,45 +1344,45 @@ static void catch_status(gpointer data, gint source, GdkInputCondition cond)
 
 	if (child_info->children->action_type & (ACTION_SINGLE | ACTION_MULTIPLE)
 	    && child_info->msginfo_list) {
-	/* Actions on message *files* might change size and
-	* time stamp, and thus invalidate the cache */
-	SummaryView *summaryview  = NULL;
-	GSList      *cur;
-	MsgInfo     *msginfo, *nmi;	/* newmsginfo */
-	char        *file;
+		/* Actions on message *files* might change size and
+		* time stamp, and thus invalidate the cache */
+		SummaryView *summaryview  = NULL;
+		GSList      *cur;
+		MsgInfo     *msginfo, *nmi;	/* newmsginfo */
+		char        *file;
 
-	if (mainwindow_get_mainwindow ())
-		summaryview = mainwindow_get_mainwindow ()->summaryview;
-	for (cur = child_info->msginfo_list; cur; cur = cur->next) {
-		msginfo = (MsgInfo *)cur->data;
-		if (!(msginfo && /* Stuff used valid? */
-		    msginfo->folder && msginfo->folder->cache)) 
-			continue;
-		file = procmsg_get_message_file_path (msginfo);
-		if (!file) 
-			continue;
-		nmi = procheader_parse_file (file, msginfo->flags, FALSE, FALSE);
-		if (!nmi) 
-			continue; /* Deleted? */
-		if (msginfo->mtime != nmi->mtime || msginfo->size != nmi->size) {
-			msginfo->mtime  = nmi->mtime;
-			msginfo->size   = nmi->size;
-			msgcache_update_msg (msginfo->folder->cache, msginfo);
-		}
-		procmsg_msginfo_free (nmi);
+		if (mainwindow_get_mainwindow ())
+			summaryview = mainwindow_get_mainwindow ()->summaryview;
+		for (cur = child_info->msginfo_list; cur; cur = cur->next) {
+			msginfo = (MsgInfo *)cur->data;
+			if (!(msginfo && /* Stuff used valid? */
+			    msginfo->folder && msginfo->folder->cache)) 
+				continue;
+			file = procmsg_get_message_file_path (msginfo);
+			if (!file) 
+				continue;
+			nmi = procheader_parse_file (file, msginfo->flags, FALSE, FALSE);
+			if (!nmi) 
+				continue; /* Deleted? */
+			if (msginfo->mtime != nmi->mtime || msginfo->size != nmi->size) {
+				msginfo->mtime  = nmi->mtime;
+				msginfo->size   = nmi->size;
+				msgcache_update_msg (msginfo->folder->cache, msginfo);
+			}
+			procmsg_msginfo_free (nmi);
 
-		/* refresh the message display, but only if this message is still
-		* in focus */
-		if (summaryview
-		    && summaryview->displayed
-		    && summaryview->folder_item == msginfo->folder
-		    && summary_get_msgnum (summaryview, summaryview->displayed) == msginfo->msgnum) {
-			summary_show (summaryview, summaryview->folder_item);
-			summary_select_node (summaryview, summaryview->displayed, TRUE, TRUE);
+			/* refresh the message display, but only if this message is still
+			* in focus */
+			if (summaryview
+			    && summaryview->displayed
+			    && summaryview->folder_item == msginfo->folder
+			    && summary_get_msgnum (summaryview, summaryview->displayed) == msginfo->msgnum) {
+				summary_show (summaryview, summaryview->folder_item);
+				summary_select_node (summaryview, summaryview->displayed, TRUE, TRUE);
+			}
 		}
-	}
-	g_slist_free (child_info->msginfo_list);
-	child_info->msginfo_list = NULL;
+		g_slist_free (child_info->msginfo_list);
+		child_info->msginfo_list = NULL;
 	}
 
 	wait_for_children(child_info->children);
@@ -1461,7 +1467,7 @@ static void catch_output(gpointer data, gint source, GdkInputCondition cond)
 			if (ret_str && bytes_written > 0) {
 				gtk_text_buffer_insert
 					(textbuf, &iter, ret_str,
-					 bytes_written);
+					 -1);
 				g_free(ret_str);
 			} else
 				gtk_text_buffer_insert(textbuf, &iter, buf, c);
