@@ -53,7 +53,10 @@
  *\brief	For the GtkListStore
  */
 enum {
+	ADDR_COMPL_ICON,
 	ADDR_COMPL_ADDRESS,
+	ADDR_COMPL_ISGROUP,
+	ADDR_COMPL_GROUPLIST,
 	N_ADDR_COMPL_COLUMNS
 };
 
@@ -83,6 +86,7 @@ typedef struct
 {
 	gchar *name;
 	gchar *address;
+	GList *grp_emails;
 } address_entry;
 
 /**
@@ -102,6 +106,9 @@ static GList 	   *g_completion_list;	/* list of strings to be checked */
 static GList 	   *g_address_list;	/* address storage */
 static GCompletion *g_completion;	/* completion object */
 
+static GHashTable *_groupAddresses_ = NULL;
+static gboolean _allowCommas_ = TRUE;
+
 /* To allow for continuing completion we have to keep track of the state
  * using the following variables. No need to create a context object. */
 
@@ -113,6 +120,8 @@ static gchar	   *g_completion_prefix;	/* last prefix. (this is cached here
 						 * because the prefix passed to g_completion
 						 * is g_strdown()'ed */
 
+static gchar *completion_folder_path = NULL;
+
 /*******************************************************************************/
 
 /*
@@ -193,6 +202,7 @@ static void free_all(void)
 		address_entry *ae = (address_entry *) walk->data;
 		g_free(ae->name);
 		g_free(ae->address);
+		g_list_free(ae->grp_emails);
 		g_free(walk->data);
 	}
 	g_list_free(g_address_list);
@@ -200,6 +210,9 @@ static void free_all(void)
 	
 	g_completion_free(g_completion);
 	g_completion = NULL;
+	if (_groupAddresses_)
+		g_hash_table_destroy(_groupAddresses_);
+	_groupAddresses_ = NULL;
 }
 
 /**
@@ -224,15 +237,23 @@ static void add_address1(const char *str, address_entry *ae)
  * \param name    Recipient name.
  * \param address EMail address.
  * \param alias   Alias to append.
+ * \param grp_emails the emails in case of a group. List should be freed later, 
+ * but not its strings
  * \return <code>0</code> if entry appended successfully, or <code>-1</code>
  *         if failure.
  */
 static gint add_address(const gchar *name, const gchar *address, 
-			const gchar *nick, const gchar *alias)
+			const gchar *nick, const gchar *alias, GList *grp_emails)
 {
 	address_entry    *ae;
+	gboolean is_group = FALSE;
 
-	if (!name || !address) return -1;
+	if (!name || !address) {
+		if (!address && !nick && !alias && grp_emails) {
+			is_group = TRUE;
+		} else
+			return -1;
+	}
 
 	ae = g_new0(address_entry, 1);
 
@@ -240,11 +261,12 @@ static gint add_address(const gchar *name, const gchar *address,
 
 	ae->name    = g_strdup(name);
 	ae->address = g_strdup(address);		
-
+	ae->grp_emails = grp_emails;
 	g_address_list = g_list_prepend(g_address_list, ae);
 
 	add_address1(name, ae);
-	add_address1(address, ae);
+	if (address != NULL)
+		add_address1(address, ae);
 	
 	if (nick != NULL)
 		add_address1(nick, ae);
@@ -259,8 +281,8 @@ static gint add_address(const gchar *name, const gchar *address,
 /**
  * Read address book, creating all entries in the completion index.
  */ 
-static void read_address_book(void) {	
-	addrindex_load_completion( add_address );
+static void read_address_book(gchar *folderpath) {	
+	addrindex_load_completion( add_address, folderpath );
 	g_address_list = g_list_reverse(g_address_list);
 	g_completion_list = g_list_reverse(g_completion_list);
 }
@@ -282,8 +304,7 @@ static gboolean is_completion_pending(void)
 static void clear_completion_cache(void)
 {
 	if (is_completion_pending()) {
-		if (g_completion_prefix)
-			g_free(g_completion_prefix);
+		g_free(g_completion_prefix);
 
 		if (g_completion_addresses) {
 			g_slist_free(g_completion_addresses);
@@ -299,19 +320,42 @@ static void clear_completion_cache(void)
  * address completion.
  * \return The number of addresses in the completion list.
  */
-gint start_address_completion(void)
+gint start_address_completion(gchar *folderpath)
 {
 	clear_completion_cache();
+
+	if ((completion_folder_path == NULL && folderpath != NULL) ||
+		(completion_folder_path != NULL && folderpath == NULL) ||
+		(completion_folder_path != NULL && folderpath != NULL &&
+		 strcmp(completion_folder_path, folderpath) != 0)) {
+
+		debug_print("start_address_completion: resetting\n");
+
+		/* TODO: wwp: optimize: only reset when the new folderpath is MORE restrictive than the old one
+		  (the most easy case is when folderpath is NULL and completion_folder_path is != NULL */
+		if (g_ref_count) {
+			free_all();
+			g_ref_count = 0;
+		}
+	}
+
+	g_free(completion_folder_path);
+	if (folderpath != NULL)
+		completion_folder_path = g_strdup(folderpath);
+	else
+		completion_folder_path = NULL;
+
 	if (!g_ref_count) {
 		init_all();
 		/* open the address book */
-		read_address_book();
+		read_address_book(folderpath);
 		/* merge the completion entry list into g_completion */
 		if (g_completion_list)
 			g_completion_add_items(g_completion, g_completion_list);
 	}
 	g_ref_count++;
-	debug_print("start_address_completion ref count %d\n", g_ref_count);
+	debug_print("start_address_completion(%s) ref count %d\n",
+				folderpath, g_ref_count);
 
 	return g_list_length(g_completion_list);
 }
@@ -377,6 +421,20 @@ static gchar *get_address_from_edit(GtkEntry *entry, gint *start_pos)
 	return str;
 } 
 
+static gchar *get_complete_address_from_name_email(const gchar *name, const gchar *email)
+{
+	gchar *address = NULL;
+	if (!name || name[0] == '\0')
+		address = g_strdup_printf("<%s>", email);
+	else if (strchr_with_skip_quote(name, '"', ','))
+		address = g_strdup_printf
+			("\"%s\" <%s>", name, email);
+	else
+		address = g_strdup_printf
+			("%s <%s>", name, email);
+	return address;
+}
+
 /**
  * Replace an incompleted address with a completed one.
  * \param entry     Address entry field.
@@ -384,12 +442,35 @@ static gchar *get_address_from_edit(GtkEntry *entry, gint *start_pos)
  * \param start_pos Insertion point in entry field.
  */
 static void replace_address_in_edit(GtkEntry *entry, const gchar *newtext,
-			     gint start_pos)
+			     gint start_pos, gboolean is_group, GList *grp_emails)
 {
 	if (!newtext) return;
 	gtk_editable_delete_text(GTK_EDITABLE(entry), start_pos, -1);
-	gtk_editable_insert_text(GTK_EDITABLE(entry), newtext, strlen(newtext),
+	if (!is_group) {
+		gtk_editable_insert_text(GTK_EDITABLE(entry), newtext, strlen(newtext),
 				 &start_pos);
+	} else {
+		gchar *addresses = NULL;
+		GList *cur = grp_emails;
+		for (; cur; cur = cur->next) {
+			gchar *tmp;
+			ItemEMail *email = (ItemEMail *)cur->data;
+			ItemPerson *person = ( ItemPerson * ) ADDRITEM_PARENT(email);
+			
+			gchar *addr = get_complete_address_from_name_email(
+				ADDRITEM_NAME(person), email->address);
+			if (addresses)
+				tmp = g_strdup_printf("%s, %s", addresses, addr);
+			else
+				tmp = g_strdup_printf("%s", addr);
+			g_free(addr);
+			g_free(addresses);
+			addresses = tmp;
+		}
+		gtk_editable_insert_text(GTK_EDITABLE(entry), addresses, strlen(addresses),
+				 &start_pos);
+		g_free(addresses);
+	}
 	gtk_editable_set_position(GTK_EDITABLE(entry), -1);
 }
 
@@ -403,10 +484,11 @@ static void replace_address_in_edit(GtkEntry *entry, const gchar *newtext,
  */
 guint complete_address(const gchar *str)
 {
-	GList *result;
-	gchar *d;
-	guint  count, cpl;
-	completion_entry *ce;
+	GList *result = NULL;
+	gchar *d = NULL;
+	guint  count = 0;
+	guint  cpl = 0;
+	completion_entry *ce = NULL;
 
 	g_return_val_if_fail(str != NULL, 0);
 
@@ -465,15 +547,17 @@ gchar *get_complete_address(gint index)
 			/* get something from the unique addresses */
 			p = (address_entry *)g_slist_nth_data
 				(g_completion_addresses, index - 1);
-			if (p != NULL) {
-				if (!p->name || p->name[0] == '\0')
-					address = g_strdup_printf(p->address);
-				else if (strchr_with_skip_quote(p->name, '"', ','))
-					address = g_strdup_printf
-						("\"%s\" <%s>", p->name, p->address);
-				else
-					address = g_strdup_printf
-						("%s <%s>", p->name, p->address);
+			if (p != NULL && p->address != NULL) {
+				address = get_complete_address_from_name_email(p->name, p->address);
+			} else if (p != NULL && p->address == NULL && p->name != NULL) {
+				/* that's a group */
+				address = g_strdup_printf("%s%s <!--___group___-->", p->name, _(" (Group)"));
+				if (!_groupAddresses_) {
+					_groupAddresses_ = g_hash_table_new(NULL, g_direct_equal);
+				}
+				if (!g_hash_table_lookup(_groupAddresses_, GINT_TO_POINTER(g_str_hash(address)))) {
+					g_hash_table_insert(_groupAddresses_, GINT_TO_POINTER(g_str_hash(address)), p->grp_emails);
+				}
 			}
 		}
 	}
@@ -525,7 +609,8 @@ gint invalidate_address_completion(void)
 		debug_print("Invalidation request for address completion\n");
 		free_all();
 		init_all();
-		read_address_book();
+		read_address_book(completion_folder_path);
+		if (g_completion_list)
 		g_completion_add_items(g_completion, g_completion_list);
 		clear_completion_cache();
 	}
@@ -564,7 +649,6 @@ static pthread_mutex_t _completionMutex_ = PTHREAD_MUTEX_INITIALIZER;
  * Completion queue list.
  */
 static GList *_displayQueue_ = NULL;
-
 /**
  * Current query ID.
  */
@@ -581,6 +665,7 @@ static guint _completionIdleID_ = 0;
  */
 
 #define ENTRY_DATA_TAB_HOOK	"tab_hook"	/* used to lookup entry */
+#define ENTRY_DATA_ALLOW_COMMAS	"allowcommas"	/* used to know whether to present groups */
 
 static void address_completion_mainwindow_set_focus	(GtkWindow   *window,
 							 GtkWidget   *widget,
@@ -733,6 +818,9 @@ static void addrcompl_resize_window( CompletionWindow *cw ) {
 		gtk_widget_set_size_request(cw->window, width, r.height);
 }
 
+static GdkPixbuf *group_pixbuf = NULL;
+static GdkPixbuf *email_pixbuf = NULL;
+
 /**
  * Add an address the completion window address list.
  * \param cw      Completion window.
@@ -742,12 +830,38 @@ static void addrcompl_add_entry( CompletionWindow *cw, gchar *address ) {
 	GtkListStore *store;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
-
+	gboolean is_group = FALSE;
+	GList *grp_emails = NULL;
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(cw->list_view)));
-	gtk_list_store_append(store, &iter);
+	GdkPixbuf *pixbuf;
+	
+	if (!group_pixbuf)
+		stock_pixbuf_gdk(cw->list_view, STOCK_PIXMAP_ADDR_TWO, &group_pixbuf);
+	if (!email_pixbuf)
+		stock_pixbuf_gdk(cw->list_view, STOCK_PIXMAP_ADDR_ONE, &email_pixbuf);
 
 	/* printf( "\t\tAdding :%s\n", address ); */
-	gtk_list_store_set(store, &iter, ADDR_COMPL_ADDRESS, address, -1);
+	if (strstr(address, " <!--___group___-->")) {
+		is_group = TRUE;
+		if (_groupAddresses_)
+			grp_emails = g_hash_table_lookup(_groupAddresses_, GINT_TO_POINTER(g_str_hash(address)));
+		*(strstr(address, " <!--___group___-->")) = '\0';
+		pixbuf = group_pixbuf;
+	} else if (strchr(address, '@') && strchr(address, '<') &&
+		   strchr(address, '>')) {
+		pixbuf = email_pixbuf;
+	} else
+		pixbuf = NULL;
+	
+	if (is_group && !_allowCommas_)
+		return;
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 
+				ADDR_COMPL_ICON, pixbuf,
+				ADDR_COMPL_ADDRESS, address, 
+				ADDR_COMPL_ISGROUP, is_group, 
+				ADDR_COMPL_GROUPLIST, grp_emails,
+				-1);
 	cw->listCount++;
 
 	/* Resize window */
@@ -905,8 +1019,11 @@ static void addrcompl_start_search( void ) {
  * moved to the next widget so that Tab key works correctly.
  * \param list_view List to process.
  * \param entry Address entry field.
+ * \param move_focus Move focus to the next widget ?
  */
-static void completion_window_apply_selection(GtkTreeView *list_view, GtkEntry *entry)
+static void completion_window_apply_selection(GtkTreeView *list_view,
+						GtkEntry *entry,
+						gboolean move_focus)
 {
 	gchar *address = NULL, *text = NULL;
 	gint   cursor_pos;
@@ -914,9 +1031,10 @@ static void completion_window_apply_selection(GtkTreeView *list_view, GtkEntry *
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-
+	gboolean is_group = FALSE;
 	g_return_if_fail(list_view != NULL);
 	g_return_if_fail(entry != NULL);
+	GList *grp_emails = NULL;
 
 	selection = gtk_tree_view_get_selection(list_view);
 	if (! gtk_tree_selection_get_selected(selection, &model, &iter))
@@ -929,16 +1047,19 @@ static void completion_window_apply_selection(GtkTreeView *list_view, GtkEntry *
 	}
 
 	/* Process selected item */
-	gtk_tree_model_get(model, &iter, ADDR_COMPL_ADDRESS, &text, -1);
+	gtk_tree_model_get(model, &iter, ADDR_COMPL_ADDRESS, &text, 
+				ADDR_COMPL_ISGROUP, &is_group, 
+				ADDR_COMPL_GROUPLIST, &grp_emails,
+				-1);
 
 	address = get_address_from_edit(entry, &cursor_pos);
 	g_free(address);
-	replace_address_in_edit(entry, text, cursor_pos);
+	replace_address_in_edit(entry, text, cursor_pos, is_group, grp_emails);
 	g_free(text);
 
 	/* Move focus to next widget */
 	parent = GTK_WIDGET(entry)->parent;
-	if( parent ) {
+	if( parent && move_focus) {
 		gtk_widget_child_focus( parent, GTK_DIR_TAB_FORWARD );
 	}
 }
@@ -950,7 +1071,7 @@ static void completion_window_apply_selection(GtkTreeView *list_view, GtkEntry *
  */
 void address_completion_start(GtkWidget *mainwindow)
 {
-	start_address_completion();
+	start_address_completion(NULL);
 
 	/* register focus change hook */
 	g_signal_connect(G_OBJECT(mainwindow), "set_focus",
@@ -968,13 +1089,14 @@ void address_completion_start(GtkWidget *mainwindow)
  * Register specified entry widget for address completion.
  * \param entry Address entry field.
  */
-void address_completion_register_entry(GtkEntry *entry)
+void address_completion_register_entry(GtkEntry *entry, gboolean allow_commas)
 {
 	g_return_if_fail(entry != NULL);
 	g_return_if_fail(GTK_IS_ENTRY(entry));
 
 	/* add hooked property */
 	g_object_set_data(G_OBJECT(entry), ENTRY_DATA_TAB_HOOK, entry);
+	g_object_set_data(G_OBJECT(entry), ENTRY_DATA_ALLOW_COMMAS, GINT_TO_POINTER(allow_commas));
 
 	/* add keypress event */
 	g_signal_connect_closure
@@ -1030,6 +1152,7 @@ static void address_completion_mainwindow_set_focus(GtkWindow *window,
 	
 	if (widget && GTK_IS_ENTRY(widget) &&
 	    g_object_get_data(G_OBJECT(widget), ENTRY_DATA_TAB_HOOK)) {
+		_allowCommas_ = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), ENTRY_DATA_ALLOW_COMMAS));
 		clear_completion_cache();
 	}
 }
@@ -1047,7 +1170,7 @@ static gboolean address_completion_entry_key_pressed(GtkEntry    *entry,
 {
 	if (ev->keyval == GDK_Tab) {
 		addrcompl_clear_queue();
-
+		_allowCommas_ = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), ENTRY_DATA_ALLOW_COMMAS));
 		if( address_completion_complete_address_in_entry( entry, TRUE ) ) {
 			/* route a void character to the default handler */
 			/* this is a dirty hack; we're actually changing a key
@@ -1102,9 +1225,7 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 	/* printf( "search for :::%s:::\n", searchTerm ); */
 
 	/* Clear any existing search */
-	if( _compWindow_->searchTerm ) {
-		g_free( _compWindow_->searchTerm );
-	}
+	g_free( _compWindow_->searchTerm );
 	_compWindow_->searchTerm = g_strdup( searchTerm );
 
 	/* Perform search on local completion index */
@@ -1113,20 +1234,17 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 		new = get_next_complete_address();
 		g_free( new );
 	}
-
 #ifndef USE_LDAP
 	/* Select the address if there is only one match */
 	if (ncount == 2) {
 		/* Display selected address in entry field */		
 		gchar *addr = get_complete_address(1);
-
-		if (addr) {
-			replace_address_in_edit(entry, addr, cursor_pos);
-			g_free(addr);
-		}
-
-		/* Discard the window */
-		clear_completion_cache();
+		if (addr && !strstr(addr, " <!--___group___-->")) {
+			replace_address_in_edit(entry, addr, cursor_pos, FALSE, NULL);
+			/* Discard the window */
+			clear_completion_cache();
+		} 
+		g_free(addr);
 	}
 	/* Make sure that drop-down appears uniform! */
 	else 
@@ -1170,7 +1288,8 @@ static void address_completion_create_completion_window( GtkEntry *entry_ )
 				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(window), scroll);
 	gtk_container_add(GTK_CONTAINER(scroll), list_view);
-
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
+		GTK_SHADOW_OUT);
 	/* Use entry widget to create initial window */
 	gdk_window_get_geometry(entry->window, &x, &y, &width, &height, &depth);
 	gdk_window_get_deskrelative_origin (entry->window, &x, &y);
@@ -1254,7 +1373,7 @@ static gboolean completion_window_button_press(GtkWidget *widget,
 		/* Clicked outside of completion window - restore */
 		searchTerm = _compWindow_->searchTerm;
 		g_free(get_address_from_edit(GTK_ENTRY(entry), &cursor_pos));
-		replace_address_in_edit(GTK_ENTRY(entry), searchTerm, cursor_pos);
+		replace_address_in_edit(GTK_ENTRY(entry), searchTerm, cursor_pos, FALSE, NULL);
 	}
 
 	clear_completion_cache();
@@ -1279,7 +1398,6 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	gint cursor_pos;
 	GtkWidget *list_view;
 	GtkWidget *parent;
-
 	g_return_val_if_fail(compWin != NULL, FALSE);
 
 	entry = compWin->entry;
@@ -1327,14 +1445,23 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 		}
 		return FALSE;
 	}
+	_allowCommas_ = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(entry), ENTRY_DATA_ALLOW_COMMAS));
 
 	/* look for presses that accept the selection */
-	if (event->keyval == GDK_Return || event->keyval == GDK_space) {
+	if (event->keyval == GDK_Return || event->keyval == GDK_space
+			|| (_allowCommas_ && event->keyval == GDK_comma)) {
 		/* User selected address with a key press */
 
 		/* Display selected address in entry field */		
 		completion_window_apply_selection(
-			GTK_TREE_VIEW(list_view), GTK_ENTRY(entry) );
+			GTK_TREE_VIEW(list_view), GTK_ENTRY(entry),
+			event->keyval != GDK_comma);
+
+		if (event->keyval == GDK_comma) {
+			gint pos = gtk_editable_get_position(GTK_EDITABLE(entry));
+			gtk_editable_insert_text(GTK_EDITABLE(entry), ", ", 2, &pos);
+			gtk_editable_set_position(GTK_EDITABLE(entry), pos + 1);
+		}
 
 		/* Discard the window */
 		clear_completion_cache();
@@ -1359,7 +1486,7 @@ static gboolean completion_window_key_press(GtkWidget *widget,
 	/* some other key, let's restore the searchTerm (orignal text) */
 	searchTerm = _compWindow_->searchTerm;
 	g_free(get_address_from_edit(GTK_ENTRY(entry), &cursor_pos));
-	replace_address_in_edit(GTK_ENTRY(entry), searchTerm, cursor_pos);
+	replace_address_in_edit(GTK_ENTRY(entry), searchTerm, cursor_pos, FALSE, NULL);
 
 	/* make sure anything we typed comes in the edit box */
 	tmp_event.type       = event->type;
@@ -1420,7 +1547,10 @@ void addrcompl_teardown( void ) {
 static GtkListStore *addr_compl_create_store(void)
 {
 	return gtk_list_store_new(N_ADDR_COMPL_COLUMNS,
+				  GDK_TYPE_PIXBUF,
 				  G_TYPE_STRING,
+				  G_TYPE_BOOLEAN,
+				  G_TYPE_POINTER,
 				  -1);
 }
 					     
@@ -1434,7 +1564,7 @@ static GtkWidget *addr_compl_list_view_create(CompletionWindow *window)
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);	
 	
-	gtk_tree_view_set_rules_hint(list_view, prefs_common.enable_rules_hint);
+	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 	gtk_tree_view_set_headers_visible(list_view, FALSE);
 	
 	selector = gtk_tree_view_get_selection(list_view);
@@ -1453,6 +1583,11 @@ static void addr_compl_create_list_view_columns(GtkWidget *list_view)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 
+	renderer = gtk_cell_renderer_pixbuf_new();
+	column = gtk_tree_view_column_new_with_attributes
+		("", renderer,
+	         "pixbuf", ADDR_COMPL_ICON, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);		
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes
 		("", renderer, "text", ADDR_COMPL_ADDRESS, NULL);
@@ -1509,7 +1644,7 @@ static gboolean addr_compl_defer_select_destruct(CompletionWindow *window)
 	GtkEntry *entry = GTK_ENTRY(window->entry);
 
 	completion_window_apply_selection(GTK_TREE_VIEW(window->list_view), 
-					  entry);
+					  entry, TRUE);
 
 	clear_completion_cache();
 
