@@ -23,7 +23,7 @@
 
 #include <gtk/gtk.h>
 
-gint start_address_completion		(void);
+gint start_address_completion		(gchar *folderpath);
 guint complete_address			(const gchar *str);
 gchar *get_complete_address		(gint index);
 gint invalidate_address_completion	(void);
@@ -31,7 +31,7 @@ gint end_address_completion		(void);
 
 /* ui functions */
 void address_completion_start		(GtkWidget *mainwindow);
-void address_completion_register_entry	(GtkEntry  *entry);
+void address_completion_register_entry	(GtkEntry  *entry, gboolean allow_commas);
 void address_completion_unregister_entry(GtkEntry  *entry);
 void address_completion_end		(GtkWidget *mainwindow);
 
