@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2002 Match Grun
+ * Copyright (C) 2002-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -46,8 +46,8 @@ static gchar *_headerTo_       = HEADER_TO;
 #define DFL_FOLDER_SIZE  20
 
 /* Noise strings included by some other E-Mail clients */
-#define REM_NAME_STRING  "(E-mail)"
-#define REM_NAME_STRING2 "(E-mail 2)"
+#define REM_NAME_STRING  "(Email)"
+#define REM_NAME_STRING2 "(Email 2)"
 
 /* Directories to ignore */
 #define DIR_IGNORE ".\t..\t.sylpheed_mark\t.sylpheed_cache"
