@@ -34,6 +34,7 @@
 #include "addritem.h"
 #include "addrcache.h"
 #include "addrbook.h"
+#include "addressbook.h"
 #include "addrindex.h"
 #include "xml.h"
 #include "addrquery.h"
@@ -98,6 +99,7 @@
 #define ATTAG_LDAP_DYN_SEARCH "dyn-search"
 #define ATTAG_LDAP_MATCH_OPT  "match-opt"
 #define ATTAG_LDAP_ENABLE_TLS "enable-tls"
+#define ATTAG_LDAP_ENABLE_SSL "enable-ssl"
 
 #define ELTAG_LDAP_ATTR_SRCH  "attribute"
 #define ATTAG_LDAP_ATTR_NAME  "name"
@@ -217,6 +219,7 @@ static void addrindex_build_if_list( AddressIndex *addrIndex ) {
 	iface->getListFolder = ( void * ) addrbook_get_list_folder;
 	iface->getListPerson = ( void * ) addrbook_get_list_person;
 	iface->getAllPersons = ( void * ) addrbook_get_all_persons;
+	iface->getAllGroups  = ( void * ) addrbook_get_all_groups;
 	iface->getName       = ( void * ) addrbook_get_name;
 	iface->setAccessFlag = ( void * ) addrbook_set_accessed;
 	iface->searchOrder   = 0;
@@ -1315,13 +1318,14 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 	gchar *serverName = NULL;
 	gchar *criteria = NULL;
 	gboolean bDynSearch;
-	gboolean bTLS;
+	gboolean bTLS, bSSL;
 	gint iMatch;
 
 	/* printf( "addrindex_parse_ldap\n" ); */
 	/* Set up some defaults */
 	bDynSearch = FALSE;
 	bTLS = FALSE;
+	bSSL = FALSE;
 	iMatch = LDAPCTL_MATCH_BEGINWITH;
 
 	ds = addrindex_create_datasource( ADDR_IF_LDAP );
@@ -1333,7 +1337,7 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 		gint ivalue = atoi( value );
 
 		if( strcmp( name, ATTAG_LDAP_NAME ) == 0 ) {
-			if( serverName ) g_free( serverName );
+			g_free( serverName );
 			serverName = g_strdup( value );
 		}
 		else if( strcmp( name, ATTAG_LDAP_HOST ) == 0 ) {
@@ -1352,7 +1356,7 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 			ldapctl_set_bind_password( ctl, value );
 		}
 		else if( strcmp( name, ATTAG_LDAP_CRITERIA ) == 0 ) {
-			if( criteria ) g_free( criteria );
+			g_free( criteria );
 			criteria = g_strdup( value );
 		}
 		else if( strcmp( name, ATTAG_LDAP_MAX_ENTRY ) == 0 ) {
@@ -1382,6 +1386,12 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 				bTLS = TRUE;
 			}
 		}
+		else if( strcmp( name, ATTAG_LDAP_ENABLE_SSL ) == 0 ) {
+			bSSL = FALSE;
+			if( strcmp( value, ATVAL_BOOLEAN_YES ) == 0 ) {
+				bSSL = TRUE;
+			}
+		}
 		attr = g_list_next( attr );
 	}
 
@@ -1389,7 +1399,10 @@ static AddressDataSource *addrindex_parse_ldap( XMLFile *file ) {
 	ldapsvr_set_name( server, serverName );
 	ldapsvr_set_search_flag( server, bDynSearch );
 	ldapctl_set_matching_option( ctl, iMatch );
+#ifdef USE_LDAP_TLS
 	ldapctl_set_tls( ctl, bTLS );
+	ldapctl_set_ssl( ctl, bSSL );
+#endif
 	g_free( serverName );
 	ldapsvr_set_control( server, ctl );
 	ds->rawDataSource = server;
@@ -1452,6 +1465,9 @@ static void addrindex_write_ldap( FILE *fp, AddressDataSource *ds, gint lvl ) {
 	addrindex_write_attr( fp, ATTAG_LDAP_ENABLE_TLS,
 			ctl->enableTLS ?
 			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO );
+	addrindex_write_attr( fp, ATTAG_LDAP_ENABLE_SSL,
+			ctl->enableSSL ?
+			ATVAL_BOOLEAN_YES : ATVAL_BOOLEAN_NO );
 
 	fputs(" >\n", fp);
 
@@ -2774,78 +2790,174 @@ void addrindex_remove_results( AddressDataSource *ds, ItemFolder *folder ) {
 * ***********************************************************************
 */
 
+static void addrindex_load_completion_load_persons(
+		gint (*callBackFunc) ( const gchar *, const gchar *, 
+				       const gchar *, const gchar *, GList * ),
+		AddressDataSource *ds)
+{
+	GList *listP, *nodeP;
+	GList *nodeM;
+	gchar *sName;
+
+	/* Read address book */
+	if( addrindex_ds_get_modify_flag( ds ) ) {
+		addrindex_ds_read_data( ds );
+	}
+
+	if( ! addrindex_ds_get_read_flag( ds ) ) {
+		addrindex_ds_read_data( ds );
+	}
+
+	/* Get all groups */
+	listP = addrindex_ds_get_all_groups( ds );
+	nodeP = listP;
+	while( nodeP ) {
+		ItemGroup *group = nodeP->data;
+		GList *emails = NULL;
+		for (nodeM = group->listEMail; nodeM; nodeM = g_list_next(nodeM)) {
+			ItemEMail *email = nodeM->data;
+			if (email->address)
+				emails = g_list_append(emails, email);
+		}
+		callBackFunc( ((AddrItemObject *)group)->name, NULL,
+			      NULL, NULL, emails );
+		nodeP = g_list_next( nodeP );
+	}
+
+	/* Free up the list */
+	g_list_free( listP );
+	/* Get all persons */
+	listP = addrindex_ds_get_all_persons( ds );
+	nodeP = listP;
+	while( nodeP ) {
+		ItemPerson *person = nodeP->data;
+		nodeM = person->listEMail;
+
+		/* Figure out name to use */
+		sName = ADDRITEM_NAME(person);
+		if( sName == NULL || *sName == '\0' ) {
+			sName = person->nickName;
+		}
+
+		/* Process each E-Mail address */
+		while( nodeM ) {
+			ItemEMail *email = nodeM->data;
+
+			callBackFunc( sName, email->address, person->nickName, 
+				      ADDRITEM_NAME(email), NULL );
+
+			nodeM = g_list_next( nodeM );
+		}
+		nodeP = g_list_next( nodeP );
+	}
+
+	/* Free up the list */
+	g_list_free( listP );
+}		
+
 /**
  * This function is used by the address completion function to load
  * addresses for all non-external address book interfaces.
  *
  * \param callBackFunc Function to be called when an address is
  *                     to be loaded.
+ * \param folderpath Addressbook's Book/folder path to restrict to (if NULL or ""
+ *                     or "Any", assume the whole addressbook
  * \return <i>TRUE</i> if data loaded, <i>FALSE</i> if address index not loaded.
  */
+
 gboolean addrindex_load_completion(
 		gint (*callBackFunc) ( const gchar *, const gchar *, 
-				       const gchar *, const gchar * ) )
+				       const gchar *, const gchar *, GList * ),
+		gchar *folderpath )
 {
-	AddressDataSource *ds;
 	GList *nodeIf, *nodeDS;
-	GList *listP, *nodeP;
-	GList *nodeM;
-	gchar *sName;
 
-	nodeIf = addrindex_get_interface_list( _addressIndex_ );
-	while( nodeIf ) {
-		AddressInterface *iface = nodeIf->data;
+	if( folderpath != NULL ) {
+		AddressDataSource *book;
+		ItemFolder* folder;
 
-		nodeIf = g_list_next( nodeIf );
-		if( ! iface->useInterface ) {
-			continue;
+		/* split the folder path we've received, we'll try to match this path, subpath by
+		   subpath against the book/folder structure in order and restrict loading of
+		   addresses to that subpart (if matches). book/folder path must exist and
+		   folderpath must not be empty or NULL */
+		
+		if( ! addressbook_peek_folder_exists( folderpath, &book, &folder ) ) {
+			g_warning("addrindex_load_completion: folder path '%s' doesn't exist\n", folderpath);
+			return FALSE;
 		}
-		if( iface->externalQuery ) {
-			continue;
+
+		if( book != NULL ) {
+			AddressBookFile *abf = book->rawDataSource;
+
+			debug_print("addrindex_load_completion: book %p '%s'\n", book, abf->fileName);
+
+			addrindex_load_completion_load_persons( callBackFunc, book );
+
+			return TRUE;
+
+		} else {
+
+			if( folder != NULL ) {
+				GList *items;
+				GList *nodeM;
+				gchar *sName;
+				ItemPerson *person;
+
+				debug_print("addrindex_load_completion: folder %p '%s'\n", folder, folder->obj.name);
+
+				/* Load email addresses */
+				items = addritem_folder_get_person_list( folder );
+				for( ; items != NULL; items = g_list_next( items ) ) {
+					person = items->data;
+					nodeM = person->listEMail;
+
+					/* Figure out name to use */
+					sName = ADDRITEM_NAME(person);
+					if( sName == NULL || *sName == '\0' ) {
+						sName = person->nickName;
+					}
+
+					/* Process each E-Mail address */
+					while( nodeM ) {
+						ItemEMail *email = nodeM->data;
+
+						callBackFunc( sName, email->address, person->nickName, 
+								  ADDRITEM_NAME(email), NULL );
+
+						nodeM = g_list_next( nodeM );
+					}
+				}
+				/* Free up the list */
+				mgu_clear_list( items );
+				g_list_free( items );
+
+				return TRUE;
+
+			} else {
+				g_warning("addrindex_load_completion: book/folder path is valid but got no pointer\n");
+			}
 		}
-		nodeDS = iface->listSource;
-		while( nodeDS ) {
-			ds = nodeDS->data;
+		return FALSE;
 
-			/* Read address book */
-			if( addrindex_ds_get_modify_flag( ds ) ) {
-				addrindex_ds_read_data( ds );
-			}
+	} else {
 
-			if( ! addrindex_ds_get_read_flag( ds ) ) {
-				addrindex_ds_read_data( ds );
-			}
+		nodeIf = addrindex_get_interface_list( _addressIndex_ );
+		while( nodeIf ) {
+			AddressInterface *iface = nodeIf->data;
 
-			/* Get all persons */
-			listP = addrindex_ds_get_all_persons( ds );
-			nodeP = listP;
-			while( nodeP ) {
-				ItemPerson *person = nodeP->data;
-				nodeM = person->listEMail;
+			nodeIf = g_list_next( nodeIf );
 
-				/* Figure out name to use */
-				sName = ADDRITEM_NAME(person);
-				if( sName == NULL || *sName == '\0' ) {
-					sName = person->nickName;
-				}
-
-				/* Process each E-Mail address */
-				while( nodeM ) {
-					ItemEMail *email = nodeM->data;
-					
-					callBackFunc( sName, email->address, person->nickName, 
-						      ADDRITEM_NAME(email) );
-					
-					nodeM = g_list_next( nodeM );
-				}
-				nodeP = g_list_next( nodeP );
-			}
-			/* Free up the list */
-			g_list_free( listP );
+			if( ! iface->useInterface || iface->externalQuery )
+				continue;
 
+			nodeDS = iface->listSource;
+			while( nodeDS ) {
+				addrindex_load_completion_load_persons( callBackFunc, nodeDS->data );
 			nodeDS = g_list_next( nodeDS );
 		}
 	}
+	}
 
 	return TRUE;
 }
@@ -2873,12 +2985,10 @@ gboolean addrindex_load_person_attribute(
 		AddressInterface *iface = nodeIf->data;
 
 		nodeIf = g_list_next( nodeIf );
-		if( ! iface->useInterface ) {
+
+		if( ! iface->useInterface || iface->externalQuery )
 			continue;
-		}
-		if( iface->externalQuery ) {
-			continue;
-		}
+
 		nodeDS = iface->listSource;
 		while( nodeDS ) {
 			ds = nodeDS->data;
