@@ -73,11 +73,12 @@ AlertValue alertpanel_with_widget(const gchar *title,
 				  const gchar *button1_label,
 				  const gchar *button2_label,
 				  const gchar *button3_label,
+				  gboolean     can_disable,
 				  GtkWidget *widget)
 {
 	return alertpanel_full(title, message, button1_label,
 				    button2_label, button3_label,
-				    FALSE, widget, ALERT_QUESTION,
+				    can_disable, widget, ALERT_QUESTION,
 				    G_ALERTDEFAULT);
 }
 
@@ -156,7 +157,7 @@ void alertpanel_warning(const gchar *format, ...)
 void alertpanel_error(const gchar *format, ...)
 {
 	va_list args;
-	gchar buf[256];
+	gchar buf[512];
 
 	va_start(args, format);
 	g_vsnprintf(buf, sizeof(buf), format, args);
@@ -185,6 +186,7 @@ void alertpanel_error_log(const gchar *format, ...)
 	mainwin = mainwindow_get_mainwindow();
 	
 	if (mainwin && mainwin->logwin) {
+		mainwindow_clear_error(mainwin);
 		val = alertpanel_full(_("Error"), buf, GTK_STOCK_CLOSE,
 				      _("_View log"), NULL, FALSE, NULL,
 				      ALERT_ERROR, G_ALERTDEFAULT);
@@ -246,7 +248,7 @@ static void alertpanel_create(const gchar *title,
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
 	gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
 
-	gtk_window_set_default_size(GTK_WINDOW(dialog), 300, 100);
+	gtk_window_set_default_size(GTK_WINDOW(dialog), 375, 100);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
@@ -316,6 +318,7 @@ static void alertpanel_create(const gchar *title,
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 	GTK_WIDGET_UNSET_FLAGS(label, GTK_CAN_FOCUS);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
