@@ -42,10 +42,23 @@ struct _Plugin
 	const gchar *(*version) (void);
 	const gchar *(*type) (void);
 	const gchar *(*licence) (void);
+	struct PluginFeature *(*provides) (void);
+	
 	GSList *rdeps;
 	gchar *error;
 };
 
+const gchar *plugin_feature_names[] =
+	{ N_("Nothing"),
+	  N_("a viewer"),
+	  N_("folders"),
+	  N_("filtering"),
+	  N_("a privacy interface"),
+	  N_("a notifier"),
+	  N_("an utility"),
+	  N_("things"),
+	  NULL };
+
 /**
  * List of all loaded plugins
  */
@@ -64,10 +77,15 @@ static gint list_find_by_string(gconstpointer data, gconstpointer str)
 
 static gint list_find_by_plugin_filename(const Plugin *plugin, const gchar *filename)
 {
-        g_return_val_if_fail(plugin, 1);
-        g_return_val_if_fail(plugin->filename, 1);
-        g_return_val_if_fail(filename, 1);
-        return strcmp(filename, plugin->filename);
+	/* FIXME: There is a problem in case of symlinks or when a
+	   user tries to load a plugin with the same name from a
+	   different directory.  I think it would be better to compare
+	   only the basename of the filename here (case-insensitive on
+	   W32). */
+	g_return_val_if_fail(plugin, 1);
+	g_return_val_if_fail(plugin->filename, 1);
+	g_return_val_if_fail(filename, 1);
+	return strcmp(filename, plugin->filename);
 }
 
 void plugin_save_list(void)
@@ -116,7 +134,7 @@ void plugin_save_list(void)
 static gboolean plugin_is_loaded(const gchar *filename)
 {
 	return (g_slist_find_custom(plugins, filename, 
-                  (GCompareFunc)list_find_by_plugin_filename) != NULL);
+		  (GCompareFunc)list_find_by_plugin_filename) != NULL);
 }
 
 static Plugin *plugin_get_by_filename(const gchar *filename)
@@ -146,9 +164,11 @@ static gint plugin_load_deps(const gchar *filename, gchar **error)
 	gchar *deps_file = NULL;
 	FILE *fp = NULL;
 	gchar buf[BUFFSIZE];
+	gchar *p;
 
-        tmp = g_strdup(filename);
-	*strrchr(tmp, '.') = '\0';
+	tmp = g_strdup(filename);
+	if( (p = strrchr(tmp, '.')) )
+	  *p = '\0';
 	deps_file = g_strconcat(tmp, ".deps", NULL);
 	g_free(tmp);
 	
@@ -220,6 +240,32 @@ static void plugin_remove_from_unloaded_list (const gchar *filename)
 	}
 }
 
+static gchar *plugin_check_features(struct PluginFeature *features) {
+	int i = 0, j = 0;
+	GSList *cur = plugins;
+
+	if (features == NULL)
+		return NULL;
+	for(; cur; cur = cur->next) {
+		Plugin *p = (Plugin *)cur->data;
+		struct PluginFeature *cur_features = p->provides();
+		for (j = 0; cur_features[j].type != PLUGIN_NOTHING; j++) {
+			for (i = 0; features[i].type != PLUGIN_NOTHING; i++) {
+				if (cur_features[j].type == features[i].type &&
+				    !strcmp(cur_features[j].subtype, features[i].subtype)) {
+					return g_strdup_printf(_(
+						"This plugin provides %s (%s), which is "
+						"already provided by the %s plugin."),
+						_(plugin_feature_names[features[i].type]), 
+						_(features[i].subtype),
+						p->name());
+				}
+			}
+		}
+	}
+
+	return NULL;
+}
 /**
  * Loads a plugin
  *
@@ -234,6 +280,8 @@ Plugin *plugin_load(const gchar *filename, gchar **error)
 	gpointer plugin_name, plugin_desc, plugin_version;
 	const gchar *(*plugin_type)(void);
 	const gchar *(*plugin_licence)(void);
+	struct PluginFeature *(*plugin_provides)(void);
+
 	gint ok;
 
 	g_return_val_if_fail(filename != NULL, NULL);
@@ -242,8 +290,8 @@ Plugin *plugin_load(const gchar *filename, gchar **error)
 	/* check duplicate plugin path name */
 	if (plugin_is_loaded(filename)) {
 		*error = g_strdup(_("Plugin already loaded"));
-		return NULL;                
-	}                               
+		return NULL;		
+	}			       
 	
 	plugin_remove_from_unloaded_list(filename);
 	
@@ -255,7 +303,7 @@ Plugin *plugin_load(const gchar *filename, gchar **error)
 		return NULL;
 	}
 
-        debug_print("trying to load `%s'\n", filename);
+	debug_print("trying to load `%s'\n", filename);
 	plugin->module = g_module_open(filename, 0);
 	if (plugin->module == NULL) {
 		*error = g_strdup(g_module_error());
@@ -268,6 +316,7 @@ Plugin *plugin_load(const gchar *filename, gchar **error)
 	    !g_module_symbol(plugin->module, "plugin_version", &plugin_version) ||
 	    !g_module_symbol(plugin->module, "plugin_type", (gpointer)&plugin_type) ||
 	    !g_module_symbol(plugin->module, "plugin_licence", (gpointer)&plugin_licence) ||
+	    !g_module_symbol(plugin->module, "plugin_provides", (gpointer)&plugin_provides) ||
 	    !g_module_symbol(plugin->module, "plugin_init", (gpointer)&plugin_init)) {
 		*error = g_strdup(g_module_error());
 		g_module_close(plugin->module);
@@ -290,11 +339,17 @@ Plugin *plugin_load(const gchar *filename, gchar **error)
 		return NULL;
 	}
 
+	if ((*error = plugin_check_features(plugin_provides())) != NULL) {
+		g_module_close(plugin->module);
+		g_free(plugin);
+		return NULL;
+	}
 	plugin->name = plugin_name;
 	plugin->desc = plugin_desc;
 	plugin->version = plugin_version;
 	plugin->type = plugin_type;
 	plugin->licence = plugin_licence;
+	plugin->provides = plugin_provides;
 	plugin->filename = g_strdup(filename);
 	plugin->error = NULL;
 	if ((ok = plugin_init(error)) < 0) {
@@ -390,6 +445,52 @@ void plugin_unload_all(const gchar *type)
 	}
 }
 
+
+/* Load those plugins we always want to use.  No error output; just
+ * try. */
+void plugin_load_standard_plugins (void)
+{
+	static const char *names[] = {
+#ifdef G_OS_WIN32 
+		"pgpmime",
+		"pgpinline",
+#else
+		/* post-2.5 maybe 
+		"bogofilter", */
+#endif
+		NULL
+	};
+	int i;
+	gchar *error, *filename;
+	
+	for (i=0; names[i]; i++) {
+		/* Simple hack to check whether the plugin has already
+		 * been loaded but checking only for the basename. */
+		GSList *cur = plugins;
+		for(; cur; cur = cur->next) {
+			Plugin *p = (Plugin *)cur->data;
+			if (strstr(p->filename, names[i]))
+				break;
+		}
+		if (!cur) { /* Not yet loaded. */
+			/* FIXME: get_plugin_dir () returns with a trailing
+			 * (back)slash; this should be fixed so that we can use
+			 * g_module_build_path here. */
+#ifdef G_OS_WIN32 
+			filename = g_strconcat (get_plugin_dir(),
+						names[i], NULL);
+#else
+			filename = g_strconcat (get_plugin_dir(),
+						names[i], ".", G_MODULE_SUFFIX, NULL);
+#endif
+			error = NULL;
+			plugin_load(filename, &error);
+			g_free (error);
+			g_free(filename);
+		}
+	}
+}
+
 GSList *plugin_get_list(void)
 {
 	return g_slist_copy(plugins);
