@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -343,7 +343,7 @@ gint session_start_tls(Session *session)
 		sock_set_nonblocking_mode(session->sock, FALSE);
 
 	if (!ssl_init_socket_with_method(session->sock, SSL_METHOD_TLSv1)) {
-		g_warning("can't start TLS session.\n");
+		g_warning("couldn't start TLS session.\n");
 		if (nb_mode)
 			sock_set_nonblocking_mode(session->sock, TRUE);
 		return -1;
