@@ -67,8 +67,10 @@ struct _SockInfo
 
 	SockFunc callback;
 	GIOCondition condition;
+	gchar *canonical_name;
 };
 
+void refresh_resolvers			(void);
 gint sock_init				(void);
 gint sock_cleanup			(void);
 
