@@ -193,7 +193,7 @@ gboolean ssl_init_socket_with_method(SockInfo *sockinfo, SSLMethod method)
 	}
 
 
-	if (!ssl_certificate_check(server_cert, sockinfo->hostname, sockinfo->port)) {
+	if (!ssl_certificate_check(server_cert, sockinfo->canonical_name, sockinfo->hostname, sockinfo->port)) {
 		X509_free(server_cert);
 		SSL_free(ssl);
 		return FALSE;
@@ -208,8 +208,9 @@ gboolean ssl_init_socket_with_method(SockInfo *sockinfo, SSLMethod method)
 
 void ssl_done_socket(SockInfo *sockinfo)
 {
-	if (sockinfo->ssl) {
+	if (sockinfo && sockinfo->ssl) {
 		SSL_free(sockinfo->ssl);
+		sockinfo->ssl = NULL;
 	}
 }
 
