@@ -34,8 +34,56 @@
 #include "socket.h"
 #include "hooks.h"
 
+static GHashTable *warned_expired = NULL;
+
 static SSLCertificate *ssl_certificate_new_lookup(X509 *x509_cert, gchar *host, gushort port, gboolean lookup);
 
+/* from Courier */
+time_t asn1toTime(ASN1_TIME *asn1Time)
+{
+	struct tm tm;
+	int offset;
+
+	if (asn1Time == NULL || asn1Time->length < 13)
+		return 0;
+
+	memset(&tm, 0, sizeof(tm));
+
+#define N2(n)	((asn1Time->data[n]-'0')*10 + asn1Time->data[(n)+1]-'0')
+
+#define CPY(f,n) (tm.f=N2(n))
+
+	CPY(tm_year,0);
+
+	if(tm.tm_year < 50)
+		tm.tm_year += 100; /* Sux */
+
+	CPY(tm_mon, 2);
+	--tm.tm_mon;
+	CPY(tm_mday, 4);
+	CPY(tm_hour, 6);
+	CPY(tm_min, 8);
+	CPY(tm_sec, 10);
+
+	offset=0;
+
+	if (asn1Time->data[12] != 'Z')
+	{
+		if (asn1Time->length < 17)
+			return 0;
+
+		offset=N2(13)*3600+N2(15)*60;
+
+		if (asn1Time->data[12] == '-')
+			offset= -offset;
+	}
+
+#undef N2
+#undef CPY
+
+	return mktime(&tm)-offset;
+}
+
 static char * get_fqdn(char *host)
 {
 	struct hostent *hp;
@@ -194,22 +242,14 @@ char* ssl_certificate_to_string(SSLCertificate *cert)
 				fingerprint,
 				(sig_status==NULL ? "correct":sig_status));
 
-	if (issuer_commonname)
-		g_free(issuer_commonname);
-	if (issuer_location)
-		g_free(issuer_location);
-	if (issuer_organization)
-		g_free(issuer_organization);
-	if (subject_commonname)
-		g_free(subject_commonname);
-	if (subject_location)
-		g_free(subject_location);
-	if (subject_organization)
-		g_free(subject_organization);
-	if (fingerprint)
-		g_free(fingerprint);
-	if (sig_status)
-		g_free(sig_status);
+	g_free(issuer_commonname);
+	g_free(issuer_location);
+	g_free(issuer_organization);
+	g_free(subject_commonname);
+	g_free(subject_location);
+	g_free(subject_organization);
+	g_free(fingerprint);
+	g_free(sig_status);
 	return ret;
 }
 	
@@ -220,8 +260,7 @@ void ssl_certificate_destroy(SSLCertificate *cert)
 
 	if (cert->x509_cert)
 		X509_free(cert->x509_cert);
-	if (cert->host)	
-		g_free(cert->host);
+	g_free(cert->host);
 	g_free(cert);
 	cert = NULL;
 }
@@ -288,7 +327,7 @@ static gboolean ssl_certificate_compare (SSLCertificate *cert_a, SSLCertificate
 	if (cert_a == NULL || cert_b == NULL)
 		return FALSE;
 	else if (!X509_cmp(cert_a->x509_cert, cert_b->x509_cert))
-		return TRUE;	
+		return TRUE;
 	else
 		return FALSE;
 }
@@ -306,7 +345,7 @@ char *ssl_certificate_check_signer (X509 *cert)
 	}
 	if (!X509_STORE_set_default_paths(store)) {
 		X509_STORE_free (store);
-		return g_strdup(_("Can't load X509 default paths"));
+		return g_strdup(_("Couldn't load X509 default paths"));
 	}
 	
 	X509_STORE_CTX_init (&store_ctx, store, cert, NULL);
@@ -325,41 +364,42 @@ char *ssl_certificate_check_signer (X509 *cert)
 	return NULL;
 }
 
-gboolean ssl_certificate_check (X509 *x509_cert, gchar *host, gushort port)
+gboolean ssl_certificate_check (X509 *x509_cert, gchar *fqdn, gchar *host, gushort port)
 {
-	SSLCertificate *current_cert = ssl_certificate_new(x509_cert, host, port);
+	SSLCertificate *current_cert = NULL;
 	SSLCertificate *known_cert;
 	SSLCertHookData cert_hook_data;
+	gchar *fqdn_host = NULL;	
+	
+	if (fqdn)
+		fqdn_host = g_strdup(fqdn);
+	else if (host)
+		fqdn_host = get_fqdn(host);
+	else {
+		g_warning("no host!\n");
+		return FALSE;
+	}
+		
+	current_cert = ssl_certificate_new_lookup(x509_cert, fqdn_host, port, FALSE);
 	
 	if (current_cert == NULL) {
 		debug_print("Buggy certificate !\n");
+		g_free(fqdn_host);
 		return FALSE;
 	}
 
-	known_cert = ssl_certificate_find (host, port);
+	known_cert = ssl_certificate_find_lookup (fqdn_host, port, FALSE);
+
+	g_free(fqdn_host);
 
 	if (known_cert == NULL) {
-		gchar *err_msg, *cur_cert_str, *sig_status;
-		
-		sig_status = ssl_certificate_check_signer(x509_cert);
-
-		g_free(sig_status);
-
-		cur_cert_str = ssl_certificate_to_string(current_cert);
-		
-		err_msg = g_strdup_printf(_("%s presented an unknown SSL certificate:\n%s"),
-					  current_cert->host,
-					  cur_cert_str);
-		g_free (cur_cert_str);
-
 		cert_hook_data.cert = current_cert;
 		cert_hook_data.old_cert = NULL;
+		cert_hook_data.expired = FALSE;
 		cert_hook_data.accept = FALSE;
 		
 		hooks_invoke(SSLCERT_ASK_HOOKLIST, &cert_hook_data);
 		
-		g_free(err_msg);
-
 		if (!cert_hook_data.accept) {
 			ssl_certificate_destroy(current_cert);
 			return FALSE;
@@ -368,26 +408,13 @@ gboolean ssl_certificate_check (X509 *x509_cert, gchar *host, gushort port)
 			ssl_certificate_destroy(current_cert);
 			return TRUE;
 		}
-	}
-	else if (!ssl_certificate_compare (current_cert, known_cert)) {
-		gchar *err_msg, *known_cert_str, *cur_cert_str;
-		
-		known_cert_str = ssl_certificate_to_string(known_cert);
-		cur_cert_str = ssl_certificate_to_string(current_cert);
-		err_msg = g_strdup_printf(_("%s's SSL certificate changed !\nWe have saved this one:\n%s\n\nIt is now:\n%s\n\nThis could mean the server answering is not the known one."),
-					  current_cert->host,
-					  known_cert_str,
-					  cur_cert_str);
-		g_free (cur_cert_str);
-		g_free (known_cert_str);
-
+	} else if (!ssl_certificate_compare (current_cert, known_cert)) {
 		cert_hook_data.cert = current_cert;
 		cert_hook_data.old_cert = known_cert;
+		cert_hook_data.expired = FALSE;
 		cert_hook_data.accept = FALSE;
 		
 		hooks_invoke(SSLCERT_ASK_HOOKLIST, &cert_hook_data);
-		
-		g_free(err_msg);
 
 		if (!cert_hook_data.accept) {
 			ssl_certificate_destroy(current_cert);
@@ -399,6 +426,37 @@ gboolean ssl_certificate_check (X509 *x509_cert, gchar *host, gushort port)
 			ssl_certificate_destroy(known_cert);
 			return TRUE;
 		}
+	} else if (asn1toTime(X509_get_notAfter(current_cert->x509_cert)) < time(NULL)) {
+		gchar *tmp = g_strdup_printf("%s:%d", current_cert->host, current_cert->port);
+		
+		if (warned_expired == NULL)
+			warned_expired = g_hash_table_new(g_str_hash, g_str_equal);
+		
+		if (g_hash_table_lookup(warned_expired, tmp)) {
+			g_free(tmp);
+			ssl_certificate_destroy(current_cert);
+			ssl_certificate_destroy(known_cert);
+			return TRUE;
+		}
+			
+		cert_hook_data.cert = current_cert;
+		cert_hook_data.old_cert = NULL;
+		cert_hook_data.expired = TRUE;
+		cert_hook_data.accept = FALSE;
+		
+		hooks_invoke(SSLCERT_ASK_HOOKLIST, &cert_hook_data);
+
+		if (!cert_hook_data.accept) {
+			g_free(tmp);
+			ssl_certificate_destroy(current_cert);
+			ssl_certificate_destroy(known_cert);
+			return FALSE;
+		} else {
+			g_hash_table_insert(warned_expired, tmp, GINT_TO_POINTER(1));
+			ssl_certificate_destroy(current_cert);
+			ssl_certificate_destroy(known_cert);
+			return TRUE;
+		}
 	}
 
 	ssl_certificate_destroy(current_cert);
