@@ -48,17 +48,19 @@ struct _SSLCertHookData
 {
 	SSLCertificate *cert;
 	SSLCertificate *old_cert;
+	gboolean expired;
 	gboolean accept;
 };
 
 SSLCertificate *ssl_certificate_find (gchar *host, gushort port);
 SSLCertificate *ssl_certificate_find_lookup (gchar *host, gushort port, gboolean lookup);
-gboolean ssl_certificate_check (X509 *x509_cert, gchar *host, gushort port);
+gboolean ssl_certificate_check (X509 *x509_cert, gchar *fqdn, gchar *host, gushort port);
 char* ssl_certificate_to_string(SSLCertificate *cert);
 void ssl_certificate_destroy(SSLCertificate *cert);
 void ssl_certificate_delete_from_disk(SSLCertificate *cert);
 char * readable_fingerprint(unsigned char *src, int len);
 char *ssl_certificate_check_signer (X509 *cert);
+time_t asn1toTime(ASN1_TIME *asn1Time);
 
 #endif /* USE_OPENSSL */
 #endif /* SSL_CERTIFICATE_H */
