@@ -257,42 +257,6 @@ gchar *strncpy2		(gchar		*dest,
 			 const gchar	*src,
 			 size_t		 n);
 
-/* wide-character functions */
-#if !HAVE_ISWALNUM
-int iswalnum		(wint_t wc);
-#endif
-#if !HAVE_ISWSPACE
-int iswspace		(wint_t wc);
-#endif
-#if !HAVE_TOWLOWER
-wint_t towlower		(wint_t wc);
-#endif
-
-#if !HAVE_WCSLEN
-size_t wcslen		(const wchar_t *s);
-#endif
-#if !HAVE_WCSCPY
-wchar_t *wcscpy		(wchar_t       *dest,
-			 const wchar_t *src);
-#endif
-#if !HAVE_WCSNCPY
-wchar_t *wcsncpy	(wchar_t       *dest,
-			 const wchar_t *src,
-			 size_t		n);
-#endif
-
-wchar_t *wcsdup			(const wchar_t *s);
-wchar_t *wcsndup		(const wchar_t *s,
-				 size_t		n);
-wchar_t *strdup_mbstowcs	(const gchar   *s);
-gchar *strdup_wcstombs		(const wchar_t *s);
-gint wcsncasecmp		(const wchar_t *s1,
-				 const wchar_t *s2,
-				 size_t		n);
-wchar_t *wcscasestr		(const wchar_t *haystack,
-				 const wchar_t *needle);
-gint get_mbs_len		(const gchar	*s);
-
 gboolean is_next_nonascii	(const gchar *s);
 gint get_next_word_len		(const gchar *s);
 
@@ -409,6 +373,7 @@ const gchar *get_template_dir		(void);
 const gchar *get_header_cache_dir	(void);
 const gchar *get_plugin_dir             (void);
 const gchar *get_tmp_dir		(void);
+const gchar *get_locale_dir		(void);
 gchar *get_tmp_file			(void);
 const gchar *get_domain_name		(void);
 
@@ -417,6 +382,8 @@ off_t get_file_size		(const gchar	*file);
 off_t get_file_size_as_crlf	(const gchar	*file);
 off_t get_left_file_size	(FILE		*fp);
 
+time_t get_file_mtime		(const gchar *file);
+
 gboolean file_exist		(const gchar	*file,
 				 gboolean	 allow_fifo);
 gboolean is_relative_filename   (const gchar *file);
@@ -509,6 +476,10 @@ size_t my_strftime		(gchar			*s,
 				 size_t			 max,
 				 const gchar		*format,
 				 const struct tm	*tm);
+size_t fast_strftime		(gchar 			*buf, 
+				 gint 			 buflen, 
+				 const gchar 		*format, 
+				 struct tm 		*lt);
 
 /* debugging */
 void debug_print_real	(const gchar *format, ...) G_GNUC_PRINTF(1, 2);
@@ -553,17 +524,31 @@ gpointer g_auto_pointer_get_ptr		(GAuto *auto_ptr);
 GAuto *g_auto_pointer_copy		(GAuto *auto_ptr);
 void g_auto_pointer_free		(GAuto *auto_ptr);
 void replace_returns			(gchar *str);
+gboolean get_uri_part	(const gchar *start,
+		    	 const gchar *scanpos,
+		     	 const gchar **bp,
+		    	 const gchar **ep,
+		   	 gboolean hdr);
+gchar *make_uri_string	(const gchar *bp,
+			 const gchar *ep);
+gboolean get_email_part	(const gchar *start, 
+			 const gchar *scanpos,
+			 const gchar **bp, 
+			 const gchar **ep,
+			 gboolean hdr);
+gchar *make_email_string(const gchar *bp,
+			 const gchar *ep);
+gchar *make_http_string (const gchar *bp,
+			 const gchar *ep);
 
-gboolean get_uri_part(const gchar *start, const gchar *scanpos,
-			     const gchar **bp, const gchar **ep, gboolean hdr);
-gchar *make_uri_string(const gchar *bp, const gchar *ep);
-gboolean get_email_part(const gchar *start, const gchar *scanpos,
-			       const gchar **bp, const gchar **ep, gboolean hdr);
-gchar *make_email_string(const gchar *bp, const gchar *ep);
-gchar *make_http_string(const gchar *bp, const gchar *ep);
-gchar *mailcap_get_command_for_type(const gchar *type);
-gboolean file_is_email (const gchar *filename);
+gchar *mailcap_get_command_for_type(const gchar *type, 
+				    const gchar *file_to_open);
+void mailcap_update_default	   (const gchar *type,
+				    const gchar *command);
 
+gboolean file_is_email(const gchar *filename);
+gboolean sc_g_list_bigger(GList *list, gint max);
+gboolean sc_g_slist_bigger(GSList *list, gint max);
 #ifdef __cplusplus
 }
 #endif
