@@ -98,23 +98,7 @@ struct _Compose
 	/* Header */
 	GtkWidget *table_vbox;
 	GtkWidget *table;
-/*
-	GtkWidget *to_hbox;
-	GtkWidget *to_entry;
-	GtkWidget *newsgroups_hbox;
-	GtkWidget *newsgroups_entry;
-*/
 	GtkWidget *subject_entry;
-/*
-	GtkWidget *cc_hbox;
-	GtkWidget *cc_entry;
-	GtkWidget *bcc_hbox;
-	GtkWidget *bcc_entry;
-	GtkWidget *reply_hbox;
-	GtkWidget *reply_entry;
-	GtkWidget *followup_hbox;
-	GtkWidget *followup_entry;
-*/
 	GtkWidget *paned;
 
 	/* Attachments */
@@ -131,6 +115,8 @@ struct _Compose
 	GtkWidget *ruler;
 	GtkWidget *scrolledwin;
 	GtkWidget *text;
+	GtkWidget *from_name;
+	GtkTooltips *tooltips;
 
 	GtkWidget *focused_editable;
 
@@ -208,6 +194,7 @@ struct _Compose
 #if USE_ASPELL
         /* GNU/aspell spell checker */
         GtkAspell *gtkaspell;
+	GtkWidget *aspell_options_menu;
 #endif
 
  	/* Priority */
@@ -227,6 +214,7 @@ struct _Compose
 	
 	gboolean updating;
 	gboolean deferred_destroy;
+	ComposeMode rmode;
 };
 
 struct _AttachInfo
@@ -245,7 +233,8 @@ Compose *compose_new			(PrefsAccount	*account,
 				 	 GPtrArray	*attach_files);
 
 Compose *compose_new_with_folderitem	(PrefsAccount	*account,
-					 FolderItem	*item);
+					 FolderItem	*item,
+					 const gchar	*mailto);
 
 Compose *compose_new_with_list		(PrefsAccount	*account,
 					 GList          *listAddress);
@@ -278,7 +267,8 @@ Compose *compose_forward_multiple	(PrefsAccount	*account,
 Compose *compose_redirect		(PrefsAccount	*account,
 					 MsgInfo	*msginfo,
 					 gboolean	 batch);
-void compose_reedit			(MsgInfo	*msginfo);
+Compose *compose_reedit			(MsgInfo	*msginfo,
+					 gboolean	 batch);
 
 GList *compose_get_compose_list		(void);
 
@@ -315,5 +305,10 @@ gboolean compose_search_string			(Compose	*compose,
 gboolean compose_search_string_backward	(Compose	*compose,
 						 const gchar	*str,
 						 gboolean	 case_sens);
-
+gint compose_queue			(Compose *compose, 
+					 gint *msgnum, 
+					 FolderItem **item, 
+					 gchar **msgpath,
+					 gboolean remove_reedit_target);
+gboolean compose_close			(Compose *compose);
 #endif /* __COMPOSE_H__ */
