@@ -8,7 +8,7 @@
 #define IMAP_SET_MAX_COUNT 100
 
 void imap_main_set_timeout(int sec);
-void imap_main_init(void);
+void imap_main_init(gboolean skip_ssl_cert_check);
 void imap_main_done(void);
 
 void imap_init(Folder * folder);
@@ -16,7 +16,7 @@ void imap_done(Folder * folder);
 
 int imap_threaded_connect(Folder * folder, const char * server, int port);
 int imap_threaded_connect_ssl(Folder * folder, const char * server, int port);
-struct mailimap_capability_data * imap_threaded_capability(Folder *folder);
+struct mailimap_capability_data * imap_threaded_capability(Folder *folder, int *ok);
 int imap_threaded_connect_cmd(Folder * folder, const char * command,
 			      const char * server, int port);
 
@@ -86,7 +86,8 @@ int imap_threaded_append(Folder * folder, const char * mailbox,
 int imap_threaded_expunge(Folder * folder);
 
 int imap_threaded_copy(Folder * folder, struct mailimap_set * set,
-		       const char * mb);
+		       const char * mb, struct mailimap_set **source,
+		       struct mailimap_set **dest);
 
 int imap_threaded_store(Folder * folder, struct mailimap_set * set,
 			struct mailimap_store_att_flags * store_att_flags);
