@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2003 Match Grun
+ * Copyright (C) 2003-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -46,6 +46,7 @@
 #include "exportldif.h"
 #include "utils.h"
 #include "manage_window.h"
+#include "filesel.h"
 
 #define PAGE_FILE_INFO             0
 #define PAGE_DN                    1
@@ -190,9 +191,9 @@ static gboolean exp_ldif_move_file( void ) {
 		"does not exist. OK to create new directory?" ),
 		_exportCtl_->dirOutput );
 	aval = alertpanel( _("Create Directory" ),
-		msg, GTK_STOCK_YES, GTK_STOCK_NO, NULL );
+		msg, GTK_STOCK_NO, GTK_STOCK_YES, NULL );
 	g_free( msg );
-	if( aval != G_ALERTDEFAULT ) return FALSE;
+	if( aval != G_ALERTALTERNATE ) return FALSE;
 
 	/* Create directory */
 	if( ! exportldif_create_dir( _exportCtl_ ) ) {
@@ -249,8 +250,8 @@ static gboolean exp_ldif_move_dn( void ) {
 				"for an LDAP server. Are you sure you wish " \
 				"to proceed without a suffix?"
 			 ),
-			GTK_STOCK_YES, GTK_STOCK_NO, NULL );
-		if( aval != G_ALERTDEFAULT ) {
+			GTK_STOCK_NO, GTK_STOCK_YES, NULL );
+		if( aval != G_ALERTALTERNATE ) {
 			gtk_widget_grab_focus( expldif_dlg.entrySuffix );
 			errFlag = TRUE;
 		}
@@ -324,6 +325,8 @@ static void export_ldif_next( GtkWidget *widget ) {
 		if( exp_ldif_move_dn() ) {
 			gtk_notebook_set_current_page(
 				GTK_NOTEBOOK(expldif_dlg.notebook), PAGE_FINISH );
+			gtk_button_set_label(GTK_BUTTON(expldif_dlg.btnCancel),
+				GTK_STOCK_CLOSE);
 			exp_ldif_finish_show();
 			exportldif_save_settings( _exportCtl_ );
 			export_ldif_message();
@@ -331,75 +334,27 @@ static void export_ldif_next( GtkWidget *widget ) {
 	}
 }
 
-/**
- * Callback function to accept LDIF file selection.
- * \param widget Widget (button).
- * \param data   User data.
- */
-static void exp_ldif_file_ok( GtkWidget *widget, gpointer data ) {
-	const gchar *sFile;
-	AddressFileSelection *afs;
-	GtkWidget *fileSel;
-
-	afs = ( AddressFileSelection * ) data;
-	fileSel = afs->fileSelector;
-	sFile = gtk_file_selection_get_filename( GTK_FILE_SELECTION(fileSel) );
-
-	afs->cancelled = FALSE;
-	gtk_entry_set_text( GTK_ENTRY(expldif_dlg.entryLdif), sFile );
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( expldif_dlg.entryLdif );
-}
-
-/**
- * Callback function to cancel LDIF file selection dialog.
- * \param widget Widget (button).
- * \param data   User data.
- */
-static void exp_ldif_file_cancel( GtkWidget *widget, gpointer data ) {
-	AddressFileSelection *afs = ( AddressFileSelection * ) data;
-	afs->cancelled = TRUE;
-	gtk_widget_hide( afs->fileSelector );
-	gtk_grab_remove( afs->fileSelector );
-	gtk_widget_grab_focus( expldif_dlg.entryLdif );
-}
-
 /**
  * Create LDIF file selection dialog.
  * \param afs Address file selection data.
  */
 static void exp_ldif_file_select_create( AddressFileSelection *afs ) {
-	GtkWidget *fileSelector;
-
-	fileSelector = gtk_file_selection_new( _("Select LDIF Output File") );
-	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(fileSelector) );
-	gtk_file_selection_complete( GTK_FILE_SELECTION(fileSelector), "*.html" );
-	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileSelector)->ok_button),
-			 "clicked", 
-			 G_CALLBACK(exp_ldif_file_ok), (gpointer)afs);
-	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileSelector)->cancel_button),
-			 "clicked", 
-			 G_CALLBACK(exp_ldif_file_cancel), (gpointer)afs);
-	afs->fileSelector = fileSelector;
-	afs->cancelled = TRUE;
+	gchar *file = filesel_select_file_save(_("Select LDIF output file"), NULL);
+	
+	if (file == NULL)
+		afs->cancelled = TRUE;
+	else {
+		afs->cancelled = FALSE;
+		gtk_entry_set_text( GTK_ENTRY(expldif_dlg.entryLdif), file );
+		g_free(file);
+	}
 }
 
 /**
  * Callback function to display LDIF file selection dialog.
  */
 static void exp_ldif_file_select( void ) {
-	gchar *sFile;
-	if( ! _exp_ldif_file_selector_.fileSelector )
-		exp_ldif_file_select_create( & _exp_ldif_file_selector_ );
-
-	sFile = gtk_editable_get_chars( GTK_EDITABLE(expldif_dlg.entryLdif), 0, -1 );
-	gtk_file_selection_set_filename(
-		GTK_FILE_SELECTION( _exp_ldif_file_selector_.fileSelector ),
-		sFile );
-	g_free( sFile );
-	gtk_widget_show( _exp_ldif_file_selector_.fileSelector );
-	gtk_grab_add( _exp_ldif_file_selector_.fileSelector );
+	exp_ldif_file_select_create( & _exp_ldif_file_selector_ );
 }
 
 /**
@@ -457,7 +412,7 @@ static void export_ldif_page_file( gint pageNum, gchar *pageLbl ) {
 	gtk_table_attach(GTK_TABLE(table), entryLdif, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
-	btnFile = gtk_button_new_with_label( _(" ... "));
+	btnFile = gtkut_get_browse_file_btn(_("B_rowse"));
 	gtk_table_attach(GTK_TABLE(table), btnFile, 2, 3, top, (top + 1),
 		GTK_FILL, 0, 3, 0);
 
@@ -563,7 +518,7 @@ static void export_ldif_page_dn( gint pageNum, gchar *pageLbl ) {
 		"  cn=John Doe,ou=people,dc=sylpheed,dc=org"
 		), NULL );
 
-	menuItem = gtk_menu_item_new_with_label( _( "E-Mail Address" ) );
+	menuItem = gtk_menu_item_new_with_label( _( "Email Address" ) );
 	gtk_object_set_user_data( GTK_OBJECT( menuItem ),
 			GINT_TO_POINTER( EXPORT_LDIF_ID_EMAIL ) );
 	gtk_menu_append( GTK_MENU(menu), menuItem );
@@ -571,7 +526,7 @@ static void export_ldif_page_dn( gint pageNum, gchar *pageLbl ) {
 	toolTip = gtk_tooltips_new();
 	gtk_tooltips_set_tip(
 		GTK_TOOLTIPS(toolTip), menuItem, _(
-		"The first E-Mail Address belonging to a person is used to " \
+		"The first Email Address belonging to a person is used to " \
 		"create a DN that is formatted similar to:\n" \
 		"  mail=john.doe@domain.com,ou=people,dc=sylpheed,dc=org"
 		), NULL );
@@ -616,7 +571,7 @@ static void export_ldif_page_dn( gint pageNum, gchar *pageLbl ) {
 	/* Fourth row */
 	top++;
 	checkEMail = gtk_check_button_new_with_label(
-			_( "Exclude record if no E-Mail Address" ) );
+			_( "Exclude record if no Email Address" ) );
 	gtk_table_attach(GTK_TABLE(table), checkEMail, 1, 2, top, (top + 1),
 		GTK_FILL, 0, 0, 0);
 
@@ -624,7 +579,7 @@ static void export_ldif_page_dn( gint pageNum, gchar *pageLbl ) {
 	gtk_tooltips_set_tip(
 		GTK_TOOLTIPS(toolTip), checkEMail, _(
 		"An addressbook may contain entries without " \
-		"E-Mail Addresses. Check this option to ignore " \
+		"Email Addresses. Check this option to ignore " \
 		"these records."
 		), NULL );
 
@@ -745,9 +700,10 @@ static void export_ldif_dialog_create( void ) {
 	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
 
 	/* Button panel */
-	gtkut_stock_button_set_create(&hbbox, &btnPrev, GTK_STOCK_GO_BACK,
-				      &btnNext, GTK_STOCK_GO_FORWARD,
-				      &btnCancel, GTK_STOCK_CANCEL);
+	gtkut_stock_button_set_create(&hbbox,
+				      &btnCancel, GTK_STOCK_CANCEL, 
+				      &btnPrev, GTK_STOCK_GO_BACK,
+				      &btnNext, GTK_STOCK_GO_FORWARD);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbbox), 2);
 	gtk_widget_grab_default(btnNext);
@@ -821,6 +777,9 @@ void addressbook_exp_ldif( AddressCache *cache ) {
 	/* Setup GUI */
 	if( ! expldif_dlg.window )
 		export_ldif_create();
+
+	gtk_button_set_label(GTK_BUTTON(expldif_dlg.btnCancel),
+			     GTK_STOCK_CANCEL);
 	expldif_dlg.cancelled = FALSE;
 	gtk_widget_show(expldif_dlg.window);
 	manage_window_set_transient(GTK_WINDOW(expldif_dlg.window));
