@@ -49,6 +49,7 @@
 #include "folder.h"
 #include "utils.h"
 #include "codeconv.h"
+#include "alertpanel.h"
 
 static GtkWidget *window;
 static GtkWidget *src_entry;
@@ -57,7 +58,7 @@ static GtkWidget *src_button;
 static GtkWidget *file_button;
 static GtkWidget *ok_button;
 static GtkWidget *cancel_button;
-static gboolean export_ack;
+static gboolean export_ok; /* see export_mbox() return values */
 
 static void export_create(void);
 static void export_ok_cb(GtkWidget *widget, gpointer data);
@@ -68,10 +69,12 @@ static gint delete_event(GtkWidget *widget, GdkEventAny *event, gpointer data);
 static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
 
 gint export_mbox(FolderItem *default_src)
+/* return values: -2 skipped/cancelled, -1 error, 0 OK */
 {
-	gint ok = 0;
 	gchar *src_id = NULL;
 
+	export_ok = -2;	// skipped or cancelled
+
 	if (!window) {
 		export_create();
 	}
@@ -98,35 +101,9 @@ gint export_mbox(FolderItem *default_src)
 
 	gtk_main();
 
-	if (export_ack) {
-		const gchar *srcdir, *utf8mbox;
-		FolderItem *src;
-
-		srcdir = gtk_entry_get_text(GTK_ENTRY(src_entry));
-		utf8mbox = gtk_entry_get_text(GTK_ENTRY(file_entry));
-		if (utf8mbox && *utf8mbox) {
-			gchar *mbox;
-
-			mbox = g_filename_from_utf8(utf8mbox, -1, NULL, NULL, NULL);
-			if (!mbox) {
-				g_warning("Failed to convert character set.\n");
-				mbox = g_strdup(utf8mbox);
-			}
-
-			src = folder_find_item_from_identifier(srcdir);
-			if (!src) {
-				g_warning("Can't find the folder.\n");
-			} else {
-				ok = export_to_mbox(src, mbox);
-			}
-
-			g_free(mbox);
-		}
-	}
-
 	gtk_widget_hide(window);
 
-	return ok;
+	return export_ok;
 }
 
 static void export_create(void)
@@ -223,14 +200,47 @@ static void export_create(void)
 
 static void export_ok_cb(GtkWidget *widget, gpointer data)
 {
-	export_ack = TRUE;
+	const gchar *srcdir, *utf8mbox;
+	FolderItem *src;
+	gchar *mbox;
+
+	srcdir = gtk_entry_get_text(GTK_ENTRY(src_entry));
+	utf8mbox = gtk_entry_get_text(GTK_ENTRY(file_entry));
+
+	if (utf8mbox && !*utf8mbox) {
+		alertpanel_error(_("Target mbox filename can't be left empty."));
+		gtk_widget_grab_focus(file_entry);
+		return;
+	}
+	if (srcdir && !*srcdir) {
+		alertpanel_error(_("Source folder can't be left empty."));
+		gtk_widget_grab_focus(src_entry);
+		return;
+	}
+
+	mbox = g_filename_from_utf8(utf8mbox, -1, NULL, NULL, NULL);
+	if (!mbox) {
+		g_warning("export_ok_cb(): failed to convert character set.\n");
+		mbox = g_strdup(utf8mbox);
+	}
+
+	src = folder_find_item_from_identifier(srcdir);
+	if (!src) {
+		alertpanel_error(_("Couldn't find the source folder."));
+		gtk_widget_grab_focus(src_entry);
+		return;
+	} else {
+		export_ok = export_to_mbox(src, mbox);
+	}
+
+	g_free(mbox);
+
 	if (gtk_main_level() > 1)
 		gtk_main_quit();
 }
 
 static void export_cancel_cb(GtkWidget *widget, gpointer data)
 {
-	export_ack = FALSE;
 	if (gtk_main_level() > 1)
 		gtk_main_quit();
 }
