@@ -620,7 +620,7 @@ static void exportldif_default_values( ExportLdifCtl *ctl ) {
 	gchar *str;
 
 	str = g_strconcat(
-		g_get_home_dir(), G_DIR_SEPARATOR_S,
+		get_home_dir(), G_DIR_SEPARATOR_S,
 		DFL_DIR_SYLPHEED_OUT, NULL );
 
 	ctl->dirOutput = mgu_replace_string( ctl->dirOutput, str );
