@@ -33,6 +33,7 @@
 #include "filtering.h"
 #include "prefs_gtk.h"
 #include "compose.h"
+#include "prefs_common.h"
 
 #define PREFSBUFSIZE		1024
 
@@ -78,19 +79,22 @@ FilteringAction * filteringaction_new(int type, int account_id,
 void filteringaction_free(FilteringAction * action)
 {
 	g_return_if_fail(action);
-	if (action->destination)
-		g_free(action->destination);
+	g_free(action->destination);
 	g_free(action);
 }
 
-FilteringProp * filteringprop_new(const gchar *name,
+FilteringProp * filteringprop_new(gboolean enabled,
+				  const gchar *name,
+				  gint account_id,
 				  MatcherList * matchers,
 				  GSList * action_list)
 {
 	FilteringProp * filtering;
 
 	filtering = g_new0(FilteringProp, 1);
+	filtering->enabled = enabled;
 	filtering->name = name ? g_strdup(name): NULL;
+	filtering->account_id = account_id;
 	filtering->matchers = matchers;
 	filtering->action_list = action_list;
 
@@ -144,6 +148,7 @@ FilteringProp * filteringprop_copy(FilteringProp *src)
                     filteringaction_copy(filtering_action));
         }
 
+	new->enabled = src->enabled;
 	new->name = g_strdup(src->name);
 
 	return new;
@@ -176,11 +181,10 @@ void filtering_move_and_copy_msgs(GSList *msgs)
 	GSList *messages = g_slist_copy(msgs);
 	FolderItem *last_item = NULL;
 	gboolean is_copy = FALSE, is_move = FALSE;
-	
+	debug_print("checking %d messages\n", g_slist_length(msgs));
 	while (messages) {
 		GSList *batch = NULL, *cur;
 		gint found = 0;
-		debug_print("%d messages to filter\n", g_slist_length(messages));
 		for (cur = messages; cur; cur = cur->next) {
 			MsgInfo *info = (MsgInfo *)cur->data;
 			if (last_item == NULL) {
@@ -188,7 +192,6 @@ void filtering_move_and_copy_msgs(GSList *msgs)
 			}
 			if (last_item == NULL)
 				continue;
-			debug_print("for %s\n", folder_item_get_path(last_item));
 			if (!is_copy && !is_move) {
 				if (info->is_copy)
 					is_copy = TRUE;
@@ -199,23 +202,23 @@ void filtering_move_and_copy_msgs(GSList *msgs)
 			if (info->to_filter_folder == last_item 
 			&&  info->is_copy == is_copy
 			&&  info->is_move == is_move) {
-				batch = g_slist_append(batch, info);
+				batch = g_slist_prepend(batch, info);
 			}
 		}
 		if (found == 0) {
 			debug_print("no more messages to move/copy\n");
 			break;
+		} else {
+			debug_print("%d messages to %s in %s\n", found,
+				is_copy ? "copy":"move", last_item->name ? last_item->name:"(noname)");
 		}
 		for (cur = batch; cur; cur = cur->next) {
 			MsgInfo *info = (MsgInfo *)cur->data;
 			messages = g_slist_remove(messages, info);
 		}
+		batch = g_slist_reverse(batch);
 		if (g_slist_length(batch)) {
 			MsgInfo *info = (MsgInfo *)batch->data;
-			debug_print("%s %d messages to %s\n",
-				is_copy?"copying":"moving",
-				g_slist_length(batch),
-				folder_item_get_path(last_item));
 			if (is_copy && last_item != info->folder) {
 				folder_item_copy_msgs(last_item, batch);
 			} else if (is_move && last_item != info->folder) {
@@ -231,7 +234,6 @@ void filtering_move_and_copy_msgs(GSList *msgs)
 		last_item = NULL;
 		is_copy = FALSE;
 		is_move = FALSE;
-		debug_print("%d messages remaining\n", g_slist_length(messages));
 	}
 	/* we don't reference the msginfos, because caller will do */
 	g_slist_free(messages);
@@ -261,10 +263,16 @@ static gboolean filteringaction_apply(FilteringAction * action, MsgInfo * info)
 			return FALSE;
 		}
 		
+		/* check if mail is set to copy already, 
+		 * in which case we have to do it */
+		if (info->is_copy && info->to_filter_folder) {
+			debug_print("should cp and mv !\n");
+			folder_item_copy_msg(info->to_filter_folder, info);
+			info->is_copy = FALSE;
+		}
 		/* mark message to be moved */		
 		info->is_move = TRUE;
 		info->to_filter_folder = dest_folder;
-		debug_print("set to move to %s\n", folder_item_get_path(dest_folder));
 		return TRUE;
 
 	case MATCHACTION_COPY:
@@ -277,10 +285,16 @@ static gboolean filteringaction_apply(FilteringAction * action, MsgInfo * info)
 			return FALSE;
 		}
 
+		/* check if mail is set to copy already, 
+		 * in which case we have to do it */
+		if (info->is_copy && info->to_filter_folder) {
+			debug_print("should cp and mv !\n");
+			folder_item_copy_msg(info->to_filter_folder, info);
+			info->is_copy = FALSE;
+		}
 		/* mark message to be copied */		
 		info->is_copy = TRUE;
 		info->to_filter_folder = dest_folder;
-		debug_print("set to copy to %s\n", folder_item_get_path(dest_folder));
 		return TRUE;
 
 	case MATCHACTION_DELETE:
@@ -397,9 +411,32 @@ gboolean filteringaction_apply_action_list(GSList *action_list, MsgInfo *info)
 	return TRUE;
 }
 
-static gboolean filtering_match_condition(FilteringProp *filtering, MsgInfo *info)
+static gboolean filtering_match_condition(FilteringProp *filtering, MsgInfo *info,
+							PrefsAccount *ac_prefs)
 {
-	return matcherlist_match(filtering->matchers, info);
+	gboolean matches = FALSE;
+
+	if (ac_prefs != NULL) {
+		matches = ((filtering->account_id == 0)
+					|| (filtering->account_id == ac_prefs->account_id));
+	} else {
+		switch (prefs_common.apply_per_account_filtering_rules) {
+		case FILTERING_ACCOUNT_RULES_FORCE:
+			/* apply filtering rules regardless to the account info */
+			matches = TRUE;
+			break;
+		case FILTERING_ACCOUNT_RULES_SKIP:
+			/* don't apply filtering rules that belong to an account */
+			matches = (filtering->account_id == 0);
+			break;
+		case FILTERING_ACCOUNT_RULES_USE_CURRENT:
+			matches = ((filtering->account_id == 0)
+					|| (filtering->account_id == cur_account->account_id));
+			break;
+		}
+	}
+
+	return matches && matcherlist_match(filtering->matchers, info);
 }
 
 /*!
@@ -459,7 +496,7 @@ static gboolean filtering_is_final_action(FilteringAction *filtering_action)
 	}
 }
 
-static gboolean filter_msginfo(GSList * filtering_list, MsgInfo * info)
+static gboolean filter_msginfo(GSList * filtering_list, MsgInfo * info, PrefsAccount* ac_prefs)
 {
 	GSList	*l;
 	gboolean final;
@@ -470,7 +507,7 @@ static gboolean filter_msginfo(GSList * filtering_list, MsgInfo * info)
 	for (l = filtering_list, final = FALSE, apply_next = FALSE; l != NULL; l = g_slist_next(l)) {
 		FilteringProp * filtering = (FilteringProp *) l->data;
 
-		if (filtering_match_condition(filtering, info)) {
+		if (filtering->enabled && filtering_match_condition(filtering, info, ac_prefs)) {
 			apply_next = filtering_apply_rule(filtering, info, &final);
                         if (final)
                                 break;
@@ -499,9 +536,9 @@ static gboolean filter_msginfo(GSList * filtering_list, MsgInfo * info)
  *		processing. E.g. \ref inc.c::inc_start moves the 
  *		message to the inbox. 	
  */
-gboolean filter_message_by_msginfo(GSList *flist, MsgInfo *info)
+gboolean filter_message_by_msginfo(GSList *flist, MsgInfo *info, PrefsAccount* ac_prefs)
 {
-	return filter_msginfo(flist, info);
+	return filter_msginfo(flist, info, ac_prefs);
 }
 
 gchar *filteringaction_to_string(gchar *dest, gint destlen, FilteringAction *action)
@@ -666,3 +703,18 @@ void prefs_filtering_clear_folder(Folder *folder)
 	/* FIXME: Note folder settings were changed, where the updates? */
 }
 
+gboolean filtering_peek_per_account_rules(GSList *filtering_list)
+/* return TRUE if there's at least one per-account filtering rule */
+{
+	GSList *l;
+
+	for (l = filtering_list; l != NULL; l = g_slist_next(l)) {
+		FilteringProp * filtering = (FilteringProp *) l->data;
+
+		if (filtering->enabled && (filtering->account_id != 0)) {
+			return TRUE;
+		}		
+	}
+
+	return FALSE;
+}
