@@ -36,13 +36,21 @@ struct _FilteringAction {
 typedef struct _FilteringAction FilteringAction;
 
 struct _FilteringProp {
+	gboolean enabled;
 	gchar *name;
+	gint account_id;
 	MatcherList * matchers;
 	GSList * action_list;
 };
 
 typedef struct _FilteringProp FilteringProp;
 
+enum {
+	FILTERING_ACCOUNT_RULES_SKIP = 0,
+	FILTERING_ACCOUNT_RULES_FORCE = 1,
+	FILTERING_ACCOUNT_RULES_USE_CURRENT = 2
+};
+
 /* extern GSList * prefs_filtering; */
 
 
@@ -53,7 +61,9 @@ void filteringaction_free(FilteringAction *action);
 FilteringAction * filteringaction_parse(gchar **str);
 gboolean filteringaction_apply_action_list (GSList *action_list, MsgInfo *info);
 
-FilteringProp * filteringprop_new(const gchar *name,
+FilteringProp * filteringprop_new(gboolean enabled,
+				  const gchar *name,
+				  gint account_id,
 				  MatcherList *matchers,
 				  GSList *action_list);
 void filteringprop_free(FilteringProp *prop);
@@ -61,7 +71,7 @@ void filteringprop_free(FilteringProp *prop);
 FilteringProp * filteringprop_parse(gchar **str);
 
 void filter_msginfo_move_or_delete(GSList *filtering_list, MsgInfo *info);
-gboolean filter_message_by_msginfo(GSList *flist, MsgInfo *info);
+gboolean filter_message_by_msginfo(GSList *flist, MsgInfo *info, PrefsAccount *ac_prefs);
 
 gchar * filteringaction_to_string(gchar *dest, gint destlen, FilteringAction *action);
 void prefs_filtering_write_config(void);
@@ -80,4 +90,6 @@ extern GSList * filtering_rules;
 extern GSList * pre_global_processing;
 extern GSList * post_global_processing;
 
+gboolean filtering_peek_per_account_rules(GSList *filtering_list);
+
 #endif
