@@ -570,12 +570,39 @@ struct _FolderClass
 						 MsgInfoList	*msglist,
 						 GRelation	*msgflags);
 	
+	/* Sets batch mode for a FolderItem. It means that numerous flags updates
+	 * could follow, and the FolderClass implementation can cache them in order
+	 * to process them later when set_false will be called again with the
+	 * batch parameter set to FALSE. 
+	 */
 	void		(*set_batch)		(Folder		*folder,
 						 FolderItem	*item,
 						 gboolean	 batch);
+	/* Called when switching offline or asking for synchronisation. the imple
+	 * mentation should do what's necessary to be able to read mails present
+	 * in the FolderItem at this time with no network connectivity. 
+	 */
 	void		(*synchronise)		(FolderItem	*item);
+	
+	/* Passed from sylpheed-claws --subscribe scheme://uri. Implementations
+	 * should check if they handle this type of URI, and return TRUE in this
+	 * case after having subscribed it.
+	 */
 	gboolean	(*subscribe)		(Folder 	*folder,
 						 const gchar	*uri);
+	
+	/* Gets the preferred sort key and type for a folderclass. */
+	void		(*get_sort_type)	(Folder		*folder,
+						 FolderSortKey	*sort_key,
+						 FolderSortType	*sort_type);
+	
+	/* Copies internal FolderItem data from one folderItem to another. Used
+	 * when moving folders (this move is in reality a folder creation, content
+	 * move, folder delettion).
+	 */
+	void		(*copy_private_data)	(Folder		*folder,
+						 FolderItem	*src,
+						 FolderItem	*dest);
 };
 
 struct _FolderItem
@@ -631,6 +658,7 @@ struct _FolderItem
 	/* for faster search of special parents */
 	SpecialFolderItemType parent_stype;
 	gboolean processing_pending;
+	gboolean scanning;
 };
 
 struct _PersistPrefs
@@ -744,6 +772,8 @@ gchar *folder_item_get_path		(FolderItem	*item);
 gint   folder_item_open			(FolderItem	*item);
 gint   folder_item_close		(FolderItem	*item);
 gint   folder_item_scan			(FolderItem	*item);
+gint   folder_item_scan_full		(FolderItem 	*item, 
+					 gboolean 	 filtering);
 gint   folder_item_syncronize_flags	(FolderItem 	*item);
 void   folder_item_scan_foreach		(GHashTable	*table);
 MsgInfo *folder_item_get_msginfo	(FolderItem 	*item,
@@ -798,7 +828,7 @@ const PersistPrefs *folder_get_persist_prefs
 					(GHashTable *pptable, const char *name);
 
 void folder_item_restore_persist_prefs	(FolderItem *item, GHashTable *pptable);
-void folder_clean_cache_memory		(void);
+void folder_clean_cache_memory		(FolderItem *protected_item);
 void folder_clean_cache_memory_force	(void);
 void folder_item_write_cache		(FolderItem *item);
 void folder_item_set_default_flags	(FolderItem *dest, MsgFlags *flags);
@@ -821,5 +851,11 @@ void folder_item_process_open		(FolderItem *item,
 					 gpointer data);
 gboolean folder_subscribe		(const gchar *uri);
 gboolean folder_have_mailbox 		(void);
+gboolean folder_item_free_cache		(FolderItem *item, gboolean force);
+void folder_item_change_type		(FolderItem *item,
+					 SpecialFolderItemType newtype);
+gboolean folder_get_sort_type		(Folder		*folder,
+					 FolderSortKey	*sort_key,
+					 FolderSortType	*sort_type);
 
 #endif /* __FOLDER_H__ */
