@@ -229,7 +229,7 @@ static void foldersel_create(void)
 	gtk_container_set_border_width(GTK_CONTAINER(window), 4);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 	gtk_window_set_wmclass
 		(GTK_WINDOW(window), "folder_selection", "Sylpheed-Claws");
 	gtk_widget_realize(window);
@@ -257,7 +257,7 @@ static void foldersel_create(void)
 					GDK_TYPE_PIXBUF,
 					GDK_TYPE_PIXBUF,
 					GDK_TYPE_COLOR,
-					G_TYPE_BOOLEAN);
+					G_TYPE_INT);
 	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(tree_store),
 					FOLDERSEL_FOLDERNAME,
 					foldersel_folder_name_compare,
@@ -267,7 +267,7 @@ static void foldersel_create(void)
 	g_object_unref(G_OBJECT(tree_store));
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview),
-	                             prefs_common.enable_rules_hint);
+	                             prefs_common.use_stripes_everywhere);
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview),
 					FOLDERSEL_FOLDERNAME);
 
@@ -298,7 +298,7 @@ static void foldersel_create(void)
 		(column, renderer,
 		 "text", FOLDERSEL_FOLDERNAME,
 		 "foreground-gdk", FOLDERSEL_FOREGROUND,
-		 "weight-set", FOLDERSEL_BOLD,
+		 "weight", FOLDERSEL_BOLD,
 		 NULL);
 	g_object_set(G_OBJECT(renderer), "weight", PANGO_WEIGHT_BOLD, NULL);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
@@ -354,7 +354,8 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 {
 	gchar *name, *tmpname;
 	GdkPixbuf *pixbuf, *pixbuf_open;
-	gboolean use_bold, use_color;
+	gboolean use_color;
+	PangoWeight weight = PANGO_WEIGHT_NORMAL;
 	GdkColor *foreground = NULL;
 	static GdkColor color_noselect = {0, COLOR_DIM, COLOR_DIM, COLOR_DIM};
 	static GdkColor color_new;
@@ -420,11 +421,14 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 	if (folder_has_parent_of_type(item, F_DRAFT) ||
 	    folder_has_parent_of_type(item, F_OUTBOX) ||
 	    folder_has_parent_of_type(item, F_TRASH)) {
-		use_bold = use_color = FALSE;
+		use_color = FALSE;
 	} else if (folder_has_parent_of_type(item, F_QUEUE)) {
-		use_bold = use_color = (item->total_msgs > 0);
+		use_color = (item->total_msgs > 0);
+		if (item->total_msgs > 0)
+			weight = PANGO_WEIGHT_BOLD;
 	} else {
-		use_bold = (item->unread_msgs > 0);
+		if (item->unread_msgs > 0)
+			weight = PANGO_WEIGHT_BOLD;
 		use_color = (item->new_msgs > 0);
 	}
 
@@ -441,7 +445,7 @@ static void foldersel_append_item(GtkTreeStore *store, FolderItem *item,
 			   FOLDERSEL_PIXBUF, pixbuf,
 			   FOLDERSEL_PIXBUF_OPEN, pixbuf_open,
 			   FOLDERSEL_FOREGROUND, foreground,
-			   FOLDERSEL_BOLD, use_bold,
+			   FOLDERSEL_BOLD, weight,
 			   -1);
         
         g_free(tmpname);
