@@ -24,6 +24,8 @@
 #include "folderutils.h"
 #include "prefs_account.h"
 #include "account.h"
+#include "mainwindow.h"
+#include "summaryview.h"
 
 gint folderutils_delete_duplicates(FolderItem *item,
 				   DeleteDuplicatesMode mode)
@@ -89,7 +91,8 @@ gint folderutils_delete_duplicates(FolderItem *item,
 				MsgInfo *msginfo = (MsgInfo *) cur->data;
 
 				procmsg_msginfo_set_to_folder(msginfo, NULL);
-				procmsg_msginfo_unset_flags(msginfo, MSG_MARKED, MSG_MOVE | MSG_COPY);
+				procmsg_msginfo_unset_flags(msginfo, MSG_MARKED, 
+					MSG_MOVE | MSG_COPY | MSG_MOVE_DONE);
 				procmsg_msginfo_set_flags(msginfo, MSG_DELETED, 0);
 			}
 			break;
@@ -119,24 +122,39 @@ gint folderutils_delete_duplicates(FolderItem *item,
 void folderutils_mark_all_read(FolderItem *item)
 {
 	MsgInfoList *msglist, *cur;
-
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+	int i = 0, m = 0;
+	debug_print("marking all read in item %s\n", (item==NULL)?"NULL":item->name);
 	g_return_if_fail(item != NULL);
 
-	msglist = folder_item_get_msg_list(item);
-	if (msglist == NULL)
-		return;
 
 	folder_item_update_freeze();
-	folder_item_set_batch(item, TRUE);
-	for (cur = msglist; cur != NULL; cur = g_slist_next(cur)) {
-		MsgInfo *msginfo = cur->data;
+	if (mainwin && mainwin->summaryview &&
+	    mainwin->summaryview->folder_item == item) {
+		debug_print("folder opened, using summary\n");
+		summary_mark_all_read(mainwin->summaryview);
+	} else {
+		msglist = folder_item_get_msg_list(item);
+		debug_print("got msglist %p\n", msglist);
+		if (msglist == NULL) {
+			folder_item_update_thaw();
+			return;
+		}
+		folder_item_set_batch(item, TRUE);
+		for (cur = msglist; cur != NULL; cur = g_slist_next(cur)) {
+			MsgInfo *msginfo = cur->data;
 
-		if (msginfo->flags.perm_flags & (MSG_NEW | MSG_UNREAD))
-			procmsg_msginfo_unset_flags(msginfo, MSG_NEW | MSG_UNREAD, 0);
-		procmsg_msginfo_free(msginfo);
+			if (msginfo->flags.perm_flags & (MSG_NEW | MSG_UNREAD)) {
+				procmsg_msginfo_unset_flags(msginfo, MSG_NEW | MSG_UNREAD, 0);
+				m++;
+			}
+			i++;
+			procmsg_msginfo_free(msginfo);
+		}
+		folder_item_set_batch(item, FALSE);
+		folder_item_close(item);
+		debug_print("marked %d messages out of %d as read\n", m, i);
+		g_slist_free(msglist);
 	}
-	folder_item_set_batch(item, FALSE);
 	folder_item_update_thaw();
-
-	g_slist_free(msglist);
 }
