@@ -65,6 +65,9 @@
 #include "partial_download.h"
 #include "prefs_folder_column.h"
 #include "filtering.h"
+#include "quicksearch.h"
+#include "manual.h"
+#include "timing.h"
 
 #define COL_FOLDER_WIDTH	150
 #define COL_NUM_WIDTH		32
@@ -227,6 +230,10 @@ static void folderview_empty_trash_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
 
+static void folderview_send_queue_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static void folderview_search_cb	(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -273,6 +280,8 @@ gboolean folderview_update_item_claws	 (gpointer 	    source,
 					  gpointer	    data);
 static void folderview_processing_cb(FolderView *folderview, guint action,
 				     GtkWidget *widget);
+static void folderview_set_sens_and_popup_menu(FolderView *folderview, gint row, 
+				GdkEventButton *event);
 
 GHashTable *folderview_popups;
 
@@ -285,10 +294,15 @@ static GtkItemFactoryEntry folderview_common_popup_entries[] =
 };
 
 static GtkItemFactoryEntry folder_view_trash_popup_entries[] = {
-	{N_("/------"),			NULL, NULL, 0, "<Separator>"},
+	{"/------trashsep",		NULL, NULL, 0, "<Separator>"},
 	{N_("/Empty _trash..."),	NULL, folderview_empty_trash_cb, 0, NULL},
 };
 
+static GtkItemFactoryEntry folder_view_queue_popup_entries[] = {
+	{"/------queuesep",	NULL, NULL, 0, "<Separator>"},
+	{N_("/Send _queue..."),	NULL, folderview_send_queue_cb, 0, NULL},
+};
+
 
 GtkTargetEntry folderview_drag_types[] =
 {
@@ -365,7 +379,7 @@ static void folderview_column_set_titles(FolderView *folderview)
 	GtkWidget *hbox_unread;
 	GtkWidget *hbox_total;
 	gint *col_pos = folderview->col_pos;
-	
+
 	debug_print("setting titles...\n");
 	gtk_widget_realize(folderview->ctree);
 	gtk_widget_show_all(folderview->scrolledwin);
@@ -382,7 +396,9 @@ static void folderview_column_set_titles(FolderView *folderview)
 	label_new = gtk_pixmap_new(newxpm, newxpmmask);
 	label_unread = gtk_pixmap_new(unreadxpm, unreadxpmmask);
 	label_total = gtk_pixmap_new(readxpm, readxpmmask);
-
+	
+	gtk_clist_column_titles_active(GTK_CLIST(ctree));
+	 
 	hbox_new = gtk_hbox_new(FALSE, 4);
 	hbox_unread = gtk_hbox_new(FALSE, 4);
 	hbox_total = gtk_hbox_new(FALSE, 4);
@@ -402,8 +418,29 @@ static void folderview_column_set_titles(FolderView *folderview)
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_NEW],hbox_new);
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_UNREAD],hbox_unread);
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),col_pos[F_COL_TOTAL],hbox_total);
+
+	gtk_sctree_set_column_tooltip(GTK_SCTREE(ctree), col_pos[F_COL_NEW], _("New"));
+	gtk_sctree_set_column_tooltip(GTK_SCTREE(ctree), col_pos[F_COL_UNREAD], _("Unread"));
+	gtk_sctree_set_column_tooltip(GTK_SCTREE(ctree), col_pos[F_COL_TOTAL], _("Total"));
 }
 
+static gboolean folderview_popup_menu(GtkWidget *widget, gpointer data)
+{
+	FolderView *folderview = (FolderView *)data;
+	GdkEventButton event;
+	if (folderview_get_selected_item(folderview) == NULL)
+		return FALSE;
+	
+	event.button = 3;
+	event.time = gtk_get_current_event_time();
+	
+	folderview_set_sens_and_popup_menu(folderview, -1, 
+				&event);
+
+	return TRUE;
+}
+
+
 GtkWidget *folderview_ctree_create(FolderView *folderview)
 {
 	GtkWidget *ctree;
@@ -431,11 +468,12 @@ GtkWidget *folderview_ctree_create(FolderView *folderview)
 	titles[col_pos[F_COL_FOLDER]] = _("Folder");
 	titles[col_pos[F_COL_NEW]]    = _("New");
 	titles[col_pos[F_COL_UNREAD]] = _("Unread");
+	/* TRANSLATORS: This in Number sign in American style */
 	titles[col_pos[F_COL_TOTAL]]  = _("#");
 
 	ctree = gtk_sctree_new_with_titles(N_FOLDER_COLS, col_pos[F_COL_FOLDER],
 					   titles);
-	
+
 	gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), col_pos[F_COL_NEW],
 					   GTK_JUSTIFY_RIGHT);
@@ -445,9 +483,19 @@ GtkWidget *folderview_ctree_create(FolderView *folderview)
 	gtk_clist_set_column_justification(GTK_CLIST(ctree), 
 					   col_pos[F_COL_TOTAL],
 					   GTK_JUSTIFY_RIGHT);
-	gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_DOTTED);
-	gtk_ctree_set_expander_style(GTK_CTREE(ctree),
+	if (prefs_common.enable_dotted_lines) {
+		gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_DOTTED);
+		gtk_ctree_set_expander_style(GTK_CTREE(ctree),
 				     GTK_CTREE_EXPANDER_SQUARE);
+	} else {
+		gtk_ctree_set_line_style(GTK_CTREE(ctree), GTK_CTREE_LINES_NONE);
+		gtk_ctree_set_expander_style(GTK_CTREE(ctree),
+				     GTK_CTREE_EXPANDER_TRIANGLE);
+	}
+
+	gtk_sctree_set_stripes(GTK_SCTREE(ctree), prefs_common.use_stripes_in_summaries);
+	gtk_sctree_set_recursive_expand(GTK_SCTREE(ctree), FALSE);
+
 	gtk_ctree_set_indent(GTK_CTREE(ctree), CTREE_INDENT);
 	gtk_clist_set_compare_func(GTK_CLIST(ctree), folderview_clist_compare);
 
@@ -467,6 +515,8 @@ GtkWidget *folderview_ctree_create(FolderView *folderview)
 	g_signal_connect(G_OBJECT(ctree), "button_press_event",
 			 G_CALLBACK(folderview_button_pressed),
 			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "popup-menu",
+			 G_CALLBACK(folderview_popup_menu), folderview);
 	g_signal_connect(G_OBJECT(ctree), "button_release_event",
 			 G_CALLBACK(folderview_button_released),
 			 folderview);
@@ -716,26 +766,26 @@ static void mark_all_read_cb(FolderView *folderview, guint action,
 	if (item == NULL)
 		return;
 
-	if (prefs_common.ask_mark_all_read) {
+	if (folderview->summaryview->folder_item != item
+	&&  prefs_common.ask_mark_all_read) {
 		val = alertpanel_full(_("Mark all as read"),
 			_("Do you really want to mark all mails in this "
 			  "folder as read ?"), GTK_STOCK_NO, GTK_STOCK_YES, NULL,
-			  TRUE, NULL, ALERT_QUESTION, G_ALERTALTERNATE);
+			  TRUE, NULL, ALERT_QUESTION, G_ALERTDEFAULT);
 
-		if (val == G_ALERTDEFAULT ||
-		    val == (G_ALERTDEFAULT|G_ALERTDISABLE))
+		if ((val & ~G_ALERTDISABLE) != G_ALERTALTERNATE)
 			return;
-		else if (val == (G_ALERTALTERNATE|G_ALERTDISABLE))
+		else if (val & G_ALERTDISABLE)
 			prefs_common.ask_mark_all_read = FALSE;
 	}
-	
+
 	summary_lock(folderview->summaryview);
 	folder_item_update_freeze();
 	if (folderview->summaryview->folder_item == item)
-		gtk_clist_freeze(GTK_CLIST(folderview->summaryview->ctree));
+		summary_freeze(folderview->summaryview);
 	folderutils_mark_all_read(item);
 	if (folderview->summaryview->folder_item == item)
-		gtk_clist_thaw(GTK_CLIST(folderview->summaryview->ctree));
+		summary_thaw(folderview->summaryview);
 	folder_item_update_thaw();
 	summary_unlock(folderview->summaryview);
 }
@@ -746,12 +796,16 @@ static void folderview_select_node(FolderView *folderview, GtkCTreeNode *node)
 
 	g_return_if_fail(node != NULL);
 
+	if (folderview->open_folder) {
+		return;
+	}
+
 	folderview->open_folder = TRUE;
 	gtkut_ctree_set_focus_row(ctree, node);
 	gtk_ctree_select(ctree, node);
 	if (folderview->summaryview->folder_item &&
 	    folderview->summaryview->folder_item->total_msgs > 0)
-		gtk_widget_grab_focus(folderview->summaryview->ctree);
+		summary_grab_focus(folderview->summaryview);
 	else
 		gtk_widget_grab_focus(folderview->ctree);
 
@@ -767,6 +821,54 @@ void folderview_unselect(FolderView *folderview)
 	folderview->selected = folderview->opened = NULL;
 }
 
+static GtkCTreeNode *folderview_find_next_marked(GtkCTree *ctree,
+						 GtkCTreeNode *node)
+{
+	FolderItem *item;
+
+	if (node)
+		node = gtkut_ctree_node_next(ctree, node);
+	else
+		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+
+	for (; node != NULL; node = gtkut_ctree_node_next(ctree, node)) {
+		item = gtk_ctree_node_get_row_data(ctree, node);
+		if (item && item->marked_msgs > 0 && item->stype != F_TRASH)
+			return node;
+	}
+
+	return NULL;
+}
+
+void folderview_select_next_marked(FolderView *folderview)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	GtkCTreeNode *node = NULL;
+	SelectOnEntry last_sel = prefs_common.select_on_entry;
+	gboolean last_open = prefs_common.always_show_msg;
+	
+	prefs_common.select_on_entry = SELECTONENTRY_MNU;
+	prefs_common.always_show_msg = TRUE;
+
+	if ((node = folderview_find_next_marked(ctree, folderview->opened))
+	    != NULL) {
+		folderview_select_node(folderview, node);
+		goto out;
+	}
+
+	if (!folderview->opened ||
+	    folderview->opened == GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list)) {
+		goto out;
+	}
+	/* search again from the first node */
+	if ((node = folderview_find_next_marked(ctree, NULL)) != NULL)
+		folderview_select_node(folderview, node);
+
+out:
+	prefs_common.select_on_entry = last_sel;
+	prefs_common.always_show_msg = last_open;
+}
+
 static GtkCTreeNode *folderview_find_next_unread(GtkCTree *ctree,
 						 GtkCTreeNode *node)
 {
@@ -786,23 +888,33 @@ static GtkCTreeNode *folderview_find_next_unread(GtkCTree *ctree,
 	return NULL;
 }
 
-void folderview_select_next_unread(FolderView *folderview)
+void folderview_select_next_unread(FolderView *folderview, gboolean force_open)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	GtkCTreeNode *node = NULL;
+	SelectOnEntry last_sel = prefs_common.select_on_entry;
+	gboolean last_open = prefs_common.always_show_msg;
+	
+	prefs_common.select_on_entry = SELECTONENTRY_UNM;
+	prefs_common.always_show_msg = force_open ? TRUE : last_open;
 
 	if ((node = folderview_find_next_unread(ctree, folderview->opened))
 	    != NULL) {
 		folderview_select_node(folderview, node);
-		return;
+		goto out;
 	}
 
 	if (!folderview->opened ||
-	    folderview->opened == GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list))
-		return;
+	    folderview->opened == GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list)) {
+		goto out;
+	}
 	/* search again from the first node */
 	if ((node = folderview_find_next_unread(ctree, NULL)) != NULL)
 		folderview_select_node(folderview, node);
+
+out:
+	prefs_common.select_on_entry = last_sel;
+	prefs_common.always_show_msg = last_open;
 }
 
 static GtkCTreeNode *folderview_find_next_new(GtkCTree *ctree,
@@ -828,19 +940,29 @@ void folderview_select_next_new(FolderView *folderview)
 {
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	GtkCTreeNode *node = NULL;
+	SelectOnEntry last_sel = prefs_common.select_on_entry;
+	gboolean last_open = prefs_common.always_show_msg;
+	
+	prefs_common.select_on_entry = SELECTONENTRY_NUM;
+	prefs_common.always_show_msg = TRUE;
 
 	if ((node = folderview_find_next_new(ctree, folderview->opened))
 	    != NULL) {
 		folderview_select_node(folderview, node);
-		return;
+		goto out;
 	}
 
 	if (!folderview->opened ||
-	    folderview->opened == GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list))
-		return;
+	    folderview->opened == GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list)) {
+		goto out;
+	}
 	/* search again from the first node */
 	if ((node = folderview_find_next_new(ctree, NULL)) != NULL)
 		folderview_select_node(folderview, node);
+
+out:
+	prefs_common.select_on_entry = last_sel;
+	prefs_common.always_show_msg = last_open;
 }
 
 FolderItem *folderview_get_selected_item(FolderView *folderview)
@@ -923,48 +1045,31 @@ void folderview_append_item(FolderItem *item)
 static void folderview_set_folders(FolderView *folderview)
 {
 	GList *list;
-	static gboolean missing_imap_warning = TRUE;
 	list = folder_get_list();
 
 	for (; list != NULL; list = list->next) {
-#ifndef HAVE_LIBETPAN
-		if ((FOLDER(list->data))
-		&&  (FOLDER(list->data))->klass
-		&&  (FOLDER(list->data))->klass->type == F_IMAP
-		&&  missing_imap_warning) {
-			missing_imap_warning = FALSE;
-			alertpanel_error(
-				_("You have one or more IMAP accounts "
-				  "defined. However this version of "
-				  "Sylpheed-Claws has been built without "
-				  "IMAP support; your IMAP account(s) are "
-				  "disabled.\n\n"
-				  "You probably need to "
-				  "install libetpan and recompile "
-				  "Sylpheed-Claws."));
-		}
-#endif
 		folderview_append_folder(folderview, FOLDER(list->data));
 	}
 }
 
+static gchar *get_scan_str(FolderItem *item)
+{
+	if (item->path)
+		return g_strdup_printf(_("Scanning folder %s%c%s ..."),
+				      item->folder->name, G_DIR_SEPARATOR,
+				      item->path);
+	else
+		return g_strdup_printf(_("Scanning folder %s ..."),
+				      item->folder->name);	
+}
 static void folderview_scan_tree_func(Folder *folder, FolderItem *item,
 				      gpointer data)
 {
 	GList *list;
-
 	for (list = folderview_list; list != NULL; list = list->next) {
 		FolderView *folderview = (FolderView *)list->data;
 		MainWindow *mainwin = folderview->mainwin;
-		gchar *str;
-
-		if (item->path)
-			str = g_strdup_printf(_("Scanning folder %s%c%s ..."),
-					      item->folder->name, G_DIR_SEPARATOR,
-					      item->path);
-		else
-			str = g_strdup_printf(_("Scanning folder %s ..."),
-					      item->folder->name);
+		gchar *str = get_scan_str(item);
 
 		STATUSBAR_PUSH(mainwin, str);
 		STATUSBAR_POP(mainwin);
@@ -1032,24 +1137,51 @@ gint folderview_check_new(Folder *folder)
 
 		for (node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 		     node != NULL; node = gtkut_ctree_node_next(ctree, node)) {
+			gchar *str = NULL;
 			item = gtk_ctree_node_get_row_data(ctree, node);
 			if (!item || !item->path || !item->folder) continue;
 			if (item->no_select) continue;
 			if (folder && folder != item->folder) continue;
 			if (!folder && !FOLDER_IS_LOCAL(item->folder)) continue;
 			if (!item->prefs->newmailcheck) continue;
+			if (item->processing_pending == TRUE) {
+				debug_print("skipping %s, processing pending\n",
+					item->path ? item->path : item->name);
+				continue;
+			}
+
+			str = get_scan_str(item);
+
+			STATUSBAR_PUSH(folderview->mainwin, str);
+			GTK_EVENTS_FLUSH();
+			g_free(str);
 
 			folderview_scan_tree_func(item->folder, item, NULL);
 			former_new    = item->new_msgs;
 			former_unread = item->unread_msgs;
 			former_total  = item->total_msgs;
 
-			if (folder_item_scan(item) < 0) {
-				summaryview_unlock(folderview->summaryview, item);
-				if (folder && !FOLDER_IS_LOCAL(folder))
-					break;
+			if (item->folder->klass->scan_required &&
+			    (item->folder->klass->scan_required(item->folder, item) ||
+			     item->folder->inbox == item ||
+			     item->opened == TRUE ||
+			     item->processing_pending == TRUE)) {
+				if (folder_item_scan(item) < 0) {
+					summaryview_unlock(folderview->summaryview, item);
+					if (folder && !FOLDER_IS_LOCAL(folder)) {
+						STATUSBAR_POP(folderview->mainwin);
+						break;
+					}
+				}
+			} else if (!item->folder->klass->scan_required) {
+				if (folder_item_scan(item) < 0) {
+					summaryview_unlock(folderview->summaryview, item);
+					if (folder && !FOLDER_IS_LOCAL(folder)) {
+						STATUSBAR_POP(folderview->mainwin);
+						break;
+					}
+				}
 			}
-
 			if (former_new    != item->new_msgs ||
 			    former_unread != item->unread_msgs ||
 			    former_total  != item->total_msgs)
@@ -1057,6 +1189,7 @@ gint folderview_check_new(Folder *folder)
 
 			new_msgs += item->new_msgs;
 			former_new_msgs += former_new;
+			STATUSBAR_POP(folderview->mainwin);
 		}
 
 		main_window_unlock(folderview->mainwin);
@@ -1464,9 +1597,6 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 			color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 			style = color_style;
 		} else if (use_color) {
-			gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
-			bold_color_style = gtk_style_copy(bold_style);
-			bold_color_style->fg[GTK_STATE_NORMAL] = gdk_color;
 			style = bold_color_style;
 		} else
 			style = bold_style;
@@ -1474,11 +1604,7 @@ static void folderview_update_node(FolderView *folderview, GtkCTreeNode *node)
 			style = bold_tgtfold_style;
 		}
 	} else if (use_color) {
-		GdkColor gdk_color;
-
-		gtkut_convert_int_to_gdk_color(prefs_common.color_new, &gdk_color);
-		style = gtk_style_copy(normal_color_style);
-		style->fg[GTK_STATE_NORMAL] = gdk_color;
+		style = normal_color_style;
 		gtk_ctree_node_set_foreground(ctree, node,
 					      &folderview->color_new);
 	} else if (item->op_count > 0) {
@@ -1562,7 +1688,8 @@ gboolean folderview_update_item_claws(gpointer source, gpointer data)
 		if (update_info->update_flags & (F_ITEM_UPDATE_MSGCNT | F_ITEM_UPDATE_NAME))
 			folderview_update_node(folderview, node);
 		if ((update_info->update_flags & F_ITEM_UPDATE_CONTENT) && (folderview->opened == node))
-			summary_show(folderview->summaryview, update_info->item);
+			if (!quicksearch_is_active(folderview->summaryview->quicksearch))
+				summary_show(folderview->summaryview, update_info->item);
 	}
 	
 	return FALSE;
@@ -1659,15 +1786,18 @@ static void folderview_sort_folders(FolderView *folderview, GtkCTreeNode *root,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	GtkCTreeNode *prev = NULL;
 
+	gtk_clist_freeze(GTK_CLIST(ctree));
 	gtk_sctree_sort_recursive(ctree, root);
-
-	if (root && GTK_CTREE_ROW(root)->parent) return;
-
+	if (root && GTK_CTREE_ROW(root)->parent) {
+		gtk_clist_thaw(GTK_CLIST(ctree));
+		return;
+	}
 	set_special_folder(ctree, folder->inbox, root, &prev);
 	set_special_folder(ctree, folder->outbox, root, &prev);
 	set_special_folder(ctree, folder->draft, root, &prev);
 	set_special_folder(ctree, folder->queue, root, &prev);
 	set_special_folder(ctree, folder->trash, root, &prev);
+	gtk_clist_thaw(GTK_CLIST(ctree));
 }
 
 static void folderview_append_folder(FolderView *folderview, Folder *folder)
@@ -1677,7 +1807,7 @@ static void folderview_append_folder(FolderView *folderview, Folder *folder)
 
 	g_return_if_fail(folder != NULL);
 
-	root = gtk_ctree_insert_gnode(ctree, NULL, NULL, folder->node,
+	root = gtk_sctree_insert_gnode(ctree, NULL, NULL, folder->node,
 				      folderview_gnode_func, folderview);
 	gtk_ctree_pre_recursive(ctree, root, folderview_expand_func,
 				folderview);
@@ -1685,34 +1815,117 @@ static void folderview_append_folder(FolderView *folderview, Folder *folder)
 }
 
 /* callback functions */
+static void folderview_set_sens_and_popup_menu(FolderView *folderview, gint row, 
+				GdkEventButton *event)
+{
+	GtkCList *clist = GTK_CLIST(folderview->ctree);
+	FolderItem *item;
+	Folder *folder;
+	FolderViewPopup *fpopup;
+	GtkItemFactory *fpopup_factory;
+	GtkWidget *popup;
+	FolderItem *special_trash = NULL, *special_queue = NULL;
+	PrefsAccount *ac;
+
+	if (row > 0)
+		item = gtk_clist_get_row_data(clist, row);
+	else
+		item = folderview_get_selected_item(folderview);
+
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+	folder = item->folder;
+
+	fpopup = g_hash_table_lookup(folderview_popups, folder->klass->idstr);
+	if (fpopup != NULL)
+		fpopup_factory = g_hash_table_lookup(folderview->popups, folder->klass->idstr);
+	else {
+		fpopup = g_hash_table_lookup(folderview_popups, "common");
+		fpopup_factory = g_hash_table_lookup(folderview->popups, "common");
+	}
+
+	if (fpopup->set_sensitivity != NULL)
+		fpopup->set_sensitivity(fpopup_factory, item);
+
+	if (NULL != (ac = account_find_from_item(item))) {
+		special_trash = account_get_special_folder(ac, F_TRASH);
+		special_queue = account_get_special_folder(ac, F_QUEUE);
+	}
+	
+	if ((item == folder->trash || item == special_trash
+	     || folder_has_parent_of_type(item, F_TRASH)) &&
+	    gtk_item_factory_get_item(fpopup_factory, "/Empty trash...") == NULL) {
+		gtk_item_factory_create_item(fpopup_factory, &folder_view_trash_popup_entries[0], folderview, 1);
+		gtk_item_factory_create_item(fpopup_factory, &folder_view_trash_popup_entries[1], folderview, 1);
+	} else if (item != folder->trash && (special_trash == NULL || item != special_trash)
+	        && !folder_has_parent_of_type(item, F_TRASH)) {
+		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_trash_popup_entries[0]);
+		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_trash_popup_entries[1]);
+	}
+	
+	if ((item == folder->queue || item == special_queue
+	     || folder_has_parent_of_type(item, F_QUEUE)) &&
+	    gtk_item_factory_get_item(fpopup_factory, "/Send queue...") == NULL) {
+		gtk_item_factory_create_item(fpopup_factory, &folder_view_queue_popup_entries[0], folderview, 1);
+		gtk_item_factory_create_item(fpopup_factory, &folder_view_queue_popup_entries[1], folderview, 1);
+	} else if (item != folder->queue && (special_queue == NULL || item != special_queue)
+	        && !folder_has_parent_of_type(item, F_QUEUE)) {
+		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_queue_popup_entries[0]);
+		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_queue_popup_entries[1]);
+	}
+	
+#define SET_SENS(name, sens) \
+	menu_set_sensitive(fpopup_factory, name, sens)
+
+	SET_SENS("/Mark all read", item->unread_msgs >= 1);
+	SET_SENS("/Search folder...", item->total_msgs >= 1 && 
+		 folderview->selected == folderview->opened);
+	SET_SENS("/Properties...", TRUE);
+	SET_SENS("/Processing...", item->node->parent != NULL);
+	if (item == folder->trash || item == special_trash
+	    || folder_has_parent_of_type(item, F_TRASH)) {
+		GSList *msglist = folder_item_get_msg_list(item);
+		SET_SENS("/Empty trash...", msglist != NULL);
+		procmsg_msg_list_free(msglist);
+	}
+	if (item == folder->queue || item == special_queue
+	    || folder_has_parent_of_type(item, F_QUEUE)) {
+		GSList *msglist = folder_item_get_msg_list(item);
+		SET_SENS("/Send queue...", msglist != NULL);
+		procmsg_msg_list_free(msglist);
+	}
+#undef SET_SENS
+
+	popup = gtk_item_factory_get_widget(fpopup_factory, fpopup->path);
+	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
+		       event->button, event->time);
+
+
+}
 
 static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 					  FolderView *folderview)
 {
 	GtkCList *clist = GTK_CLIST(ctree);
 	gint prev_row = -1, row = -1, column = -1;
-	FolderItem *item;
-	Folder *folder;
-	FolderViewPopup *fpopup;
-	GtkItemFactory *fpopup_factory;
-	GtkWidget *popup;
-	FolderItem *special_trash = NULL;
-	PrefsAccount *ac;
 
 	if (!event) return FALSE;
 
 	if (event->button == 1 || event->button == 2) {
-		folderview->open_folder = TRUE;
+		if (!gtk_ctree_is_hot_spot (GTK_CTREE(clist), event->x, event->y))
+			folderview->open_folder = TRUE;
 
 	        if (event->type == GDK_2BUTTON_PRESS) {
 			if (clist->selection) {
 				GtkCTreeNode *node;
 
 				node = GTK_CTREE_NODE(clist->selection->data);
-				if (node)
+				if (node) {
 					gtk_ctree_toggle_expansion(
 						GTK_CTREE(ctree),
 						node);
+					folderview->open_folder = FALSE;
+				}
 			}
 		}
 		return FALSE;
@@ -1746,56 +1959,7 @@ static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *even
 
 	if (event->button != 3) return FALSE;
 
-	item = gtk_clist_get_row_data(clist, row);
-	g_return_val_if_fail(item != NULL, FALSE);
-	g_return_val_if_fail(item->folder != NULL, FALSE);
-	folder = item->folder;
-
-	fpopup = g_hash_table_lookup(folderview_popups, folder->klass->idstr);
-	if (fpopup != NULL)
-		fpopup_factory = g_hash_table_lookup(folderview->popups, folder->klass->idstr);
-	else {
-		fpopup = g_hash_table_lookup(folderview_popups, "common");
-		fpopup_factory = g_hash_table_lookup(folderview->popups, "common");
-	}
-
-	if (fpopup->set_sensitivity != NULL)
-		fpopup->set_sensitivity(fpopup_factory, item);
-
-	if (NULL != (ac = account_find_from_item(item)))
-		special_trash = account_get_special_folder(ac, F_TRASH);
-
-	if ((item == folder->trash || item == special_trash
-	     || folder_has_parent_of_type(item, F_TRASH)) &&
-	    gtk_item_factory_get_item(fpopup_factory, "/Empty trash...") == NULL) {
-		gtk_item_factory_create_item(fpopup_factory, &folder_view_trash_popup_entries[0], folderview, 1);
-		gtk_item_factory_create_item(fpopup_factory, &folder_view_trash_popup_entries[1], folderview, 1);
-	} else if (item != folder->trash && (special_trash == NULL || item != special_trash)
-	        && !folder_has_parent_of_type(item, F_TRASH)) {
-		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_trash_popup_entries[0]);
-		gtk_item_factory_delete_entry(fpopup_factory, &folder_view_trash_popup_entries[1]);
-	}
-	
-#define SET_SENS(name, sens) \
-	menu_set_sensitive(fpopup_factory, name, sens)
-
-	SET_SENS("/Mark all read", item->unread_msgs >= 1);
-	SET_SENS("/Search folder...", item->total_msgs >= 1 && 
-		 folderview->selected == folderview->opened);
-	SET_SENS("/Properties...", item->node->parent != NULL);
-	SET_SENS("/Processing...", item->node->parent != NULL);
-	if (item == folder->trash || item == special_trash
-	    || folder_has_parent_of_type(item, F_TRASH)) {
-		GSList *msglist = folder_item_get_msg_list(item);
-		SET_SENS("/Empty trash...", msglist != NULL);
-		procmsg_msg_list_free(msglist);
-	}
-#undef SET_SENS
-
-	popup = gtk_item_factory_get_widget(fpopup_factory, fpopup->path);
-	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
-		       event->button, event->time);
-
+	folderview_set_sens_and_popup_menu(folderview, row, event);
 	return FALSE;
 }
 
@@ -1834,7 +1998,7 @@ static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 			if (folderview->opened == folderview->selected &&
 			    (!folderview->summaryview->folder_item ||
 			     folderview->summaryview->folder_item->total_msgs == 0))
-				folderview_select_next_unread(folderview);
+				folderview_select_next_unread(folderview, TRUE);
 			else
 				folderview_select_node(folderview,
 						       folderview->selected);
@@ -1847,6 +2011,27 @@ static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	return FALSE;
 }
 
+typedef struct _PostponedSelectData
+{
+	GtkCTree *ctree;
+	GtkCTreeNode *row;
+	gint column;
+	FolderView *folderview;
+} PostponedSelectData;
+
+static gboolean postpone_select(void *data)
+{
+	PostponedSelectData *psdata = (PostponedSelectData *)data;
+	debug_print("trying again\n");
+	psdata->folderview->open_folder = TRUE;
+	main_window_cursor_normal(psdata->folderview->mainwin);
+	STATUSBAR_POP(psdata->folderview->mainwin);
+	folderview_selected(psdata->ctree, psdata->row,
+			    psdata->column, psdata->folderview);
+	g_free(psdata);
+	return FALSE;
+}
+
 static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 				gint column, FolderView *folderview)
 {
@@ -1854,24 +2039,35 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 	gboolean opened;
 	FolderItem *item;
 	gchar *buf;
-
+	int res = 0;
+	START_TIMING("");
 	folderview->selected = row;
 
 	if (folderview->opened == row) {
 		folderview->open_folder = FALSE;
+		END_TIMING();
 		return;
 	}
 	
 	if (!can_select || summary_is_locked(folderview->summaryview)) {
-		gtkut_ctree_set_focus_row(ctree, folderview->opened);
-		gtk_ctree_select(ctree, folderview->opened);
+		if (folderview->opened) {
+			gtkut_ctree_set_focus_row(ctree, folderview->opened);
+			gtk_ctree_select(ctree, folderview->opened);
+		}
+		END_TIMING();
 		return;
 	}
 
-	if (!folderview->open_folder) return;
-
+	if (!folderview->open_folder) {
+		END_TIMING();
+		return;
+	}
 	item = gtk_ctree_node_get_row_data(ctree, row);
-	if (!item || item->no_select) return;
+	if (!item || item->no_select) {
+		END_TIMING();
+		folderview->open_folder = FALSE;
+		return;
+	}
 
 	can_select = FALSE;
 
@@ -1882,10 +2078,17 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 		
 		olditem = gtk_ctree_node_get_row_data(ctree, folderview->opened);
 		if (olditem) {
+			buf = g_strdup_printf(_("Closing Folder %s..."), 
+				olditem->path ? olditem->path:olditem->name);
 			/* will be null if we just moved the previously opened folder */
+			STATUSBAR_PUSH(folderview->mainwin, buf);
+			main_window_cursor_wait(folderview->mainwin);
+			g_free(buf);
 			summary_save_prefs_to_folderitem(folderview->summaryview, olditem);
 			summary_show(folderview->summaryview, NULL);
 			folder_item_close(olditem);
+			main_window_cursor_normal(folderview->mainwin);
+			STATUSBAR_POP(folderview->mainwin);
 		}
 	}
 
@@ -1922,7 +2125,8 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 	main_window_cursor_wait(folderview->mainwin);
 
-	if (folder_item_open(item) != 0) {
+	res = folder_item_open(item);
+	if (res == -1) {
 		main_window_cursor_normal(folderview->mainwin);
 		STATUSBAR_POP(folderview->mainwin);
 
@@ -1930,9 +2134,22 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 		folderview->open_folder = FALSE;
 		can_select = TRUE;
-
+		END_TIMING();
 		return;
-        }
+        } else if (res == -2) {
+		PostponedSelectData *data = g_new0(PostponedSelectData, 1);
+		data->ctree = ctree;
+		data->row = row;
+		data->column = column;
+		data->folderview = folderview;
+		debug_print("postponing open of %s till end of scan\n",
+			item->path ? item->path:item->name);
+		folderview->open_folder = FALSE;
+		can_select = TRUE;
+		g_timeout_add(500, postpone_select, data);
+		END_TIMING();
+		return;
+	}
 	
 	main_window_cursor_normal(folderview->mainwin);
 
@@ -1940,7 +2157,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 	summary_set_prefs_from_folderitem(folderview->summaryview, item);
 	opened = summary_show(folderview->summaryview, item);
 	
-	folder_clean_cache_memory();
+	folder_clean_cache_memory(item);
 
 	if (!opened) {
 		gtkut_ctree_set_focus_row(ctree, folderview->opened);
@@ -1956,6 +2173,7 @@ static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
 
 	folderview->open_folder = FALSE;
 	can_select = TRUE;
+	END_TIMING();
 }
 
 static void folderview_tree_expanded(GtkCTree *ctree, GtkCTreeNode *node,
@@ -2095,6 +2313,64 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	folder_item_remove_all_msg(item);
 }
 
+static void folderview_send_queue_cb(FolderView *folderview, guint action,
+				      GtkWidget *widget)
+{
+	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
+	FolderItem *item;
+	FolderItem *special_queue = NULL;
+	PrefsAccount *ac;
+	gchar *errstr = NULL;
+
+	if (!folderview->selected) return;
+	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
+	g_return_if_fail(item != NULL);
+	g_return_if_fail(item->folder != NULL);
+
+	if (NULL != (ac = account_find_from_item(item)))
+		special_queue = account_get_special_folder(ac, F_QUEUE);
+
+	if (item != item->folder->queue && item != special_queue
+	&&  !folder_has_parent_of_type(item, F_QUEUE)) return;
+	
+	if (procmsg_queue_is_empty(item))
+		return;
+
+	if (prefs_common.work_offline)
+		if (alertpanel(_("Offline warning"), 
+			       _("You're working offline. Override?"),
+			       GTK_STOCK_NO, GTK_STOCK_YES,
+			       NULL) != G_ALERTALTERNATE)
+		return;
+
+	/* ask for confirmation before sending queued messages only
+	   in online mode and if there is at least one message queued
+	   in any of the folder queue
+	*/
+	if (prefs_common.confirm_send_queued_messages) {
+		if (!prefs_common.work_offline) {
+			if (alertpanel(_("Send queued messages"), 
+			    	   _("Send all queued messages?"),
+			    	   GTK_STOCK_CANCEL, _("_Send"),
+				   NULL) != G_ALERTALTERNATE)
+				return;
+		}
+	}
+
+	if (procmsg_send_queue(item, prefs_common.savemsg, &errstr) < 0) {
+		if (!errstr)
+			alertpanel_error_log(_("Some errors occurred while "
+					   "sending queued messages."));
+		else {
+			gchar *tmp = g_strdup_printf(_("Some errors occurred "
+					"while sending queued messages:\n%s"), errstr);
+			g_free(errstr);
+			alertpanel_error_log(tmp);
+			g_free(tmp);
+		}
+	}
+}
+
 static void folderview_search_cb(FolderView *folderview, guint action,
 				 GtkWidget *widget)
 {
@@ -2113,9 +2389,6 @@ static void folderview_property_cb(FolderView *folderview, guint action,
 	g_return_if_fail(item != NULL);
 	g_return_if_fail(item->folder != NULL);
 
-	if (folder_item_parent(item) == NULL)
-		return;
-
 	prefs_folder_item_open(item);
 }
 
@@ -2164,10 +2437,9 @@ void folderview_move_folder(FolderView *folderview, FolderItem *from_folder,
 				       	 NULL, ALERT_QUESTION, G_ALERTDEFAULT);
 		g_free(buf);
 
-		if (status != G_ALERTALTERNATE
-		 && status != (G_ALERTALTERNATE | G_ALERTDISABLE))
+		if ((status & ~G_ALERTDISABLE) != G_ALERTALTERNATE)
 			return;
-		if (status & G_ALERTDISABLE)
+		else if (status & G_ALERTDISABLE)
 			prefs_common.warn_dnd = FALSE;
 	}
 
@@ -2248,7 +2520,8 @@ static void folderview_processing_cb(FolderView *folderview, guint action,
 	title = g_strdup_printf (_("Processing configuration for folder %s"), id);
 	g_free (id);
 
-	prefs_filtering_open(&item->prefs->processing, title, NULL, NULL);
+	prefs_filtering_open(&item->prefs->processing, title,
+			MANUAL_ANCHOR_PROCESSING, NULL, NULL, FALSE);
 	g_free (title);
 }
 
@@ -2273,8 +2546,7 @@ static gchar *last_font = NULL;
 void folderview_reflect_prefs_pixmap_theme(FolderView *folderview)
 {
 	/* force reinit */
-	if (last_font) 
-		g_free(last_font);
+	g_free(last_font);
 	last_font = NULL;
 	
 }
@@ -2291,9 +2563,7 @@ void folderview_reflect_prefs(void)
 	if (last_font && !strcmp(last_font, NORMAL_FONT))
 		update_font = FALSE;
 
-	if (last_font)
-		g_free(last_font);
-	
+	g_free(last_font);
 	last_font = g_strdup(NORMAL_FONT);
 
 	if (update_font) {		
@@ -2315,6 +2585,7 @@ void folderview_reflect_prefs(void)
 			GTK_CTREE(folderview->ctree), NULL, item);
 
 		folderview_select(folderview, item);
+		folderview->open_folder = FALSE;
 		folderview->selected = node;
 	}
 
@@ -2476,7 +2747,7 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		src_item = folderview->summaryview->folder_item;
 
 		srcwidget = gtk_drag_get_source_widget(context);
-		if (srcwidget == folderview->summaryview->ctree) {
+		if (srcwidget == summary_get_main_widget(folderview->summaryview)) {
 			/* comes from summaryview */
 			/* we are copying messages, so only accept folder items that are not
 			   the source item, are no root items and can copy messages */
