@@ -92,6 +92,7 @@ struct _GeditPrintJobInfo
 	GtkWidget         *progressbar;
 
 	GtkWindow	  *parent;
+	gboolean	   done;
 };
 
 static GeditPrintJobInfo* gedit_print_job_info_new 	(GtkTextView       *view);
@@ -200,7 +201,7 @@ gedit_print_dialog_response (GtkWidget *dialog, int response, GeditPrintJobInfo
 
 	default:
 		gtk_widget_destroy (dialog);
-		gedit_print_job_info_destroy (pji, FALSE);
+		pji->done = TRUE;
         }
 } 
 
@@ -276,7 +277,7 @@ page_cb (GtkSourcePrintJob *job, GeditPrintJobInfo *pji)
 	g_free (str);
 
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (pji->progressbar), 
-				       1.0 * page_num / total);
+				       (total == 0) ? 0 : 1.0 * page_num / total);
 
 	/* Update UI */
 	while (gtk_events_pending ())
@@ -304,7 +305,7 @@ preview_finished_cb (GtkSourcePrintJob *job, GeditPrintJobInfo *pji)
 	gtk_widget_destroy (pji->dialog);
 
 	if (pji->preview == PREVIEW)
-		gedit_print_job_info_destroy (pji, FALSE);
+		pji->done = TRUE;
 	else
 	{
 		g_signal_handlers_disconnect_by_func (pji->pjob, (GCallback) page_cb, pji);
@@ -327,7 +328,7 @@ print_finished_cb (GtkSourcePrintJob *job, GeditPrintJobInfo *pji)
 
 	gtk_widget_destroy (pji->dialog);
 
-	gedit_print_job_info_destroy (pji, TRUE);
+	pji->done = TRUE;
 }
 
 void 
@@ -355,6 +356,10 @@ gedit_print (GtkTextView *view)
 			  pji);
 
 	gtk_widget_show (dialog);
+	while (pji->done != TRUE)
+		gtk_main_iteration();
+	gedit_print_job_info_destroy (pji, pji->preview ? FALSE:TRUE);
+	
 }
 
 static void 
@@ -457,9 +462,20 @@ gedit_print_job_info_new (GtkTextView* view)
 	gtk_source_print_job_set_print_footer (pjob, TRUE);
 
 	pango_context = gtk_widget_get_pango_context (GTK_WIDGET (view));
-	font_desc = pango_context_get_font_description (pango_context);
+	if (prefs_common.use_different_print_font) 
+	{
+		font_desc = pango_font_description_from_string (prefs_common.printfont);
+	}
+	else
+	{
+		font_desc = pango_context_get_font_description (pango_context);
+	}
 
 	gtk_source_print_job_set_font_desc (pjob, font_desc);
+	if (prefs_common.use_different_print_font) 
+	{
+        	pango_font_description_free (font_desc);
+	}
 
 	pji = g_new0 (GeditPrintJobInfo, 1);
 
