@@ -132,7 +132,7 @@ static void about_create(void)
 	table = gtk_table_new (1, 2, FALSE);
 	gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 0);
 
-	image = stock_pixmap_widget(window, STOCK_PIXMAP_SYLPHEED_LOGO);
+	image = stock_pixmap_widget(window, STOCK_PIXMAP_SYLPHEED_CLAWS_LOGO);
 	gtk_table_attach(GTK_TABLE(table), image, 0, 1, 0, 1,
 			 (GtkAttachOptions) (GTK_SHRINK),
 			 (GtkAttachOptions) (GTK_SHRINK), 8, 0);
@@ -159,23 +159,29 @@ static void about_create(void)
 	uname(&utsbuf);
 	g_snprintf(buf, sizeof(buf),
 		   _("GTK+ %d.%d.%d / GLib %d.%d.%d\n"
-		   "Operating System: %s %s (%s)"),
+		     "Locale: %s (charset: %s)\n"
+		     "Operating System: %s %s (%s)"),
 		   gtk_major_version, gtk_minor_version, gtk_micro_version,
 		   glib_major_version, glib_minor_version, glib_micro_version,
+		   conv_get_current_locale(), conv_get_locale_charset_str(),
 		   utsbuf.sysname, utsbuf.release, utsbuf.machine);
 #elif defined(G_OS_WIN32)
 	g_snprintf(buf, sizeof(buf),
 		   _("GTK+ %d.%d.%d / GLib %d.%d.%d\n"
-		   "Operating System: %s"),
+		     "Locale: %s (charset: %s)\n"
+		     "Operating System: %s"),
 		   gtk_major_version, gtk_minor_version, gtk_micro_version,
 		   glib_major_version, glib_minor_version, glib_micro_version,
+		   conv_get_current_locale(), conv_get_locale_charset_str(),
 		   "Win32");
 #else
 	g_snprintf(buf, sizeof(buf),
 		   _("GTK+ %d.%d.%d / GLib %d.%d.%d\n"
-		   "Operating System: unknown"),
+		     "Locale: %s (charset: %s)\n"
+		     "Operating System: unknown"),
 		   gtk_major_version, gtk_minor_version, gtk_micro_version,
-		   glib_major_version, glib_minor_version, glib_micro_version);
+		   glib_major_version, glib_minor_version, glib_micro_version,
+		   conv_get_current_locale(), conv_get_locale_charset_str());
 #endif
 
 	label = gtk_label_new(buf);
@@ -215,6 +221,9 @@ static void about_create(void)
 #endif
 #if USE_GNOMEPRINT
 		   " libgnomeprint"
+#endif
+#if HAVE_LIBSM
+		   " libSM"
 #endif
 	"");
 
@@ -272,7 +281,7 @@ static void about_create(void)
 
 	gtk_text_buffer_insert(buffer, &iter, _(
 				"Sylpheed-Claws is a lightweight, fast and "
-				"highly-configurable e-mail client.\n\n"
+				"highly-configurable email client.\n\n"
 				"For further information visit the Sylpheed-"
 				"Claws website:\n"), -1);
 	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, HOMEPAGE_URI, -1,
@@ -492,7 +501,7 @@ static void about_create(void)
 	gtk_text_buffer_insert(buffer, &iter,
 		_("This product includes software developed by the OpenSSL Project "
 		  "for use in the OpenSSL Toolkit ("), -1);
-	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, "http://www.openssl.org/", -1,
+	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, OPENSSL_URI, -1,
 		"link", NULL);
 	gtk_text_buffer_insert(buffer, &iter, _(").\n"), -1);
 
@@ -684,7 +693,7 @@ static void about_textview_uri_update(GtkWidget *textview, gint x, gint y)
 	}
 
 	/* compare previous hovered link and this one
-	   (here links must be unique in text buffer otherwise RemoteURI structures should be
+	   (here links must be unique in text buffer otherwise ClickableText structures should be
 	   used as in textview.c) */
 	same = (uri != NULL && uri_hover != NULL
 		&& strcmp((char*)uri, (char*)uri_hover) == 0);
