@@ -3,11 +3,11 @@ static char *TEAM_LIST[] = {
 "Ho Vit Dinh <hoa@users.sf.net>",
 "Werner Koch <wk@gnupg.org>",
 "Darko Koruga <darko@users.sf.net>",
-"Ricardo Mones Lastra <mones@users.sf.net>",
-"Colin Leroy <colinleroy@users.sf.net>",
-"Paul Mangan <twb@users.sf.net>",
+"Ricardo Mones Lastra <ricardo@mones.org>",
+"Colin Leroy <colin@colino.net>",
+"Paul Mangan <claws@thewildbeast.co.uk>",
 "Martin Schaaf <mascha1@users.sf.net>",
-"wwp <wwp@users.sf.net>",
+"Tristan Chabredier <wwp@users.sf.net>",
 NULL
 };
 
@@ -32,9 +32,9 @@ static char *TRANS_TEAM_LIST[] = {
 "[ca] Miquel Oliete <ktalanet@yahoo.es>",
 "[cs] Tim <timbrain@post.cz>",
 "[de] Stephan Sachse <white@dev-zero.com>",
-"[en_GB] Paul Mangan <twb@users.sf.net>",
+"[en_GB] Paul Mangan <claws@thewildbeast.co.uk>",
 "[el] Stavros Giannouris <stavrosg2002@freemail.gr>",
-"[es] Ricardo Mones Lastra <mones@users.sf.net>",
+"[es] Ricardo Mones Lastra <ricardo@mones.org>",
 "[fi] Flammie Pirinen <flammie@iki.fi>",
 "[fr] Fabien Vantard <fzzzzz@gmail.com>",
 "[hr] Dragan <drleskov@inet.hr>",
@@ -56,9 +56,13 @@ NULL
 };
 
 static char *DOC_TEAM_LIST[] = {
-"Ricardo Mones Lastra <mones@users.sf.net>",
-"Colin Leroy <colinleroy@users.sf.net>",
-"Paul Mangan <twb@users.sf.net>",
+"[en] Ricardo Mones Lastra <ricardo@mones.org>",
+"     Colin Leroy <colin@colino.net>",
+"     Paul Mangan <claws@thewildbeast.co.uk>",
+"     Tristan Chabredier <wwp@users.sf.net>",
+"[fr] Colin Leroy <colin@colino.net>",
+"     Tristan Chabredier <wwp@users.sf.net>",
+"[pl] Pawel Pekala <c0rn@gazeta.pl>",
 NULL
 };
 
@@ -94,9 +98,11 @@ static char *CONTRIBS_LIST[] = {
 "Matthieu Dazy",
 "Pierric Descamps",
 "Felix Eckhofer",
+"Charles A. Edwards",
 "Stefaan Eeckels",
 "Willem van Engen",
 "Paul Evans",
+"Leonid Evdokimov",
 "Xavier FACQ",
 "Bob Forsman",
 "Matthias Frste",
@@ -116,10 +122,12 @@ static char *CONTRIBS_LIST[] = {
 "Marc Hoper",
 "Michihide Hotta",
 "Shawn Houston",
+"Michael Hughes",
 "Chideok Hwang",
 "John E.P. Hynes",
 "Hironori IWANE",
 "Motonobu Ichimura",
+"Alexey Illarionov",
 "Yoichi Imai",
 "Mathias Intemann",
 "Ishihara",
@@ -163,6 +171,7 @@ static char *CONTRIBS_LIST[] = {
 "Nishimura",
 "Nishiyama",
 "Masaaki Noro",
+"Ales Nosek",
 "James Noyes",
 "Jens Oberender",
 "Ohmasa",
@@ -215,6 +224,7 @@ static char *CONTRIBS_LIST[] = {
 "Radek Vybiral",
 "Jonathan Ware",
 "Watanabe",
+"Rafal Weglarz",
 "Martin Wicke",
 "Bob Woodside",
 "YAMAGUCHI",
@@ -225,6 +235,5 @@ static char *CONTRIBS_LIST[] = {
 "Gl Zoltn",
 "Marien Zwart",
 "Martin Zwickel",
-"Alexey Illarionov",
 NULL
 };
