@@ -33,7 +33,13 @@ struct _GtkSCTree {
 	gint dnd_select_pending;
 	guint dnd_select_pending_state;
 	gint dnd_select_pending_row;
-	gboolean selecting_range;
+	gint selecting_range;
+	gboolean sorting;
+
+	/* (dis)allow fancy color stripes */
+	gboolean show_stripes;
+	GtkTooltips *tips;
+	gboolean always_expand_recursively;
 };
 
 struct _GtkSCTreeClass {
@@ -71,6 +77,9 @@ void gtk_sctree_set_anchor_row		(GtkSCTree	*sctree,
 void gtk_sctree_remove_node		(GtkSCTree	*sctree,
 					 GtkCTreeNode	*node);
 
+void gtk_sctree_set_stripes(GtkSCTree  *sctree, gboolean show_stripes);
+void gtk_sctree_set_recursive_expand(GtkSCTree  *sctree, gboolean rec_exp);
+
 /***********************************************************
  *             Tree sorting functions                      *
  ***********************************************************/
@@ -100,5 +109,15 @@ void        gtk_sctree_set_node_info        (GtkCTree *ctree,
                                              GdkBitmap *mask_opened,
                                              gboolean is_leaf,
                                              gboolean expanded);
+GtkCTreeNode *
+gtk_sctree_insert_gnode 		    (GtkCTree          *ctree,
+					     GtkCTreeNode      *parent,
+					     GtkCTreeNode      *sibling,
+					     GNode             *gnode,
+					     GtkCTreeGNodeFunc  func,
+					     gpointer           data);
 
+void gtk_sctree_set_column_tooltip	    (GtkSCTree		*sctree,
+					     int		 column,
+					     const gchar 	*tip);
 #endif /* __GTK_SCTREE_H__ */
