@@ -60,6 +60,7 @@
 #include "prefs_common.h"
 #include "manage_window.h"
 #include "base64.h"
+#include "manual.h"
 
 gboolean gtkut_get_font_size(GtkWidget *widget,
 			     gint *width, gint *height)
@@ -104,6 +105,36 @@ void gtkut_convert_int_to_gdk_color(gint rgbvalue, GdkColor *color)
 	color->blue  = (int) (((gdouble) (rgbvalue & 0x0000ff)        / 255.0) * 65535.0);
 }
 
+void gtkut_stock_button_add_help(GtkWidget *bbox, GtkWidget **help_btn)
+{
+	g_return_if_fail(bbox != NULL);
+
+	*help_btn = gtk_button_new_from_stock(GTK_STOCK_HELP);
+
+	GTK_WIDGET_SET_FLAGS(*help_btn, GTK_CAN_DEFAULT);
+	gtk_box_pack_end(GTK_BOX (bbox), *help_btn, TRUE, TRUE, 0);
+	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX (bbox),
+			*help_btn, TRUE);
+	gtk_widget_set_sensitive(*help_btn,
+			manual_available(MANUAL_MANUAL_LOCAL));
+	gtk_widget_show(*help_btn);
+}
+
+void gtkut_stock_button_set_create_with_help(GtkWidget **bbox,
+		GtkWidget **help_button,
+		GtkWidget **button1, const gchar *label1,
+		GtkWidget **button2, const gchar *label2,
+		GtkWidget **button3, const gchar *label3)
+{
+	g_return_if_fail(bbox != NULL);
+	g_return_if_fail(button1 != NULL);
+
+	gtkut_stock_button_set_create(bbox, button1, label1,
+			button2, label2, button3, label3);
+
+	gtkut_stock_button_add_help(*bbox, help_button);
+}
+
 void gtkut_stock_button_set_create(GtkWidget **bbox,
 				   GtkWidget **button1, const gchar *label1,
 				   GtkWidget **button2, const gchar *label2,
@@ -244,7 +275,7 @@ ComboButton *gtkut_combo_button_create(GtkWidget *button,
 #define ROW_FROM_YPIXEL(clist, y) (((y) - (clist)->voffset) / \
 				   ((clist)->row_height + CELL_SPACING))
 
-void gtkut_ctree_node_move_if_on_the_edge(GtkCTree *ctree, GtkCTreeNode *node)
+void gtkut_ctree_node_move_if_on_the_edge(GtkCTree *ctree, GtkCTreeNode *node, gint _row)
 {
 	GtkCList *clist = GTK_CLIST(ctree);
 	gint row;
@@ -253,7 +284,8 @@ void gtkut_ctree_node_move_if_on_the_edge(GtkCTree *ctree, GtkCTreeNode *node)
 	g_return_if_fail(ctree != NULL);
 	g_return_if_fail(node != NULL);
 
-	row = g_list_position(clist->row_list, (GList *)node);
+	row = (_row != -1 ? _row : g_list_position(clist->row_list, (GList *)node));
+
 	if (row < 0 || row >= clist->rows || clist->row_height == 0) return;
 	row_visibility = gtk_clist_row_is_visible(clist, row);
 	prev_row_visibility = gtk_clist_row_is_visible(clist, row - 1);
@@ -387,6 +419,8 @@ gboolean gtkut_ctree_node_is_parent(GtkCTreeNode *parent, GtkCTreeNode *node)
 
 void gtkut_ctree_set_focus_row(GtkCTree *ctree, GtkCTreeNode *node)
 {
+	if (node == NULL)
+		return;
 	gtkut_clist_set_focus_row(GTK_CLIST(ctree),
 				  gtkut_ctree_get_nth_from_node(ctree, node));
 }
@@ -757,16 +791,17 @@ void gtkut_widget_init(void)
 
 void gtkut_widget_set_app_icon(GtkWidget *widget)
 {
-#include "pixmaps/sylpheed.xpm"	
-	static GdkPixmap *sylpheedxpm;
-	static GdkBitmap *sylpheedxpmmask;
+#include "pixmaps/sylpheed-claws.xpm"
+	static GdkPixmap *sylpheedclawsxpm;
+	static GdkBitmap *sylpheedclawsxpmmask;
 	
 	g_return_if_fail(widget != NULL);
 	g_return_if_fail(widget->window != NULL);
-	if (!sylpheedxpm) {
-		PIXMAP_CREATE(widget, sylpheedxpm, sylpheedxpmmask, sylpheed_xpm);
+	if (!sylpheedclawsxpm) {
+		PIXMAP_CREATE(widget, sylpheedclawsxpm, sylpheedclawsxpmmask,
+			      sylpheed_claws_xpm);
 	}		
-	gdk_window_set_icon(widget->window, NULL, sylpheedxpm, sylpheedxpmmask);
+	gdk_window_set_icon(widget->window, NULL, sylpheedclawsxpm, sylpheedclawsxpmmask);
 }
 
 void gtkut_widget_set_composer_icon(GtkWidget *widget)
@@ -798,9 +833,12 @@ GtkWidget *label_window_create(const gchar *str)
 
 	label = gtk_label_new(str);
 	gtk_container_add(GTK_CONTAINER(window), label);
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
 	gtk_widget_show(label);
 
 	gtk_widget_show_now(window);
+	
+	GTK_EVENTS_FLUSH();
 
 	return window;
 }
@@ -911,30 +949,10 @@ GtkWidget *gtkut_get_browse_file_btn(const gchar *button_label)
 {
 	GtkWidget *button;
 
-#if GTK_CHECK_VERSION(2, 6, 0)
 	button = gtk_button_new_with_mnemonic(button_label);
 	gtk_button_set_image(GTK_BUTTON(button),
 		gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_BUTTON));
-#else
-	GtkWidget* image;
-	GtkWidget* box;
-	GtkWidget* label;
 
-	button = gtk_button_new();
-	box = gtk_hbox_new(FALSE, 0);
-
-	image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
-	label = gtk_label_new(NULL);
-	gtk_label_set_text_with_mnemonic(GTK_LABEL(label), button_label);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
-
-	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 1);
-	gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 1);
-	gtk_widget_show(label);
-	gtk_widget_show(image);
-	gtk_widget_show(box);
-	gtk_container_add(GTK_CONTAINER(button), box);
-#endif
 	return button;
 }
 
@@ -945,30 +963,21 @@ GtkWidget *gtkut_get_browse_directory_btn(const gchar *button_label)
 {
 	GtkWidget *button;
 
-#if GTK_CHECK_VERSION(2, 6, 0)
 	button = gtk_button_new_with_mnemonic(button_label);
 	gtk_button_set_image(GTK_BUTTON(button),
 		gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_BUTTON));
-#else
-	GtkWidget* image;
-	GtkWidget* box;
-	GtkWidget* label;
 
-	button = gtk_button_new();
-	box = gtk_hbox_new(FALSE, 0);
+	return button;
+}
 
-	image = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
-	label = gtk_label_new(NULL);
-	gtk_label_set_text_with_mnemonic(GTK_LABEL(label), button_label);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
+GtkWidget *gtkut_get_replace_btn(const gchar *button_label)
+{
+	GtkWidget *button;
+
+	button = gtk_button_new_with_mnemonic(button_label);
+	gtk_button_set_image(GTK_BUTTON(button),
+		gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON));
 
-	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 1);
-	gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 1);
-	gtk_widget_show(label);
-	gtk_widget_show(image);
-	gtk_widget_show(box);
-	gtk_container_add(GTK_CONTAINER(button), box);
-#endif
 	return button;
 }
 
@@ -1080,6 +1089,10 @@ GtkWidget *xface_get_from_header(const gchar *o_xface, GdkColor *background,
 	gchar xface[2048];
 	strncpy(xface, o_xface, sizeof(xface));
 
+	if (!window) {
+		g_warning("no window\n");
+		return NULL;
+	}
 	if (uncompface(xface) < 0) {
 		g_warning("uncompface failed\n");
 		return NULL;
@@ -1114,6 +1127,9 @@ GtkWidget *face_get_from_header(const gchar *o_face)
 	GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();
 	GtkWidget *image;
 	
+	if (o_face == NULL || strlen(o_face) == 0)
+		return NULL;
+
 	strncpy2(face, o_face, sizeof(face));
 
 	unfold_line(face); /* strip all whitespace and linebreaks */
@@ -1205,7 +1221,6 @@ GtkWidget *gtkut_get_link_btn(GtkWidget *window, const gchar *url, const gchar *
 	GdkColormap *cmap;
 	GdkColor uri_color[2] = {{0, 0, 0, 0xffff}, {0, 0xffff, 0, 0}};
 	gboolean success[2];
-	gchar *underlined = NULL;
 	gchar *local_url = NULL;
 	if (!url)
 		return NULL;
@@ -1215,12 +1230,9 @@ GtkWidget *gtkut_get_link_btn(GtkWidget *window, const gchar *url, const gchar *
 	gtkut_convert_int_to_gdk_color(prefs_common.uri_col,
 					       &uri_color[1]);
 
-	underlined = g_markup_printf_escaped("<u>%s</u>", label?label:url);
-	btn = gtk_button_new_with_label(underlined);
-	g_free(underlined);
+	btn = gtk_button_new_with_label(label?label:url);
 	gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_NONE);
 	btn_label = GTK_BIN(btn)->child;
-	gtk_label_set_use_markup(GTK_LABEL(btn_label), TRUE);
 	cmap = gdk_drawable_get_colormap(window->window);
 	gdk_colormap_alloc_colors(cmap, uri_color, 2, FALSE, TRUE, success);
 	if (success[0] == TRUE && success[1] == TRUE) {
@@ -1251,3 +1263,186 @@ GtkWidget *gtkut_get_link_btn(GtkWidget *window, const gchar *url, const gchar *
 			 G_CALLBACK(link_btn_unrealize), local_url);
 	return btn;
 }
+
+GtkWidget *gtkut_sc_combobox_create(GtkWidget *eventbox, gboolean focus_on_click)
+{
+	GtkWidget *combobox;
+	GtkListStore *menu;
+	GtkCellRenderer *rend;
+
+	menu = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
+
+	combobox = gtk_combo_box_new_with_model(GTK_TREE_MODEL(menu));
+
+	rend = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combobox), rend, TRUE);
+	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combobox), rend,
+			"markup", 0,
+			NULL);
+
+	if( eventbox != NULL )
+		gtk_container_add(GTK_CONTAINER(eventbox), combobox);
+#if GTK_CHECK_VERSION(2,6,0)
+	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(combobox), focus_on_click);
+#endif
+	return combobox;
+}
+
+static void gtkutils_smooth_scroll_do(GtkWidget *widget, GtkAdjustment *vadj,
+				      gfloat old_value, gfloat last_value,
+				      gint step)
+{
+	gint change_value;
+	gboolean up;
+	gint i;
+
+	if (old_value < last_value) {
+		change_value = last_value - old_value;
+		up = FALSE;
+	} else {
+		change_value = old_value - last_value;
+		up = TRUE;
+	}
+
+	for (i = step; i <= change_value; i += step) {
+		vadj->value = old_value + (up ? -i : i);
+		g_signal_emit_by_name(G_OBJECT(vadj),
+				      "value_changed", 0);
+	}
+
+	vadj->value = last_value;
+	g_signal_emit_by_name(G_OBJECT(vadj), "value_changed", 0);
+
+	gtk_widget_queue_draw(widget);
+}
+
+static gboolean gtkutils_smooth_scroll_page(GtkWidget *widget, GtkAdjustment *vadj, gboolean up)
+{
+	gfloat upper;
+	gfloat page_incr;
+	gfloat old_value;
+	gfloat last_value;
+
+	if (prefs_common.scroll_halfpage)
+		page_incr = vadj->page_increment / 2;
+	else
+		page_incr = vadj->page_increment;
+
+	if (!up) {
+		upper = vadj->upper - vadj->page_size;
+		if (vadj->value < upper) {
+			old_value = vadj->value;
+			last_value = vadj->value + page_incr;
+			last_value = MIN(last_value, upper);
+
+			gtkutils_smooth_scroll_do(widget, vadj, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		} else
+			return FALSE;
+	} else {
+		if (vadj->value > 0.0) {
+			old_value = vadj->value;
+			last_value = vadj->value - page_incr;
+			last_value = MAX(last_value, 0.0);
+
+			gtkutils_smooth_scroll_do(widget, vadj, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		} else
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+gboolean gtkutils_scroll_page(GtkWidget *widget, GtkAdjustment *vadj, gboolean up)
+{
+	gfloat upper;
+	gfloat page_incr;
+
+	if (prefs_common.enable_smooth_scroll)
+		return gtkutils_smooth_scroll_page(widget, vadj, up);
+
+	if (prefs_common.scroll_halfpage)
+		page_incr = vadj->page_increment / 2;
+	else
+		page_incr = vadj->page_increment;
+
+	if (!up) {
+		upper = vadj->upper - vadj->page_size;
+		if (vadj->value < upper) {
+			vadj->value += page_incr;
+			vadj->value = MIN(vadj->value, upper);
+			g_signal_emit_by_name(G_OBJECT(vadj),
+					      "value_changed", 0);
+		} else
+			return FALSE;
+	} else {
+		if (vadj->value > 0.0) {
+			vadj->value -= page_incr;
+			vadj->value = MAX(vadj->value, 0.0);
+			g_signal_emit_by_name(G_OBJECT(vadj),
+					      "value_changed", 0);
+		} else
+			return FALSE;
+	}
+	return TRUE;
+}
+
+static void gtkutils_smooth_scroll_one_line(GtkWidget *widget, GtkAdjustment *vadj, gboolean up)
+{
+	gfloat upper;
+	gfloat old_value;
+	gfloat last_value;
+
+	if (!up) {
+		upper = vadj->upper - vadj->page_size;
+		if (vadj->value < upper) {
+			old_value = vadj->value;
+			last_value = vadj->value + vadj->step_increment;
+			last_value = MIN(last_value, upper);
+
+			gtkutils_smooth_scroll_do(widget, vadj, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		}
+	} else {
+		if (vadj->value > 0.0) {
+			old_value = vadj->value;
+			last_value = vadj->value - vadj->step_increment;
+			last_value = MAX(last_value, 0.0);
+
+			gtkutils_smooth_scroll_do(widget, vadj, old_value,
+						  last_value,
+						  prefs_common.scroll_step);
+		}
+	}
+}
+
+void gtkutils_scroll_one_line(GtkWidget *widget, GtkAdjustment *vadj, gboolean up)
+{
+	gfloat upper;
+
+	if (prefs_common.enable_smooth_scroll) {
+		gtkutils_smooth_scroll_one_line(widget, vadj, up);
+		return;
+	}
+
+	if (!up) {
+		upper = vadj->upper - vadj->page_size;
+		if (vadj->value < upper) {
+			vadj->value += vadj->step_increment;
+			vadj->value = MIN(vadj->value, upper);
+			g_signal_emit_by_name(G_OBJECT(vadj),
+					      "value_changed", 0);
+		}
+	} else {
+		if (vadj->value > 0.0) {
+			vadj->value -= vadj->step_increment;
+			vadj->value = MAX(vadj->value, 0.0);
+			g_signal_emit_by_name(G_OBJECT(vadj),
+					      "value_changed", 0);
+		}
+	}
+}
