@@ -96,6 +96,14 @@ gboolean gtkut_get_font_size		(GtkWidget	*widget,
 void gtkut_convert_int_to_gdk_color	(gint		 rgbvalue,
 					 GdkColor	*color);
 
+void gtkut_stock_button_add_help(GtkWidget *bbox, GtkWidget **help_btn);
+
+void gtkut_stock_button_set_create_with_help(GtkWidget **bbox,
+		GtkWidget **help_button,
+		GtkWidget **button1, const gchar *label1,
+		GtkWidget **button2, const gchar *label2,
+		GtkWidget **button3, const gchar *label3);
+
 void gtkut_stock_button_set_create	(GtkWidget	**bbox,
 					 GtkWidget	**button1,
 					 const gchar	 *label1,
@@ -112,7 +120,8 @@ ComboButton *gtkut_combo_button_create	(GtkWidget		*button,
 
 void gtkut_ctree_node_move_if_on_the_edge
 					(GtkCTree	*ctree,
-					 GtkCTreeNode	*node);
+					 GtkCTreeNode	*node,
+					 gint		 _row);
 gint gtkut_ctree_get_nth_from_node	(GtkCTree	*ctree,
 					 GtkCTreeNode	*node);
 GtkCTreeNode *gtkut_ctree_node_next	(GtkCTree	*ctree,
@@ -190,6 +199,7 @@ GtkWidget *gtkut_get_focused_child	(GtkContainer 	*parent);
 
 GtkWidget *gtkut_get_browse_file_btn(const gchar *label);
 GtkWidget *gtkut_get_browse_directory_btn(const gchar *label);
+GtkWidget *gtkut_get_replace_btn(const gchar *label);
 #if HAVE_LIBCOMPFACE
 gint create_xpm_from_xface(gchar *xpm[], const gchar *xface);
 GtkWidget *xface_get_from_header(const gchar *o_xface, GdkColor *background,
@@ -203,4 +213,11 @@ gboolean get_tag_range(GtkTextIter *iter,
 GtkWidget *face_get_from_header(const gchar *o_face);
 GtkWidget *gtkut_get_link_btn(GtkWidget *window, const gchar *url, const gchar *label);
 
+GtkWidget *gtkut_sc_combobox_create(GtkWidget *eventbox, gboolean focus_on_click);
+void gtkutils_scroll_one_line	(GtkWidget *widget, 
+				 GtkAdjustment *vadj, 
+				 gboolean up);
+gboolean gtkutils_scroll_page	(GtkWidget *widget, 
+				 GtkAdjustment *vadj, 
+				 gboolean up);
 #endif /* __GTKUTILS_H__ */
