@@ -49,7 +49,7 @@ static void size_allocate_cb	(GtkWidget *widget,
 					 GtkAllocation *allocation);
 static gboolean log_window_append		(gpointer 	 source,
 						 gpointer   	 data);
-static void log_window_clip			(GtkWidget 	*text,
+static void log_window_clip			(LogWindow	*logwin,
 						 guint		 glip_length);
 static void log_window_clear			(GtkWidget	*widget,
 						 LogWindow	*logwin);
@@ -90,7 +90,7 @@ LogWindow *log_window_create(void)
 			 G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			 G_CALLBACK(key_pressed), logwin);
-	g_signal_connect(G_OBJECT(window), "hide",
+	g_signal_connect_after(G_OBJECT(window), "hide",
 			 G_CALLBACK(hide_cb), logwin);
 	gtk_widget_realize(window);
 
@@ -105,9 +105,15 @@ LogWindow *log_window_create(void)
 	text = gtk_text_view_new();
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+	logwin->buffer = buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+
+	g_object_ref(G_OBJECT(logwin->buffer));
+	gtk_text_view_set_buffer(GTK_TEXT_VIEW(text), NULL);
+	logwin->hidden = TRUE;
+
 	gtk_text_buffer_get_start_iter(buffer, &iter);
-	gtk_text_buffer_create_mark(buffer, "end", &iter, FALSE);
+	logwin->end_mark = gtk_text_buffer_create_mark(buffer, "end", &iter, FALSE);
+
 	g_signal_connect(G_OBJECT(text), "populate-popup",
 			 G_CALLBACK(log_window_popup_menu_extend), logwin);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), text);
@@ -138,54 +144,88 @@ void log_window_init(LogWindow *logwin)
 {
 	GtkTextBuffer *buffer;
 	GdkColormap *colormap;
-	GdkColor color[3] =
-		{{0, 0, 0xafff, 0}, {0, 0xefff, 0, 0}, {0, 0xefff, 0, 0}};
-	gboolean success[3];
+	GdkColor color[5];
+	gboolean success[5];
 	gint i;
 
-	logwin->msg_color   = color[0];
-	logwin->warn_color  = color[1];
+	gtkut_convert_int_to_gdk_color(prefs_common.log_msg_color, &color[0]);
+	gtkut_convert_int_to_gdk_color(prefs_common.log_warn_color, &color[1]);
+	gtkut_convert_int_to_gdk_color(prefs_common.log_error_color, &color[2]);
+	gtkut_convert_int_to_gdk_color(prefs_common.log_in_color, &color[3]);
+	gtkut_convert_int_to_gdk_color(prefs_common.log_out_color, &color[4]);
+
+	logwin->msg_color = color[0];
+	logwin->warn_color = color[1];
 	logwin->error_color = color[2];
+	logwin->in_color = color[3];
+	logwin->out_color = color[4];
 
 	colormap = gdk_drawable_get_colormap(logwin->window->window);
-	gdk_colormap_alloc_colors(colormap, color, 3, FALSE, TRUE, success);
+	gdk_colormap_alloc_colors(colormap, color, 5, FALSE, TRUE, success);
 
-	for (i = 0; i < 3; i++) {
+	for (i = 0; i < 5; i++) {
 		if (success[i] == FALSE) {
 			GtkStyle *style;
 
 			g_warning("LogWindow: color allocation failed\n");
 			style = gtk_widget_get_style(logwin->window);
 			logwin->msg_color = logwin->warn_color =
-			logwin->error_color = style->black;
+			logwin->error_color = logwin->in_color =
+			logwin->out_color = style->black;
 			break;
 		}
 	}
 
-	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(logwin->text));
+	buffer = logwin->buffer;
 	gtk_text_buffer_create_tag(buffer, "message",
 				   "foreground-gdk", &logwin->msg_color,
 				   NULL);
 	gtk_text_buffer_create_tag(buffer, "warn",
 				   "foreground-gdk", &logwin->warn_color,
 				   NULL);
-	gtk_text_buffer_create_tag(buffer, "error",
+	logwin->error_tag = gtk_text_buffer_create_tag(buffer, "error",
 				   "foreground-gdk", &logwin->error_color,
 				   NULL);
+	gtk_text_buffer_create_tag(buffer, "input",
+				   "foreground-gdk", &logwin->in_color,
+				   NULL);
+	gtk_text_buffer_create_tag(buffer, "output",
+				   "foreground-gdk", &logwin->out_color,
+				   NULL);
 }
 
 void log_window_show(LogWindow *logwin)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(logwin->text);
-	GtkTextBuffer *buffer = gtk_text_view_get_buffer(text);
+	GtkTextBuffer *buffer = logwin->buffer;
 	GtkTextMark *mark;
 
+	logwin->hidden = FALSE;
+	gtk_text_view_set_buffer(GTK_TEXT_VIEW(logwin->text), logwin->buffer);
+
 	mark = gtk_text_buffer_get_mark(buffer, "end");
 	gtk_text_view_scroll_mark_onscreen(text, mark);
 
 	gtk_widget_show(logwin->window);
 }
 
+static void log_window_jump_to_error(LogWindow *logwin)
+{
+	GtkTextIter iter;
+	gtk_text_buffer_get_end_iter(logwin->buffer, &iter);
+	if (!gtk_text_iter_backward_to_tag_toggle(&iter, logwin->error_tag))
+		return;
+
+	gtk_text_iter_backward_line(&iter);
+	gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(logwin->text), &iter, 0, TRUE, 0, 0);
+}
+
+void log_window_show_error(LogWindow *logwin)
+{
+	log_window_show(logwin);
+	log_window_jump_to_error(logwin);
+}
+
 void log_window_set_clipping(LogWindow *logwin, gboolean clip, guint clip_length)
 {
 	g_return_if_fail(logwin != NULL);
@@ -201,7 +241,6 @@ static gboolean log_window_append(gpointer source, gpointer data)
 	GtkTextView *text;
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
-	GdkColor *color = NULL;
 	gchar *head = NULL;
 	const gchar *tag;
 
@@ -213,22 +252,19 @@ static gboolean log_window_append(gpointer source, gpointer data)
 		return FALSE;
 
 	text = GTK_TEXT_VIEW(logwindow->text);
-	buffer = gtk_text_view_get_buffer(text);
+	buffer = logwindow->buffer;
 	gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
 
 	switch (logtext->type) {
 	case LOG_MSG:
-		color = &logwindow->msg_color;
 		tag = "message";
 		head = "* ";
 		break;
 	case LOG_WARN:
-		color = &logwindow->warn_color;
 		tag = "warn";
 		head = "** ";
 		break;
 	case LOG_ERROR:
-		color = &logwindow->error_color;
 		tag = "error";
 		head = "*** ";
 		break;
@@ -237,6 +273,21 @@ static gboolean log_window_append(gpointer source, gpointer data)
 		break;
 	}
   
+	if (tag == NULL) {
+		if (strstr(logtext->text, "] POP3>")
+		||  strstr(logtext->text, "] IMAP4>")
+		||  strstr(logtext->text, "] SMTP>")
+		||  strstr(logtext->text, "] ESMTP>")
+		||  strstr(logtext->text, "] NNTP>"))
+			tag = "output";
+		if (strstr(logtext->text, "] POP3<")
+		||  strstr(logtext->text, "] IMAP4<")
+		||  strstr(logtext->text, "] SMTP<")
+		||  strstr(logtext->text, "] ESMTP<")
+		||  strstr(logtext->text, "] NNTP<"))
+			tag = "input";
+	}
+
 	if (head)
 		gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, head, -1,
 							 tag, NULL);
@@ -246,17 +297,22 @@ static gboolean log_window_append(gpointer source, gpointer data)
 	gtk_text_buffer_get_start_iter(buffer, &iter);
 
 	if (logwindow->clip)
-	       log_window_clip (GTK_WIDGET (text), logwindow->clip_length);
+	       log_window_clip (logwindow, logwindow->clip_length);
 
-	gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
-	gtk_text_view_scroll_to_iter(text, &iter, 0, TRUE, 0, 0);
-	gtk_text_buffer_place_cursor(buffer, &iter);
+	if (!logwindow->hidden) {
+		GtkAdjustment *vadj = text->vadjustment;
+		gfloat upper = vadj->upper - vadj->page_size;
+		if (vadj->value == upper)
+			gtk_text_view_scroll_mark_onscreen(text, logwindow->end_mark);
+	}
 
 	return FALSE;
 }
 
 static void hide_cb(GtkWidget *widget, LogWindow *logwin)
 {
+	gtk_text_view_set_buffer(GTK_TEXT_VIEW(logwin->text), NULL);
+	logwin->hidden = TRUE;
 }
 
 static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
@@ -270,12 +326,11 @@ static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 	return FALSE;
 }
 
-static void log_window_clip(GtkWidget *textw, guint clip_length)
+static void log_window_clip(LogWindow *logwin, guint clip_length)
 {
         guint length;
 	guint point;
-	GtkTextView *textview = GTK_TEXT_VIEW(textw);
-	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(textview);
+	GtkTextBuffer *textbuf = logwin->buffer;
 	GtkTextIter start_iter, end_iter;
 	
 	length = gtk_text_buffer_get_line_count(textbuf);
@@ -295,8 +350,7 @@ static void log_window_clip(GtkWidget *textw, guint clip_length)
 
 static void log_window_clear(GtkWidget *widget, LogWindow *logwin)
 {
-	GtkTextView *textview = GTK_TEXT_VIEW(logwin->text);
-	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(textview);
+	GtkTextBuffer *textbuf = logwin->buffer;
 	GtkTextIter start_iter, end_iter;
 	
 	gtk_text_buffer_get_start_iter(textbuf, &start_iter);
