@@ -34,15 +34,22 @@ struct _LogWindow
 	GdkColor msg_color;
 	GdkColor warn_color;
 	GdkColor error_color;
+	GdkColor in_color;
+	GdkColor out_color;
 
 	gboolean clip;
 	guint	 clip_length;
 	guint 	 hook_id;
+	GtkTextBuffer *buffer;
+	GtkTextTag *error_tag;
+	GtkTextMark *end_mark;
+	gboolean hidden;
 };
 
 LogWindow *log_window_create(void);
 void log_window_init(LogWindow *logwin);
 void log_window_show(LogWindow *logwin);
+void log_window_show_error(LogWindow *logwin);
 void log_window_set_clipping(LogWindow *logwin, gboolean clip, guint clip_length);
 
 #endif /* __LOGWINDOW_H__ */
