@@ -35,6 +35,7 @@
 #include "alertpanel.h"
 #include "prefs_common.h"
 #include "../inc.h"
+#include "manual.h"
 
 enum {
 	PLUGINWINDOW_NAME,		/*<! plugin name */
@@ -181,7 +182,7 @@ static void load_cb(GtkButton *button, PluginWindow *pluginwindow)
 	GList *file_list;
 
 	file_list = filesel_select_multiple_files_open_with_filter(
-			_("Select Plugin to load"), get_plugin_dir(), 
+			_("Select the Plugins to load"), get_plugin_dir(), 
 			"*." G_MODULE_SUFFIX);
 
 	if (file_list) {
@@ -194,9 +195,11 @@ static void load_cb(GtkButton *button, PluginWindow *pluginwindow)
 			if (!file) continue;
 			plugin_load(file, &error);
 			if (error != NULL) {
+				gchar *basename = g_path_get_basename(file);
 				alertpanel_error(
-				_("The following error occured while loading the plugin [%s] :\n%s\n"),
-				file, error);
+				_("The following error occured while loading %s :\n\n%s\n"),
+				basename, error);
+				g_free(basename);
 				g_free(error);
 			}
 
@@ -254,7 +257,6 @@ static void pluginwindow_size_allocate_cb(GtkWidget *widget,
 void pluginwindow_create()
 {
 	PluginWindow *pluginwindow;
-	/* ---------------------- code made by glade ---------------------- */
 	GtkWidget *window;
 	GtkWidget *vbox1;
 	GtkWidget *hbox2;
@@ -266,11 +268,14 @@ void pluginwindow_create()
 	GtkWidget *scrolledwindow3;
 	GtkWidget *plugin_desc;
 	GtkWidget *hbuttonbox1, *hbox3;
+	GtkWidget *help_btn;
 	GtkWidget *load_btn;
 	GtkWidget *unload_btn;
 	GtkWidget *close_btn;
 	GtkWidget *get_more_btn;
+	GtkWidget *desc_lbl;
 	static GdkGeometry geometry;
+	GtkTooltips *tooltips;
 	
 	debug_print("Creating plugins window...\n");
 
@@ -284,6 +289,8 @@ void pluginwindow_create()
 	vbox1 = gtk_vbox_new(FALSE, 4);
 	gtk_widget_show(vbox1);
 	gtk_container_add(GTK_CONTAINER(window), vbox1);
+	gtk_box_set_homogeneous(GTK_BOX(vbox1), FALSE);
+	gtk_widget_realize(window);
 
 	hbox2 = gtk_hbox_new(FALSE, 8);
 	gtk_widget_show(hbox2);
@@ -326,54 +333,60 @@ void pluginwindow_create()
 	gtk_widget_show(plugin_desc);
 	gtk_container_add(GTK_CONTAINER(scrolledwindow3), plugin_desc);
 
+	desc_lbl = gtk_label_new(_("More plugins are available from the "
+			           "Sylpheed-Claws website."));
+	gtk_misc_set_alignment(GTK_MISC(desc_lbl), 0, 0.5);
+	gtk_widget_show(desc_lbl);
+	gtk_box_pack_start(GTK_BOX(vbox1), desc_lbl, FALSE, FALSE, 0);
+
+	get_more_btn = gtkut_get_link_btn(window, PLUGINS_URI, _("Get more..."));
+	gtk_misc_set_alignment(GTK_MISC(GTK_BIN(get_more_btn)->child), 0, 0.5);
+	gtk_widget_show(get_more_btn);
+	gtk_box_pack_start(GTK_BOX(vbox1), get_more_btn, FALSE, FALSE, 0);
+
 	hbox3 = gtk_hbox_new(FALSE, 0);
 	gtk_widget_show(hbox3);
 	hbuttonbox1 = gtk_hbutton_box_new();
 	gtk_widget_show(hbuttonbox1);
 
-	gtk_widget_realize(window);
-	get_more_btn = gtkut_get_link_btn(window, PLUGINS_URI, _("Get more..."));
-	gtk_misc_set_alignment(GTK_MISC(GTK_BIN(get_more_btn)->child), 0, 0.5);
-	gtk_widget_show(get_more_btn);
-
-	gtk_box_pack_start(GTK_BOX(hbox3), get_more_btn, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox3), hbuttonbox1, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox3, FALSE, FALSE, 0);
-	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox1),
-				  GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox1), 6);
 
-	load_btn = gtk_button_new_with_label(_("Load Plugin..."));
-	gtk_widget_show(load_btn);
-	gtk_container_add(GTK_CONTAINER(hbuttonbox1), load_btn);
-	GTK_WIDGET_SET_FLAGS(load_btn, GTK_CAN_DEFAULT);
-
-	unload_btn = gtk_button_new_with_label(_("Unload Plugin"));
-	gtk_widget_show(unload_btn);
-	gtk_container_add(GTK_CONTAINER(hbuttonbox1), unload_btn);
-	GTK_WIDGET_SET_FLAGS(unload_btn, GTK_CAN_DEFAULT);
-
-	close_btn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-	gtk_widget_show(close_btn);
-	gtk_container_add(GTK_CONTAINER(hbuttonbox1), close_btn);
-	GTK_WIDGET_SET_FLAGS(close_btn, GTK_CAN_DEFAULT);
-	/* ----------------------------------------------------------- */
+	gtkut_stock_button_set_create_with_help(&hbuttonbox1, &help_btn,
+			&load_btn, _("Load Plugin..."),
+			&unload_btn, _("Unload Plugin"),
+			&close_btn, GTK_STOCK_CLOSE);
+	gtk_box_set_spacing(GTK_BOX(hbuttonbox1), 6);
+	gtk_widget_show(hbuttonbox1);
+	gtk_box_pack_end (GTK_BOX (hbox3), hbuttonbox1, TRUE, TRUE, 0);
 
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(plugin_desc), GTK_WRAP_WORD);
 	gtk_widget_set_sensitive(GTK_WIDGET(unload_btn), FALSE);
 
-
-	g_signal_connect(G_OBJECT(load_btn), "released",
+	g_signal_connect(G_OBJECT(help_btn), "clicked",
+			 G_CALLBACK(manual_open_with_anchor_cb),
+			 MANUAL_ANCHOR_PLUGINS);
+	g_signal_connect(G_OBJECT(load_btn), "clicked",
 			 G_CALLBACK(load_cb), pluginwindow);
-	g_signal_connect(G_OBJECT(unload_btn), "released",
+	g_signal_connect(G_OBJECT(unload_btn), "clicked",
 			 G_CALLBACK(unload_cb), pluginwindow);
-	g_signal_connect(G_OBJECT(close_btn), "released",
+	g_signal_connect(G_OBJECT(close_btn), "clicked",
 			 G_CALLBACK(close_cb), pluginwindow);
 	g_signal_connect(G_OBJECT(window), "size_allocate",
 			 G_CALLBACK(pluginwindow_size_allocate_cb), NULL);
 	g_signal_connect(G_OBJECT(window), "key_press_event",
 			   G_CALLBACK(pluginwindow_key_pressed), pluginwindow);
 
+	tooltips = gtk_tooltips_new();
+
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips),
+			load_btn,
+			_("Click here to load one or more plugins"), NULL);
+
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips),
+			unload_btn,
+			_("Unload the selected plugin"), NULL);
+
 	pluginwindow->window = window;
 	pluginwindow->plugin_list_view = plugin_list_view;
 	pluginwindow->plugin_desc = plugin_desc;
@@ -394,6 +407,8 @@ void pluginwindow_create()
 	gtk_widget_set_size_request(window, prefs_common.pluginswin_width,
 				    prefs_common.pluginswin_height);
 
+	gtk_widget_set_size_request(hbuttonbox1, -1, -1);
+
 	gtk_widget_show(window);
 }
 
@@ -416,7 +431,7 @@ static GtkWidget *pluginwindow_list_view_create(PluginWindow *pluginwindow)
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);	
 
-	gtk_tree_view_set_rules_hint(list_view, prefs_common.enable_rules_hint);
+	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 	gtk_tree_view_set_search_column (list_view, 0);
 
 	selector = gtk_tree_view_get_selection(list_view);
@@ -437,7 +452,7 @@ static void pluginwindow_create_list_view_columns(GtkWidget *list_view)
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes
-		(_("Plugins"),
+		(_("Loaded plugins"),
 		 renderer,
 		 "text", PLUGINWINDOW_NAME,
 		 "style", PLUGINWINDOW_STYLE,
