@@ -129,7 +129,7 @@ static void close_all_pages(GSList *prefs_pages)
 	}	
 }
 
-static void apply_button_released(GtkButton *button, gpointer user_data)
+static void apply_button_clicked(GtkButton *button, gpointer user_data)
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
@@ -149,7 +149,7 @@ static void close_prefs_window(PrefsWindow *prefswindow)
 	g_free(prefswindow);
 }
 
-static void ok_button_released(GtkButton *button, gpointer user_data)
+static void ok_button_clicked(GtkButton *button, gpointer user_data)
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
@@ -159,7 +159,7 @@ static void ok_button_released(GtkButton *button, gpointer user_data)
 	}		
 }
 
-static void cancel_button_released(GtkButton *button, gpointer user_data)
+static void cancel_button_clicked(GtkButton *button, gpointer user_data)
 {
 	PrefsWindow *prefswindow = (PrefsWindow *) user_data;
 
@@ -182,7 +182,7 @@ static gboolean prefswindow_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	if (event) {
 		switch (event->keyval) {
 			case GDK_Escape :
-				cancel_button_released(NULL, data);
+				cancel_button_clicked(NULL, data);
 				break;
 			case GDK_Return : 
 			case GDK_KP_Enter :
@@ -192,7 +192,7 @@ static gboolean prefswindow_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				 * and text (anything that accepts return) (can pass
 				 * NULL to any of the GTK_xxx() casts) */
 				if (!GTK_IS_TEXT_VIEW(focused_child))
-					ok_button_released(NULL, data);
+					ok_button_clicked(NULL, data);
 				break;
 			default:
 				break;
@@ -424,12 +424,12 @@ void prefswindow_open_full(const gchar *title, GSList *prefs_pages, gpointer dat
 
 	gtk_table_attach(GTK_TABLE(prefswindow->table1), prefswindow->confirm_area, 0, 2, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 2);
 
-	g_signal_connect(G_OBJECT(prefswindow->ok_btn), "released", 
-			 G_CALLBACK(ok_button_released), prefswindow);
-	g_signal_connect(G_OBJECT(prefswindow->cancel_btn), "released", 
-			 G_CALLBACK(cancel_button_released), prefswindow);
-	g_signal_connect(G_OBJECT(prefswindow->apply_btn), "released", 
-			 G_CALLBACK(apply_button_released), prefswindow);
+	g_signal_connect(G_OBJECT(prefswindow->ok_btn), "clicked", 
+			 G_CALLBACK(ok_button_clicked), prefswindow);
+	g_signal_connect(G_OBJECT(prefswindow->cancel_btn), "clicked", 
+			 G_CALLBACK(cancel_button_clicked), prefswindow);
+	g_signal_connect(G_OBJECT(prefswindow->apply_btn), "clicked", 
+			 G_CALLBACK(apply_button_clicked), prefswindow);
 	g_signal_connect(G_OBJECT(prefswindow->window), "delete_event", 
 			 G_CALLBACK(window_closed), prefswindow);
 	g_signal_connect(G_OBJECT(prefswindow->window), "key_press_event",
@@ -504,7 +504,7 @@ static GtkWidget *prefswindow_tree_view_create(PrefsWindow *prefswindow)
 	model = GTK_TREE_MODEL(prefswindow_create_data_store());
 	tree_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);
-	gtk_tree_view_set_rules_hint(tree_view, prefs_common.enable_rules_hint);
+	gtk_tree_view_set_rules_hint(tree_view, prefs_common.use_stripes_everywhere);
 	
 	selector = gtk_tree_view_get_selection(tree_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
@@ -543,6 +543,7 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 	gchar *labeltext;
 	gint pagenum, i;
 	GtkTreeIter iter;
+	GtkAdjustment *adj;
 
 	if (currently_selected) 
 		return TRUE;
@@ -580,6 +581,10 @@ static gboolean prefswindow_row_selected(GtkTreeSelection *selector,
 					page->widget);
 	gtk_notebook_set_page(GTK_NOTEBOOK(prefswindow->notebook), pagenum);
 
+	adj = gtk_scrolled_window_get_vadjustment(
+			GTK_SCROLLED_WINDOW(prefswindow->scrolledwindow2));
+	gtk_adjustment_set_value(adj, 0);
+
 	return TRUE;
 }
 
