@@ -73,9 +73,8 @@ ProgressDialog *progress_dialog_create(void)
 	progress = g_new0(ProgressDialog, 1);
 
 	dialog = gtk_dialog_new();
-	gtk_widget_set_size_request(dialog, 460, -1);
 	gtk_container_set_border_width(GTK_CONTAINER(dialog), 8);
-	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, TRUE, TRUE);
+	gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
 	gtk_widget_realize(dialog);
 
 	gtk_container_set_border_width
@@ -119,7 +118,7 @@ ProgressDialog *progress_dialog_create(void)
 	g_object_unref(G_OBJECT(store));
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), TRUE);
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview),
-				     prefs_common.enable_rules_hint);
+				     prefs_common.use_stripes_everywhere);
 	gtk_widget_show(treeview);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), treeview);
 	gtk_widget_set_size_request(treeview, -1, 120);
