@@ -62,12 +62,20 @@ struct _QuickSearch
 	gboolean			 matching;
 	gboolean			 deferred_free;
 	FolderItem			*root_folder_item;
+	gboolean			 is_fast;
+	gboolean			 in_typing;
+	guint				 press_timeout_id;
 };
 
 static void quicksearch_set_running(QuickSearch *quicksearch, gboolean run);
 static void quicksearch_set_active(QuickSearch *quicksearch, gboolean active);
 static void quicksearch_reset_folder_items(QuickSearch *quicksearch, FolderItem *folder_item);
 
+gboolean quicksearch_is_fast(QuickSearch *quicksearch)
+{
+	return quicksearch->is_fast;
+}
+
 static void prepare_matcher(QuickSearch *quicksearch)
 {
 	const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
@@ -95,7 +103,7 @@ static void prepare_matcher(QuickSearch *quicksearch)
 
 		newstr = expand_search_string(search_string);
 		if (newstr && newstr[0] != '\0') {
-			quicksearch->matcher_list = matcher_parser_get_cond(newstr);
+			quicksearch->matcher_list = matcher_parser_get_cond(newstr, &quicksearch->is_fast);
 			g_free(newstr);
 		} else {
 			quicksearch->matcher_list = NULL;
@@ -104,8 +112,8 @@ static void prepare_matcher(QuickSearch *quicksearch)
 			return;
 		}
 	} else {
-		if (quicksearch->search_string != NULL)
-			g_free(quicksearch->search_string);
+		quicksearch->is_fast = TRUE;
+		g_free(quicksearch->search_string);
 		quicksearch->search_string = g_strdup(search_string);
 	}
 
@@ -129,10 +137,18 @@ static void update_extended_buttons (QuickSearch *quicksearch)
 	}
 }
 
-static gboolean searchbar_focus_evt(GtkWidget *widget, GdkEventFocus *event,
-			      	  QuickSearch *quicksearch)
+static gboolean searchbar_focus_evt_in(GtkWidget *widget, GdkEventFocus *event,
+			      	  QuickSearch *qs)
 {
-	quicksearch->has_focus = (event && event->in);
+	qs->has_focus = TRUE;
+	return FALSE;
+}
+
+static gboolean searchbar_focus_evt_out(GtkWidget *widget, GdkEventFocus *event,
+			      	  QuickSearch *qs)
+{
+	qs->has_focus = FALSE;
+	qs->in_typing = FALSE;
 	return FALSE;
 }
 
@@ -141,11 +157,12 @@ gboolean quicksearch_has_focus(QuickSearch *quicksearch)
 	return quicksearch->has_focus;
 }
 
-static void searchbar_run(QuickSearch *quicksearch)
+static void searchbar_run(QuickSearch *quicksearch, gboolean run_only_if_fast)
 {
 	const gchar *search_string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
 
-	if (search_string && strlen(search_string) != 0) {
+	/* add to history */
+	if (!quicksearch->in_typing && search_string && strlen(search_string) != 0) {
 		prefs_common.summary_quicksearch_history =
 			add_history(prefs_common.summary_quicksearch_history,
 					search_string);
@@ -154,24 +171,69 @@ static void searchbar_run(QuickSearch *quicksearch)
 	}
 
 	prepare_matcher(quicksearch);
-
+	if (run_only_if_fast && !quicksearch->is_fast)
+		return;
+	if (quicksearch->matcher_list == NULL && 
+	    prefs_common.summary_quicksearch_type == QUICK_SEARCH_EXTENDED &&
+	    search_string && strlen(search_string) != 0)
+		return;
 	quicksearch_set_running(quicksearch, TRUE);
 	if (quicksearch->callback != NULL)
 		quicksearch->callback(quicksearch, quicksearch->callback_data);
 	quicksearch_set_running(quicksearch, FALSE);
 }
 
+static int searchbar_changed_timeout(void *data)
+{
+	QuickSearch *qs = (QuickSearch *)data;
+	if (qs && prefs_common.summary_quicksearch_dynamic) {
+		qs->in_typing = TRUE;
+		searchbar_run(qs, TRUE);
+	}
+	return FALSE;
+}
+
+static gboolean searchbar_changed_cb(GtkWidget *widget, QuickSearch *qs)
+{
+	if (prefs_common.summary_quicksearch_dynamic) {
+		if (qs->press_timeout_id != -1) {
+			gtk_timeout_remove(qs->press_timeout_id);
+		}
+		qs->press_timeout_id = gtk_timeout_add(500,
+				searchbar_changed_timeout, qs);
+	}
+
+	return FALSE;
+}
+
 static gboolean searchbar_pressed(GtkWidget *widget, GdkEventKey *event,
 			      	  QuickSearch *quicksearch)
 {
 	if (event != NULL && event->keyval == GDK_Escape) {
-		quicksearch_set(quicksearch, prefs_common.summary_quicksearch_type, "");
+
+		const gchar *str;
+
+		quicksearch->in_typing = FALSE;
+
+		str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry));
+		g_return_val_if_fail(str != NULL, TRUE);
+
+		/* If the string entry is empty -> hide quicksearch bar. If not -> empty it */
+		if (!*str) {
+			quicksearch_hide(quicksearch);
+		} else {
+			quicksearch_set(quicksearch, prefs_common.summary_quicksearch_type, "");
+			gtk_widget_grab_focus(
+					GTK_WIDGET(GTK_COMBO(quicksearch->search_string_entry)->entry));
+		}
+
 		return TRUE;
 	}
 
 	if (event != NULL && event->keyval == GDK_Return) {
+		quicksearch->in_typing = FALSE;
 		/* add expression to history list and exec quicksearch */
-		searchbar_run(quicksearch);
+		searchbar_run(quicksearch, FALSE);
 
 		g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
 		return TRUE;
@@ -244,6 +306,20 @@ static gboolean searchtype_sticky_changed(GtkMenuItem *widget, gpointer data)
 	return TRUE;
 }
 
+static gboolean searchtype_dynamic_changed(GtkMenuItem *widget, gpointer data)
+{
+	QuickSearch *quicksearch = (QuickSearch *)data;
+	gboolean checked = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+
+	prefs_common.summary_quicksearch_dynamic = checked;
+
+	/* reselect the search type */
+	gtk_option_menu_set_history(GTK_OPTION_MENU(quicksearch->search_type_opt),
+				    prefs_common.summary_quicksearch_type);
+
+	return TRUE;
+}
+
 /*
  * Strings describing how to use Extended Search
  *
@@ -265,6 +341,7 @@ static gchar *search_descr_strings[] = {
 	"h S",	 N_("messages which contain header S"),
 	"i S",	 N_("messages which contain S in Message-ID header"),
 	"I S",	 N_("messages which contain S in inreplyto header"),
+	"k #",	 N_("messages which are marked with color #"),
 	"L",	 N_("locked messages"),
 	"n S",	 N_("messages which are in newsgroup S"),
 	"N",	 N_("new messages"),
@@ -283,7 +360,7 @@ static gchar *search_descr_strings[] = {
 	"T",	 N_("marked messages"),
 	"U",	 N_("unread messages"),
 	"x S",	 N_("messages which contain S in References header"),
-	"X cmd", N_("messages returning 0 when passed to command"),
+	"X \"cmd args\"", N_("messages returning 0 when passed to command - %F is message file"),
 	"y S",	 N_("messages which contain S in X-Label header"),
 	"",	 "" ,
 	"&amp;",	 N_("logical AND operator"),
@@ -341,7 +418,7 @@ static void search_condition_expr_done(MatcherList * matchers)
 		g_free(str);
 
 		/* add expression to history list and exec quicksearch */
-		searchbar_run(mainwindow_get_mainwindow()->summaryview->quicksearch);
+		searchbar_run(mainwindow_get_mainwindow()->summaryview->quicksearch, FALSE);
 	}
 }
 
@@ -349,7 +426,7 @@ static gboolean search_condition_expr(GtkMenuItem *widget, gpointer data)
 {
 	const gchar * cond_str;
 	MatcherList * matchers = NULL;
-
+	
 	g_return_val_if_fail(
 			mainwindow_get_mainwindow()->summaryview->quicksearch != NULL,
 			FALSE);
@@ -360,7 +437,7 @@ static gboolean search_condition_expr(GtkMenuItem *widget, gpointer data)
 			GTK_ENTRY(GTK_COMBO(mainwindow_get_mainwindow()->summaryview->quicksearch->
 			search_string_entry)->entry));
 	if (*cond_str != '\0') {
-		matchers = matcher_parser_get_cond((gchar*)cond_str);
+		matchers = matcher_parser_get_cond((gchar*)cond_str, NULL);
 	}
 
 	prefs_matcher_open(matchers, search_condition_expr_done);
@@ -435,6 +512,16 @@ QuickSearch *quicksearch_new()
 			 G_CALLBACK(searchtype_sticky_changed),
 			 quicksearch);
 
+	menuitem = gtk_check_menu_item_new_with_label(_("Type-ahead"));
+	gtk_menu_shell_append(GTK_MENU_SHELL(search_type), menuitem);
+
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
+					prefs_common.summary_quicksearch_dynamic);
+
+	g_signal_connect(G_OBJECT(menuitem), "activate",
+			 G_CALLBACK(searchtype_dynamic_changed),
+			 quicksearch);
+
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(search_type_opt), search_type);
 
 	gtk_option_menu_set_history(GTK_OPTION_MENU(search_type_opt), prefs_common.summary_quicksearch_type);
@@ -453,14 +540,22 @@ QuickSearch *quicksearch_new()
 
 	search_hbox = gtk_hbox_new(FALSE, 5);
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+	clear_search = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
+#else
 	clear_search = gtk_button_new_with_label(_(" Clear "));
+#endif
 	gtk_box_pack_start(GTK_BOX(search_hbox), clear_search,
 			   FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(clear_search), "clicked",
 			 G_CALLBACK(clear_search_cb), quicksearch);
 	gtk_widget_show(clear_search);
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+	search_condition_expression = gtk_button_new_from_stock(GTK_STOCK_EDIT);
+#else
 	search_condition_expression = gtk_button_new_with_label(" ... ");
+#endif
 	gtk_box_pack_start(GTK_BOX(search_hbox), search_condition_expression,
 			   FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (search_condition_expression), "clicked",
@@ -472,7 +567,11 @@ QuickSearch *quicksearch_new()
 			     _("Edit search criteria"), NULL);
 	gtk_widget_show(search_condition_expression);
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+	search_description = gtk_button_new_from_stock(GTK_STOCK_INFO);
+#else
 	search_description = gtk_button_new_with_label(_(" Extended Symbols... "));
+#endif
 	gtk_box_pack_start(GTK_BOX(search_hbox), search_description,
 			   FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(search_description), "clicked",
@@ -486,13 +585,19 @@ QuickSearch *quicksearch_new()
 			   "key_press_event",
 			   G_CALLBACK(searchbar_pressed),
 			   quicksearch);
+
+	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry),
+			 "changed",
+			 G_CALLBACK(searchbar_changed_cb),
+			 quicksearch);
+
 	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry),
 			 "focus_in_event",
-			 G_CALLBACK(searchbar_focus_evt),
+			 G_CALLBACK(searchbar_focus_evt_in),
 			 quicksearch);
 	g_signal_connect(G_OBJECT(GTK_COMBO(search_string_entry)->entry),
 			 "focus_out_event",
-			 G_CALLBACK(searchbar_focus_evt),
+			 G_CALLBACK(searchbar_focus_evt_out),
 			 quicksearch);
 
 	quicksearch->hbox_search = hbox_search;
@@ -505,6 +610,8 @@ QuickSearch *quicksearch_new()
 	quicksearch->active = FALSE;
 	quicksearch->running = FALSE;
 	quicksearch->clear_search = clear_search;
+	quicksearch->in_typing = FALSE;
+	quicksearch->press_timeout_id = -1;
 
 	update_extended_buttons(quicksearch);
 
@@ -537,8 +644,14 @@ void quicksearch_set(QuickSearch *quicksearch, QuickSearchType type,
 {
 	gtk_option_menu_set_history(GTK_OPTION_MENU(quicksearch->search_type_opt),
 				    type);
+
+	g_signal_handlers_block_by_func(G_OBJECT(GTK_COMBO(quicksearch->search_string_entry)->entry),
+			G_CALLBACK(searchbar_changed_cb), quicksearch);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(quicksearch->search_string_entry)->entry),
 			   matchstring);
+	g_signal_handlers_unblock_by_func(G_OBJECT(GTK_COMBO(quicksearch->search_string_entry)->entry),
+			G_CALLBACK(searchbar_changed_cb), quicksearch);
+
 	prefs_common.summary_quicksearch_type = type;
 
 	prepare_matcher(quicksearch);
@@ -551,44 +664,58 @@ void quicksearch_set(QuickSearch *quicksearch, QuickSearchType type,
 
 gboolean quicksearch_is_active(QuickSearch *quicksearch)
 {
-	return quicksearch->active;
+	return quicksearch->active && 
+		(prefs_common.summary_quicksearch_type != QUICK_SEARCH_EXTENDED
+		 || quicksearch->matcher_list != NULL);
 }
 
 static void quicksearch_set_active(QuickSearch *quicksearch, gboolean active)
 {
 	static GdkColor yellow;
+	static GdkColor red;
 	static GdkColor black;
-	static gboolean yellow_initialised = FALSE;
+	static gboolean colors_initialised = FALSE;
+	gboolean error = FALSE;
 
-	if (!yellow_initialised) {
+	if (!colors_initialised) {
 		gdk_color_parse("#f5f6be", &yellow);
 		gdk_color_parse("#000000", &black);
-		yellow_initialised = gdk_colormap_alloc_color(
+		gdk_color_parse("#ff7070", &red);
+		colors_initialised = gdk_colormap_alloc_color(
 			gdk_colormap_get_system(), &yellow, FALSE, TRUE);
-		yellow_initialised &= gdk_colormap_alloc_color(
+		colors_initialised &= gdk_colormap_alloc_color(
 			gdk_colormap_get_system(), &black, FALSE, TRUE);
-
+		colors_initialised &= gdk_colormap_alloc_color(
+			gdk_colormap_get_system(), &red, FALSE, TRUE);
 	}
+
 	quicksearch->active = active;
 
+	if (active && 
+		(prefs_common.summary_quicksearch_type == QUICK_SEARCH_EXTENDED
+		 && quicksearch->matcher_list == NULL))
+		error = TRUE;
+
 	if (active) {
 		gtk_widget_set_sensitive(quicksearch->clear_search, TRUE);
-		if (yellow_initialised)
+		if (colors_initialised) {
 			gtk_widget_modify_base(
 				GTK_COMBO(quicksearch->search_string_entry)->entry,
-				GTK_STATE_NORMAL, &yellow);
+				GTK_STATE_NORMAL, error ? &red : &yellow);
 			gtk_widget_modify_text(
 				GTK_COMBO(quicksearch->search_string_entry)->entry,
 				GTK_STATE_NORMAL, &black);
+		}
 	} else {
 		gtk_widget_set_sensitive(quicksearch->clear_search, FALSE);
-		if (yellow_initialised)
+		if (colors_initialised) {
 			gtk_widget_modify_base(
 				GTK_COMBO(quicksearch->search_string_entry)->entry,
 				GTK_STATE_NORMAL, NULL);
 			gtk_widget_modify_text(
 				GTK_COMBO(quicksearch->search_string_entry)->entry,
 				GTK_STATE_NORMAL, NULL);
+		}
 	}
 
 	if (!active) {
@@ -679,6 +806,7 @@ gchar *expand_search_string(const gchar *search_string)
 		{ "h",	"headers_part",			1,	TRUE,	TRUE  },
 		{ "i",	"header \"Message-ID\"",	1,	TRUE,	TRUE  },
 		{ "I",	"inreplyto",			1,	TRUE,	TRUE  },
+		{ "k",	"colorlabel",			1,	FALSE,	FALSE },
 		{ "L",	"locked",			0,	FALSE,	FALSE },
 		{ "n",	"newsgroups",			1,	TRUE,	TRUE  },
 		{ "N",	"new",				0,	FALSE,	FALSE },
@@ -845,6 +973,16 @@ void quicksearch_pass_key(QuickSearch *quicksearch, guint val, GdkModifierType m
 	char key[7] = "";
 	guint char_len = 0;
 
+	if (gtk_editable_get_selection_bounds(GTK_EDITABLE(entry), NULL, NULL)) {
+		/* remove selection */
+		gtk_editable_delete_selection(GTK_EDITABLE(entry));
+		curpos = gtk_editable_get_position(GTK_EDITABLE(entry));
+		/* refresh string */
+		g_free(str);
+		str = g_strdup(gtk_entry_get_text(entry));
+		begin = str;
+	}
+
 	if (!(c = gdk_keyval_to_unicode(val))) {
 		g_free(str);
 		return;
@@ -854,7 +992,9 @@ void quicksearch_pass_key(QuickSearch *quicksearch, guint val, GdkModifierType m
 		return;
 	key[char_len] = '\0';
 	if (curpos < g_utf8_strlen(str, -1)) {
-		end = g_utf8_offset_to_pointer(str, curpos+1);
+		gchar *stop = g_utf8_offset_to_pointer(begin, curpos);
+		end = g_strdup(g_utf8_offset_to_pointer(str, curpos));
+		*stop = '\0';
 		new = g_strdup_printf("%s%s%s", begin, key, end);
 		gtk_entry_set_text(entry, new);
 		g_free(end);
@@ -875,21 +1015,26 @@ static gboolean quicksearch_match_subfolder(QuickSearch *quicksearch,
 	GSList *cur;
 	gboolean result = FALSE;
 	gint num = 0, total = src->total_msgs;
+	gint interval = quicksearch_is_fast(quicksearch) ? 5000:100;
+
 	statusbar_print_all(_("Searching in %s... \n"),
 		src->path ? src->path : "(null)");
+	folder_item_update_freeze();
 	for (cur = msglist; cur != NULL; cur = cur->next) {
 		MsgInfo *msg = (MsgInfo *)cur->data;
-		statusbar_progress_all(num++,total, 50);
+		statusbar_progress_all(num++,total, interval);
 		if (quicksearch_match(quicksearch, msg)) {
 			procmsg_msginfo_free(msg);
 			result = TRUE;
 			break;
 		}
 		procmsg_msginfo_free(msg);
-		GTK_EVENTS_FLUSH();
+		if (num % interval == 0)
+			GTK_EVENTS_FLUSH();
 		if (!quicksearch_is_active(quicksearch))
 			break;
 	}
+	folder_item_update_thaw();
 	statusbar_progress_all(0,0,0);
 	statusbar_pop_all();
 
@@ -904,7 +1049,8 @@ void quicksearch_search_subfolders(QuickSearch *quicksearch,
 	FolderItem *cur = NULL;
 	GNode *node = folder_item->node->children;
 
-	if (!prefs_common.summary_quicksearch_recurse)
+	if (!prefs_common.summary_quicksearch_recurse
+	||  quicksearch->in_typing == TRUE)
 		return;
 
 	for (; node != NULL; node = node->next) {
@@ -948,3 +1094,8 @@ void quicksearch_reset_cur_folder_item(QuickSearch *quicksearch)
 
 	quicksearch->root_folder_item = NULL;
 }
+
+gboolean quicksearch_is_in_typing(QuickSearch *quicksearch)
+{
+	return quicksearch->in_typing;
+}
