@@ -123,11 +123,10 @@ HeaderView *headerview_create(void)
 	gtk_misc_set_alignment(GTK_MISC(to_body_label), 0, 0.5);
 	gtk_misc_set_alignment(GTK_MISC(ng_body_label), 0, 0.5);
 	gtk_misc_set_alignment(GTK_MISC(subject_body_label), 0, 0.5);
-#if GTK_CHECK_VERSION(2, 6, 0)
 	gtk_label_set_ellipsize(GTK_LABEL(to_body_label), PANGO_ELLIPSIZE_END);
 	gtk_label_set_ellipsize(GTK_LABEL(ng_body_label), PANGO_ELLIPSIZE_END);
 	gtk_label_set_ellipsize(GTK_LABEL(subject_body_label), PANGO_ELLIPSIZE_END);
-#endif
+
 	headerview->hbox = hbox;
 	headerview->from_header_label    = from_header_label;
 	headerview->from_body_label      = from_body_label;
@@ -224,7 +223,9 @@ static void headerview_show_xface(HeaderView *headerview, MsgInfo *msginfo)
 	GtkWidget *hbox = headerview->hbox;
 	GtkWidget *image;
 
-	if (!msginfo->xface || strlen(msginfo->xface) < 5) {
+	if (!msginfo->extradata || 
+	    !msginfo->extradata->xface || 
+	    strlen(msginfo->extradata->xface) < 5) {
 		if (headerview->image &&
 		    GTK_WIDGET_VISIBLE(headerview->image)) {
 			gtk_widget_hide(headerview->image);
@@ -240,7 +241,7 @@ static void headerview_show_xface(HeaderView *headerview, MsgInfo *msginfo)
 	}
 	
 
-	image = xface_get_from_header(msginfo->xface, &hbox->style->white,
+	image = xface_get_from_header(msginfo->extradata->xface, &hbox->style->white,
 				hbox->window);
 
 	if (image) {
@@ -257,7 +258,7 @@ static gint headerview_show_face (HeaderView *headerview, MsgInfo *msginfo)
 	GtkWidget *hbox = headerview->hbox;
 	GtkWidget *image;
 
-	if (!msginfo->face) {
+	if (!msginfo->extradata || !msginfo->extradata->face) {
 		if (headerview->image &&
 		    GTK_WIDGET_VISIBLE(headerview->image)) {
 			gtk_widget_hide(headerview->image);
@@ -273,7 +274,7 @@ static gint headerview_show_face (HeaderView *headerview, MsgInfo *msginfo)
 	}
 	
 
-	image = face_get_from_header(msginfo->face);
+	image = face_get_from_header(msginfo->extradata->face);
 
 	if (image) {
 		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
