@@ -53,66 +53,66 @@ static SC_HTMLSymbol symbol_list[] = {
 };
 
 static SC_HTMLSymbol ascii_symbol_list[] = {
-	{"&iexcl;" , "^!"},
-	{"&brvbar;", "|"},
-	{"&copy;"  , "(C)"},
-	{"&laquo;" , "<<"},
-	{"&reg;"   , "(R)"},
+	{"&iexcl;" , "\302\241"},
+	{"&brvbar;", "\302\246"},
+	{"&copy;"  , "\302\251"},
+	{"&laquo;" , "\302\253"},
+	{"&reg;"   , "\302\256"},
 
-	{"&sup2;"  , "^2"},
-	{"&sup3;"  , "^3"},
-	{"&acute;" , "'"},
-	{"&cedil;" , ","},
-	{"&sup1;"  , "^1"},
-	{"&raquo;" , ">>"},
-	{"&frac14;", "1/4"},
-	{"&frac12;", "1/2"},
-	{"&frac34;", "3/4"},
-	{"&iquest;", "^?"},
+	{"&sup2;"  , "\302\262"},
+	{"&sup3;"  , "\302\263"},
+	{"&acute;" , "\302\264"},
+	{"&cedil;" , "\302\270"},
+	{"&sup1;"  , "\302\271"},
+	{"&raquo;" , "\302\273"},
+	{"&frac14;", "\302\274"},
+	{"&frac12;", "\302\275"},
+	{"&frac34;", "\302\276"},
+	{"&iquest;", "\302\277"},
 
-	{"&Agrave;", "A`"},
-	{"&Aacute;", "A'"},
-	{"&Acirc;" , "A^"},
-	{"&Atilde;", "A~"},
-	{"&AElig;" , "AE"},
-	{"&Egrave;", "E`"},
-	{"&Eacute;", "E'"},
-	{"&Ecirc;" , "E^"},
-	{"&Igrave;", "I`"},
-	{"&Iacute;", "I'"},
-	{"&Icirc;" , "I^"},
+	{"&Agrave;", "\303\200"},
+	{"&Aacute;", "\303\201"},
+	{"&Acirc;" , "\303\202"},
+	{"&Atilde;", "\303\203"},
+	{"&AElig;" , "\303\206"},
+	{"&Egrave;", "\303\210"},
+	{"&Eacute;", "\303\211"},
+	{"&Ecirc;" , "\303\212"},
+	{"&Igrave;", "\303\214"},
+	{"&Iacute;", "\303\215"},
+	{"&Icirc;" , "\303\216"},
 
-	{"&Ntilde;", "N~"},
-	{"&Ograve;", "O`"},
-	{"&Oacute;", "O'"},
-	{"&Ocirc;" , "O^"},
-	{"&Otilde;", "O~"},
-	{"&Ugrave;", "U`"},
-	{"&Uacute;", "U'"},
-	{"&Ucirc;" , "U^"},
-	{"&Yacute;", "Y'"},
+	{"&Ntilde;", "\303\221"},
+	{"&Ograve;", "\303\222"},
+	{"&Oacute;", "\303\223"},
+	{"&Ocirc;" , "\303\224"},
+	{"&Otilde;", "\303\225"},
+	{"&Ugrave;", "\303\231"},
+	{"&Uacute;", "\303\232"},
+	{"&Ucirc;" , "\303\233"},
+	{"&Yacute;", "\303\235"},
 
-	{"&agrave;", "a`"},
-	{"&aacute;", "a'"},
-	{"&acirc;" , "a^"},
-	{"&atilde;", "a~"},
-	{"&aelig;" , "ae"},
-	{"&egrave;", "e`"},
-	{"&eacute;", "e'"},
-	{"&ecirc;" , "e^"},
-	{"&igrave;", "i`"},
-	{"&iacute;", "i'"},
-	{"&icirc;" , "i^"},
+	{"&agrave;", "\303\240"},
+	{"&aacute;", "\303\241"},
+	{"&acirc;" , "\303\242"},
+	{"&atilde;", "\303\243"},
+	{"&aelig;" , "\303\246"},
+	{"&egrave;", "\303\250"},
+	{"&eacute;", "\303\251"},
+	{"&ecirc;" , "\303\252"},
+	{"&igrave;", "\303\254"},
+	{"&iacute;", "\303\255"},
+	{"&icirc;" , "\303\256"},
 
-	{"&ntilde;", "n~"},
-	{"&ograve;", "o`"},
-	{"&oacute;", "o'"},
-	{"&ocirc;" , "o^"},
-	{"&otilde;", "o~"},
-	{"&ugrave;", "u`"},
-	{"&uacute;", "u'"},
-	{"&ucirc;" , "u^"},
-	{"&yacute;", "y'"},
+	{"&ntilde;", "\303\261"},
+	{"&ograve;", "\303\262"},
+	{"&oacute;", "\303\263"},
+	{"&ocirc;" , "\303\264"},
+	{"&otilde;", "\303\265"},
+	{"&ugrave;", "\303\271"},
+	{"&uacute;", "\303\272"},
+	{"&ucirc;" , "\303\273"},
+	{"&yacute;", "\303\275"},
 };
 
 static GHashTable *default_symbol_table;
