@@ -25,12 +25,15 @@
 typedef enum
 {
 	IMAP_AUTH_LOGIN		= 1 << 0,
-	IMAP_AUTH_CRAM_MD5	= 1 << 1
+	IMAP_AUTH_CRAM_MD5	= 1 << 1,
+	IMAP_AUTH_ANON		= 1 << 2,
+	IMAP_AUTH_GSSAPI	= 1 << 3
 } IMAPAuthType;
 
 FolderClass *imap_get_class		(void);
 guint imap_folder_get_refcnt(Folder *folder);
 void imap_folder_ref(Folder *folder);
 void imap_folder_unref(Folder *folder);
-
+gchar imap_get_path_separator_for_item	(FolderItem *item);
+void imap_disconnect_all(void);
 #endif /* __IMAP_H__ */
