@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2003-2004 Match Grun
+ * Copyright (C) 2003-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -206,16 +206,6 @@ void ldapsvr_set_control( LdapServer *server, LdapControl *ctl ) {
 	server->control = ctl;
 }
 
-/**
- * Release LDAP control object.
- * \param server Server object.
- */
-static void ldapsvr_release_control( LdapServer *server ) {
-	g_return_if_fail( server != NULL );
-	ldapctl_free( server->control );
-	server->control = NULL;
-}
-
 /**
  * Free all queries.
  * \param server Server object.
@@ -437,7 +427,7 @@ static LdapQuery *ldapsvr_locate_query(
 {
 	LdapQuery *incomplete = NULL;
 	GList *node;	
-	g_return_if_fail( server != NULL );
+	g_return_val_if_fail( server != NULL, NULL );
 
 	node = server->listQuery;
 	node = g_list_last( node );
