@@ -31,8 +31,8 @@
 /* Function Prototypes */
 GList *ldaputil_read_basedn	( const gchar *host, const gint port,
 				  const gchar *bindDN, const gchar *bindPW,
-				  const gint tov );
-gboolean ldaputil_test_connect	( const gchar *host, const gint port );
+				  const gint tov, int ssl, int tls );
+gboolean ldaputil_test_connect	( const gchar *host, const gint port, int ssl, int tls);
 gboolean ldaputil_test_ldap_lib	( void );
 
 #endif	/* USE_LDAP */
