@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2003 Match Grun
+ * Copyright (C) 2001-2006 Match Grun and the Sylpheed-Claws team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -306,26 +306,28 @@ static void ldif_close_file( LdifFile *ldifFile ) {
 static gchar *ldif_get_line( LdifFile *ldifFile ) {
 	gchar buf[ LDIFBUFSIZE ];
 	gint ch;
-	gchar *ptr;
+	int i = 0;
 
-	if( feof( ldifFile->file ) ) return NULL;
+	if( feof( ldifFile->file ) ) 
+		return NULL;
 
-	ptr = buf;
-	while( TRUE ) {
-		*ptr = '\0';
+	while( i < LDIFBUFSIZE-1 ) {
 		ch = fgetc( ldifFile->file );
 		if( ch == '\0' || ch == EOF ) {
-			if( *buf == '\0' ) return NULL;
+			if( i == 0 ) return NULL;
 			break;
 		}
 #if HAVE_DOSISH_SYSTEM
 #else
-		if( ch == '\r' ) continue;
+		if( ch == '\r' ) 
+			continue;
 #endif
-		if( ch == '\n' ) break;
-		*ptr = ch;
-		ptr++;
+		if( ch == '\n' ) 
+			break;
+		buf[i] = ch;
+		i++;
 	}
+	buf[i] = '\0';
 
 	/* Return a copy of buffer */
 	return g_strdup( buf );
@@ -483,6 +485,14 @@ static void ldif_build_items(
 			fullName = g_strdup_printf( "%s", lastName );
 		}
 	}
+	
+	if (!fullName || strlen(fullName) == 0) {
+		g_free(fullName);
+		fullName = NULL;
+		if (rec->listCName)
+			fullName = g_strdup(rec->listCName->data);
+	}
+	
 	if( fullName ) {
 		g_strchug( fullName ); g_strchomp( fullName );
 	}
@@ -633,45 +643,6 @@ static void ldif_clear_rec( Ldif_ParsedRec *rec ) {
 	rec->listID = NULL;
 }
 
-#if 0
-/**
- * Print parsed data.
- * \param rec    LDIF field object.
- * \param stream Output stream.
- */
-static void ldif_print_record( Ldif_ParsedRec *rec, FILE *stream ) {
-	GSList *list;
-
-	fprintf( stream, "LDIF Parsed Record:\n" );
-	fprintf( stream, "common name:" );
-	mgu_print_list( rec->listCName, stream );
-	if( ! rec->listCName ) fprintf( stream, "\n" );
-	fprintf( stream, "first name:" );
-	mgu_print_list( rec->listFName, stream );
-	if( ! rec->listFName ) fprintf( stream, "\n" );
-	fprintf( stream, "last name:" );
-	mgu_print_list( rec->listLName, stream );
-	if( ! rec->listLName ) fprintf( stream, "\n" );
-	fprintf( stream, "nick name:" );
-	mgu_print_list( rec->listNName, stream );
-	if( ! rec->listNName ) fprintf( stream, "\n" );
-	fprintf( stream, "address:" );
-	mgu_print_list( rec->listAddress, stream );
-	if( ! rec->listAddress ) fprintf( stream, "\n" );
-	fprintf( stream, "id:" );
-	mgu_print_list( rec->listID, stream );
-	if( ! rec->listID ) fprintf( stream, "\n" );
-
-	list = rec->userAttr;
-	while( list ) {
-		Ldif_UserAttr *attr = list->data;
-		fprintf( stream, "n/v:\t%s:\t:%s:\n", attr->name, attr->value );
-		list = g_slist_next( list );
-	}
-	list = NULL;
-}
-#endif
-
 /**
  * Read file data into address cache.
  * Note that one LDIF record identifies one entity uniquely with the
@@ -723,7 +694,17 @@ static void ldif_read_file( LdifFile *ldifFile, AddressCache *cache ) {
 			if( lastTag ) {
 				/* Save record */
 				fullValue = mgu_list_coalesce( listValue );
-
+				if (fullValue && last64) {
+					gchar *out = g_malloc(strlen(fullValue));
+					int len = 0;
+					if ((len = base64_decode(out, fullValue,
+							strlen(fullValue))) >= 0) {
+						g_free(fullValue);
+						fullValue = out;
+						fullValue[len] = '\0';
+					} else
+						g_free(out);
+				}
 				/* Base-64 encoded data */
 				/*
 				if( last64 ) {
@@ -764,6 +745,17 @@ static void ldif_read_file( LdifFile *ldifFile, AddressCache *cache ) {
 							/* Save data */
 							fullValue =
 								mgu_list_coalesce( listValue );
+							if (fullValue && last64) {
+								gchar *out = g_malloc(strlen(fullValue));
+								int len = 0;
+								if ((len = base64_decode(out, fullValue,
+										strlen(fullValue))) >= 0) {
+									g_free(fullValue);
+									fullValue = out;
+									fullValue[len] = '\0';
+								} else
+									g_free(out);
+							}
 							/* Base-64 encoded data */
 							/*
 							if( last64 ) {
@@ -840,7 +832,7 @@ static void ldif_hash_add_list( GHashTable *table, GSList *list ) {
 			}
 			else if( g_utf8_collate( tag, LDIF_TAG_EMAIL ) == 0 ) {
 				rec->reserved = rec->selected = TRUE;
-				rec->userName = g_strdup( _( "E-Mail Address" ) );
+				rec->userName = g_strdup( _( "Email Address" ) );
 			}
 			g_hash_table_insert( table, key, rec );
 		}
@@ -908,7 +900,6 @@ static void ldif_read_tag_list( LdifFile *ldifFile ) {
 	/* Process file */
 	while( ! flagEOF ) {
 		gchar *line = ldif_get_line( ldifFile );
-
 		posCur = ftell( ldifFile->file );
 		if( ldifFile->cbProgress ) {
 			/* Call progress indicator */
@@ -948,6 +939,7 @@ static void ldif_read_tag_list( LdifFile *ldifFile ) {
 				if( tagName ) {
 					/* Add tag to list */
 					listTags = g_slist_append( listTags, tagName );
+
 					if( g_utf8_collate(
 						tagName, LDIF_TAG_EMAIL ) == 0 )
 					{
