@@ -39,6 +39,11 @@
 #ifdef G_OS_UNIX
 #  include <signal.h>
 #endif
+#ifdef HAVE_LIBSM
+#include <X11/SM/SMlib.h>
+#include <fcntl.h>
+#endif
+
 #include "wizard.h"
 #ifdef HAVE_STARTUP_NOTIFICATION
 # define SN_API_NOT_YET_FROZEN
@@ -104,10 +109,10 @@
 
 #include "crash.h"
 
+#include "timing.h"
+
 gchar *prog_version;
-#ifdef CRASH_DIALOG
 gchar *argv0;
-#endif
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 static SnLauncheeContext *sn_context = NULL;
@@ -141,6 +146,7 @@ static struct RemoteCmd {
 	gboolean exit;
 	gboolean subscribe;
 	const gchar *subscribe_uri;
+	const gchar *target;
 } cmd;
 
 static void parse_cmd_opt(int argc, char *argv[]);
@@ -182,6 +188,7 @@ static void exit_sylpheed		(MainWindow *mainwin);
 }
 
 static MainWindow *static_mainwindow;
+static gboolean emergency_exit = FALSE;
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 static void sn_error_trap_push(SnDisplay *display, Display *xdisplay)
@@ -252,6 +259,12 @@ gboolean defer_check(void *data)
 	return FALSE;
 }
 
+static gboolean defer_jump(void *data)
+{
+	mainwindow_jump_to(data);
+	return FALSE;
+}
+
 static gboolean migrate_old_config(const gchar *old_cfg_dir, const gchar *new_cfg_dir)
 {
 	gchar *message = g_strdup_printf(_("Configuration for Sylpheed-Claws %s found.\n"
@@ -276,6 +289,177 @@ static gboolean migrate_old_config(const gchar *old_cfg_dir, const gchar *new_cf
 	return (r == 0);
 }
 
+#ifdef HAVE_LIBSM
+static void
+sc_client_set_value (MainWindow *mainwin,
+		  gchar       *name,
+		  char        *type,
+		  int          num_vals,
+		  SmPropValue *vals)
+{
+	SmProp *proplist[1];
+	SmProp prop;
+
+	prop.name = name;
+	prop.type = type;
+	prop.num_vals = num_vals;
+	prop.vals = vals;
+
+	proplist[0]= &prop;
+	if (mainwin->smc_conn)
+		SmcSetProperties ((SmcConn) mainwin->smc_conn, 1, proplist);
+}
+
+static void sc_die_callback (SmcConn smc_conn, SmPointer client_data)
+{
+	clean_quit(NULL);
+}
+
+static void sc_save_complete_callback(SmcConn smc_conn, SmPointer client_data)
+{
+}
+
+static void sc_shutdown_cancelled_callback (SmcConn smc_conn, SmPointer client_data)
+{
+	MainWindow *mainwin = (MainWindow *)client_data;
+	if (mainwin->smc_conn)
+		SmcSaveYourselfDone ((SmcConn) mainwin->smc_conn, TRUE);
+}
+
+static void sc_save_yourself_callback (SmcConn   smc_conn,
+			       SmPointer client_data,
+			       int       save_style,
+			       gboolean  shutdown,
+			       int       interact_style,
+			       gboolean  fast) {
+
+	MainWindow *mainwin = (MainWindow *)client_data;
+	if (mainwin->smc_conn)
+		SmcSaveYourselfDone ((SmcConn) mainwin->smc_conn, TRUE);
+}
+
+static IceIOErrorHandler sc_ice_installed_handler;
+
+static void sc_ice_io_error_handler (IceConn connection)
+{
+	if (sc_ice_installed_handler)
+		(*sc_ice_installed_handler) (connection);
+}
+static gboolean sc_process_ice_messages (GIOChannel   *source,
+		      GIOCondition  condition,
+		      gpointer      data)
+{
+	IceConn connection = (IceConn) data;
+	IceProcessMessagesStatus status;
+
+	status = IceProcessMessages (connection, NULL, NULL);
+
+	if (status == IceProcessMessagesIOError) {
+		IcePointer context = IceGetConnectionContext (connection);
+
+		if (context && GTK_IS_OBJECT (context)) {
+		guint disconnect_id = g_signal_lookup ("disconnect", G_OBJECT_TYPE (context));
+
+		if (disconnect_id > 0)
+			g_signal_emit (context, disconnect_id, 0);
+		} else {
+			IceSetShutdownNegotiation (connection, False);
+			IceCloseConnection (connection);
+		}
+	}
+
+	return TRUE;
+}
+
+static void new_ice_connection (IceConn connection, IcePointer client_data, Bool opening,
+		    IcePointer *watch_data)
+{
+	guint input_id;
+
+	if (opening) {
+		GIOChannel *channel;
+		/* Make sure we don't pass on these file descriptors to any
+		exec'ed children */
+		fcntl(IceConnectionNumber(connection),F_SETFD,
+		fcntl(IceConnectionNumber(connection),F_GETFD,0) | FD_CLOEXEC);
+
+		channel = g_io_channel_unix_new (IceConnectionNumber (connection));
+		input_id = g_io_add_watch (channel,
+		G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI,
+		sc_process_ice_messages,
+		connection);
+		g_io_channel_unref (channel);
+
+		*watch_data = (IcePointer) GUINT_TO_POINTER (input_id);
+	} else {
+		input_id = GPOINTER_TO_UINT ((gpointer) *watch_data);
+		g_source_remove (input_id);
+	}
+}
+
+static void sc_session_manager_connect(MainWindow *mainwin)
+{
+	static gboolean connected = FALSE;
+	SmcCallbacks      callbacks;
+	gchar            *client_id;
+	IceIOErrorHandler default_handler;
+
+	if (connected)
+		return;
+	connected = TRUE;
+
+
+	sc_ice_installed_handler = IceSetIOErrorHandler (NULL);
+	default_handler = IceSetIOErrorHandler (sc_ice_io_error_handler);
+
+	if (sc_ice_installed_handler == default_handler)
+		sc_ice_installed_handler = NULL;
+
+	IceAddConnectionWatch (new_ice_connection, NULL);
+      
+      
+      	callbacks.save_yourself.callback      = sc_save_yourself_callback;
+	callbacks.die.callback                = sc_die_callback;
+	callbacks.save_complete.callback      = sc_save_complete_callback;
+	callbacks.shutdown_cancelled.callback = sc_shutdown_cancelled_callback;
+
+	callbacks.save_yourself.client_data =
+		callbacks.die.client_data =
+		callbacks.save_complete.client_data =
+		callbacks.shutdown_cancelled.client_data = (SmPointer) mainwin;
+	if (g_getenv ("SESSION_MANAGER")) {
+		gchar error_string_ret[256] = "";
+
+		mainwin->smc_conn = (gpointer)
+			SmcOpenConnection (NULL, mainwin,
+				SmProtoMajor, SmProtoMinor,
+				SmcSaveYourselfProcMask | SmcDieProcMask |
+				SmcSaveCompleteProcMask |
+				SmcShutdownCancelledProcMask,
+				&callbacks,
+				NULL, &client_id,
+				256, error_string_ret);
+
+		if (error_string_ret[0] || mainwin->smc_conn == NULL)
+			g_warning ("While connecting to session manager:\n%s.",
+				error_string_ret);
+		else {
+			SmPropValue *vals;
+			vals = g_new (SmPropValue, 1);
+			vals[0].length = strlen(argv0);
+			vals[0].value = argv0;
+			sc_client_set_value (mainwin, SmCloneCommand, SmLISTofARRAY8, 1, vals);
+			sc_client_set_value (mainwin, SmRestartCommand, SmLISTofARRAY8, 1, vals);
+			sc_client_set_value (mainwin, SmProgram, SmARRAY8, 1, vals);
+
+			vals[0].length = strlen(g_get_user_name()?g_get_user_name():"");
+			vals[0].value = g_strdup(g_get_user_name()?g_get_user_name():"");
+			sc_client_set_value (mainwin, SmUserID, SmARRAY8, 1, vals);
+		}
+	}
+}
+#endif
+
 int main(int argc, char *argv[])
 {
 	gchar *userrc;
@@ -283,18 +467,22 @@ int main(int argc, char *argv[])
 	FolderView *folderview;
 	GdkPixbuf *icon;
 	gboolean crash_file_present = FALSE;
-
+	gint num_folder_class = 0;
+	START_TIMING("startup");
+	
 	if (!sylpheed_init(&argc, &argv)) {
 		return 0;
 	}
 
+	prefs_prepare_cache();
 	prog_version = PROG_VERSION;
-#ifdef CRASH_DIALOG
 	argv0 = g_strdup(argv[0]);
-#endif
 
 	parse_cmd_opt(argc, argv);
 
+	if (!cmd.exit)
+		plugin_load_all("Common");
+
 #ifdef CRASH_DIALOG
 	if (cmd.crash) {
 		gtk_set_locale();
@@ -327,8 +515,8 @@ int main(int argc, char *argv[])
 	if (cmd.exit)
 		return 0;
 #endif
-	g_thread_init(NULL);
-	/* gdk_threads_init(); */
+	if (!g_thread_supported())
+		g_thread_init(NULL);
 
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
@@ -341,6 +529,26 @@ int main(int argc, char *argv[])
 		g_error(_("g_thread is not supported by glib.\n"));
 	}
 
+	/* check that we're not on a too recent/old gtk+ */
+#if GTK_CHECK_VERSION(2, 9, 0)
+	if (gtk_check_version(2, 9, 0) != NULL) {
+		alertpanel_error(_("Sylpheed-Claws has been compiled with "
+				   "a more recent GTK+ library than is "
+				   "currently available. This will cause "
+				   "crashes. You need to upgrade GTK+ or "
+				   "recompile Sylpheed-Claws."));
+		exit(1);
+	}
+#else
+	if (gtk_check_version(2, 9, 0) == NULL) {
+		alertpanel_error(_("Sylpheed-Claws has been compiled with "
+				   "an older GTK+ library than is "
+				   "currently available. This will cause "
+				   "crashes. You need to recompile "
+				   "Sylpheed-Claws."));
+		exit(1);
+	}
+#endif	
 	/* parse gtkrc files */
 	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc-2.0",
 			     NULL);
@@ -353,6 +561,7 @@ int main(int argc, char *argv[])
 
 	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
 	if (!is_dir_exist(RC_DIR)) {
+		prefs_destroy_cache();
 		gboolean r = FALSE;
 		if (is_dir_exist(OLD_GTK2_RC_DIR))
 			r = migrate_old_config(OLD_GTK2_RC_DIR, RC_DIR);
@@ -429,7 +638,7 @@ int main(int argc, char *argv[])
 	renderer_read_config();
 
 	gtkut_widget_init();
-	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED_ICON, &icon);
+	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED_CLAWS_ICON, &icon);
 	gtk_window_set_default_icon(icon);
 
 	folderview_initialize();
@@ -437,9 +646,10 @@ int main(int argc, char *argv[])
 	mh_gtk_init();
 	imap_gtk_init();
 	news_gtk_init();
-		
+
 	mainwin = main_window_create
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
+	manage_window_focus_in(mainwin->window, NULL, NULL);
 	folderview = mainwin->folderview;
 
 	gtk_clist_freeze(GTK_CLIST(mainwin->folderview->ctree));
@@ -457,12 +667,15 @@ int main(int argc, char *argv[])
 	account_read_config_all();
 
 	if (folder_read_list() < 0) {
+		prefs_destroy_cache();
 		if (!run_wizard(mainwin, TRUE))
 			exit(1);
+		main_window_reflect_prefs_all_now();
 		folder_write_list();
 	}
 
 	if (!account_get_list()) {
+		prefs_destroy_cache();
 		if (!run_wizard(mainwin, FALSE))
 			exit(1);
 		account_read_config_all();
@@ -470,10 +683,14 @@ int main(int argc, char *argv[])
 			exit_sylpheed(mainwin);
 	}
 
+	
+	toolbar_main_set_sensitive(mainwin);
+	main_window_set_menu_sensitive(mainwin);
+
 	main_window_popup(mainwin);
 
 #ifdef HAVE_LIBETPAN
-	imap_main_init();
+	imap_main_init(prefs_common.skip_ssl_cert_check);
 #endif	
 	account_set_missing_folder();
 	folder_set_missing_folders();
@@ -487,6 +704,7 @@ int main(int argc, char *argv[])
 
 	/* if Sylpheed crashed, rebuild caches */
 	if (!cmd.crash && crash_file_present) {
+		GTK_EVENTS_FLUSH();
 		debug_print("Sylpheed-Claws crashed, checking for new messages in local folders\n");
 		folder_item_update_thaw();
 		folderview_check_new(NULL);
@@ -522,15 +740,28 @@ int main(int argc, char *argv[])
 
 	prefs_toolbar_init();
 
+	num_folder_class = g_list_length(folder_get_list());
+
 	plugin_load_all("GTK2");
+
+	if (g_list_length(folder_get_list()) != num_folder_class) {
+		debug_print("new folders loaded, reloading processing rules\n");
+		prefs_matcher_read_config();
+	}
 	
 	if (plugin_get_unloaded_list() != NULL) {
+		main_window_cursor_normal(mainwin);
 		alertpanel_warning(_("Some plugin(s) failed to load. "
 				     "Check the Plugins configuration "
 				     "for more information."));
+		main_window_cursor_wait(mainwin);
 	}
-	
+
+ 	plugin_load_standard_plugins ();
+       
 	if (!folder_have_mailbox()) {
+		prefs_destroy_cache();
+		main_window_cursor_normal(mainwin);
 		alertpanel_error(_("Sylpheed-Claws has detected a configured "
 				   "mailbox, but could not load it. It is "
 				   "probably provided by an out-of-date "
@@ -543,6 +774,9 @@ int main(int argc, char *argv[])
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 	startup_notification_complete(FALSE);
+#endif
+#ifdef HAVE_LIBSM
+	sc_session_manager_connect(mainwin);
 #endif
 	folder_item_update_thaw();
 	gtk_clist_thaw(GTK_CLIST(mainwin->folderview->ctree));
@@ -574,6 +808,12 @@ int main(int argc, char *argv[])
 		send_queue();
 	}
 	
+	if (cmd.target) {
+		g_timeout_add(500, defer_jump, (gpointer)cmd.target);
+	}
+
+	prefs_destroy_cache();
+	END_TIMING();
 	gtk_main();
 
 	exit_sylpheed(mainwin);
@@ -586,17 +826,25 @@ static void save_all_caches(FolderItem *item, gpointer data)
 	if (!item->cache) {
 		return;
 	}
-	folder_item_write_cache(item);
+
+	if (item->opened)
+		folder_item_close(item);
+	
+	folder_item_free_cache(item, TRUE);
 }
 
+static gboolean sc_exiting = FALSE;
+
 static void exit_sylpheed(MainWindow *mainwin)
 {
 	gchar *filename;
 
+	sc_exiting = TRUE;
+
 	debug_print("shutting down\n");
 	inc_autocheck_timer_remove();
 
-	if (prefs_common.clean_on_exit) {
+	if (prefs_common.clean_on_exit && !emergency_exit) {
 		main_window_empty_trash(mainwin, prefs_common.ask_on_clean);
 	}
 
@@ -609,8 +857,8 @@ static void exit_sylpheed(MainWindow *mainwin)
 	}
 
 	/* save all state before exiting */
-	folder_write_list();
 	folder_func_to_all_folders(save_all_caches, NULL);
+	folder_write_list();
 
 	main_window_get_size(mainwin);
 	main_window_get_position(mainwin);
@@ -637,6 +885,12 @@ static void exit_sylpheed(MainWindow *mainwin)
 
 	lock_socket_remove();
 
+#ifdef HAVE_LIBSM
+	if (mainwin->smc_conn)
+		SmcCloseConnection ((SmcConn)mainwin->smc_conn, 0, NULL);
+	mainwin->smc_conn = NULL;
+#endif
+
 	main_window_destroy_all();
 	
 	plugin_unload_all("GTK2");
@@ -663,8 +917,8 @@ static void exit_sylpheed(MainWindow *mainwin)
 	prefs_spelling_done();
 	gtkaspell_checkers_quit();
 #endif
+	plugin_unload_all("Common");
 	sylpheed_done();
-
 }
 
 static void parse_cmd_opt(int argc, char *argv[])
@@ -764,6 +1018,8 @@ static void parse_cmd_opt(int argc, char *argv[])
  			g_print("%s\n", _("  --status [folder]...   show the total number of messages"));
  			g_print("%s\n", _("  --status-full [folder]...\n"
  			       "                         show the status of each folder"));
+			g_print("%s\n", _("  --select folder[/msg]  jumps to the specified folder/message\n" 
+			                  "                         folder is a folder id like '#mh/Mailbox/inbox'"));
 			g_print("%s\n", _("  --online               switch to online mode"));
 			g_print("%s\n", _("  --offline              switch to offline mode"));
 			g_print("%s\n", _("  --exit                 exit Sylpheed-Claws"));
@@ -783,6 +1039,8 @@ static void parse_cmd_opt(int argc, char *argv[])
 			exit(0);
 		} else if (!strncmp(argv[i], "--exit", 6)) {
 			cmd.exit = TRUE;
+		} else if (!strncmp(argv[i], "--select", 8) && i+1 < argc) {
+			cmd.target = argv[i+1];
 		} else if (i == 1 && argc == 2) {
 			/* only one parameter. Do something intelligent about it */
 			if (strstr(argv[i], "@") && !strstr(argv[i], "://")) {
@@ -837,7 +1095,6 @@ static void initial_processing(FolderItem *item, gpointer data)
 			      item->path 
 			      ? item->path 
 			      : _("top level folder"));
-	debug_print("%s\n", buf);
 	g_free(buf);
 
 	
@@ -845,7 +1102,6 @@ static void initial_processing(FolderItem *item, gpointer data)
 		folder_item_apply_processing(item);
 	}
 
-	debug_print("done.\n");
 	STATUSBAR_POP(mainwin);
 }
 
@@ -858,7 +1114,6 @@ static void draft_all_messages(void)
 		compose_draft(c);
 	}	
 }
-
 gboolean clean_quit(gpointer data)
 {
 	static gboolean firstrun = TRUE;
@@ -885,7 +1140,7 @@ gboolean clean_quit(gpointer data)
 	}
 		
 	draft_all_messages();
-
+	emergency_exit = TRUE;
 	exit_sylpheed(static_mainwindow);
 	exit(0);
 
@@ -895,13 +1150,20 @@ gboolean clean_quit(gpointer data)
 void app_will_exit(GtkWidget *widget, gpointer data)
 {
 	MainWindow *mainwin = data;
-
+	
+	if (sc_exiting == TRUE) {
+		debug_print("exit pending\n");
+		return;
+	}
+	sc_exiting = TRUE;
+	debug_print("exiting\n");
 	if (compose_get_compose_list()) {
 		gint val = alertpanel(_("Really quit?"),
 			       _("Composing message exists."),
 			       _("_Save to Draft"), _("_Discard them"), _("Do_n't quit"));
 		switch (val) {
 			case G_ALERTOTHER:
+				sc_exiting = FALSE;
 				return;
 			case G_ALERTALTERNATE:
 				break;
@@ -917,16 +1179,23 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 			       _("Some unsent messages are queued. Exit now?"),
 			       GTK_STOCK_CANCEL, GTK_STOCK_OK, NULL)
 		    != G_ALERTALTERNATE) {
+		    	sc_exiting = FALSE;
 			return;
 		}
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
 
 	sock_cleanup();
-
+	if (folderview_get_selected_item(mainwin->folderview))
+		folder_item_close(folderview_get_selected_item(mainwin->folderview));
 	gtk_main_quit();
 }
 
+gboolean sylpheed_is_exiting(void)
+{
+	return sc_exiting;
+}
+
 /*
  * CLAWS: want this public so crash dialog can delete the
  * lock file too
@@ -1051,6 +1320,10 @@ static gint prohibit_duplicate_launch(void)
  		}
 	} else if (cmd.exit) {
 		fd_write_all(uxsock, "exit\n", 5);
+	} else if (cmd.target) {
+		gchar *str = g_strdup_printf("select %s\n", cmd.target);
+		fd_write_all(uxsock, str, strlen(str));
+		g_free(str);
 	} else
 		fd_write_all(uxsock, "popup\n", 6);
 
@@ -1162,6 +1435,9 @@ static void lock_socket_input_cb(gpointer data,
  		fd_write_all(sock, ".\n", 2);
  		g_free(status);
  		if (folders) g_ptr_array_free(folders, TRUE);
+	} else if (!strncmp(buf, "select ", 7)) {
+		const gchar *target = buf+7;
+		mainwindow_jump_to(target);
 	} else if (!strncmp(buf, "exit", 4)) {
 		app_will_exit(NULL, mainwin);
 	}
@@ -1184,22 +1460,30 @@ static void open_compose_new(const gchar *address, GPtrArray *attach_files)
 static void send_queue(void)
 {
 	GList *list;
-
+	gchar *errstr = NULL;
 	for (list = folder_get_list(); list != NULL; list = list->next) {
 		Folder *folder = list->data;
 
 		if (folder->queue) {
 			gint res = procmsg_send_queue
-				(folder->queue, prefs_common.savemsg);
+				(folder->queue, prefs_common.savemsg,
+				&errstr);
 
-			if (res < 0) {
-				alertpanel_error(_("Some errors occurred while sending queued messages."));
-			}
 			if (res) {
 				folder_item_scan(folder->queue);
 			}
 		}
 	}
+	if (errstr) {
+		gchar *tmp = g_strdup_printf(_("Some errors occurred "
+				"while sending queued messages:\n%s"), errstr);
+		g_free(errstr);
+		alertpanel_error_log(tmp);
+		g_free(tmp);
+	} else {
+		alertpanel_error_log("Some errors occurred "
+				"while sending queued messages.");
+	}
 }
 
 static void quit_signal_handler(int sig)
