@@ -31,6 +31,8 @@ typedef struct _MainWindow  MainWindow;
 #include "logwindow.h"
 #include "toolbar.h"
 
+#define OFFLINE_SWITCH_HOOKLIST "offline_switch"
+
 typedef enum
 {
 	M_UNLOCKED            = 1 << 0,
@@ -49,7 +51,9 @@ typedef enum
 	M_HIDE_READ_MSG	      = 1 << 13,
 	M_DELAY_EXEC	      = 1 << 14,
 	M_NOT_NEWS	      = 1 << 15,
-	M_CAN_LEARN_SPAM      = 1 << 16
+	M_CAN_LEARN_SPAM      = 1 << 16,
+	M_ACTIONS_EXIST       = 1 << 17,
+	M_HAVE_QUEUED_MAILS   = 1 << 18
 } SensitiveCond;
 
 typedef enum
@@ -137,6 +141,11 @@ struct _MainWindow
 	gint	progressindicator_hook;
 	
 	GtkWidget 	*colorlabel_menu;
+	GtkWidget	*warning_btn;
+	
+#ifdef HAVE_LIBSM
+	gpointer smc_conn;
+#endif
 };
 
 MainWindow *main_window_create		(SeparateType	 type);
@@ -153,6 +162,7 @@ void main_window_unlock			(MainWindow	*mainwin);
 
 void main_window_reflect_prefs_all_real	(gboolean 	 pixmap_theme_changed);
 void main_window_reflect_prefs_all	(void);
+void main_window_reflect_prefs_all_now	(void);
 void main_window_reflect_prefs_custom_colors(MainWindow 	*mainwindow);
 void main_window_set_summary_column	(void);
 void main_window_set_folder_column	(void);
@@ -162,6 +172,10 @@ void main_window_set_account_menu_only_toolbar	(GList		*account_list);
 GtkWidget *main_window_get_folder_window	(MainWindow	*mainwin);
 GtkWidget *main_window_get_message_window	(MainWindow	*mainwin);
 
+/* Mailing list support */
+void main_create_mailing_list_menu 	(MainWindow *mainwin, MsgInfo *msginfo);
+gint mailing_list_get_list_post_mailto 	(gchar **url, gchar *mailto, gint maxlen);
+
 void main_window_separation_change	(MainWindow	*mainwin,
 					 SeparateType	 type);
 
@@ -207,4 +221,7 @@ gboolean mainwindow_key_pressed		      (GtkWidget 	 *widget,
 MainWindow *mainwindow_get_mainwindow 	      (void);
 void mainwindow_learn			      (MainWindow *mainwin,
 					       gboolean is_spam);
+void mainwindow_jump_to			      (const gchar 	 *target);
+void mainwindow_show_error		      (void);
+void mainwindow_clear_error		      (MainWindow *mainwin);
 #endif /* __MAINWINDOW_H__ */
