@@ -1,7 +1,9 @@
-/* A Bison parser, made by GNU Bison 2.1.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
-/* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+/* Skeleton interface for Bison's Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,10 +20,18 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.  */
 
-/* As a special exception, when this file is copied by Bison into a
-   Bison output file, you may use that output file without restriction.
-   This special exception was added by the Free Software Foundation
-   in version 1.24 of Bison.  */
+/* As a special exception, you may create a larger work that contains
+   part or all of the Bison parser skeleton and distribute that work
+   under terms of your choice, so long as that work isn't itself a
+   parser generator using the skeleton or a modified version thereof
+   as a parser skeleton.  Alternatively, if you modify or redistribute
+   the parser skeleton itself, you may (at your option) remove this
+   special exception, which will cause the skeleton and the resulting
+   Bison output files to be licensed under the GNU General Public
+   License without this special exception.
+
+   This special exception was added by the Free Software Foundation in
+   version 2.2 of Bison.  */
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -78,45 +88,49 @@
      MATCHER_REGEXP = 304,
      MATCHER_SCORE = 305,
      MATCHER_MOVE = 306,
-     MATCHER_ANY_IN_ADDRESSBOOK = 307,
-     MATCHER_ALL_IN_ADDRESSBOOK = 308,
-     MATCHER_COPY = 309,
-     MATCHER_DELETE = 310,
-     MATCHER_MARK = 311,
-     MATCHER_UNMARK = 312,
-     MATCHER_LOCK = 313,
-     MATCHER_UNLOCK = 314,
-     MATCHER_EXECUTE = 315,
-     MATCHER_MARK_AS_READ = 316,
-     MATCHER_MARK_AS_UNREAD = 317,
-     MATCHER_FORWARD = 318,
-     MATCHER_FORWARD_AS_ATTACHMENT = 319,
-     MATCHER_EOL = 320,
-     MATCHER_STRING = 321,
-     MATCHER_OR = 322,
-     MATCHER_AND = 323,
-     MATCHER_COLOR = 324,
-     MATCHER_SCORE_EQUAL = 325,
-     MATCHER_REDIRECT = 326,
-     MATCHER_SIZE_GREATER = 327,
-     MATCHER_SIZE_SMALLER = 328,
-     MATCHER_SIZE_EQUAL = 329,
-     MATCHER_LOCKED = 330,
-     MATCHER_NOT_LOCKED = 331,
-     MATCHER_PARTIAL = 332,
-     MATCHER_NOT_PARTIAL = 333,
-     MATCHER_COLORLABEL = 334,
-     MATCHER_NOT_COLORLABEL = 335,
-     MATCHER_IGNORE_THREAD = 336,
-     MATCHER_NOT_IGNORE_THREAD = 337,
-     MATCHER_CHANGE_SCORE = 338,
-     MATCHER_SET_SCORE = 339,
-     MATCHER_STOP = 340,
-     MATCHER_HIDE = 341,
-     MATCHER_IGNORE = 342,
-     MATCHER_RULENAME = 343,
-     MATCHER_SECTION = 344,
-     MATCHER_INTEGER = 345
+     MATCHER_FOUND_IN_ADDRESSBOOK = 307,
+     MATCHER_NOT_FOUND_IN_ADDRESSBOOK = 308,
+     MATCHER_IN = 309,
+     MATCHER_COPY = 310,
+     MATCHER_DELETE = 311,
+     MATCHER_MARK = 312,
+     MATCHER_UNMARK = 313,
+     MATCHER_LOCK = 314,
+     MATCHER_UNLOCK = 315,
+     MATCHER_EXECUTE = 316,
+     MATCHER_MARK_AS_READ = 317,
+     MATCHER_MARK_AS_UNREAD = 318,
+     MATCHER_FORWARD = 319,
+     MATCHER_FORWARD_AS_ATTACHMENT = 320,
+     MATCHER_EOL = 321,
+     MATCHER_STRING = 322,
+     MATCHER_OR = 323,
+     MATCHER_AND = 324,
+     MATCHER_COLOR = 325,
+     MATCHER_SCORE_EQUAL = 326,
+     MATCHER_REDIRECT = 327,
+     MATCHER_SIZE_GREATER = 328,
+     MATCHER_SIZE_SMALLER = 329,
+     MATCHER_SIZE_EQUAL = 330,
+     MATCHER_LOCKED = 331,
+     MATCHER_NOT_LOCKED = 332,
+     MATCHER_PARTIAL = 333,
+     MATCHER_NOT_PARTIAL = 334,
+     MATCHER_COLORLABEL = 335,
+     MATCHER_NOT_COLORLABEL = 336,
+     MATCHER_IGNORE_THREAD = 337,
+     MATCHER_NOT_IGNORE_THREAD = 338,
+     MATCHER_CHANGE_SCORE = 339,
+     MATCHER_SET_SCORE = 340,
+     MATCHER_STOP = 341,
+     MATCHER_HIDE = 342,
+     MATCHER_IGNORE = 343,
+     MATCHER_ENABLED = 344,
+     MATCHER_DISABLED = 345,
+     MATCHER_RULENAME = 346,
+     MATCHER_ACCOUNT = 347,
+     MATCHER_SECTION = 348,
+     MATCHER_INTEGER = 349
    };
 #endif
 /* Tokens.  */
@@ -169,57 +183,63 @@
 #define MATCHER_REGEXP 304
 #define MATCHER_SCORE 305
 #define MATCHER_MOVE 306
-#define MATCHER_ANY_IN_ADDRESSBOOK 307
-#define MATCHER_ALL_IN_ADDRESSBOOK 308
-#define MATCHER_COPY 309
-#define MATCHER_DELETE 310
-#define MATCHER_MARK 311
-#define MATCHER_UNMARK 312
-#define MATCHER_LOCK 313
-#define MATCHER_UNLOCK 314
-#define MATCHER_EXECUTE 315
-#define MATCHER_MARK_AS_READ 316
-#define MATCHER_MARK_AS_UNREAD 317
-#define MATCHER_FORWARD 318
-#define MATCHER_FORWARD_AS_ATTACHMENT 319
-#define MATCHER_EOL 320
-#define MATCHER_STRING 321
-#define MATCHER_OR 322
-#define MATCHER_AND 323
-#define MATCHER_COLOR 324
-#define MATCHER_SCORE_EQUAL 325
-#define MATCHER_REDIRECT 326
-#define MATCHER_SIZE_GREATER 327
-#define MATCHER_SIZE_SMALLER 328
-#define MATCHER_SIZE_EQUAL 329
-#define MATCHER_LOCKED 330
-#define MATCHER_NOT_LOCKED 331
-#define MATCHER_PARTIAL 332
-#define MATCHER_NOT_PARTIAL 333
-#define MATCHER_COLORLABEL 334
-#define MATCHER_NOT_COLORLABEL 335
-#define MATCHER_IGNORE_THREAD 336
-#define MATCHER_NOT_IGNORE_THREAD 337
-#define MATCHER_CHANGE_SCORE 338
-#define MATCHER_SET_SCORE 339
-#define MATCHER_STOP 340
-#define MATCHER_HIDE 341
-#define MATCHER_IGNORE 342
-#define MATCHER_RULENAME 343
-#define MATCHER_SECTION 344
-#define MATCHER_INTEGER 345
+#define MATCHER_FOUND_IN_ADDRESSBOOK 307
+#define MATCHER_NOT_FOUND_IN_ADDRESSBOOK 308
+#define MATCHER_IN 309
+#define MATCHER_COPY 310
+#define MATCHER_DELETE 311
+#define MATCHER_MARK 312
+#define MATCHER_UNMARK 313
+#define MATCHER_LOCK 314
+#define MATCHER_UNLOCK 315
+#define MATCHER_EXECUTE 316
+#define MATCHER_MARK_AS_READ 317
+#define MATCHER_MARK_AS_UNREAD 318
+#define MATCHER_FORWARD 319
+#define MATCHER_FORWARD_AS_ATTACHMENT 320
+#define MATCHER_EOL 321
+#define MATCHER_STRING 322
+#define MATCHER_OR 323
+#define MATCHER_AND 324
+#define MATCHER_COLOR 325
+#define MATCHER_SCORE_EQUAL 326
+#define MATCHER_REDIRECT 327
+#define MATCHER_SIZE_GREATER 328
+#define MATCHER_SIZE_SMALLER 329
+#define MATCHER_SIZE_EQUAL 330
+#define MATCHER_LOCKED 331
+#define MATCHER_NOT_LOCKED 332
+#define MATCHER_PARTIAL 333
+#define MATCHER_NOT_PARTIAL 334
+#define MATCHER_COLORLABEL 335
+#define MATCHER_NOT_COLORLABEL 336
+#define MATCHER_IGNORE_THREAD 337
+#define MATCHER_NOT_IGNORE_THREAD 338
+#define MATCHER_CHANGE_SCORE 339
+#define MATCHER_SET_SCORE 340
+#define MATCHER_STOP 341
+#define MATCHER_HIDE 342
+#define MATCHER_IGNORE 343
+#define MATCHER_ENABLED 344
+#define MATCHER_DISABLED 345
+#define MATCHER_RULENAME 346
+#define MATCHER_ACCOUNT 347
+#define MATCHER_SECTION 348
+#define MATCHER_INTEGER 349
 
 
 
 
-#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 245 "matcher_parser_parse.y"
-typedef union YYSTYPE {
+#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
+typedef union YYSTYPE
+#line 298 "matcher_parser_parse.y"
+{
 	char *str;
 	int value;
-} YYSTYPE;
-/* Line 1447 of yacc.c.  */
-#line 223 "matcher_parser_parse.h"
+}
+/* Line 1529 of yacc.c.  */
+#line 242 "matcher_parser_parse.h"
+	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -227,5 +247,3 @@ typedef union YYSTYPE {
 
 extern YYSTYPE yylval;
 
-
-
