@@ -32,6 +32,7 @@
 #include <sys/file.h>
 #include <ctype.h>
 #include <time.h>
+#include <errno.h>
 
 #include "mbox.h"
 #include "procmsg.h"
@@ -42,6 +43,7 @@
 #include "utils.h"
 #include "filtering.h"
 #include "alertpanel.h"
+#include "statusbar.h"
 
 #define MSGBUFSIZE	8192
 
@@ -58,7 +60,9 @@
 	} \
 }
 
-gint proc_mbox(FolderItem *dest, const gchar *mbox, gboolean apply_filter)
+gint proc_mbox(FolderItem *dest, const gchar *mbox, gboolean apply_filter,
+	       PrefsAccount *account)
+/* return values: -1 error, >=0 number of msgs added */
 {
 	FILE *mbox_fp;
 	gchar buf[MSGBUFSIZE];
@@ -67,7 +71,9 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, gboolean apply_filter)
 	gint lines;
 	MsgInfo *msginfo;
 	gboolean more;
-	GSList *to_filter = NULL, *cur;
+	GSList *to_filter = NULL, *filtered = NULL, *unfiltered = NULL, *cur, *to_add = NULL;
+	gboolean printed = FALSE;
+	FolderItem *dropfolder;
 
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(mbox != NULL, -1);
@@ -98,13 +104,25 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, gboolean apply_filter)
 	tmp_file = get_tmp_file();
 
 	folder_item_update_freeze();
+
+	if (apply_filter)
+		dropfolder = folder_get_default_processing();
+	else
+		dropfolder = dest;
 	
 	do {
 		FILE *tmp_fp;
-		FolderItem *dropfolder;
 		gint empty_lines;
 		gint msgnum;
-
+		
+		if (msgs > 0 && msgs%500 == 0) {
+			if (printed)
+				statusbar_pop_all();
+			statusbar_print_all(_("Importing from mbox... (%d mails imported)"), msgs);
+			printed=TRUE;
+			GTK_EVENTS_FLUSH();
+		}
+	
 		if ((tmp_fp = g_fopen(tmp_file, "wb")) == NULL) {
 			FILE_OP_ERROR(tmp_file, "fopen");
 			g_warning("can't open temporary file\n");
@@ -201,29 +219,59 @@ gint proc_mbox(FolderItem *dest, const gchar *mbox, gboolean apply_filter)
 			return -1;
 		}
 
-		dropfolder = folder_get_default_processing();
+		if (apply_filter) {
+			if ((msgnum = folder_item_add_msg(dropfolder, tmp_file, NULL, TRUE)) < 0) {
+				fclose(mbox_fp);
+				g_unlink(tmp_file);
+				g_free(tmp_file);
+				return -1;
+			}
+			msginfo = folder_item_get_msginfo(dropfolder, msgnum);
+			to_filter = g_slist_prepend(to_filter, msginfo);
+		} else {
+			MsgFileInfo *finfo = g_new0(MsgFileInfo, 1);
+			finfo->file = tmp_file;
 			
-		if ((msgnum = folder_item_add_msg(dropfolder, tmp_file, NULL, TRUE)) < 0) {
-			fclose(mbox_fp);
-			g_unlink(tmp_file);
-			g_free(tmp_file);
-			return -1;
+			to_add = g_slist_prepend(to_add, finfo);
+			tmp_file = get_tmp_file();
+			
+			/* flush every 500 */
+			if (msgs > 0 && msgs % 500 == 0) {
+				folder_item_add_msgs(dropfolder, to_add, TRUE);
+				procmsg_message_file_list_free(to_add);
+				to_add = NULL;
+			}
 		}
-
-		msginfo = folder_item_get_msginfo(dropfolder, msgnum);
-                if (!apply_filter || !procmsg_msginfo_filter(msginfo)) {
-			folder_item_move_msg(dest, msginfo);
-			procmsg_msginfo_free(msginfo);
-		} else
-			to_filter = g_slist_append(to_filter, msginfo);
-
 		msgs++;
 	} while (more);
 
-	filtering_move_and_copy_msgs(to_filter);
-	for (cur = to_filter; cur; cur = g_slist_next(cur)) {
-		MsgInfo *info = (MsgInfo *)cur->data;
-		procmsg_msginfo_free(info);
+	if (printed)
+		statusbar_pop_all();
+
+	if (apply_filter) {
+		procmsg_msglist_filter(to_filter, account, 
+				&filtered, &unfiltered, TRUE);
+
+		filtering_move_and_copy_msgs(to_filter);
+		for (cur = filtered; cur; cur = g_slist_next(cur)) {
+			MsgInfo *info = (MsgInfo *)cur->data;
+			procmsg_msginfo_free(info);
+		}
+
+		unfiltered = g_slist_reverse(unfiltered);
+		folder_item_move_msgs(dest, unfiltered);
+		for (cur = unfiltered; cur; cur = g_slist_next(cur)) {
+			MsgInfo *info = (MsgInfo *)cur->data;
+			procmsg_msginfo_free(info);
+		}
+
+		g_slist_free(unfiltered);
+		g_slist_free(filtered);
+		g_slist_free(to_filter);
+	} else if (to_add) {
+		folder_item_add_msgs(dropfolder, to_add, TRUE);
+		procmsg_message_file_list_free(to_add);
+		to_add = NULL;
 	}
 
 	folder_item_update_thaw();
@@ -276,21 +324,39 @@ gint lock_mbox(const gchar *base, LockType type)
 		g_free(lockfile);
 	} else if (type == LOCK_FLOCK) {
 		gint lockfd;
+		gboolean fcntled = FALSE;
+#if HAVE_FCNTL_H && !defined(G_OS_WIN32)
+		struct flock fl;
+		fl.l_type = F_WRLCK;
+		fl.l_whence = SEEK_SET;
+		fl.l_start = 0;
+		fl.l_len = 0;
+#endif
 
 #if HAVE_FLOCK
-		if ((lockfd = open(base, O_RDONLY)) < 0) {
+		if ((lockfd = open(base, O_RDWR)) < 0) {
 #else
 		if ((lockfd = open(base, O_RDWR)) < 0) {
 #endif
 			FILE_OP_ERROR(base, "open");
 			return -1;
 		}
+		
+#if HAVE_FCNTL_H && !defined(G_OS_WIN32)
+		if (fcntl(lockfd, F_SETLK, &fl) == -1) {
+			g_warning("can't fnctl %s (%s)", base, strerror(errno));
+			return -1;
+		} else {
+			fcntled = TRUE;
+		}
+#endif
+
 #if HAVE_FLOCK
-		if (flock(lockfd, LOCK_EX|LOCK_NB) < 0) {
+		if (flock(lockfd, LOCK_EX|LOCK_NB) < 0 && !fcntled) {
 			perror("flock");
 #else
 #if HAVE_LOCKF
-		if (lockf(lockfd, F_TLOCK, 0) < 0) {
+		if (lockf(lockfd, F_TLOCK, 0) < 0 && !fcntled) {
 			perror("lockf");
 #else
 		{
@@ -328,12 +394,27 @@ gint unlock_mbox(const gchar *base, gint fd, LockType type)
 
 		return 0;
 	} else if (type == LOCK_FLOCK) {
+		gboolean fcntled = FALSE;
+#if HAVE_FCNTL_H && !defined(G_OS_WIN32)
+		struct flock fl;
+		fl.l_type = F_UNLCK;
+		fl.l_whence = SEEK_SET;
+		fl.l_start = 0;
+		fl.l_len = 0;
+
+		if (fcntl(fd, F_SETLK, &fl) == -1) {
+			g_warning("can't fnctl %s", base);
+			return -1;
+		} else {
+			fcntled = TRUE;
+		}
+#endif
 #if HAVE_FLOCK
-		if (flock(fd, LOCK_UN) < 0) {
+		if (flock(fd, LOCK_UN) < 0 && !fcntled) {
 			perror("flock");
 #else
 #if HAVE_LOCKF
-		if (lockf(fd, F_ULOCK, 0) < 0) {
+		if (lockf(fd, F_ULOCK, 0) < 0 && !fcntled) {
 			perror("lockf");
 #else
 		{
@@ -375,21 +456,22 @@ void empty_mbox(const gchar *mbox)
 }
 
 gint export_list_to_mbox(GSList *mlist, const gchar *mbox)
+/* return values: -2 skipped, -1 error, 0 OK */
 {
 	GSList *cur;
 	MsgInfo *msginfo;
 	FILE *msg_fp;
 	FILE *mbox_fp;
 	gchar buf[BUFFSIZE];
-
+	gint msgs = 1, total = g_slist_length(mlist);
 	if (g_file_test(mbox, G_FILE_TEST_EXISTS) == TRUE) {
 		if (alertpanel_full(_("Overwrite mbox file"),
-							_("This file already exists. Do you want to overwrite it?"),
-							_("Overwrite"), GTK_STOCK_CANCEL, NULL, FALSE,
-							NULL, ALERT_WARNING, G_ALERTALTERNATE)
-			== G_ALERTALTERNATE) {
-		return -1;
-	}
+					_("This file already exists. Do you want to overwrite it?"),
+					GTK_STOCK_CANCEL, _("Overwrite"), NULL, FALSE,
+					NULL, ALERT_WARNING, G_ALERTDEFAULT)
+				!= G_ALERTALTERNATE) {
+			return -2;
+		}
 	}
 
 	if ((mbox_fp = g_fopen(mbox, "wb")) == NULL) {
@@ -398,13 +480,13 @@ gint export_list_to_mbox(GSList *mlist, const gchar *mbox)
 		return -1;
 	}
 
+	statusbar_print_all(_("Exporting to mbox..."));
 	for (cur = mlist; cur != NULL; cur = cur->next) {
-		msginfo = (MsgInfo *)cur->data;
 		int len;
+		msginfo = (MsgInfo *)cur->data;
 
 		msg_fp = procmsg_open_message(msginfo);
 		if (!msg_fp) {
-			procmsg_msginfo_free(msginfo);
 			continue;
 		}
 
@@ -447,18 +529,24 @@ gint export_list_to_mbox(GSList *mlist, const gchar *mbox)
 		fputc('\n', mbox_fp);
 
 		fclose(msg_fp);
-		procmsg_msginfo_free(msginfo);
+		statusbar_progress_all(msgs++,total, 500);
+		if (msgs%500 == 0)
+			GTK_EVENTS_FLUSH();
 	}
-	
+	statusbar_progress_all(0,0,0);
+	statusbar_pop_all();
+
 	fclose(mbox_fp);
 
 	return 0;
 }
 
 /* read all messages in SRC, and store them into one MBOX file. */
+/* return values: -2 skipped, -1 error, 0 OK */
 gint export_to_mbox(FolderItem *src, const gchar *mbox)
 {
 	GSList *mlist;
+	gint ret;
 	
 	g_return_val_if_fail(src != NULL, -1);
 	g_return_val_if_fail(src->folder != NULL, -1);
@@ -469,9 +557,9 @@ gint export_to_mbox(FolderItem *src, const gchar *mbox)
 
 	mlist = folder_item_get_msg_list(src);
 
-	export_list_to_mbox(mlist, mbox);
+	ret = export_list_to_mbox(mlist, mbox);
 
 	procmsg_msg_list_free(mlist);
 
-	return 0;
+	return ret;
 }
