@@ -46,6 +46,7 @@
 #include "addressbook.h"
 #include "alertpanel.h"
 #include "inputdialog.h"
+#include "mainwindow.h"
 #include "manage_window.h"
 #include "procmsg.h"
 #include "procheader.h"
@@ -119,6 +120,9 @@ static void view_source_cb		(gpointer	 data,
 static void show_all_header_cb		(gpointer	 data,
 					 guint		 action,
 					 GtkWidget	*widget);
+static void msg_hide_quotes_cb		(gpointer	 data,
+					 guint		 action,
+					 GtkWidget	*widget);
 
 static void compose_cb			(gpointer	 data,
 					 guint		 action,
@@ -126,9 +130,6 @@ static void compose_cb			(gpointer	 data,
 static void reply_cb			(gpointer	 data,
 					 guint		 action,
 					 GtkWidget	*widget);
-static void reedit_cb			(gpointer	 data,
-					 guint		 action,
-					 GtkWidget	*widget);
 
 static PrefsAccount *select_account_from_list
 					(GList		*ac_list);
@@ -276,6 +277,10 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_View/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Mess_age source"),	"<control>U", view_source_cb, 0, NULL},
 	{N_("/_View/Show all _headers"),"<control>H", show_all_header_cb, 0, "<ToggleItem>"},
+	{N_("/_View/Quotes"),			NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/Quotes/_Fold all"),		"<control><shift>Q", msg_hide_quotes_cb, 1, "<ToggleItem>"},
+	{N_("/_View/Quotes/Fold from level _2"),NULL, msg_hide_quotes_cb, 2, "<ToggleItem>"},
+	{N_("/_View/Quotes/Fold from level _3"),NULL, msg_hide_quotes_cb, 3, "<ToggleItem>"},
 
 	{N_("/_Message"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/Compose _new message"),
@@ -293,8 +298,6 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_Message/For_ward as attachment"),
 					NULL, reply_cb, COMPOSE_FORWARD_AS_ATTACH, NULL},
 	{N_("/_Message/Redirec_t"),	NULL, reply_cb, COMPOSE_REDIRECT, NULL},
-	{N_("/_Message/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/Re-_edit"),	NULL, reedit_cb, 0, NULL},
 
 	{N_("/_Tools"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/_Address book"),	"<control><shift>A", addressbook_open_cb, 0, NULL},
@@ -436,14 +439,14 @@ void messageview_add_toolbar(MessageView *msgview, GtkWidget *window)
 	msgview_list = g_list_append(msgview_list, msgview);
 }
 
-MessageView *messageview_create_with_new_window(MainWindow *mainwin)
+static MessageView *messageview_create_with_new_window_visible(MainWindow *mainwin, gboolean show)
 {
 	MessageView *msgview;
 	GtkWidget *window;
 	static GdkGeometry geometry;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(window), _("Sylpheed - Message View"));
+	gtk_window_set_title(GTK_WINDOW(window), _("Sylpheed-Claws - Message View"));
 	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 
 	if (!geometry.min_height) {
@@ -468,8 +471,12 @@ MessageView *messageview_create_with_new_window(MainWindow *mainwin)
 
 	messageview_add_toolbar(msgview, window);
 
-	gtk_widget_grab_focus(msgview->mimeview->textview->text);
-	gtk_widget_show(window);
+	if (show) {
+		gtk_widget_grab_focus(msgview->mimeview->textview->text);
+		gtk_widget_show(window);
+	} else {
+		gtk_widget_realize(window);
+	}
 
 	msgview->new_window = TRUE;
 	msgview->window = window;
@@ -482,6 +489,10 @@ MessageView *messageview_create_with_new_window(MainWindow *mainwin)
 	return msgview;
 }
 
+MessageView *messageview_create_with_new_window(MainWindow *mainwin)
+{
+	return messageview_create_with_new_window_visible(mainwin, TRUE);
+}
 void messageview_init(MessageView *messageview)
 {
 	headerview_init(messageview->headerview);
@@ -528,16 +539,18 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 	gchar *path;
         gchar *addr;
         gchar *addrp;
-
-	if ((!msginfo->returnreceiptto) && 
-	    (!msginfo->dispositionnotificationto)) 
+	gchar *foo = NULL;
+	if (!msginfo->extradata)
+		return -1;
+	if (!msginfo->extradata->returnreceiptto && 
+	    !msginfo->extradata->dispositionnotificationto) 
 		return -1;
 
 	/* RFC2298: Test for Return-Path */
-	if (msginfo->dispositionnotificationto)
-		to = msginfo->dispositionnotificationto;
+	if (msginfo->extradata->dispositionnotificationto)
+		to = msginfo->extradata->dispositionnotificationto;
 	else
-		to = msginfo->returnreceiptto;
+		to = msginfo->extradata->returnreceiptto;
 
 	ok = procheader_get_header_from_msginfo(msginfo, buf, sizeof(buf),
 				"Return-Path:");
@@ -545,7 +558,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		gchar *to_addr = g_strdup(to);
 		extract_address(to_addr);
 		extract_address(buf);
-		ok = strcmp(to_addr, buf);
+		ok = strcasecmp(to_addr, buf);
 		g_free(to_addr);
 	} else {
 		strncpy(buf, _("<No Return-Path found>"), 
@@ -562,11 +575,11 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		    "Return path: %s\n"
 		    "It is advised to not to send the return receipt."),
 		  to, buf);
-		val = alertpanel_full(_("Warning"), message, _("_Send"),
-					_("+_Don't Send"), NULL, FALSE,
-				       	NULL, ALERT_WARNING, G_ALERTALTERNATE);
+		val = alertpanel_full(_("Warning"), message,
+				_("_Don't Send"), _("_Send"), NULL, FALSE,
+				NULL, ALERT_WARNING, G_ALERTDEFAULT);
 		g_free(message);				
-		if (val != G_ALERTDEFAULT)
+		if (val != G_ALERTALTERNATE)
 			return -1;
 	}
 
@@ -580,9 +593,9 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		    "but according to its 'To:' and 'CC:' headers it was not\n"
 		    "officially addressed to you.\n"
 		    "It is advised to not to send the return receipt."),
-		  _("_Send"), _("+_Don't Send"),NULL, FALSE,
-		  NULL, ALERT_WARNING, G_ALERTALTERNATE);
-		if (val != G_ALERTDEFAULT)
+		  _("_Don't Send"), _("_Send"), NULL, FALSE,
+		  NULL, ALERT_WARNING, G_ALERTDEFAULT);
+		if (val != G_ALERTALTERNATE)
 			return -1;
 	}
 
@@ -653,7 +666,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		g_free(path);
 	}		
 
-	fprintf(fp, "\n");
+	fprintf(fp, "X-Sylpheed-End-Special-Headers: 1\n");
 	
 	/* Date */
 	get_rfc822_date(buf, sizeof(buf));
@@ -708,8 +721,9 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 
 	/* send it */
 	path = folder_item_fetch_msg(queue, num);
-	ok = procmsg_send_message_queue(path);
+	ok = procmsg_send_message_queue(path, &foo, queue, num);
 	g_free(path);
+	g_free(foo);
 	folder_item_remove_msg(queue, num);
 
 	return ok;
@@ -751,6 +765,11 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	gchar *subject = NULL;
 	g_return_val_if_fail(msginfo != NULL, -1);
 
+	if (messageview->mimeview->textview &&
+	    messageview->mimeview->textview->loading) {
+		messageview->mimeview->textview->stop_loading = TRUE;
+		return 0;
+	}
 	if (messageview->toolbar)
 		toolbar_set_learn_button
 			(messageview->toolbar,
@@ -786,8 +805,11 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 
 	while ((encinfo = find_encrypted_part(mimeinfo)) != NULL) {
 		debug_print("decrypting message part\n");
-		if (privacy_mimeinfo_decrypt(encinfo) < 0)
+		if (privacy_mimeinfo_decrypt(encinfo) < 0) {
+			alertpanel_error(_("Couldn't decrypt: %s"),
+				privacy_get_error());
 			break;
+		}
 	}
 	
 	messageview->updating = TRUE;
@@ -809,12 +831,20 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	
 	if (messageview->msginfo != msginfo) {
 		procmsg_msginfo_free(messageview->msginfo);
+		messageview->msginfo = NULL;
+		messageview_set_menu_sensitive(messageview);
 		messageview->msginfo = procmsg_msginfo_get_full_info(msginfo);
 		if (!messageview->msginfo)
 			messageview->msginfo = procmsg_msginfo_copy(msginfo);
+	} else {
+		messageview->msginfo = NULL;
+		messageview_set_menu_sensitive(messageview);
+		messageview->msginfo = msginfo;
 	}
 	headerview_show(messageview->headerview, messageview->msginfo);
 
+	messageview_set_position(messageview, 0);
+
 	textview_set_all_headers(messageview->mimeview->textview, 
 			messageview->all_headers);
 
@@ -837,14 +867,17 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 		g_free(subject);
 	}
 
-	messageview_set_position(messageview, 0);
+	main_create_mailing_list_menu(messageview->mainwin, messageview->msginfo);
 
-	if (messageview->msginfo->partial_recv)
+	if (messageview->msginfo->extradata
+	    && messageview->msginfo->extradata->partial_recv)
 		partial_recv_show(messageview->noticeview, 
 				  messageview->msginfo);
-	else if ((messageview->msginfo->dispositionnotificationto || 
-	     messageview->msginfo->returnreceiptto) &&
-	    !MSG_IS_RETRCPT_SENT(messageview->msginfo->flags))
+	else if (messageview->msginfo->extradata &&
+	    (messageview->msginfo->extradata->dispositionnotificationto || 
+	     messageview->msginfo->extradata->returnreceiptto) &&
+	    !MSG_IS_RETRCPT_SENT(messageview->msginfo->flags) &&
+	    !prefs_common.never_send_retrcpt)
 		return_receipt_show(messageview->noticeview, 
 				    messageview->msginfo);
 	else 
@@ -852,11 +885,11 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 
 	mimeinfo = procmime_mimeinfo_next(mimeinfo);
 	if (mimeinfo && (mimeinfo->type != MIMETYPE_TEXT || 
-	    strcmp(mimeinfo->subtype, "plain"))) {
-	    	if (strcmp(mimeinfo->subtype, "html"))
+	    strcasecmp(mimeinfo->subtype, "plain"))) {
+	    	if (strcasecmp(mimeinfo->subtype, "html"))
 			mimeview_show_part(messageview->mimeview,mimeinfo);
 		else if (prefs_common.invoke_plugin_on_html)
-			mimeview_show_part(messageview->mimeview,mimeinfo);
+			mimeview_select_mimepart_icon(messageview->mimeview,mimeinfo);
 	}
 
 	g_free(file);
@@ -897,11 +930,20 @@ void messageview_destroy(MessageView *messageview)
 	}
 
 	if (messageview->updating) {
-		debug_print("uh oh, better not touch that now\n");
+		debug_print("uh oh, better not touch that now (fetching)\n");
 		messageview->deferred_destroy = TRUE;
 		gtk_widget_hide(messageview->window);
 		return;
 	}
+	
+	if (messageview->mimeview->textview
+	&&  messageview->mimeview->textview->loading) {
+		debug_print("uh oh, better not touch that now (loading text)\n");
+		messageview->deferred_destroy = TRUE;
+		messageview->mimeview->textview->stop_loading = TRUE;
+		gtk_widget_hide(messageview->window);
+		return;
+	}
 
 	headerview_destroy(messageview->headerview);
 	mimeview_destroy(messageview->mimeview);
@@ -978,9 +1020,7 @@ static void messageview_update(MessageView *msgview)
 	g_return_if_fail(summaryview != NULL);
 	
 	if (summaryview->selected) {
-		GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
-		MsgInfo *msginfo = gtk_ctree_node_get_row_data(ctree, 
-						      summaryview->selected);
+		MsgInfo *msginfo = summary_get_selected_msg(summaryview);
 		if (msginfo == NULL)
 			return;
 
@@ -1109,7 +1149,7 @@ void messageview_save_as(MessageView *messageview)
 	src = procmsg_get_message_file(msginfo);
 	if (copy_file(src, dest, TRUE) < 0) {
 		tmp =  g_path_get_basename(dest);
-		alertpanel_error(_("Can't save the file '%s'."), tmp);
+		alertpanel_error(_("Couldn't save the file '%s'."), tmp);
 		g_free(tmp);
 	}
 	g_free(dest);
@@ -1155,7 +1195,6 @@ void messageview_toggle_view_real(MessageView *messageview)
 	union CompositeWin *cwin = &mainwin->win;
 	GtkWidget *vpaned = NULL;
 	GtkWidget *container = NULL;
-	GtkItemFactory *ifactory = gtk_item_factory_from_widget(mainwin->menubar);
 	
 	switch (mainwin->type) {
 	case SEPARATE_NONE:
@@ -1170,33 +1209,37 @@ void messageview_toggle_view_real(MessageView *messageview)
 	case SEPARATE_BOTH:
 		return;
 	}
-
-	if (vpaned->parent != NULL) {
-		gtk_widget_ref(vpaned);
-		gtkut_container_remove(GTK_CONTAINER(container), vpaned);
-		gtk_widget_reparent(GTK_WIDGET_PTR(messageview), container);
-		menu_set_sensitive(ifactory, "/View/Expand Summary View", FALSE);
-	} else {
-		gtk_widget_reparent(GTK_WIDGET_PTR(messageview), vpaned);
-		gtk_container_add(GTK_CONTAINER(container), vpaned);
-		gtk_widget_unref(vpaned);
-		menu_set_sensitive(ifactory, "/View/Expand Summary View", TRUE);
-		gtk_widget_grab_focus(GTK_WIDGET(mainwin->summaryview->ctree));
-	}
 }
 
 static void return_receipt_show(NoticeView *noticeview, MsgInfo *msginfo)
 {
+	gchar *addr = NULL;
+	gboolean from_me = FALSE;
 	if (msginfo->folder 
 		&& (folder_has_parent_of_type(msginfo->folder, F_QUEUE)
 		 || folder_has_parent_of_type(msginfo->folder, F_DRAFT)))
 		return;
 
-	noticeview_set_text(noticeview, _("This message asks for a return receipt."));
-	noticeview_set_button_text(noticeview, _("Send receipt"));
-	noticeview_set_button_press_callback(noticeview,
-					     G_CALLBACK(return_receipt_send_clicked),
-					     (gpointer) msginfo);
+	addr = g_strdup(msginfo->from);
+	if (addr) {
+		extract_address(addr);
+		if (account_find_from_address(addr)) {
+			from_me = TRUE;
+		}
+		g_free(addr);
+	}
+
+	if (from_me) {
+		noticeview_set_text(noticeview, _("You asked for a return receipt in this message."));
+		noticeview_set_button_text(noticeview, NULL);
+		noticeview_set_button_press_callback(noticeview, NULL, NULL);
+	} else {
+		noticeview_set_text(noticeview, _("This message asks for a return receipt."));
+		noticeview_set_button_text(noticeview, _("Send receipt"));
+		noticeview_set_button_press_callback(noticeview,
+						     G_CALLBACK(return_receipt_send_clicked),
+						     (gpointer) msginfo);
+	}
 	noticeview_show(noticeview);
 }
 
@@ -1232,6 +1275,8 @@ static void partial_recv_show(NoticeView *noticeview, MsgInfo *msginfo)
 	void  *button1_cb = NULL;
 	void  *button2_cb = NULL;
 
+	if (!msginfo->extradata)
+		return;
 	if (!partial_msg_in_uidl_list(msginfo)) {
 		text = g_strdup_printf(_("This message has been partially "
 				"retrieved,\nand has been deleted from the "
@@ -1342,7 +1387,7 @@ static PrefsAccount *select_account_from_list(GList *ac_list)
 				  "Please choose which account do you want to "
 				  "use for sending the receipt notification:"),
 			        _("_Send Notification"), _("+_Cancel"), NULL,
-			        optmenu) != G_ALERTDEFAULT)
+			        FALSE, optmenu) != G_ALERTDEFAULT)
 		return NULL;
 	return account_find_from_id(account_id);
 }
@@ -1365,6 +1410,15 @@ gchar *messageview_get_selection(MessageView *msgview)
 	
 	g_return_val_if_fail(msgview != NULL, NULL);
 
+	if (msgview->mimeview->type == MIMEVIEW_VIEWER) {
+		MimeViewer *viewer = msgview->mimeview->mimeviewer;
+		if (viewer && viewer->get_selection) {
+			text = viewer->get_selection(viewer);
+			if (text)
+				return text;
+		}
+	}
+
 	textview = messageview_get_current_textview(msgview);
 	g_return_val_if_fail(textview != NULL, NULL);
 
@@ -1393,6 +1447,48 @@ static void save_as_cb(gpointer data, guint action, GtkWidget *widget)
 	messageview_save_as(messageview);
 }
 
+#ifdef USE_GNOMEPRINT
+static void print_mimeview(MimeView *mimeview) 
+{
+	if (!mimeview 
+	||  !mimeview->textview
+	||  !mimeview->textview->text)
+		alertpanel_warning(_("Cannot print: the message doesn't "
+				     "contain text."));
+	else {
+		gtk_widget_realize(mimeview->textview->text);
+		gedit_print(GTK_TEXT_VIEW(mimeview->textview->text));
+	}
+}
+
+void messageview_print(MsgInfo *msginfo, gboolean all_headers) 
+{
+	PangoFontDescription *font_desc = NULL;
+	MessageView *tmpview = messageview_create_with_new_window_visible(
+				mainwindow_get_mainwindow(), FALSE);
+
+	if (prefs_common.use_different_print_font) {
+		font_desc = pango_font_description_from_string
+						(prefs_common.printfont);
+	} else {
+		font_desc = pango_font_description_from_string
+						(prefs_common.textfont);
+	}
+	if (font_desc) {
+		gtk_widget_modify_font(tmpview->mimeview->textview->text, 
+			font_desc);
+		pango_font_description_free(font_desc);
+	}
+
+	tmpview->all_headers = all_headers;
+	if (msginfo && messageview_show(tmpview, msginfo, 
+		tmpview->all_headers) >= 0) {
+			print_mimeview(tmpview->mimeview);
+	}
+	messageview_destroy(tmpview);
+}
+#endif
+
 static void print_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MessageView *messageview = (MessageView *)data;
@@ -1418,15 +1514,7 @@ static void print_cb(gpointer data, guint action, GtkWidget *widget)
 	procmsg_print_message(messageview->msginfo, cmdline);
 	g_free(cmdline);
 #else
-	if (!messageview->mimeview 
-	||  !messageview->mimeview->textview
-	||  !messageview->mimeview->textview->text)
-		alertpanel_warning(_("Cannot print: the message doesn't "
-				     "contain text."));
-
-	gedit_print(
-		GTK_TEXT_VIEW(messageview->mimeview
-				->textview->text));
+	messageview_print(messageview->msginfo, messageview->all_headers);
 #endif
 }
 
@@ -1509,6 +1597,43 @@ static void show_all_header_cb(gpointer data, guint action, GtkWidget *widget)
 	main_window_set_menu_sensitive(messageview->mainwin);
 }
 
+#define SET_CHECK_MENU_ACTIVE(path, active) \
+{ \
+	menuitem = gtk_item_factory_get_widget(ifactory, path); \
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), active); \
+}
+
+static void msg_hide_quotes_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	MessageView *messageview = (MessageView *)data;
+	MsgInfo *msginfo = messageview->msginfo;
+	static gboolean updating_menu = FALSE;
+	GtkItemFactory *ifactory = gtk_item_factory_from_widget(messageview->menubar);
+	GtkWidget *menuitem;
+	if (updating_menu)
+		return;
+
+	prefs_common.hide_quotes = 
+			GTK_CHECK_MENU_ITEM(widget)->active ? action : 0;
+	
+	updating_menu=TRUE;
+	SET_CHECK_MENU_ACTIVE("/View/Quotes/Fold all", FALSE);
+	SET_CHECK_MENU_ACTIVE("/View/Quotes/Fold from level 2", FALSE);
+	SET_CHECK_MENU_ACTIVE("/View/Quotes/Fold from level 3", FALSE);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), prefs_common.hide_quotes > 0);	
+	updating_menu=FALSE;
+	if (!msginfo) return;
+	messageview->msginfo = NULL;
+	messageview_show(messageview, msginfo,
+			 messageview->all_headers);
+	procmsg_msginfo_free(msginfo);
+	
+	/* update main window */
+	main_window_set_menu_sensitive(messageview->mainwin);
+	summary_redisplay_msg(messageview->mainwin->summaryview);
+}
+#undef SET_CHECK_MENU_ACTIVE
+
 static void compose_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MessageView *messageview = (MessageView *)data;
@@ -1542,22 +1667,6 @@ static void reply_cb(gpointer data, guint action, GtkWidget *widget)
 	g_slist_free(msginfo_list);
 }
 
-static void reedit_cb(gpointer data, guint action, GtkWidget *widget)
-{
-	MessageView *messageview = (MessageView *)data;
-	MsgInfo *msginfo;
-
-	if (!messageview->msginfo) return;
-	msginfo = messageview->msginfo;
-	if (!msginfo->folder) return;
-	if (!folder_has_parent_of_type(msginfo->folder, F_DRAFT) &&
-	    !folder_has_parent_of_type(msginfo->folder, F_OUTBOX) &&
-	    !folder_has_parent_of_type(msginfo->folder, F_QUEUE)) 
-		return;
-
-	compose_reedit(msginfo);
-}
-
 static void addressbook_open_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	addressbook_open(NULL);
@@ -1626,7 +1735,7 @@ static gboolean messageview_update_msg(gpointer source, gpointer data)
 void messageview_set_menu_sensitive(MessageView *messageview)
 {
 	GtkItemFactory *ifactory;
-	GtkWidget *menuitem;
+	GtkWidget *menuitem = NULL;
 
 	if (!messageview || !messageview->new_window) 
 		return;
@@ -1635,11 +1744,26 @@ void messageview_set_menu_sensitive(MessageView *messageview)
 	ifactory = gtk_item_factory_from_widget(messageview->menubar);
 	if (!ifactory) return;
 	if (messageview->mainwin->type == SEPARATE_MESSAGE) {
-		menuitem = gtk_item_factory_get_widget(ifactory, "/View/Show all headers");
+		menuitem = gtk_item_factory_get_widget(ifactory, "/View/All headers");
 		gtk_check_menu_item_set_active
 			(GTK_CHECK_MENU_ITEM(menuitem),
 			 messageview->mimeview->textview->show_all_headers);
 	}
+	if (prefs_common.hide_quotes) {
+		menuitem = NULL;
+		if (prefs_common.hide_quotes == 1)
+			menuitem = gtk_item_factory_get_widget(ifactory, 
+					"/View/Quotes/Fold all");
+		if (prefs_common.hide_quotes == 2)
+			menuitem = gtk_item_factory_get_widget(ifactory, 
+					"/View/Quotes/Fold from level 2");
+		if (prefs_common.hide_quotes == 3)
+			menuitem = gtk_item_factory_get_widget(ifactory, 
+					"/View/Quotes/Fold from level 3");
+		gtk_check_menu_item_set_active
+			(GTK_CHECK_MENU_ITEM(menuitem),
+			 TRUE);
+	}
 }
 
 void messageview_learn (MessageView *msgview, gboolean is_spam)
