@@ -30,12 +30,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-
-#undef MEASURE_TIME
-
-#ifdef MEASURE_TIME
-#  include <sys/time.h>
-#endif
+#include <time.h>
 
 #include "folder.h"
 #include "mh.h"
@@ -43,6 +38,9 @@
 #include "procheader.h"
 #include "utils.h"
 #include "codeconv.h"
+#include "statusbar.h"
+#include "gtkutils.h"
+#include "timing.h"
 
 /* Define possible missing constants for Windows. */
 #ifdef G_OS_WIN32
@@ -88,6 +86,10 @@ static gint	mh_copy_msgs		(Folder 	*folder,
 static gint     mh_remove_msg		(Folder		*folder,
 					 FolderItem	*item,
 					 gint 		 num);
+static gint 	mh_remove_msgs		(Folder 	*folder, 
+					 FolderItem 	*item, 
+		    			 MsgInfoList 	*msglist, 
+					 GRelation 	*relation);
 static gint     mh_remove_all_msg	(Folder		*folder,
 					 FolderItem	*item);
 static gboolean mh_is_msg_changed	(Folder		*folder,
@@ -126,6 +128,13 @@ static gchar   *mh_item_get_path		(Folder *folder,
 
 static gboolean mh_scan_required	(Folder		*folder,
 					 FolderItem	*item);
+static int mh_item_close		(Folder		*folder,
+					 FolderItem	*item);
+#if 0
+static gint mh_get_flags		(Folder *folder, FolderItem *item,
+                           		 MsgInfoList *msginfo_list, GRelation *msgflags);
+#endif
+static void mh_write_sequences		(FolderItem 	*item, gboolean remove_unseen);
 
 static FolderClass mh_class;
 
@@ -151,7 +160,9 @@ FolderClass *mh_get_class(void)
 		mh_class.remove_folder = mh_remove_folder;
 		mh_class.get_num_list = mh_get_num_list;
 		mh_class.scan_required = mh_scan_required;
-		
+		mh_class.close = mh_item_close;
+		mh_class.get_flags = NULL; /*mh_get_flags */;
+
 		/* Message functions */
 		mh_class.get_msginfo = mh_get_msginfo;
 		mh_class.fetch_msg = mh_fetch_msg;
@@ -160,6 +171,7 @@ FolderClass *mh_get_class(void)
 		mh_class.copy_msg = mh_copy_msg;
 		mh_class.copy_msgs = mh_copy_msgs;
 		mh_class.remove_msg = mh_remove_msg;
+		mh_class.remove_msgs = mh_remove_msgs;
 		mh_class.remove_all_msg = mh_remove_all_msg;
 		mh_class.is_msg_changed = mh_is_msg_changed;
 	}
@@ -294,7 +306,7 @@ gint mh_get_num_list(Folder *folder, FolderItem *item, GSList **list, gboolean *
 	}
 	closedir(dp);
 
-	item->mtime = time(NULL);
+	mh_set_mtime(item);
 	return nummsgs;
 }
 
@@ -324,7 +336,8 @@ static MsgInfo *mh_get_msginfo(Folder *folder, FolderItem *item, gint num)
 	gchar *file;
 
 	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(num > 0, NULL);
+	if (num <= 0)
+		return NULL;
 
 	file = mh_fetch_msg(folder, item, num);
 	if (!file) return NULL;
@@ -421,7 +434,7 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 		g_free(destfile);
 		dest->last_num++;
 	}
-
+	mh_write_sequences(dest, TRUE);
 	return dest->last_num;
 }
 
@@ -446,8 +459,12 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 	gint filemode = 0;
 	FolderItemPrefs *prefs;
 	MsgInfo *msginfo = NULL;
-	gboolean remove_special_headers = FALSE;
 	MsgInfoList *cur = NULL;
+	gint curnum = 0, total = 0;
+	gchar *srcpath = NULL;
+	gboolean full_fetch = FALSE;
+	time_t last_mtime = (time_t)0;
+
 	g_return_val_if_fail(dest != NULL, -1);
 	g_return_val_if_fail(msglist != NULL, -1);
 	
@@ -460,35 +477,54 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 		return -1;
 	}
 
+	if (msginfo->folder->folder != dest->folder)
+		full_fetch = TRUE;
+
 	if (dest->last_num < 0) {
 		mh_get_last_num(folder, dest);
 		if (dest->last_num < 0) return -1;
 	}
 
-	if ((MSG_IS_QUEUED(msginfo->flags) || MSG_IS_DRAFT(msginfo->flags))
-	&& !folder_has_parent_of_type(dest, F_QUEUE)
-	&& !folder_has_parent_of_type(dest, F_DRAFT)) {
-		/* as every msginfo comes from the same folder, it is supposed they
-		 * will either match the preceding condition either all or none.
-		 */
-		remove_special_headers = TRUE;
-	} else if (!(MSG_IS_QUEUED(msginfo->flags) || MSG_IS_DRAFT(msginfo->flags))
-	&& (folder_has_parent_of_type(dest, F_QUEUE)
-	 || folder_has_parent_of_type(dest, F_DRAFT))) {
-		return -1;
-	} 
-
 	prefs = dest->prefs;
 
+	srcpath = folder_item_get_path(msginfo->folder);
+
+	dest_need_scan = mh_scan_required(dest->folder, dest);
+	last_mtime = dest->mtime;
+
+	total = g_slist_length(msglist);
+	if (total > 100) {
+		if (MSG_IS_MOVE(msginfo->flags))
+			statusbar_print_all(_("Moving messages..."));
+		else
+			statusbar_print_all(_("Copying messages..."));
+	}
 	for (cur = msglist; cur; cur = cur->next) {
 		msginfo = (MsgInfo *)cur->data;
-		if (!msginfo)
-			continue;
-		srcfile = procmsg_get_message_file(msginfo);
+		if (!msginfo) {
+			goto err_reset_status;
+		}
+		if (!full_fetch) {
+			srcfile = g_strconcat(srcpath, 
+				G_DIR_SEPARATOR_S, 
+				itos(msginfo->msgnum), NULL);
+		} else {
+			srcfile = procmsg_get_message_file(msginfo);
+		}
+		if (!srcfile) {
+			goto err_reset_status;
+		}
 		destfile = mh_get_new_msg_filename(dest);
 		if (!destfile) {
 			g_free(srcfile);
-			continue;
+			goto err_reset_status;
+		}
+
+		if (total > 100) {
+			statusbar_progress_all(curnum, total, 100);
+			if (curnum % 100 == 0)
+				GTK_EVENTS_FLUSH();
+			curnum++;
 		}
 
 		debug_print("Copying message %s%c%d to %s ...\n",
@@ -496,18 +532,26 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 			    msginfo->msgnum, dest->path);
 
 
-		if (remove_special_headers) {
-			if (procmsg_remove_special_headers(srcfile, destfile) !=0) {
-				g_free(srcfile);
-				g_free(destfile);
-				continue;
+		if (MSG_IS_MOVE(msginfo->flags)) {
+			msginfo->flags.tmp_flags &= ~MSG_MOVE_DONE;
+			if (move_file(srcfile, destfile, TRUE) < 0) {
+				FILE_OP_ERROR(srcfile, "move");
+				if (copy_file(srcfile, destfile, TRUE) < 0) {
+					FILE_OP_ERROR(srcfile, "copy");
+					g_free(srcfile);
+					g_free(destfile);
+					goto err_reset_status;
+				}
+			} else {
+				/* say unlinking's not necessary */
+				msginfo->flags.tmp_flags |= MSG_MOVE_DONE;
 			}
 		} else if (copy_file(srcfile, destfile, TRUE) < 0) {
 			FILE_OP_ERROR(srcfile, "copy");
 			g_free(srcfile);
 			g_free(destfile);
-			continue;
-		}
+			goto err_reset_status;
+		} 
 		if (prefs && prefs->enable_folder_chmod && prefs->folder_chmod) {
 			if (chmod(destfile, prefs->folder_chmod) < 0)
 				FILE_OP_ERROR(destfile, "chmod");
@@ -524,16 +568,32 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 		dest->last_num++;
 	}
 
-	dest_need_scan = mh_scan_required(dest->folder, dest);
-	if (!dest_need_scan)
-		dest->mtime = time(NULL);
+	g_free(srcpath);
+	mh_write_sequences(dest, TRUE);
 
+	if (dest->mtime == last_mtime && !dest_need_scan) {
+		mh_set_mtime(dest);
+	}
+	if (total > 100) {
+		statusbar_progress_all(0,0,0);
+		statusbar_pop_all();
+	}
 	return dest->last_num;
+err_reset_status:
+	g_free(srcpath);
+	mh_write_sequences(dest, TRUE);
+	if (total > 100) {
+		statusbar_progress_all(0,0,0);
+		statusbar_pop_all();
+	}
+	return -1;
+
 }
 
 static gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 {
 	gboolean need_scan = FALSE;
+	time_t last_mtime = (time_t)0;
 	gchar *file;
 
 	g_return_val_if_fail(item != NULL, -1);
@@ -542,6 +602,7 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 	g_return_val_if_fail(file != NULL, -1);
 
 	need_scan = mh_scan_required(folder, item);
+	last_mtime = item->mtime;
 
 	if (g_unlink(file) < 0) {
 		FILE_OP_ERROR(file, "unlink");
@@ -549,13 +610,73 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 		return -1;
 	}
 
-	if (!need_scan)
-		item->mtime = time(NULL);
-
+	if (item->mtime == last_mtime && !need_scan) {
+		mh_set_mtime(item);
+	}
 	g_free(file);
 	return 0;
 }
 
+static gint mh_remove_msgs(Folder *folder, FolderItem *item, 
+		    MsgInfoList *msglist, GRelation *relation)
+{
+	gboolean need_scan = FALSE;
+	gchar *path, *file;
+	time_t last_mtime = (time_t)0;
+	MsgInfoList *cur;
+	gint total = 0, curnum = 0;
+
+	g_return_val_if_fail(item != NULL, -1);
+
+	path = folder_item_get_path(item);
+	
+	need_scan = mh_scan_required(folder, item);
+	last_mtime = item->mtime;
+
+	total = g_slist_length(msglist);
+	if (total > 100) {
+		statusbar_print_all(_("Deleting messages..."));
+	}
+
+	for (cur = msglist; cur; cur = cur->next) {
+		MsgInfo *msginfo = (MsgInfo *)cur->data;
+		if (msginfo == NULL)
+			continue;
+		if (MSG_IS_MOVE(msginfo->flags) && MSG_IS_MOVE_DONE(msginfo->flags)) {
+			msginfo->flags.tmp_flags &= ~MSG_MOVE_DONE;
+			continue;
+		}
+		if (total > 100) {
+			statusbar_progress_all(curnum, total, 100);
+			if (curnum % 100 == 0)
+				GTK_EVENTS_FLUSH();
+			curnum++;
+		}
+
+		file = g_strconcat(path, G_DIR_SEPARATOR_S, itos(msginfo->msgnum), NULL);
+		if (file == NULL)
+			continue;
+		
+		if (g_unlink(file) < 0) {
+			g_free(file);
+			continue;
+		}
+		
+		g_free(file);
+	}
+
+	if (total > 100) {
+		statusbar_progress_all(0,0,0);
+		statusbar_pop_all();
+	}
+	if (item->mtime == last_mtime && !need_scan) {
+		mh_set_mtime(item);
+	}
+
+	g_free(path);
+	return 0;
+}
+
 static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 {
 	gchar *path;
@@ -568,6 +689,8 @@ static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 	val = remove_all_numbered_files(path);
 	g_free(path);
 
+	mh_write_sequences(item, TRUE);
+
 	return val;
 }
 
@@ -947,10 +1070,13 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			node = item->node;
 			for (node = node->children; node != NULL; node = node->next) {
 				FolderItem *cur_item = FOLDER_ITEM(node->data);
-				if (!strcmp2(cur_item->path, entry)) {
+				gchar *curpath = mh_filename_from_utf8(cur_item->path);
+				if (!strcmp2(curpath, entry)) {
 					new_item = cur_item;
+					g_free(curpath);
 					break;
 				}
+				g_free(curpath);
 			}
 			if (!new_item) {
 				debug_print("new folder '%s' found.\n", entry);
@@ -996,23 +1122,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	closedir(dp);
 #endif
 
-	item->mtime = time(NULL);
-
-/*
-	if (item->path) {
-		gint new, unread, total, min, max;
-
-		procmsg_get_mark_sum(item->path, &new, &unread, &total,
-				     &min, &max, 0);
-		if (n_msg > total) {
-			new += n_msg - total;
-			unread += n_msg - total;
-		}
-		item->new = new;
-		item->unread = unread;
-		item->total = n_msg;
-	}
-*/
+	mh_set_mtime(item);
 }
 
 static gboolean mh_rename_folder_func(GNode *node, gpointer data)
@@ -1066,3 +1176,260 @@ static gchar *mh_filename_to_utf8(const gchar *path)
 
 	return utf8path;
 }
+
+static gint sort_cache_list_by_msgnum(gconstpointer a, gconstpointer b)
+{
+	MsgInfo *msginfo_a = (MsgInfo *) a;
+	MsgInfo *msginfo_b = (MsgInfo *) b;
+
+	return (msginfo_a->msgnum - msginfo_b->msgnum);
+}
+
+static gchar *get_unseen_seq_name(void)
+{
+	static gchar *seq_name = NULL;
+	if (!seq_name) {
+		gchar buf[BUFFSIZE];
+		gchar *tmp;
+		gchar *profile_path = g_strconcat(
+			get_home_dir(), G_DIR_SEPARATOR_S,
+			".mh_profile", NULL);
+		FILE *fp = g_fopen(profile_path, "r");
+		if (fp) {
+			while (fgets(buf, sizeof(buf), fp) != NULL) {
+				if (!strncmp(buf, "Unseen-Sequence:", strlen("Unseen-Sequence:"))) {
+					gchar *seq_tmp = buf+strlen("Unseen-Sequence:");
+					while (*seq_tmp == ' ')
+						seq_tmp++;
+					seq_name = g_strdup(seq_tmp);
+					seq_name = strretchomp(seq_name);
+					break;
+				}
+			}
+			fclose(fp);
+		}
+		if (!seq_name)
+			seq_name = g_strdup("unseen");
+		tmp = g_strdup_printf("%s:", seq_name);
+		g_free(seq_name);
+		seq_name = tmp;
+	}
+	return seq_name;	
+}
+
+#if 0
+static gint mh_get_flags(Folder *folder, FolderItem *item,
+                           MsgInfoList *msginfo_list, GRelation *msgflags)
+{
+	gchar *mh_sequences_filename;
+	FILE *mh_sequences_file;
+	gchar buf[BUFFSIZE];
+	gchar *unseen_list = NULL;
+	gchar *path;
+	MsgInfoList *mcur = NULL;
+/*
+	GTimer *timer = g_timer_new();
+	g_timer_start(timer);
+*/
+	if (!item)
+		return 0;
+
+	/* don't update from .mh_sequences if the item's opened: mails may have
+	 * been marked read/unread and it's not yet written in the file. */	
+	if (item->opened)
+		return 0;
+
+	path = folder_item_get_path(item);
+
+	mh_sequences_filename = g_strconcat(path, G_DIR_SEPARATOR_S,
+					    ".mh_sequences", NULL);
+	g_free(path);
+	if ((mh_sequences_file = g_fopen(mh_sequences_filename, "r+b")) != NULL) {
+		while (fgets(buf, sizeof(buf), mh_sequences_file) != NULL) {
+			if (!strncmp(buf, get_unseen_seq_name(), strlen(get_unseen_seq_name()))) {
+				unseen_list = g_strdup(buf+strlen(get_unseen_seq_name()));
+				break;
+			}
+		}
+		fclose(mh_sequences_file);
+	}
+	
+	g_free(mh_sequences_filename);
+	
+	if (unseen_list) {
+		gchar *cur = NULL;
+		gchar *token = NULL, *next = NULL, *boundary = NULL;
+		gint num = 0;
+		GHashTable *unseen_table = g_hash_table_new(g_direct_hash, g_direct_equal);
+
+		cur = unseen_list = strretchomp(unseen_list);
+		debug_print("found unseen list in .mh_sequences: %s\n", unseen_list);
+next_token:
+		while (*cur && *cur == ' ')
+			cur++;
+		
+		if ((next = strchr(cur, ' ')) != NULL) {
+			token = cur;
+			cur = next+1;
+			*next = '\0';
+		} else {
+			token = cur;
+			cur = NULL;
+		}
+		
+		if ((boundary = strchr(token, '-')) != NULL) {
+			gchar *start, *end;
+			int i;
+			start = token;
+			end = boundary+1;
+			*boundary='\0';
+			for (i = atoi(start); i <= atoi(end); i++) {
+				g_hash_table_insert(unseen_table, GINT_TO_POINTER(i), GINT_TO_POINTER(1));
+			}
+		} else if ((num = atoi(token)) > 0) {
+			g_hash_table_insert(unseen_table, GINT_TO_POINTER(num), GINT_TO_POINTER(1));
+		}
+		
+		if (cur)
+			goto next_token;
+		for (mcur = msginfo_list; mcur; mcur = mcur->next) {
+			MsgInfo *msginfo = (MsgInfo *)mcur->data;
+			MsgPermFlags flags = msginfo->flags.perm_flags;
+			if (g_hash_table_lookup(unseen_table, GINT_TO_POINTER(msginfo->msgnum))) {
+				flags |= MSG_UNREAD;
+			} else if (!(flags & MSG_NEW)) { /* don't mark new msgs as read */
+				flags &= ~(MSG_UNREAD);
+			}
+			if (flags != msginfo->flags.perm_flags)
+				g_relation_insert(msgflags, msginfo, GINT_TO_POINTER(flags));
+		}
+		g_hash_table_destroy(unseen_table);
+		g_free(unseen_list);
+	}
+/*
+	g_timer_stop(timer);
+	printf("mh_get_flags: %f secs\n", g_timer_elapsed(timer, NULL));
+	g_timer_destroy(timer);
+*/
+	return 0;
+}
+#endif
+
+static void mh_write_sequences(FolderItem *item, gboolean remove_unseen)
+{
+	gchar *mh_sequences_old, *mh_sequences_new;
+	FILE *mh_sequences_old_fp, *mh_sequences_new_fp;
+	gchar buf[BUFFSIZE];
+	gchar *path = NULL;
+	START_TIMING("");
+
+	if (!item)
+		return;
+	
+	path = folder_item_get_path(item);
+
+	mh_sequences_old = g_strconcat(path, G_DIR_SEPARATOR_S,
+					    ".mh_sequences", NULL);
+	mh_sequences_new = g_strconcat(path, G_DIR_SEPARATOR_S,
+					    ".mh_sequences.new", NULL);
+	if ((mh_sequences_new_fp = g_fopen(mh_sequences_new, "w+b")) != NULL) {
+		GSList *msglist = folder_item_get_msg_list(item);
+		GSList *cur;
+		MsgInfo *info = NULL;
+		gint start = -1, end = -1;
+		gchar *sequence = g_strdup("");
+		gint seq_len = 0;
+		msglist = g_slist_sort(msglist, sort_cache_list_by_msgnum);
+		cur = msglist;
+		
+		/* write the unseen sequence if we don't have to scrap it */
+		if (!remove_unseen) do {
+			info = (MsgInfo *)(cur ? cur->data:NULL);
+			if (info && (MSG_IS_UNREAD(info->flags) || MSG_IS_NEW(info->flags))) {
+				if (start < 0)
+					start = end = info->msgnum;
+				else
+					end = info->msgnum;
+			} else {
+				if (start > 0 && end > 0) {
+					gchar tmp[32];
+					gint tmp_len = 0;
+					if (start != end)
+						snprintf(tmp, 31, " %d-%d", start, end);
+					else
+						snprintf(tmp, 31, " %d", start);
+					
+					tmp_len = strlen(tmp);
+					sequence = g_realloc(sequence, seq_len+tmp_len+1);
+					strcpy(sequence+seq_len, tmp);
+					seq_len += tmp_len;
+
+					start = end = -1;
+				}
+			}
+			cur = cur ? cur->next:NULL;
+		} while (cur || (start > 0 && end > 0));
+		if (sequence && *sequence) {
+			fprintf(mh_sequences_new_fp, "%s%s\n", 
+					get_unseen_seq_name(), sequence);
+			debug_print("wrote unseen sequence: '%s%s'\n", 
+					get_unseen_seq_name(), sequence);
+		}
+		/* rewrite the rest of the file */
+		if ((mh_sequences_old_fp = g_fopen(mh_sequences_old, "r+b")) != NULL) {
+			while (fgets(buf, sizeof(buf), mh_sequences_old_fp) != NULL) {
+				if (strncmp(buf, get_unseen_seq_name(), strlen(get_unseen_seq_name())))
+					fprintf(mh_sequences_new_fp, "%s", buf);
+			}
+			fclose(mh_sequences_old_fp);
+		}
+		
+		fflush(mh_sequences_new_fp);
+#if 0
+		fsync(fileno(mh_sequences_new_fp));
+#endif
+		fclose(mh_sequences_new_fp);
+
+		g_rename(mh_sequences_new, mh_sequences_old);
+		g_free(sequence);
+		procmsg_msg_list_free(msglist);
+	}
+	g_free(mh_sequences_old);
+	g_free(mh_sequences_new);
+	g_free(path);
+
+	END_TIMING();
+}
+
+static int mh_item_close(Folder *folder, FolderItem *item)
+{
+	time_t last_mtime = (time_t)0;
+	gboolean need_scan = mh_scan_required(item->folder, item);
+	last_mtime = item->mtime;
+
+	mh_write_sequences(item, FALSE);
+
+	if (item->mtime == last_mtime && !need_scan) {
+		mh_set_mtime(item);
+	}
+
+	return 0;
+}
+
+void mh_set_mtime(FolderItem *item)
+{
+	struct stat s;
+	gchar *path = folder_item_get_path(item);
+
+	g_return_if_fail(path != NULL);
+
+	if (stat(path, &s) < 0) {
+		FILE_OP_ERROR(path, "stat");
+		g_free(path);
+		return;
+	}
+
+	item->mtime = s.st_mtime;
+	debug_print("MH: forced mtime of %s to %ld\n", item->name, item->mtime);
+	g_free(path);
+}
