@@ -135,7 +135,7 @@ static gchar *news_folder_get_path	 (Folder	*folder);
 gchar *news_item_get_path		 (Folder	*folder,
 					  FolderItem	*item);
 static void news_synchronise		 (FolderItem	*item);
-static int news_dummy_remove		 (Folder 	*folder, 
+static int news_remove_msg		 (Folder 	*folder, 
 					  FolderItem 	*item, 
 					  gint 		 msgnum);
 static gint news_remove_folder		 (Folder	*folder,
@@ -164,17 +164,29 @@ FolderClass *news_get_class(void)
 		news_class.get_msginfos = news_get_msginfos;
 		news_class.fetch_msg = news_fetch_msg;
 		news_class.synchronise = news_synchronise;
-		news_class.remove_msg = news_dummy_remove;
+		news_class.remove_msg = news_remove_msg;
 	};
 
 	return &news_class;
 }
 
-static int news_dummy_remove		 (Folder 	*folder, 
+static int news_remove_msg		 (Folder 	*folder, 
 					  FolderItem 	*item, 
 					  gint 		 msgnum)
 {
-	debug_print("doing nothing on purpose\n");
+	gchar *path, *filename;
+
+	g_return_val_if_fail(folder != NULL, -1);
+	g_return_val_if_fail(item != NULL, -1);
+
+	path = folder_item_get_path(item);
+	if (!is_dir_exist(path))
+		make_dir_hier(path);
+	
+	filename = g_strconcat(path, G_DIR_SEPARATOR_S, itos(msgnum), NULL);
+	g_free(path);
+	g_unlink(filename);
+	g_free(filename);
 	return 0;
 }
 
@@ -287,10 +299,9 @@ static Session *news_session_new_for_folder(Folder *folder)
 			  "Do you want to continue connecting to this "
 			  "server? The communication would not be "
 			  "secure."),
-			  _("Con_tinue connecting"), 
-			  GTK_STOCK_CANCEL, NULL,
-			  FALSE, NULL, ALERT_WARNING,
-			  G_ALERTALTERNATE) != G_ALERTDEFAULT)
+			  GTK_STOCK_CANCEL, _("Con_tinue connecting"), 
+			  NULL, FALSE, NULL, ALERT_WARNING,
+			  G_ALERTDEFAULT) != G_ALERTALTERNATE)
 			return NULL;
 	}
 	port = ac->set_nntpport ? ac->nntpport : NNTP_PORT;
@@ -574,7 +585,7 @@ gint news_post_stream(Folder *folder, FILE *fp)
 
 	ok = nntp_post(session, fp);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't post article.\n"));
+		log_warning(_("couldn't post article.\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(folder)->session = NULL;
@@ -600,7 +611,7 @@ static gint news_get_article_cmd(NNTPSession *session, const gchar *cmd,
 
 	ok = recv_write_to_file(SESSION(session)->sock, filename);
 	if (ok < 0) {
-		log_warning(_("can't retrieve article %d\n"), num);
+		log_warning(_("couldn't retrieve article %d\n"), num);
 		if (ok == -2)
 			return NN_SOCKET;
 		else
@@ -649,7 +660,7 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 	if (ok == NN_SUCCESS)
 		session->group = g_strdup(group);
 	else
-		log_warning(_("can't select group: %s\n"), group);
+		log_warning(_("couldn't select group: %s\n"), group);
 
 	return ok;
 }
@@ -709,6 +720,10 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 
         msginfo->subject = conv_unmime_header(subject, NULL);
 
+	remove_return(msginfo->from);
+	remove_return(msginfo->fromname);
+	remove_return(msginfo->subject);
+
         if (msgid) {
                 extract_parenthesis(msgid, '<', '>');
                 remove_space(msgid);
@@ -741,15 +756,6 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
                 }
         } 
 
-	/*
-	msginfo->xref = g_strdup(xref);
-	p = msginfo->xref+strlen(msginfo->xref) - 1;
-	while (*p == '\r' || *p == '\n') {
-		*p = '\0';
-		p--;
-	}
-	*/
-
 	return msginfo;
 }
 
@@ -879,7 +885,7 @@ gint news_get_num_list(Folder *folder, FolderItem *item, GSList **msgnum_list, g
 
 	ok = news_select_group(session, item->path, &num, &first, &last);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't set group: %s\n"), item->path);
+		log_warning(_("couldn't set group: %s\n"), item->path);
 		news_folder_unlock(NEWS_FOLDER(item->folder));
 		return -1;
 	}
@@ -913,6 +919,26 @@ gint news_get_num_list(Folder *folder, FolderItem *item, GSList **msgnum_list, g
 		} \
 	}
 
+static void news_set_msg_flags(FolderItem *item, MsgInfo *msginfo)
+{
+	msginfo->flags.tmp_flags = 0;
+	if (item->folder->account->mark_crosspost_read && msginfo->msgid) {
+		if (item->folder->newsart &&
+		    g_hash_table_lookup(item->folder->newsart, msginfo->msgid) != NULL) {
+			msginfo->flags.perm_flags = MSG_COLORLABEL_TO_FLAGS(item->folder->account->crosspost_col);
+				
+		} else {
+			if (!item->folder->newsart) 
+				item->folder->newsart = g_hash_table_new(g_str_hash, g_str_equal);
+			g_hash_table_insert(item->folder->newsart,
+					g_strdup(msginfo->msgid), GINT_TO_POINTER(1));
+			msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
+		}
+	} else {
+		msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
+	}
+}
+
 MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 {
 	NNTPSession *session;
@@ -933,7 +959,7 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 	news_folder_lock(NEWS_FOLDER(item->folder));
 	
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't get xover\n"));
+		log_warning(_("couldn't get xover\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
@@ -961,13 +987,14 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 	}
 
 	msginfo->folder = item;
-	msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
-	msginfo->flags.tmp_flags = MSG_NEWS;
+	
+	news_set_msg_flags(item, msginfo);
+	msginfo->flags.tmp_flags |= MSG_NEWS;
 	msginfo->newsgroups = g_strdup(item->path);
 
 	ok = nntp_xhdr(session, "to", num, num);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't get xhdr\n"));
+		log_warning(_("couldn't get xhdr\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
@@ -988,7 +1015,7 @@ MsgInfo *news_get_msginfo(Folder *folder, FolderItem *item, gint num)
 
 	ok = nntp_xhdr(session, "cc", num, num);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't get xhdr\n"));
+		log_warning(_("couldn't get xhdr\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
@@ -1016,7 +1043,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 	GSList *newlist = NULL;
 	GSList *llast = NULL;
 	MsgInfo *msginfo;
-	guint count = 0, lines = (end - begin + 2) * 3;
+	guint count = 0, lines = (end - begin) + 2;
 	gint ok;
 
 	g_return_val_if_fail(session != NULL, NULL);
@@ -1026,7 +1053,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 		    begin, end, item->path);
 	ok = nntp_xover(session, begin, end);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't get xover\n"));
+		log_warning(_("couldn't get xover\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
@@ -1057,8 +1084,8 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 		}
 
 		msginfo->folder = item;
-		msginfo->flags.perm_flags = MSG_NEW|MSG_UNREAD;
-		msginfo->flags.tmp_flags = MSG_NEWS;
+		news_set_msg_flags(item, msginfo);
+		msginfo->flags.tmp_flags |= MSG_NEWS;
 		msginfo->newsgroups = g_strdup(item->path);
 
 		if (!newlist)
@@ -1071,7 +1098,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 
 	ok = nntp_xhdr(session, "to", begin, end);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't get xhdr\n"));
+		log_warning(_("couldn't get xhdr\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
@@ -1088,11 +1115,6 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 			news_folder_unlock(NEWS_FOLDER(item->folder));
 			return newlist;
 		}
-		count++;
-		progressindicator_set_percentage
-			(PROGRESS_TYPE_NETWORK,
-			 session->fetch_base_percentage +
-			 (((gfloat) count) / ((gfloat) lines)) * session->fetch_total_percentage);
 
 		if (buf[0] == '.' && buf[1] == '\r') break;
 		if (!llast) {
@@ -1108,7 +1130,7 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 
 	ok = nntp_xhdr(session, "cc", begin, end);
 	if (ok != NN_SUCCESS) {
-		log_warning(_("can't get xhdr\n"));
+		log_warning(_("couldn't get xhdr\n"));
 		if (ok == NN_SOCKET) {
 			session_destroy(SESSION(session));
 			REMOTE_FOLDER(item->folder)->session = NULL;
@@ -1125,11 +1147,6 @@ static GSList *news_get_msginfos_for_range(NNTPSession *session, FolderItem *ite
 			news_folder_unlock(NEWS_FOLDER(item->folder));
 			return newlist;
 		}
-		count++;
-		progressindicator_set_percentage
-			(PROGRESS_TYPE_NETWORK,
-			 session->fetch_base_percentage +
-			 (((gfloat) count) / ((gfloat) lines)) * session->fetch_total_percentage);
 
 		if (buf[0] == '.' && buf[1] == '\r') break;
 		if (!llast) {
