@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto & the Sylpheed-Claws Team
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto & the Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,13 +14,18 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef NEWS_GTK_H
 #define NEWS_GTK_H
 
-void news_gtk_init(void);
-gboolean news_gtk_should_override(void);
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include "folder.h"
+
+void news_gtk_init(void);
+void news_gtk_synchronise(FolderItem *item);
 #endif /* NEWS_GTK_H */
