@@ -141,7 +141,8 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 				    clamav_save_folder = folder_get_default_trash();
 
 			procmsg_msginfo_unset_flags(msginfo, ~0, 0);
-			folder_item_move_msg(clamav_save_folder, msginfo);
+			msginfo->is_move = TRUE;
+			msginfo->to_filter_folder = clamav_save_folder;
 		} else {
 			folder_item_remove_msg(msginfo->folder, msginfo->msgnum);
 		}
@@ -194,18 +195,18 @@ gint plugin_init(gchar **error)
 	unsigned int no;
 
 	if ((sylpheed_get_version() > VERSION_NUMERIC)) {
-		*error = g_strdup("Your version of Sylpheed-Claws is newer than the version the ClamAV plugin was built with");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is newer than the version the ClamAV plugin was built with"));
 		return -1;
 	}
 
 	if ((sylpheed_get_version() < MAKE_NUMERIC_VERSION(0, 9, 3, 86))) {
-		*error = g_strdup("Your version of Sylpheed-Claws is too old for the ClamAV plugin");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is too old for the ClamAV plugin"));
 		return -1;
 	}
 
 	hook_id = hooks_register_hook(MAIL_FILTERING_HOOKLIST, mail_filtering_hook, NULL);
 	if (hook_id == -1) {
-		*error = g_strdup("Failed to register mail filtering hook");
+		*error = g_strdup(_("Failed to register mail filtering hook"));
 		return -1;
 	}
 
@@ -276,3 +277,11 @@ const gchar *plugin_version(void)
 {
 	return VERSION;
 }
+
+struct PluginFeature *plugin_provides(void)
+{
+	static struct PluginFeature features[] = 
+		{ {PLUGIN_FILTERING, N_("Virus detection")},
+		  {PLUGIN_NOTHING, NULL}};
+	return features;
+}
