@@ -141,6 +141,7 @@ static MimeViewer *dillo_viewer_create(void)
 	viewer->mimeviewer.show_mimepart = dillo_show_mimepart;
 	viewer->mimeviewer.clear_viewer = dillo_clear_viewer;
 	viewer->mimeviewer.destroy_viewer = dillo_destroy_viewer;	
+	viewer->mimeviewer.get_selection = NULL;
 	viewer->widget = gtk_event_box_new();
 
 	gtk_widget_show(viewer->widget);
@@ -159,18 +160,18 @@ static MimeViewerFactory dillo_viewer_factory =
 	content_types,	
 	0,
 
-	dillo_viewer_create
+	dillo_viewer_create,
 };
 
 gint plugin_init(gchar **error)
 {
 	if ((sylpheed_get_version() > VERSION_NUMERIC)) {
-		*error = g_strdup("Your version of Sylpheed-Claws is newer than the version the Dillo plugin was built with");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is newer than the version the Dillo plugin was built with"));
 		return -1;
 	}
 
 	if ((sylpheed_get_version() < MAKE_NUMERIC_VERSION(0, 9, 3, 86))) {
-		*error = g_strdup("Your version of Sylpheed-Claws is too old for the Dillo plugin");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is too old for the Dillo plugin"));
 		return -1;
 	}
 
@@ -215,3 +216,11 @@ const gchar *plugin_version(void)
 {
 	return VERSION;
 }
+
+struct PluginFeature *plugin_provides(void)
+{
+	static struct PluginFeature features[] = 
+		{ {PLUGIN_MIMEVIEWER, N_("text/html")},
+		  {PLUGIN_NOTHING, NULL}};
+	return features;
+}
