@@ -29,27 +29,31 @@
 #include "common/sylpheed.h"
 #include "sgpgme.h"
 #include "prefs_gpg.h"
+#include "pgp_viewer.h"
+#include "plugin.h"
 
 gint plugin_init(gchar **error)
 {
 	if ((sylpheed_get_version() > VERSION_NUMERIC)) {
-		*error = g_strdup("Your version of Sylpheed-Claws is newer than the version the PGP/Core plugin was built with");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is newer than the version the PGP/Core plugin was built with"));
 		return -1;
 	}
 
 	if ((sylpheed_get_version() < MAKE_NUMERIC_VERSION(2, 1, 0, 1))) {
-		*error = g_strdup("Your version of Sylpheed-Claws is too old for the PGP/Core plugin");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is too old for the PGP/Core plugin"));
 		return -1;
 	}
 
 	sgpgme_init();
 	prefs_gpg_init();
-
+	sgpgme_check_create_key();
+	pgp_viewer_init();
 	return 0;	
 }
 
 void plugin_done(void)
 {
+	pgp_viewer_done();
 	prefs_gpg_done();
 	sgpgme_done();
 }
@@ -87,3 +91,10 @@ const gchar *plugin_version(void)
 	return VERSION;
 }
 
+struct PluginFeature *plugin_provides(void)
+{
+	static struct PluginFeature features[] = 
+		{ {PLUGIN_PRIVACY, N_("Core operations")},
+		  {PLUGIN_NOTHING, NULL}};
+	return features;
+}
