@@ -26,6 +26,7 @@
 #include "prefs.h"
 #include "prefs_gtk.h"
 #include "prefs_gpg.h"
+#include "sgpgme.h"
 
 struct GPGConfig prefs_gpg;
 
@@ -43,6 +44,8 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"gpg_warning", "TRUE", &prefs_gpg.gpg_warning, P_BOOL,
 	 NULL, NULL, NULL},
+	{"gpg_ask_create_key", "TRUE", &prefs_gpg.gpg_ask_create_key, P_BOOL,
+	 NULL, NULL, NULL},
 
 	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
 };
@@ -193,6 +196,9 @@ struct GPGAccountPage
 	GtkWidget *key_custom;
 	GtkWidget *keyid;
 	GtkWidget *keyid_label;
+	GtkWidget *new_key_label;
+	GtkWidget *new_key_btn;
+	GtkWidget *new_key_box;
 
 	PrefsAccount *account;
 };
@@ -209,6 +215,34 @@ void key_custom_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 		gtk_editable_delete_text(GTK_EDITABLE(page->keyid), 0, -1);
 }
 
+static void prefs_gpg_update_sens(struct GPGAccountPage *page)
+{
+	gboolean active;
+	active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->key_custom));
+	if (sgpgme_has_secret_key()) {
+		gtk_widget_hide(page->new_key_box);
+		gtk_widget_set_sensitive(page->key_default, TRUE);
+		gtk_widget_set_sensitive(page->key_by_from, TRUE);
+		gtk_widget_set_sensitive(page->key_custom, TRUE);
+		gtk_widget_set_sensitive(page->keyid, active);
+		gtk_widget_set_sensitive(page->keyid_label, active);
+	} else {
+		gtk_widget_show(page->new_key_box);
+		gtk_widget_set_sensitive(page->key_default, FALSE);
+		gtk_widget_set_sensitive(page->key_by_from, FALSE);
+		gtk_widget_set_sensitive(page->key_custom, FALSE);
+		gtk_widget_set_sensitive(page->keyid, FALSE);
+		gtk_widget_set_sensitive(page->keyid_label, FALSE);
+	}
+}
+
+static void new_key_clicked(GtkWidget *widget, gpointer user_data)
+{
+	struct GPGAccountPage *page = (struct GPGAccountPage *) user_data;
+	sgpgme_create_secret_key(page->account, FALSE);
+	prefs_gpg_update_sens(page);
+}
+
 static void prefs_gpg_account_create_widget_func(PrefsPage *_page,
 						 GtkWindow *window,
 						 gpointer data)
@@ -227,6 +261,10 @@ static void prefs_gpg_account_create_widget_func(PrefsPage *_page,
 	GtkWidget *key_custom;
 	GtkWidget *keyid_label;
 	GtkWidget *keyid;
+	GtkWidget *image;
+	GtkWidget *new_key_label;
+	GtkWidget *new_key_btn;
+	GtkWidget *new_key_box;
 
 	vbox = gtk_vbox_new(FALSE, VSPACING);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), VBOX_BORDER);
@@ -305,19 +343,42 @@ static void prefs_gpg_account_create_widget_func(PrefsPage *_page,
 		break;
 	}
 
+	hbox = gtk_hbox_new (FALSE, 5);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+
+	new_key_box = gtk_hbox_new(FALSE, 6);
+	gtk_widget_show(new_key_box);
+	gtk_box_pack_start(GTK_BOX(hbox), new_key_box, FALSE, FALSE, 0);
+
+	image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_WARNING,
+			GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+	gtk_box_pack_start(GTK_BOX(new_key_box), image, FALSE, FALSE, 0);
+	new_key_label = gtk_label_new(
+			_("No secret key found."));
+	gtk_box_pack_start(GTK_BOX(new_key_box), new_key_label, FALSE, FALSE, 0);
+
+	new_key_btn = gtk_button_new_with_label(_("Generate a new key pair"));
+	gtk_widget_show(new_key_btn);
+	gtk_box_pack_start(GTK_BOX(hbox), new_key_btn, FALSE, FALSE, 0);
+
 	if (config->sign_key_id != NULL)
 		gtk_entry_set_text(GTK_ENTRY(keyid), config->sign_key_id);
 
 	g_signal_connect(G_OBJECT(key_custom), "toggled", G_CALLBACK(key_custom_toggled), page);
+	g_signal_connect(G_OBJECT(new_key_btn), "clicked", G_CALLBACK(new_key_clicked), page);
 
 	page->key_default = key_default;
 	page->key_by_from = key_by_from;
 	page->key_custom = key_custom;
 	page->keyid = keyid;
 	page->keyid_label = keyid_label;
+	page->new_key_box = new_key_box;
 
 	page->page.widget = vbox;
 	page->account = account;
+	prefs_gpg_update_sens(page);
 }
 
 static void prefs_gpg_account_destroy_widget_func(PrefsPage *_page)
