@@ -30,6 +30,14 @@
 #include <glib/gi18n.h>
 #include <stdio.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#if (defined(__DragonFly__) || defined (__NetBSD__) || defined (__FreeBSD__) || defined (__OpenBSD__))
+#  include <sys/signal.h>
+#endif
+#ifndef G_OS_WIN32
+#include <sys/mman.h>
+#endif
 #if HAVE_LOCALE_H
 #  include <locale.h>
 #endif
@@ -41,6 +49,7 @@
 #include "alertpanel.h"
 #include "passphrase.h"
 #include "prefs_gpg.h"
+#include "account.h"
 #include "select-keys.h"
 
 static void sgpgme_disable_all(void)
@@ -57,6 +66,7 @@ gpgme_verify_result_t sgpgme_verify_signature(gpgme_ctx_t ctx, gpgme_data_t sig,
 
 	if ((err = gpgme_op_verify(ctx, sig, plain, dummy)) != GPG_ERR_NO_ERROR) {
 		debug_print("op_verify err %s\n", gpgme_strerror(err));
+		privacy_set_error(gpgme_strerror(err));
 		return GINT_TO_POINTER(-GPG_ERR_SYSTEM_ERROR);
 		
 	}
@@ -163,7 +173,7 @@ gchar *sgpgme_sigstat_info_short(gpgme_ctx_t ctx, gpgme_verify_result_t status)
 	static gboolean warned = FALSE;
 
 	if (GPOINTER_TO_INT(status) == -GPG_ERR_SYSTEM_ERROR) {
-		return g_strdup(_("The signature can't be checked - GPG error."));
+		return g_strdup_printf(_("The signature can't be checked - %s"), privacy_get_error());
 	}
 
 	if (status == NULL) {
@@ -331,10 +341,8 @@ gpgme_data_t sgpgme_data_from_mimeinfo(MimeInfo *mimeinfo)
 	tmp_file = get_tmp_file();
 	copy_file_part(fp, mimeinfo->offset, mimeinfo->length, tmp_file);
 	fclose(fp);
-	fp = g_fopen(tmp_file, "rb");
+	fp = NULL;
 	debug_print("tmp file %s\n", tmp_file);
-	if (!fp) 
-		return NULL;
 	
 	err = gpgme_data_new_from_file(&data, tmp_file, 1);
 	g_unlink(tmp_file);
@@ -344,6 +352,7 @@ gpgme_data_t sgpgme_data_from_mimeinfo(MimeInfo *mimeinfo)
 	if (err) {
 		debug_print ("gpgme_data_new_from_file failed: %s\n",
 			     gpgme_strerror (err));
+		privacy_set_error(_("Couldn't get data from message, %s"), gpgme_strerror(err));
 		return NULL;
 	}
 	return data;
@@ -357,8 +366,9 @@ gpgme_data_t sgpgme_decrypt_verify(gpgme_data_t cipher, gpgme_verify_result_t *s
 
 	memset (&info, 0, sizeof info);
 	
-	if (gpgme_data_new(&plain) != GPG_ERR_NO_ERROR) {
+	if ((err = gpgme_data_new(&plain)) != GPG_ERR_NO_ERROR) {
 		gpgme_release(ctx);
+		privacy_set_error(_("Couldn't initialize data, %s"), gpgme_strerror(err));
 		return NULL;
 	}
 	
@@ -377,6 +387,7 @@ gpgme_data_t sgpgme_decrypt_verify(gpgme_data_t cipher, gpgme_verify_result_t *s
 		err = gpgme_op_decrypt_verify(ctx, cipher, plain);
 		if (err != GPG_ERR_NO_ERROR) {
 			debug_print("can't decrypt (%s)\n", gpgme_strerror(err));
+			privacy_set_error("%s", gpgme_strerror(err));
 			gpgmegtk_free_passphrase();
 			gpgme_data_release(plain);
 			return NULL;
@@ -444,9 +455,21 @@ gboolean sgpgme_setup_signers(gpgme_ctx_t ctx, PrefsAccount *account)
 
 	config = prefs_gpg_account_get_config(account);
 
+	switch(config->sign_key) {
+	case SIGN_KEY_DEFAULT:
+		debug_print("using default gnupg key\n");
+		break;
+	case SIGN_KEY_BY_FROM:
+		debug_print("using key for %s\n", account->address);
+		break;
+	case SIGN_KEY_CUSTOM:
+		debug_print("using key for %s\n", config->sign_key_id);
+		break;
+	}
+
 	if (config->sign_key != SIGN_KEY_DEFAULT) {
 		gchar *keyid;
-		gpgme_key_t key;
+		gpgme_key_t key, key2;
 		gpgme_error_t err;
 
 		if (config->sign_key == SIGN_KEY_BY_FROM)
@@ -454,34 +477,42 @@ gboolean sgpgme_setup_signers(gpgme_ctx_t ctx, PrefsAccount *account)
 		else if (config->sign_key == SIGN_KEY_CUSTOM)
 			keyid = config->sign_key_id;
 		else
-			return FALSE;
+			goto bail;
 
 		err = gpgme_op_keylist_start(ctx, keyid, 1);
+		if (!err)
+			err = gpgme_op_keylist_next(ctx, &key);
 		if (err) {
-			g_warning("setup_signers start: %s",
-				gpg_strerror(err));
-			return FALSE;
+			g_warning("setup_signers start: %s", gpgme_strerror(err));
+			privacy_set_error(_("Secret key not found (%s)"), gpgme_strerror(err));
+			goto bail;
 		}
-		while (!(err = gpgme_op_keylist_next(ctx, &key))) {
-			gpgme_signers_add(ctx, key);
-			gpgme_key_release(key);
+		
+		err = gpgme_op_keylist_next(ctx, &key2);
+		if (!err) {
+			g_warning("ambiguous specification of secret key '%s'\n",
+				keyid);
+			privacy_set_error(_("Secret key specification is ambiguous"));
+			goto bail;
 		}
-		if (err && gpg_err_code(err) != GPG_ERR_EOF) {
-			g_warning("setup_signers next: %s",
-				gpg_strerror(err));
-			return FALSE;
-		}
-		err = gpgme_op_keylist_end(ctx);
+		
+		gpgme_op_keylist_end(ctx);
+		err = gpgme_signers_add(ctx, key);
+		gpgme_key_release(key);
+		
 		if (err) {
-			g_warning("setup_signers end: %s",
-				gpg_strerror(err));
-			return FALSE;
+			g_warning("error adding secret key: %s\n", gpgme_strerror(err));
+			privacy_set_error(_("Error setting secret key: %s"), gpgme_strerror(err));
+			goto bail;
 		}
 	}
 
 	prefs_gpg_account_free_config(config);
 
 	return TRUE;
+bail:
+	prefs_gpg_account_free_config(config);
+	return FALSE;
 }
 
 void sgpgme_init()
@@ -496,9 +527,36 @@ void sgpgme_init()
 #endif
 		if (!gpgme_get_engine_info(&engineInfo)) {
 			while (engineInfo) {
-				debug_print("GpgME Protocol: %s\n      Version: %s\n",
+				debug_print("GpgME Protocol: %s\n"
+					    "Version: %s (req %s)\n"
+					    "Executable: %s\n",
 					gpgme_get_protocol_name(engineInfo->protocol),
-					engineInfo->version);
+					engineInfo->version ? engineInfo->version:"???",
+					engineInfo->req_version ? engineInfo->req_version:"???",
+					engineInfo->file_name ? engineInfo->file_name:"???");
+				if (engineInfo->protocol == GPGME_PROTOCOL_OpenPGP
+				&&  gpgme_engine_check_version(engineInfo->protocol) != 
+					GPG_ERR_NO_ERROR) {
+					if (engineInfo->file_name && !engineInfo->version) {
+						alertpanel_error(_("Gpgme protocol '%s' is unusable: "
+								   "Engine '%s' isn't installed properly."),
+								   gpgme_get_protocol_name(engineInfo->protocol),
+								   engineInfo->file_name);
+					} else if (engineInfo->file_name && engineInfo->version
+					  && engineInfo->req_version) {
+						alertpanel_error(_("Gpgme protocol '%s' is unusable: "
+								   "Engine '%s' version %s is installed, "
+								   "but version %s is required.\n"),
+								   gpgme_get_protocol_name(engineInfo->protocol),
+								   engineInfo->file_name,
+								   engineInfo->version,
+								   engineInfo->req_version);
+					} else {
+						alertpanel_error(_("Gpgme protocol '%s' is unusable "
+								   "(unknown problem)"),
+								   gpgme_get_protocol_name(engineInfo->protocol));
+					}
+				}
 				engineInfo = engineInfo->next;
 			}
 		}
@@ -526,4 +584,214 @@ void sgpgme_done()
         gpgmegtk_free_passphrase();
 }
 
+void sgpgme_create_secret_key(PrefsAccount *account, gboolean ask_create)
+{
+	AlertValue val = G_ALERTDEFAULT;
+	gchar *key_parms = NULL;
+	gchar *name = NULL;
+	gchar *email = NULL;
+	gchar *passphrase = NULL, *passphrase_second = NULL;
+	gint prev_bad = 0;
+	gchar *tmp = NULL;
+	gpgme_error_t err = 0;
+	gpgme_ctx_t ctx;
+	GtkWidget *window = NULL;
+	gpgme_genkey_result_t key;
+
+	if (account == NULL)
+		account = account_get_default();
+
+	if (account->address == NULL) {
+		alertpanel_error(_("You have to save the account's information with \"OK\" "
+				   "before being able to generate a key pair.\n"));
+		return;
+	}
+	if (ask_create) {
+		val = alertpanel(_("No PGP key found"),
+				_("Sylpheed-Claws did not find a secret PGP key, "
+				  "which means that you won't be able to sign "
+				  "emails or receive encrypted emails.\n"
+				  "Do you want to create a new key pair now?"),
+				  GTK_STOCK_NO, "+" GTK_STOCK_YES, NULL);
+		if (val == G_ALERTDEFAULT) {
+			prefs_gpg_get_config()->gpg_ask_create_key = FALSE;
+			prefs_gpg_save_config();
+			return;
+		}
+	}
+
+	if (account->name) {
+		name = g_strdup(account->name);
+	} else {
+		name = g_strdup(account->address);
+	}
+	email = g_strdup(account->address);
+	tmp = g_strdup_printf("%s <%s>", account->name?account->name:account->address, account->address);
+again:
+	passphrase = passphrase_mbox(tmp, NULL, prev_bad, 1);
+	if (passphrase == NULL) {
+		g_free(tmp);
+		g_free(email);
+		g_free(name);		
+		return;
+	}
+	passphrase_second = passphrase_mbox(tmp, NULL, 0, 2);
+	if (passphrase_second == NULL) {
+		g_free(tmp);
+		g_free(email);
+		g_free(passphrase);		
+		g_free(name);		
+		return;
+	}
+	if (strcmp(passphrase, passphrase_second)) {
+		g_free(passphrase);
+		g_free(passphrase_second);
+		prev_bad = 1;
+		goto again;
+	}
+	
+	key_parms = g_strdup_printf("<GnupgKeyParms format=\"internal\">\n"
+					"Key-Type: DSA\n"
+					"Key-Length: 1024\n"
+					"Subkey-Type: ELG-E\n"
+					"Subkey-Length: 2048\n"
+					"Name-Real: %s\n"
+					"Name-Email: %s\n"
+					"Expire-Date: 0\n"
+					"%s%s%s"
+					"</GnupgKeyParms>\n",
+					name, email, 
+					strlen(passphrase)?"Passphrase: ":"",
+					passphrase,
+					strlen(passphrase)?"\n":"");
+#ifndef G_PLATFORM_WIN32
+	if (mlock(passphrase, strlen(passphrase)) == -1)
+		debug_print("couldn't lock passphrase\n");
+	if (mlock(passphrase_second, strlen(passphrase_second)) == -1)
+		debug_print("couldn't lock passphrase2\n");
+#endif
+	g_free(tmp);
+	g_free(email);
+	g_free(name);
+	g_free(passphrase_second);
+	g_free(passphrase);
+	
+	err = gpgme_new (&ctx);
+	if (err) {
+		alertpanel_error(_("Couldn't generate a new key pair: %s"),
+				 gpgme_strerror(err));
+		g_free(key_parms);
+		return;
+	}
+	
+
+	window = label_window_create(_("Generating your new key pair... Please move the mouse "
+			      "around to help generate entropy..."));
+
+	err = gpgme_op_genkey(ctx, key_parms, NULL, NULL);
+	g_free(key_parms);
+
+	gtk_widget_destroy(window);
+
+	if (err) {
+		alertpanel_error(_("Couldn't generate a new key pair: %s"), gpgme_strerror(err));
+		gpgme_release(ctx);
+		return;
+	}
+	key = gpgme_op_genkey_result(ctx);
+	if (key == NULL) {
+		alertpanel_error(_("Couldn't generate a new key pair: unknown error"));
+		gpgme_release(ctx);
+		return;
+	} else {
+		gchar *buf = g_strdup_printf(_("Your new key pair has been generated. "
+				    "Its fingerprint is:\n%s\n\nDo you want to export it "
+				    "to a keyserver?"),
+				    key->fpr ? key->fpr:"null");
+		AlertValue val = alertpanel(_("Key generated"), buf,
+				  GTK_STOCK_NO, "+" GTK_STOCK_YES, NULL);
+		g_free(buf);
+		if (val == G_ALERTALTERNATE) {
+#ifndef G_OS_WIN32
+			gchar *cmd = g_strdup_printf("gpg --no-tty --send-keys %s", key->fpr);
+			int res = 0;
+			pid_t pid = 0;
+			pid = fork();
+			if (pid == -1) {
+				res = -1;
+			} else if (pid == 0) {
+				/* son */
+				res = system(cmd);
+				res = WEXITSTATUS(res);
+				_exit(res);
+			} else {
+				int status = 0;
+				time_t start_wait = time(NULL);
+				res = -1;
+				do {
+					if (waitpid(pid, &status, WNOHANG) == 0 || !WIFEXITED(status)) {
+						usleep(200000);
+					} else {
+						res = WEXITSTATUS(status);
+						break;
+					}
+					if (time(NULL) - start_wait > 5) {
+						debug_print("SIGTERM'ing gpg\n");
+						kill(pid, SIGTERM);
+					}
+					if (time(NULL) - start_wait > 6) {
+						debug_print("SIGKILL'ing gpg\n");
+						kill(pid, SIGKILL);
+						break;
+					}
+				} while(1);
+			}
+			if (res == 0) {
+				alertpanel_notice(_("Key exported."));
+			} else {
+				alertpanel_error(_("Couldn't export key."));
+			}
+			g_free(cmd);
+#else
+			alertpanel_error(_("Key export isn't implemented in Windows."));
+#endif
+		}
+	}
+	prefs_gpg_get_config()->gpg_ask_create_key = FALSE;
+	prefs_gpg_save_config();
+	gpgme_release(ctx);
+}
+
+gboolean sgpgme_has_secret_key(void)
+{
+	gpgme_error_t err = 0;
+	gpgme_ctx_t ctx;
+	gpgme_key_t key;
+
+	err = gpgme_new (&ctx);
+	if (err) {
+		debug_print("err : %s\n", gpgme_strerror(err));
+		return TRUE;
+	}
+	err = gpgme_op_keylist_start(ctx, NULL, TRUE);
+	if (!err)
+		err = gpgme_op_keylist_next(ctx, &key);
+	gpgme_op_keylist_end(ctx);
+	gpgme_release(ctx);
+	if (gpg_err_code(err) == GPG_ERR_EOF)
+		return FALSE;
+	else
+		return TRUE;
+}
+
+void sgpgme_check_create_key(void)
+{
+	if (prefs_gpg_get_config()->gpg_ask_create_key &&
+	    !sgpgme_has_secret_key()) {
+		sgpgme_create_secret_key(NULL, TRUE);
+	} else {
+		prefs_gpg_get_config()->gpg_ask_create_key = FALSE;
+		prefs_gpg_save_config();
+	}	
+}
 #endif /* USE_GPGME */
