@@ -26,6 +26,8 @@
 
 #include "defs.h"
 #include <glib.h>
+#include <glib/gi18n.h>
+#include <errno.h>
 #include <gpgme.h>
 
 #include "utils.h"
@@ -38,6 +40,7 @@
 #include "quoted-printable.h"
 #include "base64.h"
 #include "codeconv.h"
+#include "plugin.h"
 
 extern struct GPGConfig prefs_gpg;
 
@@ -231,6 +234,7 @@ static gint pgpinline_check_signature(MimeInfo *mimeinfo)
 	
 	if (!textdata) {
 		g_free(textdata);
+		privacy_set_error(_("Couldn't get text data."));
 		return 0;
 	}
 
@@ -247,9 +251,10 @@ static gint pgpinline_check_signature(MimeInfo *mimeinfo)
 	}
 	g_free(textdata);
 
-	if (!tmp)
+	if (!tmp) {
+		privacy_set_error(_("Couldn't convert text data to any sane charset."));
 		return 0;
-
+	}
 	textdata = g_strdup(tmp);
 	g_free(tmp);
 	
@@ -370,12 +375,14 @@ static MimeInfo *pgpinline_decrypt(MimeInfo *mimeinfo)
 	if (procmime_mimeinfo_parent(mimeinfo) == NULL ||
 	    mimeinfo->type != MIMETYPE_TEXT) {
 		gpgme_release(ctx);
+		privacy_set_error(_("Couldn't parse mime part."));
 		return NULL;
 	}
 
 	textdata = get_part_as_string(mimeinfo);
 	if (!textdata) {
 		gpgme_release(ctx);
+		privacy_set_error(_("Couldn't get text data."));
 		return NULL;
 	}
 
@@ -398,6 +405,7 @@ static MimeInfo *pgpinline_decrypt(MimeInfo *mimeinfo)
 
     	if ((dstfp = g_fopen(fname, "wb")) == NULL) {
         	FILE_OP_ERROR(fname, "fopen");
+		privacy_set_error(_("Couldn't open decrypted file %s"), fname);
         	g_free(fname);
         	gpgme_data_release(plain);
 		gpgme_release(ctx);
@@ -427,6 +435,7 @@ static MimeInfo *pgpinline_decrypt(MimeInfo *mimeinfo)
 	
 	if (parseinfo == NULL) {
 		gpgme_release(ctx);
+		privacy_set_error(_("Couldn't scan decrypted file."));
 		return NULL;
 	}
 	decinfo = g_node_first_child(parseinfo->node) != NULL ?
@@ -434,6 +443,7 @@ static MimeInfo *pgpinline_decrypt(MimeInfo *mimeinfo)
 		
 	if (decinfo == NULL) {
 		gpgme_release(ctx);
+		privacy_set_error(_("Couldn't scan decrypted file parts."));
 		return NULL;
 	}
 
@@ -470,7 +480,9 @@ static gboolean pgpinline_sign(MimeInfo *mimeinfo, PrefsAccount *account)
 	gpgme_ctx_t ctx;
 	gpgme_data_t gpgtext, gpgsig;
 	guint len;
+	gpgme_error_t err;
 	struct passphrase_cb_info_s info;
+	gpgme_sign_result_t result = NULL;
 
 	memset (&info, 0, sizeof info);
 
@@ -485,6 +497,7 @@ static gboolean pgpinline_sign(MimeInfo *mimeinfo, PrefsAccount *account)
 	fp = my_tmpfile();
 	if (fp == NULL) {
 		perror("my_tmpfile");
+		privacy_set_error(_("Couldn't create temporary file."));
 		return FALSE;
 	}
 	procmime_write_mimeinfo(msgcontent, fp);
@@ -511,14 +524,59 @@ static gboolean pgpinline_sign(MimeInfo *mimeinfo, PrefsAccount *account)
     		gpgme_set_passphrase_cb (ctx, gpgmegtk_passphrase_cb, &info);
 	}
 
-	if (gpgme_op_sign(ctx, gpgtext, gpgsig, GPGME_SIG_MODE_CLEAR) 
-	    != GPG_ERR_NO_ERROR) {
-	    	gpgme_release(ctx);
+	err = gpgme_op_sign(ctx, gpgtext, gpgsig, GPGME_SIG_MODE_CLEAR);
+	if (err != GPG_ERR_NO_ERROR) {
+		if (err == GPG_ERR_CANCELED) {
+			/* ignore cancelled signing */
+			privacy_reset_error();
+			debug_print("gpgme_op_sign cancelled\n");
+		} else {
+			privacy_set_error(_("Data signing failed, %s"), gpgme_strerror(err));
+			debug_print("gpgme_op_sign error : %x\n", err);
+		}
+		gpgme_release(ctx);
+		return FALSE;
+	}
+	result = gpgme_op_sign_result(ctx);
+	if (result && result->signatures) {
+		gpgme_new_signature_t sig = result->signatures;
+		while (sig) {
+			debug_print("valid signature: %s\n", sig->fpr);
+			sig = sig->next;
+		}
+	} else if (result && result->invalid_signers) {
+		gpgme_invalid_key_t invalid = result->invalid_signers;
+		while (invalid) {
+			g_warning("invalid signer: %s (%s)", invalid->fpr, 
+				gpgme_strerror(invalid->reason));
+			privacy_set_error(_("Data signing failed due to invalid signer: %s"), 
+				gpgme_strerror(invalid->reason));
+			invalid = invalid->next;
+		}
+		gpgme_release(ctx);
+		return FALSE;
+	} else {
+		/* can't get result (maybe no signing key?) */
+		debug_print("gpgme_op_sign_result error\n");
+		privacy_set_error(_("Data signing failed, no results."));
+		gpgme_release(ctx);
 		return FALSE;
 	}
 
-	gpgme_release(ctx);
+
 	sigcontent = gpgme_data_release_and_get_mem(gpgsig, &len);
+	
+	gpgme_release(ctx);
+	
+	if (sigcontent == NULL || len <= 0) {
+		g_warning("gpgme_data_release_and_get_mem failed");
+		privacy_set_error(_("Data signing failed, no contents."));
+		gpgme_data_release(gpgtext);
+		g_free(textstr);
+		g_free(sigcontent);
+		return FALSE;
+	}
+
 	tmp = g_malloc(len+1);
 	g_memmove(tmp, sigcontent, len+1);
 	tmp[len] = '\0';
@@ -528,7 +586,8 @@ static gboolean pgpinline_sign(MimeInfo *mimeinfo, PrefsAccount *account)
 
 	if (msgcontent->content == MIMECONTENT_FILE &&
 	    msgcontent->data.filename != NULL) {
-		g_unlink(msgcontent->data.filename);
+	    	if (msgcontent->tmp == TRUE)
+			g_unlink(msgcontent->data.filename);
 		g_free(msgcontent->data.filename);
 	}
 	msgcontent->data.mem = g_strdup(tmp);
@@ -559,7 +618,9 @@ static gboolean pgpinline_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	gpgme_ctx_t ctx;
 	gpgme_key_t *kset = NULL;
 	gchar **fprs = g_strsplit(encrypt_data, " ", -1);
+	gpgme_error_t err;
 	gint i = 0;
+
 	while (fprs[i] && strlen(fprs[i])) {
 		i++;
 	}
@@ -570,11 +631,11 @@ static gboolean pgpinline_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	i = 0;
 	while (fprs[i] && strlen(fprs[i])) {
 		gpgme_key_t key;
-		gpgme_error_t err;
 		err = gpgme_get_key(ctx, fprs[i], &key, 0);
 		if (err) {
 			debug_print("can't add key '%s'[%d] (%s)\n", fprs[i],i, gpgme_strerror(err));
-			break;
+			privacy_set_error(_("Couldn't add GPG key %s, %s"), fprs[i], gpgme_strerror(err));
+			return FALSE;
 		}
 		debug_print("found %s at %d\n", fprs[i], i);
 		kset[i] = key;
@@ -594,6 +655,7 @@ static gboolean pgpinline_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 
 	fp = my_tmpfile();
 	if (fp == NULL) {
+		privacy_set_error(_("Couldn't create temporary file, %s"), strerror(errno));
 		perror("my_tmpfile");
 		return FALSE;
 	}
@@ -611,11 +673,19 @@ static gboolean pgpinline_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 	gpgme_new(&ctx);
 	gpgme_set_armor(ctx, 1);
 
-	gpgme_op_encrypt(ctx, kset, GPGME_ENCRYPT_ALWAYS_TRUST, gpgtext, gpgenc);
+	err = gpgme_op_encrypt(ctx, kset, GPGME_ENCRYPT_ALWAYS_TRUST, gpgtext, gpgenc);
 
 	gpgme_release(ctx);
 	enccontent = gpgme_data_release_and_get_mem(gpgenc, &len);
 
+	if (enccontent == NULL || len <= 0) {
+		g_warning("gpgme_data_release_and_get_mem failed");
+		privacy_set_error(_("Encryption failed, %s"), gpgme_strerror(err));
+		gpgme_data_release(gpgtext);
+		g_free(textstr);
+		return FALSE;
+	}
+
 	tmp = g_malloc(len+1);
 	g_memmove(tmp, enccontent, len+1);
 	tmp[len] = '\0';
@@ -626,7 +696,8 @@ static gboolean pgpinline_encrypt(MimeInfo *mimeinfo, const gchar *encrypt_data)
 
 	if (msgcontent->content == MIMECONTENT_FILE &&
 	    msgcontent->data.filename != NULL) {
-		g_unlink(msgcontent->data.filename);
+	    	if (msgcontent->tmp == TRUE)
+			g_unlink(msgcontent->data.filename);
 		g_free(msgcontent->data.filename);
 	}
 	msgcontent->data.mem = g_strdup(tmp);
@@ -669,4 +740,11 @@ void pgpinline_done()
 	privacy_unregister_system(&pgpinline_system);
 }
 
+struct PluginFeature *plugin_provides(void)
+{
+	static struct PluginFeature features[] = 
+		{ {PLUGIN_PRIVACY, N_("PGP/Inline")},
+		  {PLUGIN_NOTHING, NULL}};
+	return features;
+}
 #endif /* USE_GPGME */
