@@ -32,12 +32,12 @@
 gint plugin_init(gchar **error)
 {
 	if ((sylpheed_get_version() > VERSION_NUMERIC)) {
-		*error = g_strdup("Your version of Sylpheed-Claws is newer than the version the PGP/MIME plugin was built with");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is newer than the version the PGP/MIME plugin was built with"));
 		return -1;
 	}
 
 	if ((sylpheed_get_version() < MAKE_NUMERIC_VERSION(2, 1, 0, 1))) {
-		*error = g_strdup("Your version of Sylpheed-Claws is too old for the PGP/MIME plugin");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is too old for the PGP/MIME plugin"));
 		return -1;
 	}
 
@@ -62,8 +62,6 @@ const gchar *plugin_desc(void)
 		 "mails. You can decrypt mails, verify signatures or "
                  "sign and encrypt your own mails.\n"
 		 "\n"
-		 "It also verifies S/MIME signatures.\n"
-		 "\n"
 		 "It can be selected as the Default Privacy System in "
 		 "/Configuration/[Account Preferences]/Privacy and when "
 		 "composing a message from /Options/Privacy System\n"
