@@ -48,6 +48,7 @@
 #include "inc.h"
 #include "log.h"
 #include "prefs_common.h"
+#include "alertpanel.h"
 
 #ifdef HAVE_SYSEXITS_H
 #include <sysexits.h>
@@ -120,14 +121,20 @@ gboolean timeout_func(gpointer data)
 	return FALSE;
 }
 
-static gboolean msg_is_spam(FILE *fp)
+typedef enum {
+	MSG_IS_HAM = 0,
+	MSG_IS_SPAM = 1,
+	MSG_FILTERING_ERROR = 2
+} MsgStatus;
+
+static MsgStatus msg_is_spam(FILE *fp)
 {
 	struct transport trans;
 	struct message m;
 	gboolean is_spam = FALSE;
 
 	if (!config.enable)
-		return FALSE;
+		return MSG_IS_HAM;
 
 	transport_init(&trans);
 	switch (config.transport) {
@@ -145,13 +152,13 @@ static gboolean msg_is_spam(FILE *fp)
 		trans.socketpath = config.socket;
 		break;
 	default:
-		return FALSE;
+		return MSG_IS_HAM;
 	}
 
 	if (transport_setup(&trans, flags) != EX_OK) {
-		log_error("Spamassassin plugin couldn't connect to spamd.\n");
+		log_error(_("SpamAssassin plugin couldn't connect to spamd.\n"));
 		debug_print("failed to setup transport\n");
-		return FALSE;
+		return MSG_FILTERING_ERROR;
 	}
 
 	m.type = MESSAGE_NONE;
@@ -161,13 +168,14 @@ static gboolean msg_is_spam(FILE *fp)
 	if (message_read(fileno(fp), flags, &m) != EX_OK) {
 		debug_print("failed to read message\n");
 		message_cleanup(&m);
-		return FALSE;
+		return MSG_FILTERING_ERROR;
 	}
 
 	if (message_filter(&trans, config.username, flags, &m) != EX_OK) {
+		log_error(_("SpamAssassin plugin filtering failed.\n"));
 		debug_print("filtering the message failed\n");
 		message_cleanup(&m);
-		return FALSE;
+		return MSG_FILTERING_ERROR;
 	}
 
 	if (m.is_spam == EX_ISSPAM)
@@ -175,21 +183,22 @@ static gboolean msg_is_spam(FILE *fp)
 
 	message_cleanup(&m);
 
-	return is_spam;
+	return is_spam ? MSG_IS_SPAM:MSG_IS_HAM;
 }
 
 static gboolean mail_filtering_hook(gpointer source, gpointer data)
 {
 	MailFilteringData *mail_filtering_data = (MailFilteringData *) source;
 	MsgInfo *msginfo = mail_filtering_data->msginfo;
-	gboolean is_spam = FALSE;
+	gboolean is_spam = FALSE, error = FALSE;
+	static gboolean warned_error = FALSE;
 	FILE *fp = NULL;
 	int pid = 0;
 	int status;
 
 	/* SPAMASSASSIN_DISABLED : keep test for compatibility purpose */
 	if (!config.enable || config.transport == SPAMASSASSIN_DISABLED) {
-		log_error("Spamassassin plugin is disabled by its preferences.\n");
+		log_warning(_("SpamAssassin plugin is disabled by its preferences.\n"));
 		return FALSE;
 	}
 	debug_print("Filtering message %d\n", msginfo->msgnum);
@@ -203,7 +212,7 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 
 	pid = fork();
 	if (pid == 0) {
-		_exit(msg_is_spam(fp) ? 1 : 0);
+		_exit(msg_is_spam(fp));
 	} else {
 		gint running = 0;
 
@@ -218,8 +227,11 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 			ret = waitpid(pid, &status, WNOHANG);
 			if (ret == pid) {
 				if (WIFEXITED(status)) {
+					MsgStatus result = MSG_IS_HAM;
 					running &= ~CHILD_RUNNING;
-    					is_spam = WEXITSTATUS(status) == 1 ? TRUE : FALSE;
+					result = WEXITSTATUS(status);
+    					is_spam = (result == MSG_IS_SPAM) ? TRUE : FALSE;
+					error = (result == MSG_FILTERING_ERROR);
 				}
 			} if (ret < 0) {
 				running &= ~CHILD_RUNNING;
@@ -247,7 +259,8 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 
 			procmsg_msginfo_unset_flags(msginfo, ~0, 0);
 			procmsg_msginfo_set_flags(msginfo, MSG_SPAM, 0);
-			folder_item_move_msg(save_folder, msginfo);
+			msginfo->is_move = TRUE;
+			msginfo->to_filter_folder = save_folder;
 		} else {
 			folder_item_remove_msg(msginfo->folder, msginfo->msgnum);
 		}
@@ -257,6 +270,24 @@ static gboolean mail_filtering_hook(gpointer source, gpointer data)
 		debug_print("message is ham\n");
 		procmsg_msginfo_unset_flags(msginfo, MSG_SPAM, 0);
 	}
+	
+	if (error) {
+		gchar *msg = _("The SpamAssassin plugin couldn't filter "
+					   "a message. The probable cause of the error "
+					   "is an unreachable spamd daemon. Please make "
+					   "sure spamd is running and accessible.");
+		if (!prefs_common.no_recv_err_panel) {
+			if (!warned_error) {
+				alertpanel_error(msg);
+			}
+			warned_error = TRUE;
+		} else {
+			gchar *tmp = g_strdup_printf("%s\n", msg);
+			log_error(tmp);
+			g_free(tmp);
+		}
+	}
+	
 	return FALSE;
 }
 
@@ -346,13 +377,9 @@ int spamassassin_learn(MsgInfo *msginfo, GSList *msglist, gboolean spam)
 					execute_command_line(tmpcmd, FALSE);
 					g_free(tmpcmd);
 				}
-				if (tmpfile != NULL) {
-					g_free(tmpfile);
-				}
-			}
-			if (spamc_wrapper != NULL) {
-				g_free(spamc_wrapper);
+				g_free(tmpfile);
 			}
+			g_free(spamc_wrapper);
 			return 0;
 		} else {
 			cmd = g_strdup_printf("sa-learn -u %s %s %s",
@@ -372,9 +399,7 @@ int spamassassin_learn(MsgInfo *msginfo, GSList *msglist, gboolean spam)
 					g_free(cmd);
 					cmd = tmpcmd;
 				}
-				if (tmpfile != NULL) {
-					g_free(tmpfile);
-				}
+				g_free(tmpfile);
 			}
 		}
 	}
@@ -385,9 +410,8 @@ int spamassassin_learn(MsgInfo *msginfo, GSList *msglist, gboolean spam)
 	/* only run sync calls to sa-learn/spamc to prevent system lockdown */
 	execute_command_line(cmd, FALSE);
 	g_free(cmd);
-	if (spamc_wrapper != NULL) {
-		g_free(spamc_wrapper);
-	}
+	g_free(spamc_wrapper);
+
 	return 0;
 }
 
@@ -442,12 +466,12 @@ gint plugin_init(gchar **error)
 	hook_id = -1;
 
 	if ((sylpheed_get_version() > VERSION_NUMERIC)) {
-		*error = g_strdup("Your version of Sylpheed-Claws is newer than the version the SpamAssassin plugin was built with");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is newer than the version the SpamAssassin plugin was built with"));
 		return -1;
 	}
 
 	if ((sylpheed_get_version() < MAKE_NUMERIC_VERSION(0, 9, 3, 86))) {
-		*error = g_strdup("Your version of Sylpheed-Claws is too old for the SpamAssassin plugin");
+		*error = g_strdup(_("Your version of Sylpheed-Claws is too old for the SpamAssassin plugin"));
 		return -1;
 	}
 
@@ -456,19 +480,19 @@ gint plugin_init(gchar **error)
 	prefs_read_config(param, "SpamAssassin", rcpath, NULL);
 	g_free(rcpath);
 	if (!spamassassin_check_username()) {
-		*error = g_strdup("Failed to get username");
+		*error = g_strdup(_("Failed to get username"));
 		return -1;
 	}
 	spamassassin_gtk_init();
 		
-	debug_print("Spamassassin plugin loaded\n");
+	debug_print("SpamAssassin plugin loaded\n");
 
 	if (config.process_emails) {
 		spamassassin_register_hook();
 	}
 
 	if (!config.enable || config.transport == SPAMASSASSIN_DISABLED) {
-		log_error("Spamassassin plugin is loaded but disabled by its preferences.\n");
+		log_warning(_("SpamAssassin plugin is loaded but disabled by its preferences.\n"));
 	}
 	else {
 		if (config.transport == SPAMASSASSIN_TRANSPORT_TCP)
@@ -491,7 +515,7 @@ void plugin_done(void)
 	spamassassin_gtk_done();
 	procmsg_unregister_spam_learner(spamassassin_learn);
 	procmsg_spam_set_folder(NULL);
-	debug_print("Spamassassin plugin unloaded\n");
+	debug_print("SpamAssassin plugin unloaded\n");
 }
 
 const gchar *plugin_name(void)
@@ -529,9 +553,19 @@ const gchar *plugin_version(void)
 	return VERSION;
 }
 
+struct PluginFeature *plugin_provides(void)
+{
+	static struct PluginFeature features[] = 
+		{ {PLUGIN_FILTERING, N_("Spam detection")},
+		  {PLUGIN_FILTERING, N_("Spam learning")},
+		  {PLUGIN_NOTHING, NULL}};
+	return features;
+}
+
 void spamassassin_register_hook(void)
 {
-	hook_id = hooks_register_hook(MAIL_FILTERING_HOOKLIST, mail_filtering_hook, NULL);
+	if (hook_id == -1)
+		hook_id = hooks_register_hook(MAIL_FILTERING_HOOKLIST, mail_filtering_hook, NULL);
 	if (hook_id == -1) {
 		g_warning("Failed to register mail filtering hook");
 		config.process_emails = FALSE;
@@ -543,4 +577,5 @@ void spamassassin_unregister_hook(void)
 	if (hook_id != -1) {
 		hooks_unregister_hook(MAIL_FILTERING_HOOKLIST, hook_id);
 	}
+	hook_id = -1;
 }
