@@ -777,6 +777,7 @@ static Pop3State pop3_lookup_next(Pop3Session *session)
 			log_message
 				(_("POP3: Deleting expired message "
 				   "%d\n"), session->cur_msg);
+			session->cur_total_bytes += size;
 			pop3_delete_send(session);
 			return POP3_DELETE;
 		}
@@ -831,7 +832,7 @@ static Pop3ErrorValue pop3_ok(Pop3Session *session, const gchar *msg)
 			switch (session->state) {
 #if USE_OPENSSL
 			case POP3_STLS:
-				log_error(_("can't start TLS session\n"));
+				log_error(_("couldn't start TLS session\n"));
 				ok = PS_ERROR;
 				break;
 #endif
