@@ -33,7 +33,8 @@ typedef enum {
 	A_IMAP4,
 	A_NNTP,
 	A_LOCAL,
-	A_NONE		/* SMTP only */
+	A_NONE,	/* SMTP only */
+	NUM_RECV_PROTOCOLS
 } RecvProtocol;
 
 typedef enum {
@@ -98,7 +99,6 @@ struct _PrefsAccount
 	gint imap_auth_type;
 
 	/* Send */
-	gboolean add_date;
 	gboolean gen_msgid;
 	gboolean add_customhdr;
 	gboolean use_smtp_auth;
@@ -133,6 +133,7 @@ struct _PrefsAccount
 	gboolean  default_encrypt_reply;
 	gboolean  default_sign;
 	gboolean  save_encrypted_as_clear_text;
+	gboolean  encrypt_to_self;
 
 	/* Advanced */
 	gboolean  set_smtpport;
