@@ -44,6 +44,7 @@
 #include "action.h"
 #include "description_window.h"
 #include "gtkutils.h"
+#include "manual.h"
 
 enum {
 	PREFS_ACTIONS_STRING,	/*!< string pointer managed by list store, 
@@ -76,7 +77,7 @@ static void prefs_actions_set_dialog	(void);
 static gint prefs_actions_clist_set_row	(GtkTreeIter *row);
 
 /* callback functions */
-static void prefs_actions_help_cb	(GtkWidget	*w,
+static void prefs_actions_info_cb	(GtkWidget	*w,
 					 GtkWidget	*window);
 static void prefs_actions_register_cb	(GtkWidget	*w,
 					 gpointer	 data);
@@ -145,14 +146,14 @@ static void prefs_actions_create(MainWindow *mainwin)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
+	GtkWidget *help_btn;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *confirm_area;
 
 	GtkWidget *vbox1;
+	GtkWidget *table;
 
-	GtkWidget *entry_vbox;
-	GtkWidget *hbox;
 	GtkWidget *name_label;
 	GtkWidget *name_entry;
 	GtkWidget *cmd_label;
@@ -169,7 +170,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 	GtkWidget *cond_scrolledwin;
 	GtkWidget *cond_list_view;
 
-	GtkWidget *help_button;
+	GtkWidget *info_btn;
 
 	GtkWidget *btn_vbox;
 	GtkWidget *up_btn;
@@ -189,9 +190,10 @@ static void prefs_actions_create(MainWindow *mainwin)
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
-				      &ok_btn, GTK_STOCK_OK,
-				      NULL, NULL);
+	gtkut_stock_button_set_create_with_help(&confirm_area, &help_btn,
+			&cancel_btn, GTK_STOCK_CANCEL,
+			&ok_btn, GTK_STOCK_OK,
+			NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -208,34 +210,46 @@ static void prefs_actions_create(MainWindow *mainwin)
 			 G_CALLBACK(prefs_actions_ok), mainwin);
 	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
 			 G_CALLBACK(prefs_actions_cancel), NULL);
+	g_signal_connect(G_OBJECT(help_btn), "clicked",
+			 G_CALLBACK(manual_open_with_anchor_cb),
+			 MANUAL_ANCHOR_ACTIONS);
 
-	vbox1 = gtk_vbox_new(FALSE, 8);
+	vbox1 = gtk_vbox_new(FALSE, VSPACING);
 	gtk_widget_show(vbox1);
 	gtk_box_pack_start(GTK_BOX(vbox), vbox1, TRUE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 2);
 
-	entry_vbox = gtk_vbox_new(FALSE, 4);
-	gtk_box_pack_start(GTK_BOX(vbox1), entry_vbox, FALSE, FALSE, 0);
+	table = gtk_table_new(2, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), VSPACING_NARROW_2);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+	gtk_widget_show(table);
+	gtk_box_pack_start (GTK_BOX (vbox1), table, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 8);
-	gtk_box_pack_start(GTK_BOX(entry_vbox), hbox, FALSE, FALSE, 0);
+	name_label = gtk_label_new (_("Menu name"));
+	gtk_widget_show (name_label);
+	gtk_misc_set_alignment (GTK_MISC (name_label), 1, 0.5);
+  	gtk_table_attach (GTK_TABLE (table), name_label, 0, 1, 0, 1,
+                    	  (GtkAttachOptions) (GTK_FILL),
+                    	  (GtkAttachOptions) (0), 0, 0);
 
-	name_label = gtk_label_new(_("Menu name:"));
-	gtk_box_pack_start(GTK_BOX(hbox), name_label, FALSE, FALSE, 0);
+	name_entry = gtk_entry_new ();
+	gtk_widget_show (name_entry);
+  	gtk_table_attach (GTK_TABLE (table), name_entry, 1, 2, 0, 1,
+                    	  (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+                    	  (GtkAttachOptions) (0), 0, 0);
 
-	name_entry = gtk_entry_new();
-	gtk_box_pack_start(GTK_BOX(hbox), name_entry, TRUE, TRUE, 0);
+	cmd_label = gtk_label_new (_("Command line"));
+	gtk_widget_show (cmd_label);
+	gtk_misc_set_alignment (GTK_MISC (cmd_label), 1, 0.5);
+  	gtk_table_attach (GTK_TABLE (table), cmd_label, 0, 1, 1, 2,
+                    	  (GtkAttachOptions) (GTK_FILL),
+                    	  (GtkAttachOptions) (0), 0, 0);
 
-	hbox = gtk_hbox_new(FALSE, 8);
-	gtk_box_pack_start(GTK_BOX(entry_vbox), hbox, TRUE, TRUE, 0);
-
-	cmd_label = gtk_label_new(_("Command line:"));
-	gtk_box_pack_start(GTK_BOX(hbox), cmd_label, FALSE, FALSE, 0);
-
-	cmd_entry = gtk_entry_new();
-	gtk_box_pack_start(GTK_BOX(hbox), cmd_entry, TRUE, TRUE, 0);
-
-	gtk_widget_show_all(entry_vbox);
+	cmd_entry = gtk_entry_new ();
+	gtk_widget_show (cmd_entry);
+  	gtk_table_attach (GTK_TABLE (table), cmd_entry, 1, 2, 1, 2,
+                    	  (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+                    	  (GtkAttachOptions) (0), 0, 0);
 
 	/* register / substitute / delete */
 
@@ -258,7 +272,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 	g_signal_connect(G_OBJECT(reg_btn), "clicked",
 			 G_CALLBACK(prefs_actions_register_cb), NULL);
 
-	subst_btn = gtk_button_new_with_label(_(" Replace "));
+	subst_btn = gtkut_get_replace_btn(_("Replace"));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), subst_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(subst_btn), "clicked",
@@ -271,11 +285,15 @@ static void prefs_actions_create(MainWindow *mainwin)
 	g_signal_connect(G_OBJECT(del_btn), "clicked",
 			 G_CALLBACK(prefs_actions_delete_cb), NULL);
 
-	help_button = gtk_button_new_from_stock(GTK_STOCK_HELP);
-	gtk_widget_show(help_button);
-	gtk_box_pack_end(GTK_BOX(reg_hbox), help_button, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(help_button), "clicked",
-			 G_CALLBACK(prefs_actions_help_cb), GTK_WINDOW(window));
+#if GTK_CHECK_VERSION(2, 8, 0)
+	info_btn = gtk_button_new_from_stock(GTK_STOCK_INFO);
+#else
+	info_btn = gtk_button_new_with_label(_("Info..."));
+#endif
+	gtk_widget_show(info_btn);
+	gtk_box_pack_end(GTK_BOX(reg_hbox), info_btn, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(info_btn), "clicked",
+			 G_CALLBACK(prefs_actions_info_cb), GTK_WINDOW(window));
 
 	cond_hbox = gtk_hbox_new(FALSE, 8);
 	gtk_widget_show(cond_hbox);
@@ -437,7 +455,7 @@ static void prefs_actions_set_dialog(void)
 	gtk_list_store_clear(store);
 
 	prefs_actions_list_view_insert_action(actions.actions_list_view,
-					      NULL, _("New"), FALSE);
+					      NULL, _("(New)"), FALSE);
 
 	for (cur = prefs_common.actions_list; cur != NULL; cur = cur->next) {
 		gchar *action = (gchar *) cur->data;
@@ -730,7 +748,7 @@ static void prefs_actions_cancel(GtkWidget *w, gpointer data)
 {
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("_Continue editing"),
+				 GTK_STOCK_CLOSE, _("+_Continue editing"),
 				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
@@ -750,7 +768,7 @@ static void prefs_actions_ok(GtkWidget *widget, gpointer data)
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("_Continue editing"),
+				 GTK_STOCK_CLOSE, _("+_Continue editing"),
 				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
@@ -821,7 +839,7 @@ static DescriptionWindow actions_desc_win = {
 };
 
 
-static void prefs_actions_help_cb(GtkWidget *w, GtkWidget *window)
+static void prefs_actions_info_cb(GtkWidget *w, GtkWidget *window)
 {
 	actions_desc_win.parent = window;
 	description_window_create(&actions_desc_win);
@@ -879,7 +897,7 @@ static GtkWidget *prefs_actions_list_view_create(void)
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);	
 	
-	gtk_tree_view_set_rules_hint(list_view, prefs_common.enable_rules_hint);
+	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 	gtk_tree_view_set_reorderable(list_view, TRUE);
 
 	selector = gtk_tree_view_get_selection(list_view);
