@@ -95,6 +95,8 @@ static PrefParam param_os_specific[] = {
 	  &prefs_common.widgetfont,		P_STRING, NULL, NULL, NULL},
 	{"message_font_gtk2",	"Monospace 9",
 	 &prefs_common.textfont,		P_STRING, NULL, NULL, NULL},
+        {"print_font_gtk2",     "Monospace 9",
+         &prefs_common.printfont,		P_STRING, NULL, NULL, NULL},
 	{"small_font_gtk2",	"Sans 9",
 	  &prefs_common.smallfont,		P_STRING, NULL, NULL, NULL},
 	{"normal_font_gtk2",	"Sans 9",
@@ -107,12 +109,10 @@ static PrefParam param_os_specific[] = {
 	 &prefs_common.attach_load_dir, P_STRING, NULL, NULL, NULL},
 
 	/* MIME viewer */
-	{"mime_image_viewer", NULL,
-	 &prefs_common.mime_image_viewer, P_STRING, NULL, NULL, NULL},
-	{"mime_audio_player", NULL,
-	 &prefs_common.mime_audio_player, P_STRING, NULL, NULL, NULL},
+	{"mime_textviewer", NULL,
+	 &prefs_common.mime_textviewer,   P_STRING, NULL, NULL, NULL},
 	{"mime_open_command", "notepad '%s'",
-	 &prefs_common.mime_open_cmd, P_STRING, NULL, NULL, NULL},
+	 &prefs_common.mime_open_cmd,     P_STRING, NULL, NULL, NULL},
 
 	/* Interface */
 	{"pixmap_theme_path", DEFAULT_PIXMAP_THEME, 
@@ -164,6 +164,10 @@ static PrefParam param[] = {
  	 NULL, NULL, NULL},
 	{"receive_dialog_mode", "1", &prefs_common.recv_dialog_mode, P_ENUM,
 	 NULL, NULL, NULL},
+	{"receivewin_width", "460", &prefs_common.receivewin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"receivewin_height", "-1", &prefs_common.receivewin_height, P_INT,
+	 NULL, NULL, NULL},
 	{"no_receive_error_panel", "FALSE", &prefs_common.no_recv_err_panel,
 	 P_BOOL, NULL, NULL, NULL},
 	{"close_receive_dialog", "TRUE", &prefs_common.close_recv_dialog,
@@ -176,6 +180,10 @@ static PrefParam param[] = {
 	 P_BOOL, NULL, NULL, NULL},
 	{"send_dialog_mode", "0", &prefs_common.send_dialog_mode, P_ENUM,
 	 NULL, NULL, NULL},
+	{"sendwin_width", "460", &prefs_common.sendwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"sendwin_height", "-1", &prefs_common.sendwin_height, P_INT,
+	 NULL, NULL, NULL},
 
 	{"outgoing_charset", CS_AUTO, &prefs_common.outgoing_charset, P_STRING,
 	 NULL, NULL, NULL},
@@ -228,6 +236,8 @@ static PrefParam param[] = {
 #endif
 	{"reply_with_quote", "TRUE", &prefs_common.reply_with_quote, P_BOOL,
 	 NULL, NULL, NULL},
+	{"compose_dnd_insert_or_attach", "0", &prefs_common.compose_dnd_mode, P_ENUM,
+	 NULL, NULL, NULL},
 
 	/* Account autoselection */
 	{"reply_account_autoselect", "TRUE",
@@ -280,6 +290,8 @@ static PrefParam param[] = {
 	  &SPECIFIC_PREFS.widgetfont,		P_STRING, NULL, NULL, NULL},
 	{"message_font_gtk2",	"Monospace 9",
 	 &SPECIFIC_PREFS.textfont,			P_STRING, NULL, NULL, NULL},
+        {"print_font_gtk2",     "Monospace 9",
+         &SPECIFIC_PREFS.printfont,             P_STRING, NULL, NULL, NULL},
 	{"small_font_gtk2",	"Sans 9",
 	  &SPECIFIC_PREFS.smallfont,		P_STRING, NULL, NULL, NULL},
 	{"normal_font_gtk2",	"Sans 9",
@@ -522,6 +534,14 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"quote_level3_color", "179", &prefs_common.quote_level3_col, P_COLOR,
 	 NULL, NULL, NULL},
+	{"enable_bgcolor", "FALSE", &prefs_common.enable_bgcolor, P_BOOL,
+	 NULL, NULL, NULL},
+	{"quote_level1_bgcolor", "13421772", &prefs_common.quote_level1_bgcol, P_COLOR,
+	 NULL, NULL, NULL},
+	{"quote_level2_bgcolor", "13948116", &prefs_common.quote_level2_bgcol, P_COLOR,
+	 NULL, NULL, NULL},
+	{"quote_level3_bgcolor", "14540253", &prefs_common.quote_level3_bgcol, P_COLOR,
+	 NULL, NULL, NULL},
 	{"uri_color", "32512", &prefs_common.uri_col, P_COLOR,
 	 NULL, NULL, NULL},
 	{"target_folder_color", "14294218", &prefs_common.tgt_folder_col, P_COLOR,
@@ -543,6 +563,8 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"line_space", "2", &prefs_common.line_space, P_INT,
 	 NULL, NULL, NULL},
+	{"never_send_retrcpt", "FALSE", &prefs_common.never_send_retrcpt, P_BOOL,
+	 NULL, NULL, NULL},
 
 	{"enable_smooth_scroll", "FALSE",
 	 &prefs_common.enable_smooth_scroll, P_BOOL,
@@ -551,10 +573,15 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"scroll_half_page", "FALSE", &prefs_common.scroll_halfpage, P_BOOL,
 	 NULL, NULL, NULL},
+	{"respect_flowed_format", "FALSE", &prefs_common.respect_flowed_format, P_BOOL,
+	 NULL, NULL, NULL},
 
 	{"show_other_header", "FALSE", &prefs_common.show_other_header, P_BOOL,
 	 NULL, NULL, NULL},
 
+	{"use_different_print_font", "FALSE", &prefs_common.use_different_print_font, P_BOOL,
+	 NULL, NULL, NULL},
+
 	{"attach_desc", "TRUE", &prefs_common.attach_desc, P_BOOL,
 	 NULL, NULL, NULL},
 	{"attach_save_directory", NULL,
@@ -563,12 +590,10 @@ static PrefParam param[] = {
 	 &SPECIFIC_PREFS.attach_load_dir, P_STRING, NULL, NULL, NULL},
 
 	/* MIME viewer */
-	{"mime_image_viewer", DEFAULT_IMAGE_VIEWER_CMD,
-	 &SPECIFIC_PREFS.mime_image_viewer,	P_STRING, NULL, NULL, NULL},
-	{"mime_audio_player", DEFAULT_AUDIO_PLAYER_CMD,
-	 &SPECIFIC_PREFS.mime_audio_player, P_STRING, NULL, NULL, NULL},
+	{"mime_textviewer",   NULL,
+	 &SPECIFIC_PREFS.mime_textviewer,   P_STRING, NULL, NULL, NULL},
 	{"mime_open_command", "gedit '%s'",
-	 &SPECIFIC_PREFS.mime_open_cmd, P_STRING, NULL, NULL, NULL},
+	 &SPECIFIC_PREFS.mime_open_cmd,     P_STRING, NULL, NULL, NULL},
 
 	/* Interface */
 	{"separate_folder", "FALSE", &prefs_common.sep_folder, P_BOOL,
@@ -601,6 +626,11 @@ static PrefParam param[] = {
 	{"ask_mark_all_read", "TRUE", &prefs_common.ask_mark_all_read, P_BOOL,
 	 NULL, NULL, NULL},
 
+	{"ask_apply_per_account_filtering_rules", "TRUE", &prefs_common.ask_apply_per_account_filtering_rules, P_BOOL,
+	 NULL, NULL, NULL},
+	{"apply_per_account_filtering_rules", "0", &prefs_common.apply_per_account_filtering_rules, P_ENUM,
+	 NULL, NULL, NULL},
+
 	/* Other */
 	{"uri_open_command", DEFAULT_BROWSER_CMD,
 	 &SPECIFIC_PREFS.uri_cmd, P_STRING, NULL, NULL, NULL},
@@ -636,6 +666,16 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"log_length", "500", &prefs_common.loglength, P_INT,
 	 NULL, NULL, NULL},
+	{"log_msg_color", "#00af00", &prefs_common.log_msg_color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"log_warn_color", "#af0000", &prefs_common.log_warn_color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"log_error_color", "#af0000", &prefs_common.log_error_color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"log_in_color", "#000000", &prefs_common.log_in_color, P_COLOR,
+	 NULL, NULL, NULL},
+	{"log_out_color", "#0000ef", &prefs_common.log_out_color, P_COLOR,
+	 NULL, NULL, NULL},
 
 	{"color_new", "179", &prefs_common.color_new, P_COLOR,
 	 NULL, NULL, NULL},
@@ -676,6 +716,11 @@ static PrefParam param[] = {
 	{"addressbookeditpersonwin_height", "-1", &prefs_common.addressbookeditpersonwin_height, P_INT,
 	 NULL, NULL, NULL},
 
+	{"addressbookeditgroupwin_width", "580", &prefs_common.addressbookeditgroupwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"addressbookeditgroupwin_height", "340", &prefs_common.addressbookeditgroupwin_height, P_INT,
+	 NULL, NULL, NULL},
+
 	{"pluginswin_width", "480", &prefs_common.pluginswin_width, P_INT,
 	 NULL, NULL, NULL},
 	{"pluginswin_height", "-1", &prefs_common.pluginswin_height, P_INT,
@@ -716,6 +761,11 @@ static PrefParam param[] = {
 	{"addressaddwin_height", "-1", &prefs_common.addressaddwin_height, P_INT,
 	 NULL, NULL, NULL},
 
+	{"addressbook_folderselwin_width", "300", &prefs_common.addressbook_folderselwin_width, P_INT,
+	 NULL, NULL, NULL},
+	{"addressbook_folderselwin_height", "-1", &prefs_common.addressbook_folderselwin_height, P_INT,
+	 NULL, NULL, NULL},
+
 	/* Hidden */
 	{"warn_dnd", "1", &prefs_common.warn_dnd, P_INT,
 	 NULL, NULL, NULL},
@@ -724,7 +774,13 @@ static PrefParam param[] = {
 	 NULL, NULL, NULL},
 	{"enable_swap_from", "FALSE", &prefs_common.swap_from, P_BOOL,
 	 NULL, NULL, NULL},
-	{"enable_rules_hint", "TRUE", &prefs_common.enable_rules_hint, P_BOOL,
+	{"use_stripes_everywhere", "TRUE", &prefs_common.use_stripes_everywhere, P_BOOL,
+	 NULL, NULL, NULL},
+	{"use_stripes_in_summaries", "TRUE", &prefs_common.use_stripes_in_summaries, P_BOOL,
+	 NULL, NULL, NULL},
+	{"stripes_color_offset", "4000", &prefs_common.stripes_color_offset, P_INT,
+	 NULL, NULL, NULL},
+	{"enable_dotted_lines", "FALSE", &prefs_common.enable_dotted_lines, P_BOOL,
 	 NULL, NULL, NULL},
 	{"enable_hscrollbar", "TRUE", &prefs_common.enable_hscrollbar, P_BOOL,
 	 NULL, NULL, NULL},
@@ -744,8 +800,18 @@ static PrefParam param[] = {
 	P_INT, NULL, NULL, NULL },
 	{"summary_quicksearch_sticky", "1", &prefs_common.summary_quicksearch_sticky, P_INT,
 	 NULL, NULL, NULL},
+	{"summary_quicksearch_dynamic", "0", &prefs_common.summary_quicksearch_dynamic, P_INT,
+	 NULL, NULL, NULL},
 	{"statusbar_update_step", "10", &prefs_common.statusbar_update_step, P_INT,
 	 NULL, NULL, NULL},
+	{"compose_no_markup", "FALSE", &prefs_common.compose_no_markup, P_BOOL,
+	 NULL, NULL, NULL},
+	{"skip_ssl_cert_check", "FALSE", &prefs_common.skip_ssl_cert_check, P_BOOL,
+	 NULL, NULL, NULL},
+	{"live_dangerously", "FALSE", &prefs_common.live_dangerously, P_BOOL,
+	 NULL, NULL, NULL},
+	{"hide_quotes", "0", &prefs_common.hide_quotes, P_INT,
+	 NULL, NULL, NULL},
 
 	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
 };
@@ -857,30 +923,58 @@ void prefs_common_write_config(void)
 		prefs_common.summary_quicksearch_history);
 }
 
-void pref_set_textview_from_pref(GtkTextView *textview, gchar *txt)
+/* make a copy of string 'in' into buffer 'out'. un-escape \ sequences.
+   both 'in' and 'out' must be non-NULL.
+   'out' must be a pointer to a buffer whose size is at least equal
+   to strlen(txt)+1, this buffer will get cleared. */
+void pref_get_unescaped_pref(gchar *out, const gchar *in)
 {
-	GtkTextBuffer *buffer = gtk_text_view_get_buffer(textview);
-	gchar *o_out, *out = malloc(txt?(strlen(txt)+1):1);
-	gchar *t = txt;
-	memset(out, 0, strlen(txt)+1);
-	o_out = out;
-	while (*t != '\0') {
-		if (*t == '\\' && *(t+1) == 'n') {
-			*out++ = '\n';
-			t++;
-		} else if (*t == '\\') {
-			t++;
-		} else {
-			*out++ = *t;
-		}
-		t++;
-	}
-	*out='\0';
+	gchar *o, *i;
 
-	gtk_text_buffer_set_text(buffer, o_out?o_out:"", -1);
-	g_free(o_out);
+	g_return_if_fail( in != NULL );
+	g_return_if_fail( out != NULL );
+
+	i = in;
+	o = out;
+	memset(out, 0, strlen(in)+1);
+	while (*i != '\0') {
+		if (*i == '\\' && *(i+1) == 'n') {
+			*o++ = '\n';
+			i++;
+		} else if (*i == '\\' && *(i+1) == 't') {
+			*o++ = '\t';
+			i++;
+		} else if (*i == '\\' && *(i+1) == '\\') {
+			*o++ = '\\';
+			i++;
+		} else {
+			*o++ = *i;
+		}
+		i++;
+	}
+	*o='\0';
 }
 
+/* set the contents of a textview widget from the internal \-escaped
+  representation of a pref string */
+void pref_set_textview_from_pref(GtkTextView *textview, const gchar *txt)
+{
+	GtkTextBuffer *out_buffer;
+	gchar *out = NULL;
+
+	g_return_if_fail( txt != NULL );
+
+	out_buffer = gtk_text_view_get_buffer(textview);
+	out = malloc(txt?(strlen(txt)+1):1);
+
+	pref_get_unescaped_pref(out, txt);
+
+	gtk_text_buffer_set_text(out_buffer, out?out:"", -1);
+	g_free(out);
+}
+
+/* get the \-escaped internal representation of a pref from the contents of
+   a textview widget */
 gchar *pref_get_pref_from_textview(GtkTextView *textview) 
 {
 	GtkTextBuffer *buffer;
@@ -898,6 +992,9 @@ gchar *pref_get_pref_from_textview(GtkTextView *textview)
 		if (*t == '\n') {
 			*out++ = '\\';
 			*out++ = 'n';
+		} else if (*t == '\t') {
+			*out++ = '\\';
+			*out++ = 't';
 		} else if (*t == '\\') {
 			*out++ = '\\';
 			*out++ = '\\';
