@@ -42,6 +42,12 @@ typedef enum {
 	RECV_DIALOG_NEVER
 } RecvDialogMode;
 
+typedef enum {
+	COMPOSE_DND_ASK,
+	COMPOSE_DND_INSERT,
+	COMPOSE_DND_ATTACH
+} ComposeDndInsertOrAttach;
+
 typedef enum {
 	CTE_AUTO,
 	CTE_BASE64,
@@ -57,8 +63,14 @@ typedef enum {
 
 typedef enum
 {
-	SELECTONENTRY_UNREAD = 1 << 0,
-	SELECTONENTRY_NEW    = 1 << 1
+/* U = unread, N = new, M = marked */
+	SELECTONENTRY_NOTHING,
+	SELECTONENTRY_UNM,
+	SELECTONENTRY_UMN,
+	SELECTONENTRY_NUM,
+	SELECTONENTRY_NMU,
+	SELECTONENTRY_MNU,
+	SELECTONENTRY_MUN
 } SelectOnEntry;
 
 typedef enum
@@ -82,6 +94,8 @@ struct _PrefsCommon
  	gboolean newmail_notify_manu;
  	gchar   *newmail_notify_cmd;
 	RecvDialogMode recv_dialog_mode;
+	gint receivewin_width;
+	gint receivewin_height;
 	gboolean close_recv_dialog;
 	gboolean no_recv_err_panel;
 
@@ -89,6 +103,8 @@ struct _PrefsCommon
 	gboolean savemsg;
 	gboolean confirm_send_queued_messages;
 	SendDialogMode send_dialog_mode;
+	gint sendwin_width;
+	gint sendwin_height;
 	gchar *outgoing_charset;
 	TransferEncodingMethod encoding_method;
 
@@ -108,7 +124,9 @@ struct _PrefsCommon
 	gboolean show_ruler;
 	gboolean autosave;
 	gint autosave_length;
-	
+	gboolean compose_no_markup;
+	ComposeDndInsertOrAttach compose_dnd_mode;
+
 	/* Quote */
 	gboolean reply_with_quote;
 	gchar *quotemark;
@@ -142,6 +160,7 @@ struct _PrefsCommon
 	/* new fonts */
 	gchar *widgetfont;
 	gchar *textfont;
+        gchar *printfont;
 	gchar *normalfont;
 	gchar *smallfont;
 	gchar *titlefont;
@@ -164,7 +183,10 @@ struct _PrefsCommon
 	gboolean use_addr_book;
 	gchar *date_format;
 
-	gboolean enable_rules_hint;
+	gboolean use_stripes_everywhere;
+	gboolean use_stripes_in_summaries; /* overrides if use_stripes_everywhere is set to TRUE */
+	gint stripes_color_offset;
+	gboolean enable_dotted_lines;
 	gboolean enable_hscrollbar;
 	gboolean bold_unread;
 	gboolean enable_thread;
@@ -234,9 +256,13 @@ struct _PrefsCommon
 
 	/* Message */
 	gboolean enable_color;
+	gboolean enable_bgcolor;
 	gulong quote_level1_col;
 	gulong quote_level2_col;
 	gulong quote_level3_col;
+	gulong quote_level1_bgcol;
+	gulong quote_level2_bgcol;
+	gulong quote_level3_bgcol;
 	gulong uri_col;
 	gulong tgt_folder_col;
 	gulong signature_col;
@@ -251,15 +277,16 @@ struct _PrefsCommon
 	gboolean enable_smooth_scroll;
 	gint scroll_step;
 	gboolean scroll_halfpage;
+	gboolean respect_flowed_format;
 
 	gboolean show_other_header;
+	gboolean use_different_print_font;
 	GSList *disphdr_list;
 
 	gboolean attach_desc;
 
 	/* MIME viewer */
-	gchar *mime_image_viewer;
-	gchar *mime_audio_player;
+	gchar *mime_textviewer;
 	gchar *mime_open_cmd;
 	gchar *attach_save_dir;
 	gchar *attach_load_dir;
@@ -291,16 +318,23 @@ struct _PrefsCommon
 	gchar *pixmap_theme_path;
 	int hover_timeout; /* msecs mouse hover timeout */
 	gboolean ask_mark_all_read;
+	gboolean ask_apply_per_account_filtering_rules;
+	gint apply_per_account_filtering_rules;
 
 	/* Other */
 	gchar *uri_cmd;
 	gchar *print_cmd;
 	gchar *ext_editor_cmd;
 
-        gboolean cliplog;
-        guint loglength;
+    gboolean cliplog;
+    guint loglength;
+	gulong log_msg_color;
+	gulong log_warn_color;
+	gulong log_error_color;
+	gulong log_in_color;
+	gulong log_out_color;
 
-        gboolean confirm_on_exit;
+    gboolean confirm_on_exit;
 	gboolean clean_on_exit;
 	gboolean ask_on_clean;
 	gboolean warn_queued_on_exit;
@@ -318,6 +352,7 @@ struct _PrefsCommon
 	gint summary_quicksearch_type;
 	gint summary_quicksearch_sticky;
 	gint summary_quicksearch_recurse;
+	gint summary_quicksearch_dynamic;
 	gulong color_new;
 	
 	GList *summary_quicksearch_history;
@@ -336,6 +371,8 @@ struct _PrefsCommon
 	gint addressbookwin_height;
 	gint addressbookeditpersonwin_width;
 	gint addressbookeditpersonwin_height;
+	gint addressbookeditgroupwin_width;
+	gint addressbookeditgroupwin_height;
 	gint pluginswin_width;
 	gint pluginswin_height;
 	gint prefswin_width;
@@ -352,9 +389,15 @@ struct _PrefsCommon
 	gint folderselwin_height;
 	gint addressaddwin_width;
 	gint addressaddwin_height;
+	gint addressbook_folderselwin_width;
+	gint addressbook_folderselwin_height;
 
 	gint warn_dnd;
 	gint broken_are_utf8;
+	gint skip_ssl_cert_check;
+	gint live_dangerously;
+	gint never_send_retrcpt;
+	gint hide_quotes;
 };
 
 extern PrefsCommon prefs_common;
@@ -363,7 +406,8 @@ void prefs_common_read_config	(void);
 void prefs_common_write_config	(void);
 void prefs_common_open		(void);
 PrefsCommon *prefs_common_get	(void);
-void pref_set_textview_from_pref(GtkTextView *textview, gchar *txt);
+void pref_get_unescaped_pref(gchar *out, const gchar *in);
+void pref_set_textview_from_pref(GtkTextView *textview, const gchar *txt);
 gchar *pref_get_pref_from_textview(GtkTextView *textview);
 
 #endif /* __PREFS_COMMON_H__ */
