@@ -36,6 +36,7 @@
 
 #include "gtk/gtkutils.h"
 #include "gtk/prefswindow.h"
+#include "gtk/menu.h"
 
 #include "manage_window.h"
 
@@ -56,6 +57,7 @@ typedef struct _WritingPage
 	GtkWidget *hbox_autosave;
 	GtkWidget *checkbtn_autosave;
 	GtkWidget *spinbtn_autosave_length;
+	GtkWidget *optmenu_dnd_insert_or_attach;
 } WritingPage;
 
 void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window, 
@@ -68,7 +70,7 @@ void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window,
 
 	GtkWidget *checkbtn_autoextedit;
 
-	GtkWidget *frame_autosel;
+	GtkWidget *frame;
 	GtkWidget *hbox_autosel;
 	GtkWidget *checkbtn_reply_account_autosel;
 	GtkWidget *checkbtn_forward_account_autosel;
@@ -90,34 +92,37 @@ void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window,
 	GtkWidget *spinbtn_autosave_length;
 	GtkWidget *label_autosave_length;
 	
+	GtkWidget *hbox_dnd_insert_or_attach;
+	GtkWidget *label_dnd_insert_or_attach;
+	GtkWidget *optmenu_dnd_insert_or_attach;
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), VBOX_BORDER);
 
-        /* Account autoselection */
-	PACK_FRAME(vbox1, frame_autosel, _("Automatic account selection"));
+	/* Account autoselection */
+	PACK_FRAME(vbox1, frame, _("Automatic account selection"));
 
-	hbox_autosel = gtk_hbox_new (FALSE, VSPACING_NARROW);
+	hbox_autosel = gtk_hbox_new (TRUE, VSPACING_NARROW);
 	gtk_widget_show (hbox_autosel);
-	gtk_container_add (GTK_CONTAINER (frame_autosel), hbox_autosel);
+	gtk_container_add (GTK_CONTAINER (frame), hbox_autosel);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox_autosel), 8);
 
-        PACK_CHECK_BUTTON (hbox_autosel, checkbtn_reply_account_autosel,
+	PACK_CHECK_BUTTON (hbox_autosel, checkbtn_reply_account_autosel,
 			   _("when replying"));
 	PACK_CHECK_BUTTON (hbox_autosel, checkbtn_forward_account_autosel,
 			   _("when forwarding"));
 	PACK_CHECK_BUTTON (hbox_autosel, checkbtn_reedit_account_autosel,
 			   _("when re-editing"));
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
+	PACK_FRAME(vbox1, frame, _("Forwarding"));
+	
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
 	gtk_widget_show (vbox2);
-	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
-
-	PACK_CHECK_BUTTON (vbox2, checkbtn_default_reply_list,
-			   _("Reply button invokes mailing list reply"));
-
-	PACK_CHECK_BUTTON (vbox2, checkbtn_autoextedit,
-			   _("Automatically launch the external editor"));
+	gtk_container_add (GTK_CONTAINER (frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);	
 
 	PACK_CHECK_BUTTON (vbox2, checkbtn_forward_as_attachment,
 			   _("Forward as attachment"));
@@ -125,12 +130,22 @@ void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window,
 	PACK_CHECK_BUTTON (vbox2, checkbtn_redirect_keep_from,
 			   _("Keep the original 'From' header when redirecting"));
 
+	PACK_FRAME(vbox1, frame, _("Editing"));
+
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	PACK_CHECK_BUTTON (vbox2, checkbtn_autoextedit,
+			   _("Automatically launch the external editor"));
+
 	hbox_autosave = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox_autosave);
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox_autosave, FALSE, FALSE, 0);
 
 	PACK_CHECK_BUTTON (hbox_autosave, checkbtn_autosave,
-			   _("Autosave to Drafts folder every"));
+			   _("Autosave message text to Drafts folder every"));
 
 	spinbtn_autosave_adj = gtk_adjustment_new (50, 0, 1000, 1, 10, 10);
 	spinbtn_autosave_length = gtk_spin_button_new
@@ -159,13 +174,39 @@ void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_box_pack_start (GTK_BOX (hbox_undolevel), spinbtn_undolevel, FALSE, FALSE, 0);
 	gtk_widget_set_size_request (spinbtn_undolevel, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_undolevel), TRUE);
+	
+	PACK_CHECK_BUTTON (vbox1, checkbtn_default_reply_list,
+			   _("Reply button invokes mailing list reply"));
 
+	/* dnd insert or attach */
+	label_dnd_insert_or_attach = gtk_label_new (_("When dropping files into the Compose window"));
+	gtk_misc_set_alignment(GTK_MISC(label_dnd_insert_or_attach), 0, 0.5);
+	gtk_widget_show (label_dnd_insert_or_attach);
+
+	optmenu_dnd_insert_or_attach = gtk_option_menu_new ();
+	gtk_widget_show (optmenu_dnd_insert_or_attach);
+
+	menu = gtk_menu_new ();
+	MENUITEM_ADD (menu, menuitem, _("Ask"), COMPOSE_DND_ASK);
+	MENUITEM_ADD (menu, menuitem, _("Insert"), COMPOSE_DND_INSERT);
+	MENUITEM_ADD (menu, menuitem, _("Attach"), COMPOSE_DND_ATTACH);
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu_dnd_insert_or_attach), menu);
+
+	hbox_dnd_insert_or_attach = gtk_hbox_new(FALSE, 20);
+	gtk_widget_show(hbox_dnd_insert_or_attach);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox_dnd_insert_or_attach, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox_dnd_insert_or_attach),
+			label_dnd_insert_or_attach, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox_dnd_insert_or_attach),
+			optmenu_dnd_insert_or_attach, FALSE, FALSE, 0);
+	
 	SET_TOGGLE_SENSITIVITY (checkbtn_autosave, spinbtn_autosave_length);
 	SET_TOGGLE_SENSITIVITY (checkbtn_autosave, label_autosave_length);
 
 	prefs_writing->checkbtn_autoextedit = checkbtn_autoextedit;
 
-        prefs_writing->checkbtn_reply_account_autosel   = checkbtn_reply_account_autosel;
+	prefs_writing->checkbtn_reply_account_autosel   = checkbtn_reply_account_autosel;
 	prefs_writing->checkbtn_forward_account_autosel = checkbtn_forward_account_autosel;
 	prefs_writing->checkbtn_reedit_account_autosel  = checkbtn_reedit_account_autosel;
 
@@ -180,6 +221,8 @@ void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window,
 		checkbtn_redirect_keep_from;
 	prefs_writing->checkbtn_default_reply_list = checkbtn_default_reply_list;
 
+	prefs_writing->optmenu_dnd_insert_or_attach = optmenu_dnd_insert_or_attach;
+
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_writing->checkbtn_autoextedit),
 		prefs_common.auto_exteditor);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_writing->checkbtn_forward_as_attachment),
@@ -200,12 +243,17 @@ void prefs_compose_writing_create_widget(PrefsPage *_page, GtkWindow *window,
 		prefs_common.reedit_account_autosel);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prefs_writing->checkbtn_default_reply_list),
 		prefs_common.default_reply_list);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(optmenu_dnd_insert_or_attach),
+		prefs_common.compose_dnd_mode);
 
 	prefs_writing->page.widget = vbox1;
 }
 
 void prefs_compose_writing_save(PrefsPage *_page)
 {
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+
 	WritingPage *page = (WritingPage *) _page;
 	prefs_common.auto_exteditor = 
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_autoextedit));
@@ -229,6 +277,10 @@ void prefs_compose_writing_save(PrefsPage *_page)
 	prefs_common.default_reply_list =
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_default_reply_list));
 	
+	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(page->optmenu_dnd_insert_or_attach));
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	prefs_common.compose_dnd_mode = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 }
 
 static void prefs_compose_writing_destroy_widget(PrefsPage *_page)
