@@ -458,7 +458,11 @@ static void prefs_filtering_action_create(void)
 			  G_CALLBACK(prefs_filtering_action_select_dest),
 			  NULL);
 
-	exec_btn = gtk_button_new_with_label (_("Info ..."));
+#if GTK_CHECK_VERSION(2, 8, 0)
+	exec_btn = gtk_button_new_from_stock(GTK_STOCK_INFO);
+#else
+	exec_btn = gtk_button_new_with_label (_("Info..."));
+#endif
 	gtk_widget_show (exec_btn);
 	gtk_box_pack_start (GTK_BOX (hbox1), exec_btn, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (exec_btn), "clicked",
@@ -486,7 +490,7 @@ static void prefs_filtering_action_create(void)
 	g_signal_connect(G_OBJECT(reg_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_action_register_cb), NULL);
 
-	subst_btn = gtk_button_new_with_label(_("  Replace  "));
+	subst_btn = gtkut_get_replace_btn(_("Replace"));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), subst_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(subst_btn), "clicked",
@@ -578,7 +582,7 @@ static void prefs_filtering_action_list_view_set_row(GtkTreeIter *row,
 	if (row == NULL && action == NULL) {
 		prefs_filtering_action_list_view_insert_action
 			(filtering_action.action_list_view,
-			 NULL, _("New"), FALSE);
+			 NULL, _("(New)"), FALSE);
 		return;
 	}			 
 
@@ -1134,11 +1138,15 @@ static void prefs_filtering_action_type_selection_changed(GtkList *list,
 		||  filtering_action.current_action == ACTION_REDIRECT) {
 			debug_print("unregistering address completion entry\n");
 			address_completion_unregister_entry(GTK_ENTRY(filtering_action.dest_entry));
+			address_completion_end(filtering_action.window);
 		}
 		if (value == ACTION_FORWARD || value == ACTION_FORWARD_AS_ATTACHMENT
 		||  value == ACTION_REDIRECT) {
 			debug_print("registering address completion entry\n");
-			address_completion_register_entry(GTK_ENTRY(filtering_action.dest_entry));
+			address_completion_start(filtering_action.window);
+			address_completion_register_entry(
+					GTK_ENTRY(filtering_action.dest_entry),
+					TRUE);
 		}
 		filtering_action.current_action = value;
 	}
@@ -1377,7 +1385,7 @@ static GtkWidget *prefs_filtering_action_list_view_create(void)
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);	
 	
-	gtk_tree_view_set_rules_hint(list_view, prefs_common.enable_rules_hint);
+	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
