@@ -32,6 +32,7 @@
 #include "folder.h"
 #include "prefs_folder_item.h"
 #include "folderview.h"
+#include "folder.h"
 #include "summaryview.h"
 #include "menu.h"
 #include "account.h"
@@ -44,6 +45,7 @@
 #include "filtering.h"
 #include "folder_item_prefs.h"
 #include "gtk/colorsel.h"
+#include "string_match.h"
 
 #if USE_ASPELL
 #include "gtkaspell.h"
@@ -57,6 +59,7 @@
 
 typedef struct _FolderItemGeneralPage FolderItemGeneralPage;
 typedef struct _FolderItemComposePage FolderItemComposePage;
+static gboolean can_save = TRUE;
 
 struct _FolderItemGeneralPage
 {
@@ -65,8 +68,12 @@ struct _FolderItemGeneralPage
 	FolderItem *item;
 
 	GtkWidget *table;
+	GtkWidget *no_save_warning;
+	GtkWidget *folder_type;
 	GtkWidget *checkbtn_simplify_subject;
 	GtkWidget *entry_simplify_subject;
+	GtkWidget *entry_regexp_test_string;
+	GtkWidget *entry_regexp_test_result;
 	GtkWidget *checkbtn_folder_chmod;
 	GtkWidget *entry_folder_chmod;
 	GtkWidget *folder_color_btn;
@@ -74,7 +81,7 @@ struct _FolderItemGeneralPage
 	GtkWidget *checkbtn_newmailcheck;
 	GtkWidget *checkbtn_offlinesync;
 
-	/* appy to sub folders */
+	/* apply to sub folders */
 	GtkWidget *simplify_subject_rec_checkbtn;
 	GtkWidget *folder_chmod_rec_checkbtn;
 	GtkWidget *folder_color_rec_checkbtn;
@@ -93,6 +100,7 @@ struct _FolderItemComposePage
 
 	GtkWidget *window;
 	GtkWidget *table;
+	GtkWidget *no_save_warning;
 	GtkWidget *checkbtn_request_return_receipt;
 	GtkWidget *checkbtn_save_copy_to_folder;
 	GtkWidget *checkbtn_default_to;
@@ -115,7 +123,6 @@ struct _FolderItemComposePage
 #if USE_ASPELL
 	GtkWidget *default_dictionary_rec_checkbtn;
 #endif
-
 };
 
 
@@ -128,6 +135,8 @@ static gboolean compose_save_recurse_func(GNode *node, gpointer data);
 gint prefs_folder_item_chmod_mode		(gchar *folder_chmod);
 
 static void folder_color_set_dialog(GtkWidget *widget, gpointer data);
+static void folder_regexp_test_cb(GtkWidget *widget, gpointer data);
+static void folder_regexp_set_subject_example_cb(GtkWidget *widget, gpointer data);
 
 #define SAFE_STRING(str) \
 	(str) ? (str) : ""
@@ -140,12 +149,26 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	FolderItem *item = (FolderItem *) data;
 	guint rowcount;
 
+
 	GtkWidget *table;
 	GtkWidget *hbox;
 	GtkWidget *label;
 	
+	GtkWidget *folder_type_menu;
+	GtkWidget *folder_type;
+	GtkWidget *dummy_chkbtn;
+	GtkWidget *menuitem;
+	SpecialFolderItemType type;
+	
+	GtkWidget *no_save_warning = NULL;
 	GtkWidget *checkbtn_simplify_subject;
 	GtkWidget *entry_simplify_subject;
+	GtkWidget *hbox_regexp;
+	GtkWidget *label_regexp_test;
+	GtkWidget *entry_regexp_test_string;
+	GtkWidget *left_arrow;
+	GtkWidget *entry_regexp_test_result;
+
 	GtkWidget *checkbtn_folder_chmod;
 	GtkWidget *entry_folder_chmod;
 	GtkWidget *folder_color;
@@ -160,44 +183,145 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	GtkWidget *enable_processing_rec_checkbtn;
 	GtkWidget *newmailcheck_rec_checkbtn;
 	GtkWidget *offlinesync_rec_checkbtn;
+	GtkTooltips *tooltips;
 
 	page->item	   = item;
 
 	/* Table */
-	table = gtk_table_new(6, 2, FALSE);
+	table = gtk_table_new(7, 4, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (table), VBOX_BORDER);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
 	rowcount = 0;
 
+	if (!can_save) {
+		no_save_warning = gtk_label_new(
+			_("<i>These preferences will not be saved as this folder "
+			"is a top-level one. However you can use them to set them "
+			"to the whole mailbox tree using \"Apply to subfolders\".</i>"));
+		gtk_label_set_use_markup(GTK_LABEL(no_save_warning), TRUE);
+		gtk_label_set_line_wrap(GTK_LABEL(no_save_warning), TRUE);
+		gtk_misc_set_alignment(GTK_MISC(no_save_warning), 0.0, 0.5);
+		gtk_table_attach(GTK_TABLE(table), no_save_warning, 0, 4,
+			 rowcount, rowcount + 1, GTK_FILL, 0, 0, 0);
+		rowcount++;
+	}
+	
 	/* Apply to subfolders */
 	label = gtk_label_new(_("Apply to\nsubfolders"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 2, 3,
+	gtk_table_attach(GTK_TABLE(table), label, 3, 4,
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 	rowcount++;
 
+	/* folder_type */
+	folder_type = gtk_option_menu_new ();
+	gtk_widget_show (folder_type);
+
+	type = F_NORMAL;
+	if (item->stype == F_INBOX)
+		type = F_INBOX;
+	else if (folder_has_parent_of_type(item, F_OUTBOX))
+		type = F_OUTBOX;
+	else if (folder_has_parent_of_type(item, F_DRAFT))
+		type = F_DRAFT;
+	else if (folder_has_parent_of_type(item, F_QUEUE))
+		type = F_QUEUE;
+	else if (folder_has_parent_of_type(item, F_TRASH))
+		type = F_TRASH;
+
+	folder_type_menu = gtk_menu_new ();
+
+	MENUITEM_ADD (folder_type_menu, menuitem, _("Normal"),  F_NORMAL);
+	MENUITEM_ADD (folder_type_menu, menuitem, _("Inbox"),  F_INBOX);
+	MENUITEM_ADD (folder_type_menu, menuitem, _("Outbox"),  F_OUTBOX);
+	MENUITEM_ADD (folder_type_menu, menuitem, _("Drafts"),  F_DRAFT);
+	MENUITEM_ADD (folder_type_menu, menuitem, _("Queue"),  F_QUEUE);
+	MENUITEM_ADD (folder_type_menu, menuitem, _("Trash"),  F_TRASH);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (folder_type), folder_type_menu);
+
+	gtk_option_menu_set_history(GTK_OPTION_MENU(folder_type), type);
+
+	dummy_chkbtn = gtk_check_button_new();
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dummy_chkbtn), type != F_INBOX);
+	gtk_widget_set_sensitive(dummy_chkbtn, FALSE);
+
+	if (type == item->stype && type == F_NORMAL)
+		gtk_widget_set_sensitive(folder_type, TRUE);
+	else
+		gtk_widget_set_sensitive(folder_type, FALSE);
+
+	label = gtk_label_new(_("Folder type"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 
+			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), folder_type, 1, 2, 
+			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), dummy_chkbtn, 3, 4, 
+			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
+
+	rowcount++;
+
 	/* Simplify Subject */
-	checkbtn_simplify_subject = gtk_check_button_new_with_label(_("Simplify Subject RegExp: "));
+	checkbtn_simplify_subject = gtk_check_button_new_with_label(_("Simplify Subject RegExp"));
 	gtk_table_attach(GTK_TABLE(table), checkbtn_simplify_subject, 0, 1, 
 			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_simplify_subject), 
 				     item->prefs->enable_simplify_subject);
 
+	g_signal_connect(G_OBJECT(checkbtn_simplify_subject), "toggled",
+			G_CALLBACK(folder_regexp_set_subject_example_cb), page);
+
 	entry_simplify_subject = gtk_entry_new();
-	gtk_table_attach(GTK_TABLE(table), entry_simplify_subject, 1, 2, 
+	gtk_table_attach(GTK_TABLE(table), entry_simplify_subject, 1, 3, 
 			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 	SET_TOGGLE_SENSITIVITY(checkbtn_simplify_subject, entry_simplify_subject);
 	gtk_entry_set_text(GTK_ENTRY(entry_simplify_subject), 
 			   SAFE_STRING(item->prefs->simplify_subject_regexp));
 
+	g_signal_connect(G_OBJECT(entry_simplify_subject), "changed",
+			G_CALLBACK(folder_regexp_test_cb), page);
+
 	simplify_subject_rec_checkbtn = gtk_check_button_new();
-	gtk_table_attach(GTK_TABLE(table), simplify_subject_rec_checkbtn, 2, 3, 
+	gtk_table_attach(GTK_TABLE(table), simplify_subject_rec_checkbtn, 3, 4, 
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	rowcount++;
 
+	/* Test RegExp */
+	label_regexp_test = gtk_label_new(_("Test RegExp"));
+	gtk_misc_set_alignment(GTK_MISC(label_regexp_test), 0, 0.5);
+	gtk_table_attach(GTK_TABLE(table), label_regexp_test, 0, 1, 
+			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+	SET_TOGGLE_SENSITIVITY(checkbtn_simplify_subject, label_regexp_test);
+
+	entry_regexp_test_string = gtk_entry_new();
+	gtk_table_attach(GTK_TABLE(table), entry_regexp_test_string, 1, 2, 
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+	SET_TOGGLE_SENSITIVITY(checkbtn_simplify_subject, entry_regexp_test_string);
+
+	g_signal_connect(G_OBJECT(entry_regexp_test_string), "changed",
+			G_CALLBACK(folder_regexp_test_cb), page);
+
+	hbox_regexp = gtk_hbox_new (FALSE, 4);
+	gtk_widget_show (hbox_regexp);
+	gtk_table_attach(GTK_TABLE(table), hbox_regexp, 2, 3, 
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+
+	left_arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
+	gtk_widget_show(left_arrow);
+	gtk_box_pack_start (GTK_BOX(hbox_regexp), left_arrow, FALSE, FALSE, 0);
+	SET_TOGGLE_SENSITIVITY(checkbtn_simplify_subject, left_arrow);
+
+	entry_regexp_test_result = gtk_entry_new();
+	gtk_box_pack_end (GTK_BOX(hbox_regexp), entry_regexp_test_result, TRUE, TRUE, 0);
+	SET_TOGGLE_SENSITIVITY(checkbtn_simplify_subject, entry_regexp_test_result);
+	gtk_entry_set_editable(GTK_ENTRY(entry_regexp_test_result), FALSE);
+
+	rowcount++;
+
 	/* Folder chmod */
-	checkbtn_folder_chmod = gtk_check_button_new_with_label(_("Folder chmod: "));
+	checkbtn_folder_chmod = gtk_check_button_new_with_label(_("Folder chmod"));
 	gtk_table_attach(GTK_TABLE(table), checkbtn_folder_chmod, 0, 1, 
 			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 
@@ -217,13 +341,13 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	}
 	
 	folder_chmod_rec_checkbtn = gtk_check_button_new();
-	gtk_table_attach(GTK_TABLE(table), folder_chmod_rec_checkbtn, 2, 3, 
+	gtk_table_attach(GTK_TABLE(table), folder_chmod_rec_checkbtn, 3, 4, 
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	rowcount++;
 	
 	/* Folder color */
-	folder_color = gtk_label_new(_("Folder color: "));
+	folder_color = gtk_label_new(_("Folder color"));
 	gtk_misc_set_alignment(GTK_MISC(folder_color), 0, 0.5);
 	gtk_table_attach(GTK_TABLE(table), folder_color, 0, 1, 
 			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
@@ -235,6 +359,9 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	folder_color_btn = gtk_button_new_with_label("");
 	gtk_widget_set_size_request(folder_color_btn, 36, 26);
   	gtk_box_pack_start (GTK_BOX(hbox), folder_color_btn, FALSE, FALSE, 0);
+	tooltips = gtk_tooltips_new();
+	gtk_tooltips_set_tip(tooltips, folder_color_btn,
+			     _("Pick color for folder"), NULL);
 
 	page->folder_color = item->prefs->color;
 
@@ -245,7 +372,7 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	gtkut_set_widget_bgcolor_rgb(folder_color_btn, item->prefs->color);
 
 	folder_color_rec_checkbtn = gtk_check_button_new();
-	gtk_table_attach(GTK_TABLE(table), folder_color_rec_checkbtn, 2, 3, 
+	gtk_table_attach(GTK_TABLE(table), folder_color_rec_checkbtn, 3, 4, 
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	rowcount++;
@@ -259,7 +386,7 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 				     item->prefs->enable_processing);
 
 	enable_processing_rec_checkbtn = gtk_check_button_new();
-	gtk_table_attach(GTK_TABLE(table), enable_processing_rec_checkbtn, 2, 3, 
+	gtk_table_attach(GTK_TABLE(table), enable_processing_rec_checkbtn, 3, 4, 
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 	
 	rowcount++;
@@ -272,7 +399,7 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_newmailcheck),
 								 item->prefs->newmailcheck);
 	newmailcheck_rec_checkbtn = gtk_check_button_new();
-	gtk_table_attach(GTK_TABLE(table), newmailcheck_rec_checkbtn, 2, 3, 
+	gtk_table_attach(GTK_TABLE(table), newmailcheck_rec_checkbtn, 3, 4, 
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	rowcount++;
@@ -283,7 +410,7 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 			 rowcount, rowcount+1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 	
 	offlinesync_rec_checkbtn = gtk_check_button_new();
-	gtk_table_attach(GTK_TABLE(table), offlinesync_rec_checkbtn, 2, 3, 
+	gtk_table_attach(GTK_TABLE(table), offlinesync_rec_checkbtn, 3, 4, 
 			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
 	if (item->folder && (item->folder->klass->type != F_IMAP && 
@@ -302,8 +429,12 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	gtk_widget_show_all(table);
 
 	page->table = table;
+	page->folder_type = folder_type;
+	page->no_save_warning = no_save_warning;
 	page->checkbtn_simplify_subject = checkbtn_simplify_subject;
 	page->entry_simplify_subject = entry_simplify_subject;
+	page->entry_regexp_test_string = entry_regexp_test_string;
+	page->entry_regexp_test_result = entry_regexp_test_result;
 	page->checkbtn_folder_chmod = checkbtn_folder_chmod;
 	page->entry_folder_chmod = entry_folder_chmod;
 	page->folder_color_btn = folder_color_btn;
@@ -319,6 +450,8 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	page->offlinesync_rec_checkbtn	     = offlinesync_rec_checkbtn;
 
 	page->page.widget = table;
+
+	folder_regexp_set_subject_example_cb(NULL, page);
 }
 
 void prefs_folder_item_general_destroy_widget_func(PrefsPage *page_) 
@@ -329,19 +462,33 @@ void prefs_folder_item_general_destroy_widget_func(PrefsPage *page_)
 /** \brief  Save the prefs in page to folder.
  *
  *  If the folder is not the one  specified in page->item, then only those properties 
- *  that have the relevant 'appy to sub folders' button checked are saved
+ *  that have the relevant 'apply to sub folders' button checked are saved
  */
 static void general_save_folder_prefs(FolderItem *folder, FolderItemGeneralPage *page)
 {
 	FolderItemPrefs *prefs = folder->prefs;
 	gchar *buf;
 	gboolean all = FALSE;
+	SpecialFolderItemType type = F_NORMAL;
+	GtkWidget *menu;
+	GtkWidget *menuitem;
+
+	if (folder->path == NULL)
+		return;
 
 	g_return_if_fail(prefs != NULL);
 
 	if (page->item == folder) 
 		all = TRUE;
 
+	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(page->folder_type));
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	type = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
+	if (all && folder->stype != type) {
+		folder_item_change_type(folder, type);
+	}
+
 	if (all || gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->simplify_subject_rec_checkbtn))) {
 		prefs->enable_simplify_subject =
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_simplify_subject));
@@ -439,6 +586,7 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 	GtkWidget *table;
 	GtkWidget *label;
 	
+	GtkWidget *no_save_warning = NULL;
 	GtkWidget *checkbtn_request_return_receipt = NULL;
 	GtkWidget *checkbtn_save_copy_to_folder = NULL;
 	GtkWidget *checkbtn_default_to = NULL;
@@ -477,15 +625,29 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 
 	/* Table */
 #if USE_ASPELL
-# define TABLEHEIGHT 6
+# define TABLEHEIGHT 7
 #else
-# define TABLEHEIGHT 5
+# define TABLEHEIGHT 6
 #endif
-	table = gtk_table_new(TABLEHEIGHT, 2, FALSE);
+	table = gtk_table_new(TABLEHEIGHT, 3, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (table), VBOX_BORDER);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
 	rowcount = 0;
 
+	if (!can_save) {
+		no_save_warning = gtk_label_new(
+			_("<i>These preferences will not be saved as this folder "
+			"is a top-level one. However you can use them to set them "
+			"to the whole mailbox tree using \"Apply to subfolders\".</i>"));
+		gtk_label_set_use_markup(GTK_LABEL(no_save_warning), TRUE);
+		gtk_label_set_line_wrap(GTK_LABEL(no_save_warning), TRUE);
+		gtk_misc_set_alignment(GTK_MISC(no_save_warning), 0.0, 0.5);
+		gtk_table_attach(GTK_TABLE(table), no_save_warning, 0, 3,
+			 rowcount, rowcount + 1, GTK_FILL, 0, 0, 0);
+		rowcount++;
+	}
+	
 	/* Apply to subfolders */
 	label = gtk_label_new(_("Apply to\nsubfolders"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
@@ -535,7 +697,8 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 				 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 		SET_TOGGLE_SENSITIVITY(checkbtn_default_to, entry_default_to);
 		gtk_entry_set_text(GTK_ENTRY(entry_default_to), SAFE_STRING(item->prefs->default_to));
-		address_completion_register_entry(GTK_ENTRY(entry_default_to));
+		address_completion_register_entry(GTK_ENTRY(entry_default_to),
+				TRUE);
 
 		default_to_rec_checkbtn = gtk_check_button_new();
 		gtk_table_attach(GTK_TABLE(table), default_to_rec_checkbtn, 2, 3, 
@@ -555,7 +718,8 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 				 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 		SET_TOGGLE_SENSITIVITY(checkbtn_default_reply_to, entry_default_reply_to);
 		gtk_entry_set_text(GTK_ENTRY(entry_default_reply_to), SAFE_STRING(item->prefs->default_reply_to));
-		address_completion_register_entry(GTK_ENTRY(entry_default_reply_to));
+		address_completion_register_entry(
+				GTK_ENTRY(entry_default_reply_to), TRUE);
 
 		default_reply_to_rec_checkbtn = gtk_check_button_new();
 		gtk_table_attach(GTK_TABLE(table), default_reply_to_rec_checkbtn, 2, 3, 
@@ -652,6 +816,7 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 
 	page->window = GTK_WIDGET(window);
 	page->table = table;
+	page->no_save_warning = no_save_warning;
 	page->checkbtn_request_return_receipt = checkbtn_request_return_receipt;
 	page->checkbtn_save_copy_to_folder = checkbtn_save_copy_to_folder;
 	page->checkbtn_default_to = checkbtn_default_to;
@@ -693,7 +858,7 @@ void prefs_folder_item_compose_destroy_widget_func(PrefsPage *page_)
 /** \brief  Save the prefs in page to folder.
  *
  *  If the folder is not the one  specified in page->item, then only those properties 
- *  that have the relevant 'appy to sub folders' button checked are saved
+ *  that have the relevant 'apply to sub folders' button checked are saved
  */
 static void compose_save_folder_prefs(FolderItem *folder, FolderItemComposePage *page)
 {
@@ -702,6 +867,9 @@ static void compose_save_folder_prefs(FolderItem *folder, FolderItemComposePage
 	GtkWidget *menuitem;
 	gboolean all = FALSE;
 
+	if (folder->path == NULL)
+		return;
+
 	if (page->item == folder) 
 		all = TRUE;
 
@@ -830,6 +998,100 @@ static void folder_color_set_dialog(GtkWidget *widget, gpointer data)
 	page->folder_color = rgbcolor;
 }
 
+static regex_t *summary_compile_simplify_regexp(gchar *simplify_subject_regexp)
+{
+	int err;
+	gchar buf[BUFFSIZE];
+	regex_t *preg = NULL;
+
+	preg = g_new0(regex_t, 1);
+
+	err = string_match_precompile(simplify_subject_regexp, 
+				      preg, REG_EXTENDED);
+	if (err) {
+		regerror(err, preg, buf, BUFFSIZE);
+		g_free(preg);
+		preg = NULL;
+	}
+
+	return preg;
+}
+
+static void folder_regexp_test_cb(GtkWidget *widget, gpointer data)
+{
+	static GdkColor red;
+	static gboolean colors_initialised = FALSE;
+	static gchar buf[BUFFSIZE];
+	FolderItemGeneralPage *page = (FolderItemGeneralPage *)data;
+	gchar *test_string, *regexp;
+	regex_t *preg;
+
+	regexp = g_strdup(gtk_entry_get_text(GTK_ENTRY(page->entry_simplify_subject)));
+	test_string = g_strdup(gtk_entry_get_text(GTK_ENTRY(page->entry_regexp_test_string)));
+
+	if (!regexp || !regexp[0]) {
+		if (test_string) {
+			gtk_entry_set_text(GTK_ENTRY(page->entry_regexp_test_result), test_string);
+			g_free(test_string);
+		}
+		return;
+	}
+
+	if (!test_string || !test_string[0]) {
+		g_free(regexp);
+		return;
+	}
+
+	if (!colors_initialised) {
+		gdk_color_parse("#ff7070", &red);
+		colors_initialised = gdk_colormap_alloc_color(
+			gdk_colormap_get_system(), &red, FALSE, TRUE);
+	}
+
+	preg = summary_compile_simplify_regexp(regexp);
+	if (colors_initialised) {
+		gtk_widget_modify_base(page->entry_simplify_subject,
+				GTK_STATE_NORMAL, preg ? NULL : &red);
+	}
+
+	if (preg != NULL) {
+		string_remove_match(buf, BUFFSIZE, test_string, preg);
+
+		gtk_entry_set_text(GTK_ENTRY(page->entry_regexp_test_result), buf);
+
+		regfree(preg);
+		g_free(preg);
+	}
+
+	g_free(test_string);
+	g_free(regexp);
+}
+
+static gchar *folder_regexp_get_subject_example(void)
+{
+	MsgInfo *msginfo_selected;
+	SummaryView *summaryview = NULL;
+
+	if (!mainwindow_get_mainwindow())
+		return NULL;
+	summaryview = mainwindow_get_mainwindow()->summaryview;
+
+	msginfo_selected = summary_get_selected_msg(summaryview);
+	return msginfo_selected ? g_strdup(msginfo_selected->subject) : NULL;
+}
+
+static void folder_regexp_set_subject_example_cb(GtkWidget *widget, gpointer data)
+{
+	FolderItemGeneralPage *page = (FolderItemGeneralPage *)data;
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_simplify_subject))) {
+		gchar *subject = folder_regexp_get_subject_example();
+		if (subject) {
+			gtk_entry_set_text(GTK_ENTRY(page->entry_regexp_test_string), subject);
+			g_free(subject);
+		}
+	}
+}
 
 static void register_general_page()
 {
@@ -869,17 +1131,23 @@ static GSList *prefs_pages = NULL;
 void prefs_folder_item_open(FolderItem *item)
 {
 	gchar *id, *title;
-
 	if (prefs_pages == NULL) {
 		register_general_page();
 		register_compose_page();
 	}
 
-	id = folder_item_get_identifier (item);
+	if (item->path) {
+		id = folder_item_get_identifier (item);
+		can_save = TRUE;
+	} else {
+		id = g_strdup(item->name);
+		can_save = FALSE;
+	}
 	title = g_strdup_printf (_("Properties for folder %s"), id);
 	g_free (id);
 	prefswindow_open(title, prefs_pages, item,
 			&prefs_common.folderitemwin_width, &prefs_common.folderitemwin_height);
+	
 	g_free (title);
 }
 
