@@ -127,7 +127,7 @@ void prefs_quote_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_widget_show (text_quotefmt);
 	gtk_container_add(GTK_CONTAINER(scrolledwin_quotefmt), text_quotefmt);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_quotefmt), TRUE);
-	gtk_widget_set_size_request(text_quotefmt, -1, 60);
+	gtk_widget_set_size_request(text_quotefmt, -1, 100);
 
 	/* forward */
 
@@ -171,14 +171,18 @@ void prefs_quote_create_widget(PrefsPage *_page, GtkWindow *window,
 	gtk_container_add(GTK_CONTAINER(scrolledwin_quotefmt),
 			  text_fw_quotefmt);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_fw_quotefmt), TRUE);
-	gtk_widget_set_size_request (text_fw_quotefmt, -1, 60);
+	gtk_widget_set_size_request (text_fw_quotefmt, -1, 100);
 
 	hbox1 = gtk_hbox_new (FALSE, 32);
 	gtk_widget_show (hbox1);
 	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+	btn_quotedesc = gtk_button_new_from_stock(GTK_STOCK_INFO);
+#else
 	btn_quotedesc =
 		gtk_button_new_with_label (_(" Description of symbols... "));
+#endif
 	gtk_widget_show (btn_quotedesc);
 	gtk_box_pack_start (GTK_BOX (hbox1), btn_quotedesc, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(btn_quotedesc), "clicked",
