@@ -195,7 +195,7 @@ static GtkWidget *date_format_create(GtkButton *button, void *data)
 	datefmt_list_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
 	g_object_unref(G_OBJECT(store));
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(datefmt_list_view),
-				     prefs_common.enable_rules_hint);
+				     prefs_common.use_stripes_everywhere);
 	gtk_widget_show(datefmt_list_view);
 	gtk_container_add(GTK_CONTAINER(scrolledwindow1), datefmt_list_view);
 
@@ -463,7 +463,7 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		{"<Main>/View/Go to/Other folder...",		"G"},
 		{"<Main>/View/Open in new window",		"<control><alt>N"},
 		{"<Main>/View/Message source",			"<control>U"},
-		{"<Main>/View/Show all headers",		"<control>H"},
+		{"<Main>/View/All headers",			"<control>H"},
 		{"<Main>/View/Update summary",			"<control><alt>U"},
 
 		{"<Main>/Message/Receive/Get from current account",
@@ -543,7 +543,7 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		{"<Main>/View/Go to/Other folder...",		"G"},
 		{"<Main>/View/Open in new window",		"<control><alt>N"},
 		{"<Main>/View/Message source",			"<control>U"},
-		{"<Main>/View/Show all headers",		"<shift>H"},
+		{"<Main>/View/All headers",			"<shift>H"},
 		{"<Main>/View/Update summary",			"<shift>S"},
 
 		{"<Main>/Message/Receive/Get from current account",
@@ -597,8 +597,8 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		{"<Main>/View/Go to/Other folder...",		"C"},
 		{"<Main>/View/Open in new window",		"<control><alt>N"},
 		{"<Main>/View/Message source",			"<control>U"},
-		{"<Main>/View/Show all headers",		"<control>H"},
-		{"<Main>/View/Update summary",				"<control><alt>U"},
+		{"<Main>/View/All headers",			"<control>H"},
+		{"<Main>/View/Update summary",			"<control><alt>U"},
 
 		{"<Main>/Message/Receive/Get from current account",
 								"<control>I"},
@@ -651,7 +651,7 @@ static void prefs_keybind_apply_clicked(GtkWidget *widget)
 		{"<Main>/View/Go to/Other folder...",		"<alt>G"},
 		{"<Main>/View/Open in new window",		"<shift><control>N"},
 		{"<Main>/View/Message source",			"<control>U"},
-		{"<Main>/View/Show all headers",		"<control>H"},
+		{"<Main>/View/All headers",			"<control>H"},
 		{"<Main>/View/Update summary",			"<alt>U"},
 
 		{"<Main>/Message/Receive/Get from current account",
@@ -766,8 +766,8 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 		(vbox2, chkbtn_transhdr,
 		 _("Translate header names"));
 	gtk_tooltips_set_tip(tooltips, chkbtn_transhdr,
-			     "The display of standard headers (such as 'From:', 'Subject:') "
-			     "will be translated into your language.", NULL);
+			     _("The display of standard headers (such as 'From:', 'Subject:') "
+			     "will be translated into your language."), NULL);
 
 	PACK_CHECK_BUTTON (vbox2, chkbtn_folder_unread,
 			   _("Display unread number next to folder name"));
@@ -916,11 +916,19 @@ void prefs_summaries_create_widget(PrefsPage *_page, GtkWindow *window,
 			(GtkAttachOptions) (0), 0, 0);
 
 	menu = gtk_menu_new ();
-	MENUITEM_ADD (menu, menuitem, _("Do nothing"), 0);
-	MENUITEM_ADD (menu, menuitem, _("Select first unread (or new) message"),
-		      SELECTONENTRY_UNREAD);
-	MENUITEM_ADD (menu, menuitem, _("Select first new (or unread) message"),
-		      SELECTONENTRY_NEW);
+	MENUITEM_ADD (menu, menuitem, _("Do nothing"), SELECTONENTRY_NOTHING);
+	MENUITEM_ADD (menu, menuitem, _("Select first unread (or new or marked) message"),
+		      SELECTONENTRY_UNM);
+	MENUITEM_ADD (menu, menuitem, _("Select first unread (or marked or new) message"),
+		      SELECTONENTRY_UMN);
+	MENUITEM_ADD (menu, menuitem, _("Select first new (or unread or marked) message"),
+		      SELECTONENTRY_NUM);
+	MENUITEM_ADD (menu, menuitem, _("Select first new (or marked or unread) message"),
+		      SELECTONENTRY_NMU);
+	MENUITEM_ADD (menu, menuitem, _("Select first marked (or new or unread) message"),
+		      SELECTONENTRY_MNU);
+	MENUITEM_ADD (menu, menuitem, _("Select first marked (or unread or new) message"),
+		      SELECTONENTRY_MUN);
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu_select_on_entry), menu);
 
@@ -1148,7 +1156,7 @@ static void date_format_entry_on_change(GtkEditable *editable,
 	buffer[0] = 0;
 	text = gtk_editable_get_chars(editable, 0, -1);
 	if (text)
-		strftime(buffer, sizeof buffer, text, cal_time); 
+		fast_strftime(buffer, sizeof buffer, text, cal_time); 
 	g_free(text);
 
 	text = conv_codeset_strdup(buffer,
