@@ -30,6 +30,7 @@
 
 #include "template.h"
 #include "main.h"
+#include "prefs_gtk.h"
 #include "inc.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -39,6 +40,7 @@
 #include "addr_compl.h"
 #include "quote_fmt.h"
 #include "prefs_common.h"
+#include "manual.h"
 
 enum {
 	TEMPL_TEXT,
@@ -119,34 +121,13 @@ static void prefs_template_size_allocate_cb(GtkWidget *widget,
 	prefs_common.templateswin_height = allocation->height;
 }
 
-#define ADD_ENTRY(entry, str, row) \
-{ \
-	label1 = gtk_label_new(str); \
-	gtk_widget_show(label1); \
-	gtk_table_attach(GTK_TABLE(table), label1, 0, 1, row, (row + 1), \
-			 GTK_FILL, 0, 0, 0); \
-	gtk_misc_set_alignment(GTK_MISC(label1), 1, 0.5); \
- \
-	entry = gtk_entry_new(); \
-	gtk_widget_show(entry); \
-	gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, (row + 1), \
-			 GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0); \
-}
-
 static void prefs_template_window_create(void)
 {
 	/* window structure ;) */
 	GtkWidget *window;
 	GtkWidget   *vpaned;
 	GtkWidget     *vbox1;
-	GtkWidget       *hbox1;
-	GtkWidget         *label1;
-	GtkWidget         *entry_name;
-	GtkWidget       *table;
-	GtkWidget         *entry_to;
-	GtkWidget         *entry_cc;
-	GtkWidget         *entry_bcc;		
-	GtkWidget         *entry_subject;
+	GtkWidget       *table; /* including : entry_[name|to|cc|bcc|subject] */
 	GtkWidget       *scroll2;
 	GtkWidget         *text_value;
 	GtkWidget     *vbox2;
@@ -160,9 +141,12 @@ static void prefs_template_window_create(void)
 	GtkWidget       *scroll1;
 	GtkWidget         *list_view;
 	GtkWidget       *confirm_area;
-	GtkWidget         *ok_btn;
+	GtkWidget         *help_btn;
 	GtkWidget         *cancel_btn;
+	GtkWidget         *ok_btn;
 	static GdkGeometry geometry;
+	gint i;
+	GtkTooltips *tooltips;
 
 	debug_print("Creating templates configuration window...\n");
 
@@ -183,34 +167,52 @@ static void prefs_template_window_create(void)
 	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 8);
 	gtk_paned_pack1(GTK_PANED(vpaned), vbox1, FALSE, FALSE);
 
-	hbox1 = gtk_hbox_new(FALSE, 8);
-	gtk_widget_show(hbox1);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
-
-	label1 = gtk_label_new(_("Template name"));
-	gtk_widget_show(label1);
-	gtk_box_pack_start(GTK_BOX(hbox1), label1, FALSE, FALSE, 0);
-
-	entry_name = gtk_entry_new();
-	gtk_widget_show(entry_name);
-	gtk_box_pack_start(GTK_BOX(hbox1), entry_name, TRUE, TRUE, 0);
-
-	/* table for headers */
-	table = gtk_table_new(2, 2, FALSE);
+	table = gtk_table_new(5, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), VSPACING_NARROW_2);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
 	gtk_widget_show(table);
-	gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 0);
-	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
+	gtk_box_pack_start (GTK_BOX (vbox1), table, FALSE, FALSE, 0);
 
-	ADD_ENTRY(entry_to, _("To:"), 0);
-	address_completion_register_entry(GTK_ENTRY(entry_to));
-	ADD_ENTRY(entry_cc, _("Cc:"), 1)
-	address_completion_register_entry(GTK_ENTRY(entry_cc));
-	ADD_ENTRY(entry_bcc, _("Bcc:"), 2)	
-	address_completion_register_entry(GTK_ENTRY(entry_bcc));
-	ADD_ENTRY(entry_subject, _("Subject:"), 3);
+	tooltips = gtk_tooltips_new();
 
-#undef ADD_ENTRY
+	struct
+	{
+		gchar *label;
+		GtkWidget **entry;
+		gboolean compl;
+		gchar *tooltips;
+	} tab[] = {
+		{_("Name"),	&templates.entry_name,		FALSE,
+			_("This name is used as the Menu item")},
+		{_("To"),	&templates.entry_to,		TRUE, 	NULL},
+		{_("Cc"),	&templates.entry_cc,		TRUE, 	NULL},
+		{_("Bcc"),	&templates.entry_bcc,		TRUE, 	NULL},
+		{_("Subject"),	&templates.entry_subject,	FALSE,	NULL},
+		{NULL,		NULL,				FALSE,	NULL}
+	};
+
+	for (i=0; tab[i].label; i++) {
+
+		GtkWidget *label;
+
+		label = gtk_label_new(tab[i].label);
+		gtk_widget_show(label);
+		gtk_table_attach(GTK_TABLE(table), label, 0, 1, i, (i + 1),
+				(GtkAttachOptions) (GTK_FILL),
+				(GtkAttachOptions) 0, 0, 0);
+		gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+
+		*(tab[i].entry) = gtk_entry_new();
+		gtk_widget_show(*(tab[i].entry));
+		gtk_table_attach(GTK_TABLE(table), *(tab[i].entry), 1, 2, i, (i + 1),
+				(GtkAttachOptions) (GTK_EXPAND|GTK_SHRINK|GTK_FILL),
+				(GtkAttachOptions) 0, 0, 0);
+		gtk_tooltips_set_tip(tooltips, *(tab[i].entry), tab[i].tooltips, NULL);
+
+		if (tab[i].compl)
+			address_completion_register_entry(
+				GTK_ENTRY(*(tab[i].entry)), TRUE);
+	}
 
 	/* template content */
 	scroll2 = gtk_scrolled_window_new(NULL, NULL);
@@ -255,7 +257,7 @@ static void prefs_template_window_create(void)
 	g_signal_connect(G_OBJECT (reg_btn), "clicked",
 			 G_CALLBACK (prefs_template_register_cb), NULL);
 
-	subst_btn = gtk_button_new_with_label(_("  Replace  "));
+	subst_btn = gtkut_get_replace_btn(_("Replace"));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), subst_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT(subst_btn), "clicked",
@@ -268,7 +270,11 @@ static void prefs_template_window_create(void)
 	g_signal_connect(G_OBJECT(del_btn), "clicked",
 			 G_CALLBACK(prefs_template_delete_cb), NULL);
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+	desc_btn = gtk_button_new_from_stock(GTK_STOCK_INFO);
+#else
 	desc_btn = gtk_button_new_with_label(_(" Symbols... "));
+#endif
 	gtk_widget_show(desc_btn);
 	gtk_box_pack_end(GTK_BOX(hbox2), desc_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(desc_btn), "clicked",
@@ -287,9 +293,11 @@ static void prefs_template_window_create(void)
 	gtk_widget_set_size_request(scroll1, -1, 140);
 	gtk_container_add(GTK_CONTAINER(scroll1), list_view);
 
-	/* ok | cancel */
-	gtkut_stock_button_set_create(&confirm_area, &cancel_btn, GTK_STOCK_CANCEL,
-				      &ok_btn, GTK_STOCK_OK, NULL, NULL);
+	/* help | cancel | ok */
+	gtkut_stock_button_set_create_with_help(&confirm_area, &help_btn,
+			&cancel_btn, GTK_STOCK_CANCEL,
+			&ok_btn, GTK_STOCK_OK,
+			NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox2), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -307,6 +315,9 @@ static void prefs_template_window_create(void)
 			 G_CALLBACK(prefs_template_ok_cb), NULL);
 	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
 			 G_CALLBACK(prefs_template_cancel_cb), NULL);
+	g_signal_connect(G_OBJECT(help_btn), "clicked",
+			 G_CALLBACK(manual_open_with_anchor_cb),
+			 MANUAL_ANCHOR_TEMPLATES);
 
 	address_completion_start(window);
 
@@ -323,11 +334,6 @@ static void prefs_template_window_create(void)
 	templates.window = window;
 	templates.ok_btn = ok_btn;
 	templates.list_view = list_view;
-	templates.entry_name = entry_name;
-	templates.entry_subject = entry_subject;
-	templates.entry_to = entry_to;
-	templates.entry_cc = entry_cc;
-	templates.entry_bcc = entry_bcc;	
 	templates.text_value = text_value;
 }
 
@@ -337,7 +343,8 @@ static void prefs_template_window_setup(void)
 	GSList *cur;
 	Template *tmpl;
 	GtkListStore *store;
-
+	GtkTextBuffer *buffer;
+	
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
 				(templates.list_view)));
 
@@ -359,6 +366,15 @@ static void prefs_template_window_setup(void)
 							 tmpl, FALSE);
 	}
 
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_to), "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc), "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc), "");			
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_subject), "");
+	
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(templates.text_value));
+	gtk_text_buffer_set_text(buffer, "", -1);
+
 	g_slist_free(tmpl_list);
 }
 
@@ -399,7 +415,7 @@ static void prefs_template_ok_cb(void)
 
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("_Continue editing"), 
+				 GTK_STOCK_CLOSE, _("+_Continue editing"), 
 				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
@@ -416,7 +432,7 @@ static void prefs_template_cancel_cb(void)
 {
 	if (modified && alertpanel(_("Entry not saved"),
 				 _("The entry was not saved. Close anyway?"),
-				 GTK_STOCK_CLOSE, _("_Continue editing"),
+				 GTK_STOCK_CLOSE, _("+_Continue editing"),
 				 NULL) != G_ALERTDEFAULT) {
 		return;
 	}
@@ -502,7 +518,7 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 		MsgInfo dummyinfo;
 
 		memset(&dummyinfo, 0, sizeof(MsgInfo));
-		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE);
+		quote_fmt_init(&dummyinfo, NULL, NULL, TRUE, NULL);
 		quote_fmt_scan_string(value);
 		quote_fmt_parse();
 		parsed_buf = quote_fmt_get_buffer();
@@ -511,6 +527,7 @@ static gboolean prefs_template_list_view_set_row(GtkTreeIter *row)
 			g_free(value);
 			return FALSE;
 		}
+		quote_fmt_reset_vartable();
 	}
 
 	name = gtk_editable_get_chars(GTK_EDITABLE(templates.entry_name),
@@ -723,7 +740,7 @@ static GtkWidget *prefs_template_list_view_create(void)
 	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	g_object_unref(model);	
 	
-	gtk_tree_view_set_rules_hint(list_view, prefs_common.enable_rules_hint);
+	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
 	
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
