@@ -24,6 +24,43 @@
 #include "procmime.h"
 
 static GSList *systems = NULL;
+static gchar *privacy_last_error = NULL;
+
+void privacy_set_error(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[BUFSIZ];
+
+	va_start(args, format);
+	g_vsnprintf(buf, BUFSIZ, format, args);
+	va_end(args);
+	g_free(privacy_last_error);
+	privacy_last_error = g_strdup(buf);
+}
+
+static gchar tmp_privacy_error[BUFSIZ];
+
+void privacy_reset_error(void)
+{
+	g_free(privacy_last_error);
+	privacy_last_error = NULL;
+}
+
+gboolean privacy_peek_error(void)
+{
+	return (privacy_last_error != NULL);
+}
+
+const gchar *privacy_get_error (void)
+{
+	if (privacy_last_error) {
+		strncpy2(tmp_privacy_error, privacy_last_error, BUFSIZ-1);
+		privacy_reset_error();
+		return tmp_privacy_error;
+	} else {
+		return _("Unknown error");
+	}
+}
 
 PrivacySystem *privacy_data_get_system(PrivacyData *data)
 {
@@ -365,7 +402,10 @@ gboolean privacy_encrypt(const gchar *id, MimeInfo *mimeinfo, const gchar *encda
 
 	g_return_val_if_fail(id != NULL, FALSE);
 	g_return_val_if_fail(mimeinfo != NULL, FALSE);
-	g_return_val_if_fail(encdata != NULL, FALSE);
+	if (encdata == NULL) {
+		privacy_set_error(_("No recipient keys defined."));
+		return FALSE;
+	}
 
 	system = privacy_get_system(id);
 	if (system == NULL)
