@@ -134,6 +134,7 @@ static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 		/* ([*WSP CRLF] 1*WSP) */
 		if (nexthead == ' ' || nexthead == '\t') {
 			size_t buflen;
+			gboolean skiptab = (nexthead == '\t');
 			/* trim previous trailing \n if requesting one header or
 			 * unfolding was requested */
 			if ((!hentry && unfold) || (hp && hp->unfold))
@@ -145,6 +146,9 @@ static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 			if ((len - buflen) > 2) {
 				if (getline(buf + buflen, len - buflen, data) == NULL)
 					break;
+				if (skiptab) { /* replace tab with space */
+					*(buf + buflen) = ' ';
+				}
 			} else
 				break;
 		} else {
@@ -358,6 +362,12 @@ enum
 	H_SC_PARTIALLY_RETRIEVED = 20,
 	H_SC_ACCOUNT_SERVER = 21,
 	H_SC_ACCOUNT_LOGIN = 22,
+ 	H_LIST_POST	   = 23,
+	H_LIST_SUBSCRIBE   = 24,
+ 	H_LIST_UNSUBSCRIBE = 25,
+ 	H_LIST_HELP        = 26,
+ 	H_LIST_ARCHIVE     = 27,
+ 	H_LIST_OWNER       = 28,
 };
 
 static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
@@ -383,6 +393,12 @@ static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
 				   {"SC-Partially-Retrieved:", NULL, FALSE},
 				   {"SC-Account-Server:", NULL, FALSE},
 				   {"SC-Account-Login:",NULL, FALSE},
+				   {"List-Post:",	NULL, TRUE},
+				   {"List-Subscribe:",	NULL, TRUE},
+				   {"List-Unsubscribe:",NULL, TRUE},
+				   {"List-Help:",	NULL, TRUE},
+ 				   {"List-Archive:",	NULL, TRUE},
+ 				   {"List-Owner:",	NULL, TRUE},
 				   {NULL,		NULL, FALSE}};
 
 static HeaderEntry hentry_short[] = {{"Date:",		NULL, FALSE},
@@ -423,6 +439,8 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 	gchar *hp;
 	HeaderEntry *hentry;
 	gint hnum;
+	void *orig_data = data;
+
 	get_one_field_func get_one_field =
 		isstring ? (get_one_field_func)string_get_one_field
 			 : (get_one_field_func)procheader_get_one_field;
@@ -430,8 +448,22 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 	hentry = procheader_get_headernames(full);
 
 	if (MSG_IS_QUEUED(flags) || MSG_IS_DRAFT(flags)) {
-		while (get_one_field(buf, sizeof(buf), data, NULL) != -1)
-			; /* loop */
+		while (get_one_field(buf, sizeof(buf), data, NULL) != -1) {
+			if (!strncmp(buf, "X-Sylpheed-End-Special-Headers: 1",
+				strlen("X-Sylpheed-End-Special-Headers:")))
+				break;
+			/* from other mailers */
+			if (!strncmp(buf, "Date: ", 6)
+			||  !strncmp(buf, "To: ", 4)
+			||  !strncmp(buf, "From: ", 6)
+			||  !strncmp(buf, "Subject: ", 9)) {
+				if (isstring)
+					data = orig_data;
+				else 
+					rewind((FILE *)data);
+				break;
+			}
+		}
 	}
 
 	msginfo = procmsg_msginfo_new();
@@ -474,11 +506,12 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			if (msginfo->from) break;
                         msginfo->from = conv_unmime_header(hp, NULL);
 			msginfo->fromname = procheader_get_fromname(msginfo->from);
-			replace_returns(msginfo->from);
-			replace_returns(msginfo->fromname);
+			remove_return(msginfo->from);
+			remove_return(msginfo->fromname);
 			break;
 		case H_TO:
                         tmp = conv_unmime_header(hp, NULL);
+			remove_return(tmp);
 			if (msginfo->to) {
 				p = msginfo->to;
 				msginfo->to =
@@ -490,6 +523,7 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			break;
 		case H_CC:
                         tmp = conv_unmime_header(hp, NULL);
+			remove_return(tmp);
 			if (msginfo->cc) {
 				p = msginfo->cc;
 				msginfo->cc =
@@ -511,7 +545,6 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 		case H_SUBJECT:
 			if (msginfo->subject) break;
                         msginfo->subject = conv_unmime_header(hp, NULL);
-			replace_returns(msginfo->subject);
 			break;
 		case H_MSG_ID:
 			if (msginfo->msgid) break;
@@ -548,32 +581,47 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			break;
 #endif			
 		case H_FACE:
-			if (msginfo->face) break;
-			msginfo->face = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->face) break;
+			msginfo->extradata->face = g_strdup(hp);
 			break;
 		case H_X_FACE:
-			if (msginfo->xface) break;
-			msginfo->xface = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->xface) break;
+			msginfo->extradata->xface = g_strdup(hp);
 			break;
 		case H_DISPOSITION_NOTIFICATION_TO:
-			if (msginfo->dispositionnotificationto) break;
-			msginfo->dispositionnotificationto = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->dispositionnotificationto) break;
+			msginfo->extradata->dispositionnotificationto = g_strdup(hp);
 			break;
 		case H_RETURN_RECEIPT_TO:
-			if (msginfo->returnreceiptto) break;
-			msginfo->returnreceiptto = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->returnreceiptto) break;
+			msginfo->extradata->returnreceiptto = g_strdup(hp);
 			break;
+/* partial download infos */			
 		case H_SC_PARTIALLY_RETRIEVED:
-			if (msginfo->partial_recv) break;
-			msginfo->partial_recv = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->partial_recv) break;
+			msginfo->extradata->partial_recv = g_strdup(hp);
 			break;
 		case H_SC_ACCOUNT_SERVER:
-			if (msginfo->account_server) break;
-			msginfo->account_server = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->account_server) break;
+			msginfo->extradata->account_server = g_strdup(hp);
 			break;
 		case H_SC_ACCOUNT_LOGIN:
-			if (msginfo->account_login) break;
-			msginfo->account_login = g_strdup(hp);
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->account_login) break;
+			msginfo->extradata->account_login = g_strdup(hp);
 			break;
 		case H_SC_MESSAGE_SIZE:
 			if (msginfo->total_size) break;
@@ -582,6 +630,7 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 		case H_SC_PLANNED_DOWNLOAD:
 			msginfo->planned_download = atoi(hp);
 			break;
+/* end partial download infos */
 #ifdef ALLOW_HEADER_HINT			
 		case H_STATUS:
 			if (strchr(hp, 'R') != NULL)
@@ -608,7 +657,46 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 		case H_FROM_SPACE:
 			if (msginfo->fromspace) break;
 			msginfo->fromspace = g_strdup(hp);
+			remove_return(msginfo->fromspace);
 			break;
+/* list infos */
+ 		case H_LIST_POST:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->list_post) break;
+			msginfo->extradata->list_post = g_strdup(hp);
+			break;
+		case H_LIST_SUBSCRIBE:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->list_subscribe) break;
+			msginfo->extradata->list_subscribe = g_strdup(hp);
+			break;
+		case H_LIST_UNSUBSCRIBE:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->list_unsubscribe) break;
+			msginfo->extradata->list_unsubscribe = g_strdup(hp);
+			break;
+		case H_LIST_HELP:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->list_help) break;
+			msginfo->extradata->list_help = g_strdup(hp);
+			break;
+		case H_LIST_ARCHIVE:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->list_archive) break;
+			msginfo->extradata->list_archive = g_strdup(hp);
+			break;
+		case H_LIST_OWNER:
+			if (!msginfo->extradata)
+				msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+			if (msginfo->extradata->list_owner) break;
+			msginfo->extradata->list_owner = g_strdup(hp);
+			break;
+/* end list infos */
 		default:
 			break;
 		}
@@ -827,17 +915,18 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 	lt = localtime(&timer);
 
 	if (prefs_common.date_format)
-		strftime(dest, len, prefs_common.date_format, lt);
+		fast_strftime(dest, len, prefs_common.date_format, lt);
 	else
-		strftime(dest, len, default_format, lt);
+		fast_strftime(dest, len, default_format, lt);
 
-	src_codeset = conv_get_locale_charset_str();
-	dest_codeset = CS_UTF_8;
-	str = conv_codeset_strdup(dest, src_codeset, dest_codeset);
-	if (str) {
-		g_snprintf(dest, len, "%s", str);
-		strncpy2(dest, str, len);
-		g_free(str);
+	if (!g_utf8_validate(dest, -1, NULL)) {
+		src_codeset = conv_get_locale_charset_str_no_utf8();
+		dest_codeset = CS_UTF_8;
+		str = conv_codeset_strdup(dest, src_codeset, dest_codeset);
+		if (str) {
+			strncpy2(dest, str, len);
+			g_free(str);
+		}
 	}
 }
 
