@@ -48,6 +48,7 @@
 #include "utils.h"
 #include "prefs_common.h"
 #include "prefs_gtk.h"
+#include "alertpanel.h"
 
 static GHashTable *procmime_get_mime_type_table	(void);
 
@@ -226,6 +227,24 @@ const gchar *procmime_mimeinfo_get_parameter(MimeInfo *mimeinfo, const gchar *na
 	return value;
 }
 
+#define FLUSH_LASTLINE() {							\
+	if (*lastline != '\0') {						\
+		gint llen = 0;							\
+		strretchomp(lastline);						\
+		llen = strlen(lastline);					\
+		if (lastline[llen-1] == ' ' && strcmp(lastline,"-- ")) {	\
+			/* this is flowed */					\
+			if (delsp)						\
+				lastline[llen-1] = '\0';			\
+			fputs(lastline, outfp);					\
+		} else {							\
+			fputs(lastline, outfp);					\
+			fputs("\n", outfp);					\
+		}								\
+	} 									\
+	strcpy(lastline, buf);							\
+}
+
 gboolean procmime_decode_content(MimeInfo *mimeinfo)
 {
 	gchar buf[BUFFSIZE];
@@ -234,15 +253,34 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 	FILE *outfp, *infp;
 	struct stat statbuf;
 	gboolean tmp_file = FALSE;
-
+	gboolean flowed = FALSE;
+	gboolean delsp = FALSE; 
 	EncodingType encoding = forced_encoding 
 				? forced_encoding
 				: mimeinfo->encoding_type;
+	gchar lastline[BUFFSIZE];
+	memset(lastline, 0, BUFFSIZE);
 		   
 	g_return_val_if_fail(mimeinfo != NULL, FALSE);
 
-	if (encoding == ENC_UNKNOWN ||
-	    encoding == ENC_BINARY)
+	if (prefs_common.respect_flowed_format &&
+	    mimeinfo->type == MIMETYPE_TEXT && 
+	    !strcasecmp(mimeinfo->subtype, "plain")) {
+		if (procmime_mimeinfo_get_parameter(mimeinfo, "format") != NULL &&
+		    !strcasecmp(procmime_mimeinfo_get_parameter(mimeinfo, "format"),"flowed"))
+			flowed = TRUE;
+		if (flowed &&
+		    procmime_mimeinfo_get_parameter(mimeinfo, "delsp") != NULL &&
+		    !strcasecmp(procmime_mimeinfo_get_parameter(mimeinfo, "delsp"),"yes"))
+			delsp = TRUE;
+	}
+	
+	if (!flowed && (
+	     encoding == ENC_UNKNOWN ||
+	     encoding == ENC_BINARY ||
+	     encoding == ENC_7BIT ||
+	     encoding == ENC_8BIT
+	    ))
 		return TRUE;
 
 	infp = g_fopen(mimeinfo->data.filename, "rb");
@@ -265,8 +303,15 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 		while ((ftell(infp) < readend) && (fgets(buf, sizeof(buf), infp) != NULL)) {
 			gint len;
 			len = qp_decode_line(buf);
-			fwrite(buf, 1, len, outfp);
+			buf[len]='\0';
+			if (!flowed) {
+				fwrite(buf, 1, len, outfp);
+			} else {
+				FLUSH_LASTLINE();
+			}
 		}
+		if (flowed)
+			FLUSH_LASTLINE();
 	} else if (encoding == ENC_BASE64) {
 		gchar outbuf[BUFFSIZE];
 		gint len;
@@ -336,8 +381,14 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 		}
 	} else {
 		while ((ftell(infp) < readend) && (fgets(buf, sizeof(buf), infp) != NULL)) {
-			fputs(buf, outfp);
+			if (!flowed) {
+				fputs(buf, outfp);
+			} else {
+				FLUSH_LASTLINE();
+			}
 		}
+		if (flowed)
+			FLUSH_LASTLINE();
 	}
 
 	fclose(outfp);
@@ -346,8 +397,7 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 	stat(tmpfilename, &statbuf);
 	if (mimeinfo->tmp && (mimeinfo->data.filename != NULL))
 		g_unlink(mimeinfo->data.filename);
-	if (mimeinfo->data.filename != NULL)
-		g_free(mimeinfo->data.filename);
+	g_free(mimeinfo->data.filename);
 	mimeinfo->data.filename = tmpfilename;
 	mimeinfo->tmp = TRUE;
 	mimeinfo->offset = 0;
@@ -819,8 +869,11 @@ FILE *procmime_get_first_encrypted_text_content(MsgInfo *msginfo)
 	partinfo = mimeinfo;
 	if ((encinfo = find_encrypted_part(partinfo)) != NULL) {
 		debug_print("decrypting message part\n");
-		if (privacy_mimeinfo_decrypt(encinfo) < 0)
+		if (privacy_mimeinfo_decrypt(encinfo) < 0) {
+			alertpanel_error(_("Couldn't decrypt: %s"),
+				privacy_get_error());
 			return NULL;
+		}
 	}
 	partinfo = mimeinfo;
 	while (partinfo && partinfo->type != MIMETYPE_TEXT) {
@@ -1132,7 +1185,9 @@ EncodingType procmime_get_encoding_for_charset(const gchar *charset)
 		return ENC_8BIT;
 	else if (!g_ascii_strncasecmp(charset, "ISO-8859-", 9))
 		return ENC_QUOTED_PRINTABLE;
-	else
+	else if (!g_ascii_strncasecmp(charset, "UTF-8", 5))
+		return ENC_QUOTED_PRINTABLE;
+	else 
 		return ENC_8BIT;
 }
 
@@ -1382,7 +1437,7 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
 	}
 	fseek(fp, mimeinfo->offset, SEEK_SET);
 	while ((p = fgets(buf, sizeof(buf), fp)) != NULL && result == 0) {
-		if (ftell(fp) > (mimeinfo->offset + mimeinfo->length))
+		if (ftell(fp) - 1 > (mimeinfo->offset + mimeinfo->length))
 			break;
 
 		if (IS_BOUNDARY(buf, boundary, boundary_len)) {
@@ -1842,8 +1897,22 @@ MimeInfo *procmime_scan_queue_file(const gchar *filename)
 	if ((fp = g_fopen(filename, "rb")) == NULL)
 		return NULL;
 	/* Skip queue header */
-	while (fgets(buf, sizeof(buf), fp) != NULL)
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		/* new way */
+		if (!strncmp(buf, "X-Sylpheed-End-Special-Headers: 1",
+			strlen("X-Sylpheed-End-Special-Headers:")))
+			break;
+		/* old way */
 		if (buf[0] == '\r' || buf[0] == '\n') break;
+		/* from other mailers */
+		if (!strncmp(buf, "Date: ", 6)
+		||  !strncmp(buf, "To: ", 4)
+		||  !strncmp(buf, "From: ", 6)
+		||  !strncmp(buf, "Subject: ", 9)) {
+			rewind(fp);
+			break;
+		}
+	}
 	offset = ftell(fp);
 	fclose(fp);
 
