@@ -44,8 +44,12 @@
 #include "partial_download.h"
 #include "mainwindow.h"
 #include "summaryview.h"
+#include "log.h"
+#include "timing.h"
+#include "inc.h"
 
-static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_session);
+static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_session, gchar **errstr,
+					    FolderItem *queue, gint msgnum);
 
 enum
 {
@@ -61,6 +65,7 @@ enum
 	Q_PRIVACY_SYSTEM   = 9,
 	Q_ENCRYPT 	   = 10,
 	Q_ENCRYPT_DATA	   = 11,
+	Q_SYLPHEED_HDRS    = 12,
 };
 
 GHashTable *procmsg_msg_hash_table_create(GSList *mlist)
@@ -240,15 +245,18 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 {
 	GNode *root, *parent, *node, *next;
 	GHashTable *msgid_table;
-	GRelation *subject_relation;
+	GRelation *subject_relation = NULL;
 	MsgInfo *msginfo;
 	const gchar *msgid;
         GSList *reflist;
-
+	START_TIMING("");
 	root = g_node_new(NULL);
 	msgid_table = g_hash_table_new(g_str_hash, g_str_equal);
-	subject_relation = g_relation_new(2);
-	g_relation_index(subject_relation, 0, g_str_hash, g_str_equal);
+	
+	if (prefs_common.thread_by_subject) {
+		subject_relation = g_relation_new(2);
+		g_relation_index(subject_relation, 0, g_str_hash, g_str_equal);
+	}
 
 	for (; mlist != NULL; mlist = mlist->next) {
 		msginfo = (MsgInfo *)mlist->data;
@@ -303,6 +311,7 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 	}
 
 	if (prefs_common.thread_by_subject) {
+		START_TIMING("thread by subject");
 		for (node = root->children; node && node != NULL;) {
 			next = node->next;
 			msginfo = (MsgInfo *) node->data;
@@ -326,11 +335,14 @@ GNode *procmsg_get_thread_tree(GSList *mlist)
 
 			node = next;
 		}	
+		END_TIMING();
 	}
 	
-	g_relation_destroy(subject_relation);
-	g_hash_table_destroy(msgid_table);
+	if (prefs_common.thread_by_subject)
+		g_relation_destroy(subject_relation);
 
+	g_hash_table_destroy(msgid_table);
+	END_TIMING();
 	return root;
 }
 
@@ -355,15 +367,16 @@ next_folder:
 		}
 		if (!dest) {
 			dest = msginfo->to_folder;
-			movelist = g_slist_append(movelist, msginfo);
+			movelist = g_slist_prepend(movelist, msginfo);
 		} else if (dest == msginfo->to_folder) {
-			movelist = g_slist_append(movelist, msginfo);
+			movelist = g_slist_prepend(movelist, msginfo);
 		} else {
 			continue;
 		}
 		procmsg_msginfo_set_to_folder(msginfo, NULL);
 	}
 	if (movelist) {
+		movelist = g_slist_reverse(movelist);
 		retval |= folder_item_move_msgs(dest, movelist);
 		g_slist_free(movelist);
 		movelist = NULL;
@@ -398,15 +411,16 @@ next_folder:
 		}
 		if (!dest) {
 			dest = msginfo->to_folder;
-			copylist = g_slist_append(copylist, msginfo);
+			copylist = g_slist_prepend(copylist, msginfo);
 		} else if (dest == msginfo->to_folder) {
-			copylist = g_slist_append(copylist, msginfo);
+			copylist = g_slist_prepend(copylist, msginfo);
 		} else {
 			continue;
 		}
 		procmsg_msginfo_set_to_folder(msginfo, NULL);
 	}
 	if (copylist) {
+		copylist = g_slist_reverse(copylist);
 		folder_item_copy_msgs(dest, copylist);
 		g_slist_free(copylist);
 		copylist = NULL;
@@ -534,8 +548,22 @@ FILE *procmsg_open_message(MsgInfo *msginfo)
 	if (MSG_IS_QUEUED(msginfo->flags) || MSG_IS_DRAFT(msginfo->flags)) {
 		gchar buf[BUFFSIZE];
 
-		while (fgets(buf, sizeof(buf), fp) != NULL)
+		while (fgets(buf, sizeof(buf), fp) != NULL) {
+			/* new way */
+			if (!strncmp(buf, "X-Sylpheed-End-Special-Headers: 1",
+				strlen("X-Sylpheed-End-Special-Headers:")))
+				break;
+			/* old way */
 			if (buf[0] == '\r' || buf[0] == '\n') break;
+			/* from other mailers */
+			if (!strncmp(buf, "Date: ", 6)
+			||  !strncmp(buf, "To: ", 4)
+			||  !strncmp(buf, "From: ", 6)
+			||  !strncmp(buf, "Subject: ", 9)) {
+				rewind(fp);
+				break;
+			}
+		}
 	}
 
 	return fp;
@@ -633,7 +661,7 @@ void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
 		} else if (hentry[H_SENDER].body != NULL) {
 			SET_FILTER_KEY("header \"Sender\"", H_SENDER);
 		} else if (hentry[H_LIST_POST].body != NULL) {
-			SET_FILTER_KEY("header \"Sender\"", H_LIST_POST);
+			SET_FILTER_KEY("header \"List-Post\"", H_LIST_POST);
 		} else if (msginfo->to) {
 			*header = g_strdup("to");
 			*key = g_strdup(msginfo->to);
@@ -858,6 +886,7 @@ static gboolean procmsg_is_last_for_account(FolderItem *queue, MsgInfo *msginfo,
 	return TRUE;
 }
 
+static gboolean send_queue_lock = FALSE;
 /*!
  *\brief	Send messages in queue
  *
@@ -867,16 +896,35 @@ static gboolean procmsg_is_last_for_account(FolderItem *queue, MsgInfo *msginfo,
  *\return	Number of messages sent, negative if an error occurred
  *		positive if no error occurred
  */
-gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
+gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs, gchar **errstr)
 {
 	gint sent = 0, err = 0;
 	GSList *list, *elem;
 	GSList *sorted_list = NULL;
 	GNode *node, *next;
-
+	
+	if (send_queue_lock) {
+		/* Avoid having to translate two similar strings */
+		log_warning("%s\n", _("Already trying to send."));
+		if (errstr) {
+			if (*errstr) g_free(*errstr);
+			*errstr = g_strdup_printf(_("Already trying to send."));
+		}
+		toolbar_main_set_sensitive(mainwindow_get_mainwindow());
+		return -1;
+	}
+	send_queue_lock = TRUE;
+	inc_lock();
 	if (!queue)
 		queue = folder_get_default_queue();
-	g_return_val_if_fail(queue != NULL, -1);
+	
+	if (queue == NULL) {
+		send_queue_lock = FALSE;
+		inc_unlock();
+		return -1;
+	}
+
+	toolbar_main_set_sensitive(mainwindow_get_mainwindow());
 
 	folder_item_scan(queue);
 	list = folder_item_get_msg_list(queue);
@@ -893,21 +941,12 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 			file = folder_item_fetch_msg(queue, msginfo->msgnum);
 			if (file) {
 				if (procmsg_send_message_queue_full(file, 
-						!procmsg_is_last_for_account(queue, msginfo, elem)) < 0) {
+						!procmsg_is_last_for_account(queue, msginfo, elem),
+						errstr, queue, msginfo->msgnum) < 0) {
 					g_warning("Sending queued message %d failed.\n", 
 						  msginfo->msgnum);
 					err++;
 				} else {
-					/* CLAWS: 
-					 * We save in procmsg_send_message_queue because
-					 * we need the destination folder from the queue
-					 * header
-							
-					if (save_msgs)
-						procmsg_save_to_outbox
-							(queue->folder->outbox,
-							 file, TRUE);
-					 */
 					sent++; 
 					folder_item_remove_msg(queue, msginfo->msgnum);
 				}
@@ -928,7 +967,7 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 		while (node != NULL) {
 			int res = 0;
 			next = node->next;
-			res = procmsg_send_queue(FOLDER_ITEM(node->data), save_msgs);
+			res = procmsg_send_queue(FOLDER_ITEM(node->data), save_msgs, errstr);
 			if (res < 0) 
 				err = -res;
 			else
@@ -936,10 +975,18 @@ gint procmsg_send_queue(FolderItem *queue, gboolean save_msgs)
 			node = next;
 		}
 	}
+	send_queue_lock = FALSE;
+	inc_unlock();
+	toolbar_main_set_sensitive(mainwindow_get_mainwindow());
 
 	return (err != 0 ? -err : sent);
 }
 
+gboolean procmsg_is_sending(void)
+{
+	return send_queue_lock;
+}
+
 /*!
  *\brief	Determine if a queue folder is empty
  *
@@ -989,8 +1036,22 @@ gint procmsg_remove_special_headers(const gchar *in, const gchar *out)
 		fclose(fp);
 		return -1;
 	}
-	while (fgets(buf, sizeof(buf), fp) != NULL)
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		/* new way */
+		if (!strncmp(buf, "X-Sylpheed-End-Special-Headers: 1",
+			strlen("X-Sylpheed-End-Special-Headers:")))
+			break;
+		/* old way */
 		if (buf[0] == '\r' || buf[0] == '\n') break;
+		/* from other mailers */
+		if (!strncmp(buf, "Date: ", 6)
+		||  !strncmp(buf, "To: ", 4)
+		||  !strncmp(buf, "From: ", 6)
+		||  !strncmp(buf, "Subject: ", 9)) {
+			rewind(fp);
+			break;
+		}
+	}
 	while (fgets(buf, sizeof(buf), fp) != NULL)
 		fputs(buf, outfp);
 	fclose(outfp);
@@ -1034,7 +1095,6 @@ gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file,
 			g_warning("can't save message\n");
 			return -1;
 		}
-		return -1;
 	}
 	msginfo = folder_item_get_msginfo(outbox, num);		/* refcnt++ */
 	tmp_msginfo = procmsg_msginfo_get_full_info(msginfo);	/* refcnt++ */ 
@@ -1042,11 +1102,12 @@ gint procmsg_save_to_outbox(FolderItem *outbox, const gchar *file,
 		procmsg_msginfo_unset_flags(msginfo, ~0, 0);
 		procmsg_msginfo_free(msginfo);			/* refcnt-- */
 		/* tmp_msginfo == msginfo */
-		if (tmp_msginfo && (msginfo->dispositionnotificationto || 
-		    msginfo->returnreceiptto)) {
+		if (tmp_msginfo && msginfo->extradata && 
+		    (msginfo->extradata->dispositionnotificationto || 
+		     msginfo->extradata->returnreceiptto)) {
 			procmsg_msginfo_set_flags(msginfo, MSG_RETRCPT_SENT, 0); 
-			procmsg_msginfo_free(msginfo);		/* refcnt-- */
 		}	
+		procmsg_msginfo_free(tmp_msginfo);		/* refcnt-- */
 	}
 
 	return 0;
@@ -1168,10 +1229,22 @@ MsgInfo *procmsg_msginfo_copy(MsgInfo *msginfo)
 	MEMBCOPY(folder);
 	MEMBCOPY(to_folder);
 
-	MEMBDUP(face);
-	MEMBDUP(xface);
-	MEMBDUP(dispositionnotificationto);
-	MEMBDUP(returnreceiptto);
+	if (msginfo->extradata) {
+		newmsginfo->extradata = g_new0(MsgInfoExtraData, 1);
+		MEMBDUP(extradata->face);
+		MEMBDUP(extradata->xface);
+		MEMBDUP(extradata->dispositionnotificationto);
+		MEMBDUP(extradata->returnreceiptto);
+		MEMBDUP(extradata->partial_recv);
+		MEMBDUP(extradata->account_server);
+		MEMBDUP(extradata->account_login);
+		MEMBDUP(extradata->list_post);
+		MEMBDUP(extradata->list_subscribe);
+		MEMBDUP(extradata->list_unsubscribe);
+		MEMBDUP(extradata->list_help);
+		MEMBDUP(extradata->list_archive);
+		MEMBDUP(extradata->list_owner);
+	}
 
         refs = msginfo->references;
         for (refs = msginfo->references; refs != NULL; refs = refs->next) {
@@ -1207,29 +1280,44 @@ MsgInfo *procmsg_msginfo_get_full_info(MsgInfo *msginfo)
 	g_free(file);
 	if (!full_msginfo) return NULL;
 
-	/* CLAWS: make sure we add the missing members; see: 
-	 * procheader.c::procheader_get_headernames() */
-	if (!msginfo->xface)
-		msginfo->xface = g_strdup(full_msginfo->xface);
-	if (!msginfo->face)
-		msginfo->face = g_strdup(full_msginfo->face);
-	if (!msginfo->dispositionnotificationto)
-		msginfo->dispositionnotificationto = 
-			g_strdup(full_msginfo->dispositionnotificationto);
-	if (!msginfo->returnreceiptto)
-		msginfo->returnreceiptto = g_strdup
-			(full_msginfo->returnreceiptto);
-	if (!msginfo->partial_recv && full_msginfo->partial_recv)
-		msginfo->partial_recv = g_strdup
-			(full_msginfo->partial_recv);
 	msginfo->total_size = full_msginfo->total_size;
-	if (!msginfo->account_server && full_msginfo->account_server)
-		msginfo->account_server = g_strdup
-			(full_msginfo->account_server);
-	if (!msginfo->account_login && full_msginfo->account_login)
-		msginfo->account_login = g_strdup
-			(full_msginfo->account_login);
 	msginfo->planned_download = full_msginfo->planned_download;
+
+	if (full_msginfo->extradata) {
+		if (!msginfo->extradata)
+			msginfo->extradata = g_new0(MsgInfoExtraData, 1);
+		if (!msginfo->extradata->list_post)
+			msginfo->extradata->list_post = g_strdup(full_msginfo->extradata->list_post);
+		if (!msginfo->extradata->list_subscribe)
+			msginfo->extradata->list_subscribe = g_strdup(full_msginfo->extradata->list_subscribe);
+		if (!msginfo->extradata->list_unsubscribe)
+			msginfo->extradata->list_unsubscribe = g_strdup(full_msginfo->extradata->list_unsubscribe);
+		if (!msginfo->extradata->list_help)
+			msginfo->extradata->list_help = g_strdup(full_msginfo->extradata->list_help);
+		if (!msginfo->extradata->list_archive)
+			msginfo->extradata->list_archive= g_strdup(full_msginfo->extradata->list_archive);
+		if (!msginfo->extradata->list_owner)
+			msginfo->extradata->list_owner = g_strdup(full_msginfo->extradata->list_owner);
+		if (!msginfo->extradata->xface)
+			msginfo->extradata->xface = g_strdup(full_msginfo->extradata->xface);
+		if (!msginfo->extradata->face)
+			msginfo->extradata->face = g_strdup(full_msginfo->extradata->face);
+		if (!msginfo->extradata->dispositionnotificationto)
+			msginfo->extradata->dispositionnotificationto = 
+				g_strdup(full_msginfo->extradata->dispositionnotificationto);
+		if (!msginfo->extradata->returnreceiptto)
+			msginfo->extradata->returnreceiptto = g_strdup
+				(full_msginfo->extradata->returnreceiptto);
+		if (!msginfo->extradata->partial_recv && full_msginfo->extradata->partial_recv)
+			msginfo->extradata->partial_recv = g_strdup
+				(full_msginfo->extradata->partial_recv);
+		if (!msginfo->extradata->account_server && full_msginfo->extradata->account_server)
+			msginfo->extradata->account_server = g_strdup
+				(full_msginfo->extradata->account_server);
+		if (!msginfo->extradata->account_login && full_msginfo->extradata->account_login)
+			msginfo->extradata->account_login = g_strdup
+				(full_msginfo->extradata->account_login);
+	}
 	procmsg_msginfo_free(full_msginfo);
 
 	return procmsg_msginfo_new_ref(msginfo);
@@ -1249,10 +1337,6 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 	}
 
 	g_free(msginfo->fromspace);
-	g_free(msginfo->returnreceiptto);
-	g_free(msginfo->dispositionnotificationto);
-	g_free(msginfo->xface);
-	g_free(msginfo->face);
 
 	g_free(msginfo->fromname);
 
@@ -1266,10 +1350,22 @@ void procmsg_msginfo_free(MsgInfo *msginfo)
 	g_free(msginfo->inreplyto);
 	g_free(msginfo->xref);
 
-	g_free(msginfo->partial_recv);
-	g_free(msginfo->account_server);
-	g_free(msginfo->account_login);
-	
+	if (msginfo->extradata) {
+		g_free(msginfo->extradata->returnreceiptto);
+		g_free(msginfo->extradata->dispositionnotificationto);
+		g_free(msginfo->extradata->xface);
+		g_free(msginfo->extradata->face);
+		g_free(msginfo->extradata->list_post);
+		g_free(msginfo->extradata->list_subscribe);
+		g_free(msginfo->extradata->list_unsubscribe);
+		g_free(msginfo->extradata->list_help);
+		g_free(msginfo->extradata->list_archive);
+		g_free(msginfo->extradata->list_owner);
+		g_free(msginfo->extradata->partial_recv);
+		g_free(msginfo->extradata->account_server);
+		g_free(msginfo->extradata->account_login);
+		g_free(msginfo->extradata);
+	}
 	slist_free_strings(msginfo->references);
 	g_slist_free(msginfo->references);
 
@@ -1302,14 +1398,6 @@ guint procmsg_msginfo_memusage(MsgInfo *msginfo)
 		memusage += strlen(msginfo->msgid);
 	if (msginfo->inreplyto)
 		memusage += strlen(msginfo->inreplyto);
-	if (msginfo->xface)
-		memusage += strlen(msginfo->xface);
-	if (msginfo->face)
-		memusage += strlen(msginfo->face);
-	if (msginfo->dispositionnotificationto)
-		memusage += strlen(msginfo->dispositionnotificationto);
-	if (msginfo->returnreceiptto)
-		memusage += strlen(msginfo->returnreceiptto);
 	for (refs = msginfo->references; refs; refs=refs->next) {
 		gchar *r = (gchar *)refs->data;
 		memusage += r?strlen(r):0;
@@ -1317,6 +1405,37 @@ guint procmsg_msginfo_memusage(MsgInfo *msginfo)
 	if (msginfo->fromspace)
 		memusage += strlen(msginfo->fromspace);
 
+	if (msginfo->extradata) {
+		memusage += sizeof(MsgInfoExtraData);
+		if (msginfo->extradata->xface)
+			memusage += strlen(msginfo->extradata->xface);
+		if (msginfo->extradata->face)
+			memusage += strlen(msginfo->extradata->face);
+		if (msginfo->extradata->dispositionnotificationto)
+			memusage += strlen(msginfo->extradata->dispositionnotificationto);
+		if (msginfo->extradata->returnreceiptto)
+			memusage += strlen(msginfo->extradata->returnreceiptto);
+
+		if (msginfo->extradata->partial_recv)
+			memusage += strlen(msginfo->extradata->partial_recv);
+		if (msginfo->extradata->account_server)
+			memusage += strlen(msginfo->extradata->account_server);
+		if (msginfo->extradata->account_login)
+			memusage += strlen(msginfo->extradata->account_login);
+
+		if (msginfo->extradata->list_post)
+			memusage += strlen(msginfo->extradata->list_post);
+		if (msginfo->extradata->list_subscribe)
+			memusage += strlen(msginfo->extradata->list_subscribe);
+		if (msginfo->extradata->list_unsubscribe)
+			memusage += strlen(msginfo->extradata->list_unsubscribe);
+		if (msginfo->extradata->list_help)
+			memusage += strlen(msginfo->extradata->list_help);
+		if (msginfo->extradata->list_archive)
+			memusage += strlen(msginfo->extradata->list_archive);
+		if (msginfo->extradata->list_owner)
+			memusage += strlen(msginfo->extradata->list_owner);
+	}
 	return memusage;
 }
 
@@ -1333,7 +1452,8 @@ gint procmsg_cmp_msgnum_for_sort(gconstpointer a, gconstpointer b)
 	return msginfo1->msgnum - msginfo2->msgnum;
 }
 
-static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_session)
+static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_session, gchar **errstr,
+					    FolderItem *queue, gint msgnum)
 {
 	static HeaderEntry qentry[] = {{"S:",    NULL, FALSE},
 				       {"SSV:",  NULL, FALSE},
@@ -1347,6 +1467,7 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 				       {"X-Sylpheed-Privacy-System:", NULL, FALSE},
 				       {"X-Sylpheed-Encrypt:", NULL, FALSE},
 				       {"X-Sylpheed-Encrypt-Data:", NULL, FALSE},
+				       {"X-Sylpheed-End-Special-Headers:", NULL, FALSE},
 				       {NULL,    NULL, FALSE}};
 	FILE *fp;
 	gint filepos;
@@ -1373,6 +1494,10 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 
 	if ((fp = g_fopen(file, "rb")) == NULL) {
 		FILE_OP_ERROR(file, "fopen");
+		if (errstr) {
+			if (*errstr) g_free(*errstr);
+			*errstr = g_strdup_printf(_("Couldn't open file %s."), file);
+		}
 		return -1;
 	}
 
@@ -1425,14 +1550,22 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 			if (encrypt_data == NULL) 
 				encrypt_data = g_strdup(p);
 			break;
+		case Q_SYLPHEED_HDRS:
+			/* end of special headers reached */
+			goto send_mail; /* can't "break;break;" */
 		}
 	}
+send_mail:
 	filepos = ftell(fp);
 
 	if (encrypt) {
 		MimeInfo *mimeinfo;
 
-		save_clear_text = (mailac != NULL && mailac->save_encrypted_as_clear_text);
+		if (mailac && mailac->save_encrypted_as_clear_text 
+		&&  !mailac->encrypt_to_self)
+			save_clear_text = TRUE;
+		else
+			save_clear_text = FALSE;
 
 		fclose(fp);
 		fp = NULL;
@@ -1455,6 +1588,11 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 			g_free(fwdmessageid);
 			g_free(privacy_system);
 			g_free(encrypt_data);
+			if (errstr) {
+				if (*errstr) g_free(*errstr);
+				*errstr = g_strdup_printf(_("Couldn't encrypt the email: %s"),
+						privacy_get_error());
+			}
 			return -1;
 		}
 		
@@ -1483,7 +1621,10 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 	if (to_list) {
 		debug_print("Sending message by mail\n");
 		if (!from) {
-			g_warning("Queued message header is broken.\n");
+			if (errstr) {
+				if (*errstr) g_free(*errstr);
+				*errstr = g_strdup_printf(_("Queued message header is broken."));
+			}
 			mailval = -1;
 		} else if (mailac && mailac->use_mail_command &&
 			   mailac->mail_command && (* mailac->mail_command)) {
@@ -1499,9 +1640,13 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 				}
 			}
 
-			if (mailac)
+			if (mailac) {
 				mailval = send_message_smtp_full(mailac, to_list, fp, keep_session);
-			else {
+				if (mailval == -1 && errstr) {
+					if (*errstr) g_free(*errstr);
+					*errstr = g_strdup_printf(_("An error happened during SMTP session."));
+				}
+			} else {
 				PrefsAccount tmp_ac;
 
 				g_warning("Account not found.\n");
@@ -1511,8 +1656,20 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 				tmp_ac.smtp_server = smtpserver;
 				tmp_ac.smtpport = SMTP_PORT;
 				mailval = send_message_smtp(&tmp_ac, to_list, fp);
+				if (mailval == -1 && errstr) {
+					if (*errstr) g_free(*errstr);
+					*errstr = g_strdup_printf(_("No specific account has been found to "
+							"send, and an error happened during SMTP session."));
+				}
 			}
 		}
+	} else if (!to_list && !newsgroup_list) {
+		if (errstr) {
+			if (*errstr) g_free(*errstr);
+			*errstr = g_strdup(_("Couldn't determine sending informations. "
+				"Maybe the email hasn't been generated by Sylpheed-Claws."));
+		}
+		mailval = -1;
 	}
 
 	fseek(fp, filepos, SEEK_SET);
@@ -1527,7 +1684,7 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
     		if ((tmpfp = g_fopen(tmp, "wb")) == NULL) {
             		FILE_OP_ERROR(tmp, "fopen");
             		newsval = -1;
-			alertpanel_error(_("Could not create temporary file for news sending."));
+			alertpanel_error(_("Couldn't create temporary file for news sending."));
     		} else {
     			if (change_file_mode_rw(tmpfp, tmp) < 0) {
             			FILE_OP_ERROR(tmp, "chmod");
@@ -1538,7 +1695,10 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 				if (fputs(buf, tmpfp) == EOF) {
 					FILE_OP_ERROR(tmp, "fputs");
 					newsval = -1;
-					alertpanel_error(_("Error when writing temporary file for news sending."));
+					if (errstr) {
+						if (*errstr) g_free(*errstr);
+						*errstr = g_strdup_printf(_("Error when writing temporary file for news sending."));
+					}
 				}
 			}
 			fclose(tmpfp);
@@ -1549,10 +1709,11 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 				folder = FOLDER(newsac->folder);
 
     				newsval = news_post(folder, tmp);
-    				if (newsval < 0) {
-            				alertpanel_error(_("Error occurred while posting the message to %s ."),
-                            			 newsac->nntp_server);
-    				}
+    				if (newsval < 0 && errstr)  {
+					if (*errstr) g_free(*errstr);
+					*errstr = g_strdup_printf(_("Error occurred while posting the message to %s."),
+                            		 newsac->nntp_server);
+				}
 			}
 			g_unlink(tmp);
 		}
@@ -1572,7 +1733,15 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 			outbox = folder_get_default_outbox();
 			
 		if (save_clear_text || tmp_enc_file == NULL) {
-			procmsg_save_to_outbox(outbox, file, TRUE);
+			gboolean saved = FALSE;
+			if (queue && msgnum > 0) {
+				MsgInfo *queued_mail = folder_item_get_msginfo(queue, msgnum);
+				if (folder_item_copy_msg(outbox, queued_mail) >= 0)
+					saved = TRUE;
+				procmsg_msginfo_free(queued_mail);
+			}
+			if (!saved)
+				procmsg_save_to_outbox(outbox, file, TRUE);
 		} else {
 			procmsg_save_to_outbox(outbox, tmp_enc_file, FALSE);
 		}
@@ -1641,9 +1810,11 @@ static gint procmsg_send_message_queue_full(const gchar *file, gboolean keep_ses
 	return (newsval != 0 ? newsval : mailval);
 }
 
-gint procmsg_send_message_queue(const gchar *file)
+gint procmsg_send_message_queue(const gchar *file, gchar **errstr, FolderItem *queue, gint msgnum)
 {
-	return procmsg_send_message_queue_full(file, FALSE);
+	gint result = procmsg_send_message_queue_full(file, FALSE, errstr, queue, msgnum);
+	toolbar_main_set_sensitive(mainwindow_get_mainwindow());
+	return result;
 }
 
 static void update_folder_msg_counts(FolderItem *item, MsgInfo *msginfo, MsgPermFlags old_flags)
@@ -1745,11 +1916,6 @@ void procmsg_msginfo_unset_flags(MsgInfo *msginfo, MsgPermFlags perm_flags, MsgT
 		folder_item_change_msg_flags(msginfo->folder, msginfo, perm_flags_new);
 
 		update_folder_msg_counts(item, msginfo, perm_flags_old);
-
-		msginfo_update.msginfo = msginfo;
-		msginfo_update.flags = MSGINFO_UPDATE_FLAGS;
-		hooks_invoke(MSGINFO_UPDATE_HOOKLIST, &msginfo_update);
-		folder_item_update(msginfo->folder, F_ITEM_UPDATE_MSGCNT);
 	}
 
 	/* Tmp flags hanlding */
@@ -1834,9 +2000,8 @@ gboolean procmsg_msg_has_flagged_parent_real(MsgInfo *info,
 			gboolean result;
 
 			if (g_hash_table_lookup(parentmsgs, info)) {
-				debug_print("loop detected: %s%c%d\n",
-					folder_item_get_path(info->folder),
-					G_DIR_SEPARATOR, info->msgnum);
+				debug_print("loop detected: %d\n",
+					info->msgnum);
 				result = FALSE;
 			} else {
 				g_hash_table_insert(parentmsgs, info, "1");
@@ -1869,11 +2034,14 @@ gboolean parentmsgs_hash_remove(gpointer key,
 gboolean procmsg_msg_has_flagged_parent(MsgInfo *info, MsgPermFlags perm_flags)
 {
 	gboolean result;
-	GHashTable *parentmsgs = g_hash_table_new(NULL, NULL); 
+	static GHashTable *parentmsgs = NULL;
+	
+	if (parentmsgs == NULL)
+		parentmsgs = g_hash_table_new(NULL, NULL); 
 
 	result = procmsg_msg_has_flagged_parent_real(info, perm_flags, parentmsgs);
 	g_hash_table_foreach_remove(parentmsgs, parentmsgs_hash_remove, NULL);
-	g_hash_table_destroy(parentmsgs);
+
 	return result;
 }
 
@@ -1977,24 +2145,105 @@ void procmsg_msginfo_set_to_folder(MsgInfo *msginfo, FolderItem *to_folder)
  * \return TRUE if the message was moved and MsgInfo is now invalid,
  *         FALSE otherwise
  */
-gboolean procmsg_msginfo_filter(MsgInfo *msginfo)
+gboolean procmsg_msginfo_filter(MsgInfo *msginfo, PrefsAccount* ac_prefs)
 {
 	MailFilteringData mail_filtering_data;
 			
 	mail_filtering_data.msginfo = msginfo;			
+	mail_filtering_data.msglist = NULL;			
+	mail_filtering_data.filtered = NULL;			
+	mail_filtering_data.unfiltered = NULL;			
 	if (hooks_invoke(MAIL_FILTERING_HOOKLIST, &mail_filtering_data)) {
 		return TRUE;
 	}
 
 	/* filter if enabled in prefs or move to inbox if not */
 	if((filtering_rules != NULL) &&
-	   filter_message_by_msginfo(filtering_rules, msginfo)) {
+	   filter_message_by_msginfo(filtering_rules, msginfo, ac_prefs)) {
 		return TRUE;
 	}
 		
 	return FALSE;
 }
 
+void procmsg_msglist_filter(GSList *list, PrefsAccount *ac, 
+			    GSList **filtered, GSList **unfiltered,
+			    gboolean do_filter)
+{
+	GSList *cur, *to_do = NULL;
+	gint total = 0, curnum = 0;
+	MailFilteringData mail_filtering_data;
+			
+	g_return_if_fail(filtered != NULL);
+	g_return_if_fail(unfiltered != NULL);
+
+	*filtered = NULL;
+	*unfiltered = NULL;
+	
+	if (list == NULL)
+		return;
+
+	total = g_slist_length(list);
+
+	if (!do_filter) {
+		*filtered = NULL;
+		*unfiltered = g_slist_copy(list);
+		return;
+	}
+
+	statusbar_print_all(_("Filtering messages...\n"));
+
+	mail_filtering_data.msginfo = NULL;			
+	mail_filtering_data.msglist = list;			
+	mail_filtering_data.filtered = NULL;			
+	mail_filtering_data.unfiltered = NULL;	
+			
+	hooks_invoke(MAIL_LISTFILTERING_HOOKLIST, &mail_filtering_data);
+	
+	if (mail_filtering_data.filtered == NULL &&
+	    mail_filtering_data.unfiltered == NULL) {
+	    	/* nothing happened */
+		debug_print(MAIL_LISTFILTERING_HOOKLIST " did nothing. filtering whole list normally.\n");
+		to_do = list;
+	} 
+	if (mail_filtering_data.filtered != NULL) {
+		/* keep track of what's been filtered by the hooks */
+		debug_print(MAIL_LISTFILTERING_HOOKLIST " filtered some stuff. total %d filtered %d unfilt %d.\n",
+			g_slist_length(list),
+			g_slist_length(mail_filtering_data.filtered),
+			g_slist_length(mail_filtering_data.unfiltered));
+
+		*filtered = g_slist_copy(mail_filtering_data.filtered);
+	}
+	if (mail_filtering_data.unfiltered != NULL) {
+		/* what the hooks didn't handle will go in filtered or 
+		 * unfiltered in the next loop */
+		debug_print(MAIL_LISTFILTERING_HOOKLIST " left unfiltered stuff. total %d filtered %d unfilt %d.\n",
+			g_slist_length(list),
+			g_slist_length(mail_filtering_data.filtered),
+			g_slist_length(mail_filtering_data.unfiltered));
+		to_do = mail_filtering_data.unfiltered;
+	} 
+
+	for (cur = to_do; cur; cur = cur->next) {
+		MsgInfo *info = (MsgInfo *)cur->data;
+		if (procmsg_msginfo_filter(info, ac))
+			*filtered = g_slist_prepend(*filtered, info);
+		else
+			*unfiltered = g_slist_prepend(*unfiltered, info);
+		statusbar_progress_all(curnum++, total, prefs_common.statusbar_update_step);
+	}
+
+	g_slist_free(mail_filtering_data.filtered);
+	g_slist_free(mail_filtering_data.unfiltered);
+	
+	*filtered = g_slist_reverse(*filtered);
+	*unfiltered = g_slist_reverse(*unfiltered);
+
+	statusbar_progress_all(0,0,0);
+	statusbar_pop_all();
+}
+
 MsgInfo *procmsg_msginfo_new_from_mimeinfo(MsgInfo *src_msginfo, MimeInfo *mimeinfo)
 {
 	MsgInfo *tmp_msginfo = NULL;
@@ -2078,8 +2327,7 @@ int procmsg_spam_learner_learn (MsgInfo *info, GSList *list, gboolean spam)
 static gchar *spam_folder_item = NULL;
 void procmsg_spam_set_folder (const char *item_identifier)
 {
-	if (spam_folder_item)
-		g_free(spam_folder_item);
+	g_free(spam_folder_item);
 	if (item_identifier)
 		spam_folder_item = g_strdup(item_identifier);
 	else
@@ -2091,3 +2339,19 @@ FolderItem *procmsg_spam_get_folder (void)
 	FolderItem *item = spam_folder_item ? folder_find_item_from_identifier(spam_folder_item) : NULL;
 	return item ? item : folder_get_default_trash();
 }
+
+static void item_has_queued_mails(FolderItem *item, gpointer data)
+{
+	gboolean *result = (gboolean *)data;
+	if (*result == TRUE)
+		return;
+	if (folder_has_parent_of_type(item, F_QUEUE) && item->total_msgs > 0)
+		*result = TRUE;
+}
+
+gboolean procmsg_have_queued_mails_fast (void)
+{
+	gboolean result = FALSE;
+	folder_func_to_all_folders(item_has_queued_mails, &result);
+	return result;
+}
