@@ -36,6 +36,8 @@ typedef struct _MsgFileInfo     	MsgFileInfo;
 typedef struct _MsgInfoUpdate 		MsgInfoUpdate;
 typedef struct _MailFilteringData	MailFilteringData;
 
+typedef struct _MsgInfoExtraData	MsgInfoExtraData;
+
 typedef GSList MsgInfoList;
 typedef GSList MsgNumberList;
 
@@ -84,6 +86,7 @@ typedef guint32 MsgPermFlags;
 
 #define MSG_MOVE		(1U << 0)
 #define MSG_COPY		(1U << 1)
+#define MSG_MOVE_DONE		(1U << 15)		
 #define MSG_QUEUED		(1U << 16)
 #define MSG_DRAFT		(1U << 17)
 #define MSG_ENCRYPTED		(1U << 18)
@@ -128,6 +131,7 @@ typedef guint32 MsgTmpFlags;
 
 #define MSG_IS_MOVE(msg)		(((msg).tmp_flags & MSG_MOVE) != 0)
 #define MSG_IS_COPY(msg)		(((msg).tmp_flags & MSG_COPY) != 0)
+#define MSG_IS_MOVE_DONE(msg)		(((msg).tmp_flags & MSG_MOVE_DONE) != 0)
 
 #define MSG_IS_QUEUED(msg)		(((msg).tmp_flags & MSG_QUEUED) != 0)
 #define MSG_IS_DRAFT(msg)		(((msg).tmp_flags & MSG_DRAFT) != 0)
@@ -148,6 +152,7 @@ typedef guint32 MsgTmpFlags;
 
 #define MSGINFO_UPDATE_HOOKLIST "msginfo_update"
 #define MAIL_FILTERING_HOOKLIST "mail_filtering_hooklist"
+#define MAIL_LISTFILTERING_HOOKLIST "mail_listfiltering_hooklist"
 #define MAIL_POSTFILTERING_HOOKLIST "mail_postfiltering_hooklist"
 
 typedef enum {
@@ -165,7 +170,13 @@ struct _MsgFlags
 	MsgTmpFlags  tmp_flags;
 };
 
-
+/* *********************************************************** *
+ * WARNING: When adding or removing members to this structure, *
+ * be sure to update procmsg.c::procmsg_msginfo_memusage()  to *
+ * avoid underestimating cache memory usage - especially since *
+ * this would cause an overflow and metadata loss when writing *
+ * the cache to disk.                                          *
+ * *********************************************************** */
 struct _MsgInfo
 {
 	guint refcnt;
@@ -196,12 +207,6 @@ struct _MsgInfo
 	gboolean is_move;
 	gboolean is_copy;
 
-	gchar *xface;
-	gchar *face;
-
-	gchar *dispositionnotificationto;
-	gchar *returnreceiptto;
-
 	GSList *references;
 	gchar *fromspace;
 
@@ -211,13 +216,34 @@ struct _MsgInfo
 	gchar *plaintext_file;
         
         gint hidden;
-	
+
+	/* used only for partially received messages */
+	gint total_size;
+	gint planned_download;
+
+	MsgInfoExtraData *extradata;
+};
+
+struct _MsgInfoExtraData
+{
+	gchar *xface;
+	gchar *face;
+
+	gchar *dispositionnotificationto;
+	gchar *returnreceiptto;
+
 	/* used only for partially received messages */
 	gchar *partial_recv;
-	gint total_size;
 	gchar *account_server;
 	gchar *account_login;
-	gint planned_download;
+
+ 	/* Mailing list support */
+ 	gchar *list_post;
+ 	gchar *list_subscribe;
+ 	gchar *list_unsubscribe;
+ 	gchar *list_help;
+ 	gchar *list_archive;
+ 	gchar *list_owner;
 };
 
 struct _MsgFileInfo
@@ -235,6 +261,9 @@ struct _MsgInfoUpdate {
 struct _MailFilteringData
 {
 	MsgInfo	*msginfo;
+	GSList  *msglist;
+	GSList  *filtered;
+	GSList  *unfiltered;
 };
 
 GHashTable *procmsg_msg_hash_table_create	(GSList		*mlist);
@@ -279,7 +308,8 @@ void	procmsg_empty_trash		(FolderItem	*trash);
 void	procmsg_empty_all_trash		(void);
 
 gint	procmsg_send_queue		(FolderItem	*queue,
-					 gboolean	 save_msgs);
+					 gboolean	 save_msgs,
+					 gchar		**errstr);
 gboolean procmsg_queue_is_empty	(FolderItem *queue);
 gint	procmsg_save_to_outbox		(FolderItem	*outbox,
 					 const gchar	*file,
@@ -296,7 +326,10 @@ guint	 procmsg_msginfo_memusage	(MsgInfo	*msginfo);
 
 gint procmsg_cmp_msgnum_for_sort	(gconstpointer	 a,
 					 gconstpointer	 b);
-gint procmsg_send_message_queue		(const gchar *file);
+gint procmsg_send_message_queue		(const gchar *file,
+					 gchar **errstr,
+					 FolderItem *queue, 
+					 gint msgnum);
 
 void procmsg_msginfo_set_flags		(MsgInfo *msginfo,
 					 MsgPermFlags perm_flags,
@@ -320,7 +353,14 @@ void procmsg_update_unread_children	(MsgInfo 	*info,
 					 gboolean 	 newly_marked);
 void procmsg_msginfo_set_to_folder	(MsgInfo 	*msginfo,
 					 FolderItem 	*to_folder);
-gboolean procmsg_msginfo_filter		(MsgInfo	*msginfo);
+gboolean procmsg_msginfo_filter		(MsgInfo	*msginfo,
+					 PrefsAccount	*ac_prefs);
+void procmsg_msglist_filter		(GSList 	*list, 
+					 PrefsAccount 	*ac, 
+					 GSList 	**filtered,
+					 GSList 	**unfiltered,
+					 gboolean 	 do_filter);
+
 MsgInfo *procmsg_msginfo_new_from_mimeinfo
 					(MsgInfo 	*src_msginfo, 
 					 MimeInfo	*mimeinfo);
@@ -331,5 +371,7 @@ gboolean procmsg_spam_can_learn		(void);
 void procmsg_spam_set_folder		(const char *item_identifier);
 FolderItem *procmsg_spam_get_folder	(void);
 int procmsg_spam_learner_learn 	(MsgInfo *msginfo, GSList *msglist, gboolean spam);
+gboolean procmsg_have_queued_mails_fast (void);
+gboolean procmsg_is_sending(void);
 
 #endif /* __PROCMSG_H__ */
