@@ -37,7 +37,7 @@
  * When adding new lines, remember to put 2 strings for each line
  */
 static gchar *quote_desc_strings[] = {
-	"%D{fmt}",	N_("customized date format (see man strftime)"), /* date expression */
+	"%D{fmt}",	N_("customized date format (see 'man strftime')"), /* date expression */
 	"%d",		N_("Date"), /* date */
 	"%f",		N_("From"), /* from */
 	"%N",		N_("full name of sender"), /* full name */
@@ -55,18 +55,29 @@ static gchar *quote_desc_strings[] = {
 	"%m",		N_("message body without signature"), /* message with no signature */
 	"%q",		N_("quoted message body without signature"), /* quoted message with no signature */
 	"%X",		N_("cursor position"), /* X marks the cursor spot */
-	"",		NULL,
-	"?x{expr}",	N_("insert expr if x is set\nx is one of the characters above after %"),
-	"",		NULL,
+	"%af",		N_("Account property: your name"), /* full name in compose account */
+	"%am",		N_("Account property: your email address"), /* mail address in compose account */
+	"%an",		N_("Account property: account name"), /* compose account name itself */
+	"%ao",		N_("Account property: organization"), /* organization in compose account */
 	"\\%", 		N_("literal %"),
 	"\\\\",		N_("literal backslash"),
 	"\\?",		N_("literal question mark"),
+	"\\!",		N_("literal exclamation mark"),
 	"\\|",		N_("literal pipe"),
 	"\\{",		N_("literal opening curly brace"),
 	"\\}",		N_("literal closing curly brace"),
+	"\\t", 		N_("tab"),
+	"\\n", 		N_("linefeed"),
 	"",		NULL,
-	"|f{file}",	N_("insert file"),
-	"|p{command}",  N_("insert program output"), /* insert program output */
+	"?x{expr}\n",	N_("insert expr if x is set\n(where x is one of the dfNFLIstcnri characters or af, ao)"),
+	"!x{expr}\n",	N_("insert expr if x is not set\n(where x is one of the dfNFLIstcnri characters or af, ao)"),
+	"|f{sub_expr}\n",	N_("insert file:\nsub_expr is evaluated as a filename to insert"), /* insert file */
+	"|p{sub_expr}\n\n",	N_("insert program output:\nsub_expr is evaluated as a command-line to get\nthe output from"), /* insert program output */
+	"|i{sub_expr}\n\n",	N_("insert user input:\nsub_expr is a variable to be replaced by\nuser-entered text"), /* insert user input */
+	"",		NULL,
+	N_("terms definition:"),	NULL,
+	"expr",			N_("text that can contain any of the symbols above"),
+	"sub_expr\n",	N_("text that can contain any of the symbols above\nbut ?x{}, !x{}, |f{}, |p{} and |i{}"),
 	NULL,NULL
 };
 
