@@ -148,7 +148,7 @@ gint send_message_local(const gchar *command, FILE *fp)
 				     &pid, &child_stdin, NULL, NULL,
 				     NULL) == FALSE) {
 		g_snprintf(buf, sizeof(buf),
-			   _("Can't execute command: %s"), command);
+			   _("Couldn't execute command: %s"), command);
 		log_warning("%s\n", buf);
 		alertpanel_error("%s", buf);
 		g_strfreev(argv);
@@ -202,6 +202,10 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	gchar buf[BUFFSIZE];
 	gint ret = 0;
 	gboolean was_inited = FALSE;
+	MsgInfo *tmp_msginfo = NULL;
+	MsgFlags flags = {0, 0};
+	long fp_pos = 0;
+	gchar spec_from[BUFFSIZE];
 
 	g_return_val_if_fail(ac_prefs != NULL, -1);
 	g_return_val_if_fail(ac_prefs->address != NULL, -1);
@@ -209,6 +213,23 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	g_return_val_if_fail(to_list != NULL, -1);
 	g_return_val_if_fail(fp != NULL, -1);
 
+	/* get the From address used, not necessarily the ac_prefs',
+	 * because it's editable. */
+
+	fp_pos = ftell(fp);
+	tmp_msginfo = procheader_parse_stream(fp, flags, FALSE, FALSE);
+	fseek(fp, fp_pos, SEEK_SET);
+	
+	if (tmp_msginfo && tmp_msginfo->from) {
+		strncpy2(spec_from, tmp_msginfo->from, BUFFSIZE-1);
+		extract_address(spec_from);
+	} else {
+		strncpy2(spec_from, ac_prefs->address, BUFFSIZE-1);
+	}
+	if (tmp_msginfo) {
+		procmsg_msginfo_free(tmp_msginfo);
+	}
+
 	if (!ac_prefs->session) {
 		/* we can't reuse a previously initialised session */
 		session = smtp_session_new();
@@ -282,10 +303,9 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 				  "Do you want to continue connecting to this "
 				  "server? The communication would not be "
 				  "secure."),
-				  _("Con_tinue connecting"), 
-				  GTK_STOCK_CANCEL, NULL,
-				  FALSE, NULL, ALERT_WARNING,
-				  G_ALERTALTERNATE) != G_ALERTDEFAULT) {
+				  GTK_STOCK_CANCEL, _("Con_tinue connecting"),
+				  NULL, FALSE, NULL, ALERT_WARNING,
+				  G_ALERTDEFAULT) != G_ALERTALTERNATE) {
 				session_destroy(session);
 				return -1;
 			}
@@ -322,12 +342,12 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 			(session, send_send_data_progressive, dialog);
 		session_set_send_data_notify(session, send_send_data_finished, dialog);
 
-		ac_prefs->session = SMTP_SESSION(session);
 	} else {
 		/* everything is ready to start at MAIL FROM:, just
 		 * reinit useful variables. 
 		 */
 		session = SESSION(ac_prefs->session);
+		ac_prefs->session = NULL;
 		smtp_session = SMTP_SESSION(session);
 		smtp_session->state = SMTP_HELO;
 		dialog = (SendProgressDialog *)smtp_session->dialog;
@@ -335,12 +355,14 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 	}
 
 	/* This has to be initialised for every mail sent */
-	smtp_session->from = g_strdup(ac_prefs->address);
+	smtp_session->from = g_strdup(spec_from);
 	smtp_session->to_list = to_list;
 	smtp_session->cur_to = to_list;
-	smtp_session->send_data = get_outgoing_rfc2822_str(fp);
-	smtp_session->send_data_len = strlen(smtp_session->send_data);
+	smtp_session->send_data = (guchar *)get_outgoing_rfc2822_str(fp);
+	smtp_session->send_data_len = strlen((gchar *)smtp_session->send_data);
 
+	session_set_timeout(session,
+			    prefs_common.io_timeout_secs * 1000);
 	/* connect if necessary */
 	if (!was_inited && session_connect(session, ac_prefs->smtp_server, port) < 0) {
 		session_destroy(session);
@@ -409,6 +431,9 @@ gint send_message_smtp_full(PrefsAccount *ac_prefs, GSList *to_list, FILE *fp, g
 		g_free(smtp_session->send_data);
 		g_free(smtp_session->error_msg);
 	}
+	if (keep_session && ret == 0 && ac_prefs->session == NULL)
+		ac_prefs->session = SMTP_SESSION(session);
+
 
 	statusbar_pop_all();
 	statusbar_verbosity_set(FALSE);
@@ -493,7 +518,7 @@ static gint send_send_data_progressive(Session *session, guint cur_len,
 		   cur_len, total_len);
 	progress_dialog_set_label(dialog->dialog, buf);
 	progress_dialog_set_fraction
-		(dialog->dialog, (gfloat)cur_len / (gfloat)total_len);
+		(dialog->dialog, (total_len == 0) ? 0 : (gfloat)cur_len / (gfloat)total_len);
 
 	return 0;
 }
@@ -508,10 +533,20 @@ static gint send_send_data_finished(Session *session, guint len, gpointer data)
 	return 0;
 }
 
+static void send_progress_dialog_size_allocate_cb(GtkWidget *widget,
+					 GtkAllocation *allocation)
+{
+	g_return_if_fail(allocation != NULL);
+
+	prefs_common.sendwin_width = allocation->width;
+	prefs_common.sendwin_height = allocation->height;
+}
+
 static SendProgressDialog *send_progress_dialog_create(void)
 {
 	SendProgressDialog *dialog;
 	ProgressDialog *progress;
+	static GdkGeometry geometry;
 
 	dialog = g_new0(SendProgressDialog, 1);
 
@@ -523,10 +558,22 @@ static SendProgressDialog *send_progress_dialog_create(void)
 	g_signal_connect(G_OBJECT(progress->window), "delete_event",
 			 G_CALLBACK(gtk_true), NULL);
 	gtk_window_set_modal(GTK_WINDOW(progress->window), TRUE);
+	g_signal_connect(G_OBJECT(progress->window), "size_allocate",
+			 G_CALLBACK(send_progress_dialog_size_allocate_cb), NULL);
 	manage_window_set_transient(GTK_WINDOW(progress->window));
 
 	progress_dialog_get_fraction(progress);
 
+	if (!geometry.min_height) {
+		geometry.min_width = 460;
+		geometry.min_height = 250;
+	}
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(progress->window), NULL, &geometry,
+				      GDK_HINT_MIN_SIZE);
+	gtk_widget_set_size_request(progress->window, prefs_common.sendwin_width,
+				    prefs_common.sendwin_height);
+
 	if (prefs_common.send_dialog_mode == SEND_DIALOG_ALWAYS) {
 		gtk_widget_show_now(progress->window);
 	}
@@ -600,11 +647,12 @@ static void send_put_error(Session *session)
 		break;
 	}
 
-	if (log_msg)
-		log_warning("%s\n", log_msg);
 	if (err_msg) {
-		alertpanel_error_log("%s", err_msg);
+		log_error("%s\n", err_msg);
 		g_free(err_msg);
+	} else {
+		if (log_msg)
+			log_warning("%s\n", log_msg);
 	}
 }
 
