@@ -25,6 +25,8 @@
 
 typedef enum
 {
+	STOCK_PIXMAP_ADDR_ONE,
+	STOCK_PIXMAP_ADDR_TWO,
 	STOCK_PIXMAP_ADDRESS,
 	STOCK_PIXMAP_ADDRESS_BOOK,
 	STOCK_PIXMAP_ADDRESS_SEARCH,
@@ -147,14 +149,16 @@ typedef enum
 	STOCK_PIXMAP_PRIVACY_EMBLEM_FAILED,
 	STOCK_PIXMAP_PRIVACY_EMBLEM_WARN,
 	STOCK_PIXMAP_MIME_MESSAGE,
-	STOCK_PIXMAP_SYLPHEED_ICON,
+	STOCK_PIXMAP_SYLPHEED_CLAWS_ICON,
 	STOCK_PIXMAP_READ,
 	STOCK_PIXMAP_TRASH,
-	STOCK_PIXMAP_SYLPHEED_LOGO,
+	STOCK_PIXMAP_SYLPHEED_CLAWS_LOGO,
 	STOCK_PIXMAP_DIR_NOSELECT, 
 	STOCK_PIXMAP_SPAM,
 	STOCK_PIXMAP_SPAM_BTN,
 	STOCK_PIXMAP_HAM_BTN,
+	STOCK_PIXMAP_MOVED,
+	STOCK_PIXMAP_COPIED,
 	STOCK_PIXMAP_EMPTY,              /* last entry */
 	N_STOCK_PIXMAPS
 } StockPixmap;
