@@ -56,6 +56,7 @@
 #include "matcher.h"
 #include "matcher_parser.h"
 #include "prefs_matcher.h"
+#include "manual.h"
 
 static struct SummarySearchWindow {
 	GtkWidget *window;
@@ -74,6 +75,7 @@ static struct SummarySearchWindow {
 	GtkWidget *case_checkbtn;
 
 	GtkWidget *clear_btn;
+	GtkWidget *help_btn;
 	GtkWidget *all_btn;
 	GtkWidget *prev_btn;
 	GtkWidget *next_btn;
@@ -116,6 +118,15 @@ static gboolean key_pressed		(GtkWidget	*widget,
 					 gpointer	 data);
 
 
+#define GTK_BUTTON_SET_SENSITIVE(widget,sensitive) {	\
+	gboolean in_btn = FALSE;							\
+	if (GTK_IS_BUTTON(widget))							\
+		in_btn = GTK_BUTTON(widget)->in_button;			\
+	gtk_widget_set_sensitive(widget, sensitive);		\
+	if (GTK_IS_BUTTON(widget))							\
+		GTK_BUTTON(widget)->in_button = in_btn;			\
+}
+
 void summary_search(SummaryView *summaryview)
 {
 	if (!search_window.window) {
@@ -131,6 +142,24 @@ void summary_search(SummaryView *summaryview)
 	gtk_widget_show(search_window.window);
 }
 
+static void summary_show_stop_button(void)
+{
+	gtk_widget_hide(search_window.close_btn);
+	gtk_widget_show(search_window.stop_btn);
+	GTK_BUTTON_SET_SENSITIVE(search_window.all_btn, FALSE)
+	GTK_BUTTON_SET_SENSITIVE(search_window.prev_btn, FALSE)
+	GTK_BUTTON_SET_SENSITIVE(search_window.next_btn, FALSE)
+}
+
+static void summary_hide_stop_button(void)
+{
+	gtk_widget_hide(search_window.stop_btn);
+	gtk_widget_show(search_window.close_btn);
+	gtk_widget_set_sensitive(search_window.all_btn, TRUE);
+	gtk_widget_set_sensitive(search_window.prev_btn, TRUE);
+	gtk_widget_set_sensitive(search_window.next_btn, TRUE);
+}
+
 static void summary_search_create(void)
 {
 	GtkWidget *window;
@@ -159,6 +188,7 @@ static void summary_search_create(void)
 	GtkWidget *case_checkbtn;
 
 	GtkWidget *confirm_area;
+	GtkWidget *help_btn;
 	GtkWidget *all_btn;
 	GtkWidget *prev_btn;
 	GtkWidget *next_btn;
@@ -170,7 +200,7 @@ static void summary_search_create(void)
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW (window), _("Search messages"));
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 	gtk_container_set_border_width(GTK_CONTAINER (window), 8);
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(gtk_widget_hide_on_delete), NULL);
@@ -296,7 +326,7 @@ static void summary_search_create(void)
 	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), case_checkbtn,
 			    FALSE, FALSE, 0);
 
-	adv_search_checkbtn = gtk_check_button_new_with_label (_("Advanced search"));
+	adv_search_checkbtn = gtk_check_button_new_with_label (_("Extended Search"));
 	gtk_widget_show (adv_search_checkbtn);
 	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), adv_search_checkbtn,
 			    FALSE, FALSE, 0);
@@ -307,6 +337,8 @@ static void summary_search_create(void)
 				  GTK_BUTTONBOX_END);
 	gtk_box_set_spacing(GTK_BOX(confirm_area), 5);
 
+	gtkut_stock_button_add_help(confirm_area, &help_btn);
+
 	all_btn = gtk_button_new_with_mnemonic(_("Find _all"));
 	GTK_WIDGET_SET_FLAGS(all_btn, GTK_CAN_DEFAULT);
 	gtk_box_pack_start(GTK_BOX(confirm_area), all_btn, TRUE, TRUE, 0);
@@ -345,6 +377,9 @@ static void summary_search_create(void)
 	SET_TOGGLE_SENSITIVITY(adv_search_checkbtn, adv_condition_btn)
 	SET_TOGGLE_SENSITIVITY_REVERSE(adv_search_checkbtn, case_checkbtn)
 
+	g_signal_connect(G_OBJECT(help_btn), "clicked",
+			 G_CALLBACK(manual_open_with_anchor_cb),
+			 MANUAL_ANCHOR_SEARCHING);
 	g_signal_connect(G_OBJECT(clear_btn), "clicked",
 			 G_CALLBACK(summary_search_clear), NULL);
 	g_signal_connect(G_OBJECT(all_btn), "clicked",
@@ -371,6 +406,7 @@ static void summary_search_create(void)
 	search_window.case_checkbtn = case_checkbtn;
 	search_window.adv_search_checkbtn = adv_search_checkbtn;
 	search_window.clear_btn = clear_btn;
+	search_window.help_btn = help_btn;
 	search_window.all_btn = all_btn;
 	search_window.prev_btn = prev_btn;
 	search_window.next_btn = next_btn;
@@ -395,6 +431,9 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 	const gchar *from_str = NULL, *to_str = NULL, *subject_str = NULL;
 	const gchar *body_str = NULL, *adv_condition = NULL;
 	StrFindFunc str_find_func = NULL;
+	gboolean is_fast = TRUE;
+	gint interval = 1000;
+	gint i = 0;
 
 	if (summary_is_locked(summaryview)) {
 		return;
@@ -411,7 +450,9 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 		}
 		adv_condition = gtk_entry_get_text(GTK_ENTRY(search_window.adv_condition_entry));
 		if (adv_condition[0] != '\0') {
-			search_window.matcher_list = matcher_parser_get_cond((gchar*)adv_condition);
+			search_window.matcher_list = matcher_parser_get_cond((gchar*)adv_condition, &is_fast);
+			if (!is_fast)
+				interval = 100;
 			/* TODO: check for condition parsing error and show an error dialog */
 		} else {
 			/* TODO: warn if no search condition? (or make buttons enabled only when
@@ -450,12 +491,11 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 
 	search_window.is_searching = TRUE;
 	main_window_cursor_wait(summaryview->mainwin);
-	gtk_widget_hide(search_window.close_btn);
-	gtk_widget_show(search_window.stop_btn);
+	summary_show_stop_button();
 
 	if (search_all) {
-		gtk_clist_freeze(GTK_CLIST(ctree));
-		gtk_clist_unselect_all(GTK_CLIST(ctree));
+		summary_freeze(summaryview);
+		summary_unselect_all(summaryview);
 		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 		backward = FALSE;
 	} else if (!summaryview->selected) {
@@ -467,8 +507,7 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 
 		if (!node) {
 			search_window.is_searching = FALSE;
-			gtk_widget_hide(search_window.stop_btn);
-			gtk_widget_show(search_window.close_btn);
+			summary_hide_stop_button();
 			main_window_cursor_normal(summaryview->mainwin);
 			summary_unlock(summaryview);
 			return;
@@ -483,7 +522,7 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 		}
 	}
 
-	for (; search_window.is_searching;) {
+	for (; search_window.is_searching; i++) {
 		if (!node) {
 			gchar *str;
 			AlertValue val;
@@ -506,7 +545,7 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 				str = _("End of list reached; continue from beginning?");
 
 			val = alertpanel(_("Search finished"), str,
-					 GTK_STOCK_NO, GTK_STOCK_YES, NULL);
+					 GTK_STOCK_NO, "+" GTK_STOCK_YES, NULL);
 			if (G_ALERTALTERNATE == val) {
 				if (backward) {
 					node = GTK_CTREE_NODE
@@ -614,16 +653,15 @@ static void summary_search_execute(gboolean backward, gboolean search_all)
 
 		node = backward ? gtkut_ctree_node_prev(ctree, node)
 				: gtkut_ctree_node_next(ctree, node);
-
-		GTK_EVENTS_FLUSH();
+		if (i % interval == 0)
+			GTK_EVENTS_FLUSH();
 	}
 
 	search_window.is_searching = FALSE;
-	gtk_widget_hide(search_window.stop_btn);
-	gtk_widget_show(search_window.close_btn);
+	summary_hide_stop_button();
 	main_window_cursor_normal(summaryview->mainwin);
 	if (search_all) {
-		gtk_clist_thaw(GTK_CLIST(ctree));
+		summary_thaw(summaryview);
 	}
 	summary_unlock(summaryview);
 }
@@ -702,7 +740,7 @@ static void adv_condition_btn_clicked(GtkButton *button, gpointer data)
 	cond_str = gtk_entry_get_text(
 			GTK_ENTRY(search_window.adv_condition_entry));
 	if (*cond_str != '\0') {
-		matchers = matcher_parser_get_cond((gchar*)cond_str);
+		matchers = matcher_parser_get_cond((gchar*)cond_str, NULL);
 	}
 
 	prefs_matcher_open(matchers, adv_condition_btn_done);
