@@ -108,6 +108,7 @@ struct _SummaryView
 
 	GtkCTreeNode *selected;
 	GtkCTreeNode *displayed;
+	GtkCTreeNode *last_displayed;
 
 	gboolean display_msg;
 
@@ -156,6 +157,9 @@ private:
 	GSList *mlist;
 	int msginfo_update_callback_id;
 
+	/* update folder label when renaming */
+	gint folder_item_update_callback_id;
+
 	GtkTargetList *target_list; /* DnD */
 };
 
@@ -169,6 +173,10 @@ void summary_clear_all		  (SummaryView		*summaryview);
 
 void summary_lock		  (SummaryView		*summaryview);
 void summary_unlock		  (SummaryView		*summaryview);
+void summary_freeze		  (SummaryView		*summaryview);
+void summary_thaw		  (SummaryView		*summaryview);
+void summary_grab_focus		  (SummaryView		*summaryview);
+GtkWidget *summary_get_main_widget(SummaryView 		*summaryview);
 gboolean summary_is_locked	  (SummaryView		*summaryview);
 
 SummarySelection summary_get_selection_type	(SummaryView	*summaryview);
@@ -183,6 +191,8 @@ void summary_select_prev_marked	  (SummaryView		*summaryview);
 void summary_select_next_marked	  (SummaryView		*summaryview);
 void summary_select_prev_labeled  (SummaryView		*summaryview);
 void summary_select_next_labeled  (SummaryView		*summaryview);
+void summary_select_last_read     (SummaryView		*summaryview);
+void summary_select_parent        (SummaryView		*summaryview);
 void summary_select_by_msgnum	  (SummaryView		*summaryview,
 				   guint		 msgnum);
 guint summary_get_current_msgnum  (SummaryView		*summaryview);
@@ -220,7 +230,7 @@ void summary_attract_by_subject	  (SummaryView		*summaryview);
 
 gint summary_write_cache	  (SummaryView		*summaryview);
 
-void summary_pass_key_press_event (SummaryView		*summaryview,
+gboolean summary_pass_key_press_event (SummaryView		*summaryview,
 				   GdkEventKey		*event);
 
 void summary_display_msg_selected (SummaryView		*summaryview,
@@ -255,6 +265,9 @@ void summary_mark_all_read	  (SummaryView		*summaryview);
 void summary_mark_as_spam	  (SummaryView 		*summaryview, 
 				   guint 		 action, 
 				   GtkWidget 		*widget);
+void summary_ignore_thread	  (SummaryView 		*summaryview);
+void summary_unignore_thread	  (SummaryView 		*summaryview);
+
 void summary_add_address	  (SummaryView		*summaryview);
 void summary_select_all		  (SummaryView		*summaryview);
 void summary_unselect_all	  (SummaryView		*summaryview);
@@ -282,7 +295,7 @@ void summary_set_prefs_from_folderitem
                                   (SummaryView *summaryview, FolderItem *item);
 void summary_save_prefs_to_folderitem
                                   (SummaryView *summaryview, FolderItem *item);
-void summaryview_export_mbox_list (SummaryView *summaryview);
+gint summaryview_export_mbox_list (SummaryView *summaryview);
 void summaryview_lock(SummaryView *summaryview, FolderItem *item);
 void summaryview_unlock(SummaryView *summaryview, FolderItem *item);
 void summary_reflect_prefs(void);
