@@ -389,7 +389,8 @@ static void toolbar_set_default_main(void)
 		{ A_RECEIVE_ALL,   STOCK_PIXMAP_MAIL_RECEIVE_ALL,     _("Get Mail") },
 		{ A_SEPARATOR,     0,                                 ("")         }, 
 		{ A_SEND_QUEUED,   STOCK_PIXMAP_MAIL_SEND_QUEUE,      _("Send")    },
-		{ A_COMPOSE_EMAIL, STOCK_PIXMAP_MAIL_COMPOSE,         _("Compose") },
+		{ A_COMPOSE_EMAIL, STOCK_PIXMAP_MAIL_COMPOSE,
+			(gchar*)Q_("Toolbar|Compose") },
 		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_REPLY_MESSAGE, STOCK_PIXMAP_MAIL_REPLY,           _("Reply")   }, 
 		{ A_REPLY_ALL,     STOCK_PIXMAP_MAIL_REPLY_TO_ALL,    _("All")     },
@@ -397,7 +398,7 @@ static void toolbar_set_default_main(void)
 		{ A_FORWARD,       STOCK_PIXMAP_MAIL_FORWARD,         _("Forward") },
 		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_TRASH,         STOCK_PIXMAP_TRASH,                _("Trash")   },
-#ifdef USE_SPAMASSASSIN_PLUGIN
+#if (defined(USE_SPAMASSASSIN_PLUGIN) || defined(USE_BOGOFILTER_PLUGIN))
 		{ A_LEARN_SPAM,	   STOCK_PIXMAP_SPAM_BTN,             _("Spam")    },
 #endif
 		{ A_SEPARATOR,     0,                                 ("")         },
@@ -488,7 +489,7 @@ static void toolbar_set_default_msgview(void)
 		{ A_FORWARD,       STOCK_PIXMAP_MAIL_FORWARD,         _("Forward") },
 		{ A_SEPARATOR,     0,                                 ("")         },
 		{ A_TRASH,         STOCK_PIXMAP_TRASH,                _("Trash")   },
-#ifdef USE_SPAMASSASSIN_PLUGIN
+#if (defined(USE_SPAMASSASSIN_PLUGIN) || defined(USE_BOGOFILTER_PLUGIN))
 		{ A_LEARN_SPAM,	   STOCK_PIXMAP_SPAM_BTN,             _("Spam")    },
 #endif
 		{ A_GOTO_NEXT,     STOCK_PIXMAP_DOWN_ARROW,           _("Next")    }
@@ -653,10 +654,8 @@ void toolbar_clear_list(ToolbarType source)
 		toolbar_config[source].item_list = 
 			g_slist_remove(toolbar_config[source].item_list, item);
 
-		if (item->file)
-			g_free(item->file);
-		if (item->text)
-			g_free(item->text);
+		g_free(item->file);
+		g_free(item->text);
 		g_free(item);	
 	}
 	g_slist_free(toolbar_config[source].item_list);
@@ -998,7 +997,7 @@ static void toolbar_compose_cb(GtkWidget *widget, gpointer data)
 	case TOOLBAR_MSGVIEW:
 		msgview = (MessageView*)toolbar_item->parent;
 		compose_new_with_folderitem(NULL, 
-					    msgview->msginfo->folder);
+					    msgview->msginfo->folder, NULL);
 		break;	
 	default:
 		debug_print("toolbar event not supported\n");
@@ -1140,12 +1139,10 @@ static void toolbar_prev_unread_cb(GtkWidget *widget, gpointer data)
 		
 		/* Now we need to update the messageview window */
 		if (msgview->mainwin->summaryview->selected) {
-			GtkCTree *ctree = GTK_CTREE(msgview->mainwin->summaryview->ctree);
-			
-			MsgInfo * msginfo = gtk_ctree_node_get_row_data(ctree, 
-									msgview->mainwin->summaryview->selected);
+			MsgInfo * msginfo = summary_get_selected_msg(msgview->mainwin->summaryview);
 		       
-			messageview_show(msgview, msginfo, 
+			if (msginfo)
+				messageview_show(msgview, msginfo, 
 					 msgview->all_headers);
 		} else {
 			gtk_widget_destroy(msgview->window);
@@ -1187,12 +1184,10 @@ static void toolbar_next_unread_cb(GtkWidget *widget, gpointer data)
 
 		/* Now we need to update the messageview window */
 		if (msgview->mainwin->summaryview->selected) {
-			GtkCTree *ctree = GTK_CTREE(msgview->mainwin->summaryview->ctree);
+			MsgInfo * msginfo = summary_get_selected_msg(msgview->mainwin->summaryview);
 			
-			MsgInfo * msginfo = gtk_ctree_node_get_row_data(ctree, 
-									msgview->mainwin->summaryview->selected);
-		       
-			messageview_show(msgview, msginfo, 
+			if (msginfo)
+				messageview_show(msgview, msginfo, 
 					 msgview->all_headers);
 		} else {
 			gtk_widget_destroy(msgview->window);
@@ -1434,6 +1429,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 	ComboButton *replylist_combo;
 	ComboButton *replysender_combo;
 	ComboButton *fwd_combo;
+	ComboButton *compose_combo;
 
 	GtkTooltips *toolbar_tips;
 	ToolbarSylpheedActions *action_item;
@@ -1525,6 +1521,14 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
 					     toolbar_data->compose_news_btn,
 					   _("Compose News"), NULL);
+			compose_combo = gtkut_combo_button_create(toolbar_data->compose_mail_btn, NULL, 0,
+					"<Compose>", (gpointer)toolbar_item);
+			gtk_button_set_relief(GTK_BUTTON(compose_combo->arrow),
+					      GTK_RELIEF_NONE);
+			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
+				  		  GTK_WIDGET_PTR(compose_combo),
+				 		  _("Compose with selected Account"), "Compose");
+			toolbar_data->compose_combo = compose_combo;
 			break;
 		case A_LEARN_SPAM:
 			icon_ham = stock_pixmap_widget(container, STOCK_PIXMAP_HAM_BTN);
@@ -1830,6 +1834,15 @@ void toolbar_update(ToolbarType type, gpointer data)
 	}
 }
 
+#define GTK_BUTTON_SET_SENSITIVE(widget,sensitive) {		\
+	gboolean in_btn = FALSE;				\
+	if (GTK_IS_BUTTON(widget))				\
+		in_btn = GTK_BUTTON(widget)->in_button;		\
+	gtk_widget_set_sensitive(widget, sensitive);		\
+	if (GTK_IS_BUTTON(widget))				\
+		GTK_BUTTON(widget)->in_button = in_btn;		\
+}
+
 void toolbar_main_set_sensitive(gpointer data)
 {
 	SensitiveCond state;
@@ -1854,39 +1867,61 @@ void toolbar_main_set_sensitive(gpointer data)
 	entry_list = g_slist_append(entry_list, e); \
 }
 
-	SET_WIDGET_COND(toolbar->get_btn, M_HAVE_ACCOUNT|M_UNLOCKED);
-	SET_WIDGET_COND(toolbar->getall_btn, M_HAVE_ACCOUNT|M_UNLOCKED);
-	if (toolbar->getall_btn)
+	
+	if (toolbar->get_btn)
+		SET_WIDGET_COND(toolbar->get_btn, 
+			M_HAVE_ACCOUNT|M_UNLOCKED);
+
+	if (toolbar->getall_btn) {
+		SET_WIDGET_COND(toolbar->getall_btn, 
+			M_HAVE_ACCOUNT|M_UNLOCKED);
 		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->getall_combo),
 			M_HAVE_ACCOUNT|M_UNLOCKED);
-	SET_WIDGET_COND(toolbar->compose_news_btn, M_HAVE_ACCOUNT);
-	SET_WIDGET_COND(toolbar->reply_btn,
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	if (toolbar->reply_btn)
+	}
+	if (toolbar->send_btn) {
+		SET_WIDGET_COND(toolbar->send_btn,
+			M_HAVE_QUEUED_MAILS);
+	}
+	if (toolbar->compose_mail_btn) {
+		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->compose_combo),
+			M_HAVE_ACCOUNT);
+		SET_WIDGET_COND(toolbar->compose_news_btn, 
+			M_HAVE_ACCOUNT);
+	}
+	if (toolbar->reply_btn) {
+		SET_WIDGET_COND(toolbar->reply_btn,
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
 		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->reply_combo),
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	SET_WIDGET_COND(toolbar->replyall_btn,
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	if (toolbar->replyall_btn)
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+	}
+	if (toolbar->replyall_btn) {
+		SET_WIDGET_COND(toolbar->replyall_btn,
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
 		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->replyall_combo),
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	SET_WIDGET_COND(toolbar->replylist_btn,
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	if (toolbar->replylist_btn) 
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+	}
+	if (toolbar->replylist_btn) {
+		SET_WIDGET_COND(toolbar->replylist_btn,
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
 		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->replylist_combo),
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	SET_WIDGET_COND(toolbar->replysender_btn,
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	if (toolbar->replysender_btn)
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+	}
+	if (toolbar->replysender_btn) {
+		SET_WIDGET_COND(toolbar->replysender_btn,
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
 		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->replysender_combo),
-			M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST);
-	SET_WIDGET_COND(toolbar->fwd_btn, M_HAVE_ACCOUNT|M_TARGET_EXIST);
-	if (toolbar->fwd_btn)
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
+	}
+	if (toolbar->fwd_btn) {
+		SET_WIDGET_COND(toolbar->fwd_btn, 
+			M_HAVE_ACCOUNT|M_TARGET_EXIST);
 		SET_WIDGET_COND(GTK_WIDGET_PTR(toolbar->fwd_combo),
 			M_HAVE_ACCOUNT|M_TARGET_EXIST); 
+	}
 	if (toolbar->fwd_combo) {
 		GtkWidget *submenu = gtk_item_factory_get_widget(toolbar->fwd_combo->factory, "/Redirect");
-		SET_WIDGET_COND(submenu, M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST); 
+		SET_WIDGET_COND(submenu, 
+			M_HAVE_ACCOUNT|M_TARGET_EXIST); 
 	}
 
 	if (prefs_common.next_unread_msg_dialog == NEXTUNREADMSGDIALOG_ASSUME_NO) {
@@ -1898,16 +1933,22 @@ void toolbar_main_set_sensitive(gpointer data)
 	if (toolbar->trash_btn)
 		SET_WIDGET_COND(toolbar->trash_btn,
 			M_TARGET_EXIST|M_ALLOW_DELETE);
+
 	if (toolbar->delete_btn)
 		SET_WIDGET_COND(toolbar->delete_btn,
 			M_TARGET_EXIST|M_ALLOW_DELETE);
+
 	if (toolbar->exec_btn)
-		SET_WIDGET_COND(toolbar->exec_btn, M_DELAY_EXEC);
+		SET_WIDGET_COND(toolbar->exec_btn, 
+			M_DELAY_EXEC);
 	
 	if (toolbar->learn_ham_btn)
-		SET_WIDGET_COND(toolbar->learn_ham_btn, M_TARGET_EXIST|M_CAN_LEARN_SPAM);
+		SET_WIDGET_COND(toolbar->learn_ham_btn,
+			M_TARGET_EXIST|M_CAN_LEARN_SPAM);
+
 	if (toolbar->learn_spam_btn)
-		SET_WIDGET_COND(toolbar->learn_spam_btn, M_TARGET_EXIST|M_CAN_LEARN_SPAM);
+		SET_WIDGET_COND(toolbar->learn_spam_btn, 
+			M_TARGET_EXIST|M_CAN_LEARN_SPAM);
 
 	for (cur = toolbar->action_list; cur != NULL;  cur = cur->next) {
 		ToolbarSylpheedActions *act = (ToolbarSylpheedActions*)cur->data;
@@ -1924,15 +1965,14 @@ void toolbar_main_set_sensitive(gpointer data)
 
 		if (e->widget != NULL) {
 			sensitive = ((e->cond & state) == e->cond);
-			gtk_widget_set_sensitive(e->widget, sensitive);	
+			GTK_BUTTON_SET_SENSITIVE(e->widget, sensitive);	
 		}
 	}
 	
 	while (entry_list != NULL) {
 		Entry *e = (Entry*) entry_list->data;
 
-		if (e)
-			g_free(e);
+		g_free(e);
 		entry_list = g_slist_remove(entry_list, e);
 	}
 
@@ -1941,6 +1981,7 @@ void toolbar_main_set_sensitive(gpointer data)
 	activate_compose_button(toolbar, 
 				prefs_common.toolbar_style,
 				toolbar->compose_btn_type);
+	
 }
 
 void toolbar_comp_set_sensitive(gpointer data, gboolean sensitive)
@@ -1949,32 +1990,32 @@ void toolbar_comp_set_sensitive(gpointer data, gboolean sensitive)
 	GSList *items = compose->toolbar->action_list;
 
 	if (compose->toolbar->send_btn)
-		gtk_widget_set_sensitive(compose->toolbar->send_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->send_btn, sensitive);
 	if (compose->toolbar->sendl_btn)
-		gtk_widget_set_sensitive(compose->toolbar->sendl_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->sendl_btn, sensitive);
 	if (compose->toolbar->draft_btn )
-		gtk_widget_set_sensitive(compose->toolbar->draft_btn , sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->draft_btn , sensitive);
 	if (compose->toolbar->insert_btn )
-		gtk_widget_set_sensitive(compose->toolbar->insert_btn , sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->insert_btn , sensitive);
 	if (compose->toolbar->attach_btn)
-		gtk_widget_set_sensitive(compose->toolbar->attach_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->attach_btn, sensitive);
 	if (compose->toolbar->sig_btn)
-		gtk_widget_set_sensitive(compose->toolbar->sig_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->sig_btn, sensitive);
 	if (compose->toolbar->exteditor_btn)
-		gtk_widget_set_sensitive(compose->toolbar->exteditor_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->exteditor_btn, sensitive);
 	if (compose->toolbar->linewrap_current_btn)
-		gtk_widget_set_sensitive(compose->toolbar->linewrap_current_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->linewrap_current_btn, sensitive);
 	if (compose->toolbar->linewrap_all_btn)
-		gtk_widget_set_sensitive(compose->toolbar->linewrap_all_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->linewrap_all_btn, sensitive);
 	if (compose->toolbar->addrbook_btn)
-		gtk_widget_set_sensitive(compose->toolbar->addrbook_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->addrbook_btn, sensitive);
 #ifdef USE_ASPELL
 	if (compose->toolbar->spellcheck_btn)
-		gtk_widget_set_sensitive(compose->toolbar->spellcheck_btn, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(compose->toolbar->spellcheck_btn, sensitive);
 #endif
 	for (; items != NULL; items = g_slist_next(items)) {
 		ToolbarSylpheedActions *item = (ToolbarSylpheedActions *)items->data;
-		gtk_widget_set_sensitive(item->widget, sensitive);
+		GTK_BUTTON_SET_SENSITIVE(item->widget, sensitive);
 	}
 }
 
@@ -1990,6 +2031,7 @@ void toolbar_init(Toolbar * toolbar) {
 	toolbar->send_btn         	= NULL;
 	toolbar->compose_mail_btn 	= NULL;
 	toolbar->compose_news_btn 	= NULL;
+	toolbar->compose_combo	 	= NULL;
 	toolbar->reply_btn        	= NULL;
 	toolbar->replysender_btn  	= NULL;
 	toolbar->replyall_btn     	= NULL;
@@ -2073,6 +2115,8 @@ void send_queue_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	GList *list;
 	gboolean found;
+	gboolean got_error = FALSE;
+	gchar *errstr = NULL;
 
 	if (prefs_common.work_offline)
 		if (alertpanel(_("Offline warning"), 
@@ -2108,9 +2152,21 @@ void send_queue_cb(gpointer data, guint action, GtkWidget *widget)
 
 		if (folder->queue) {
 			if (procmsg_send_queue(folder->queue, 
-					       prefs_common.savemsg) < 0)
-				alertpanel_error(_("Some errors occurred while "
-						   "sending queued messages."));
+					       prefs_common.savemsg,
+					       &errstr) < 0)
+				got_error = TRUE;
+		}
+	}
+	if (got_error) {
+		if (!errstr)
+			alertpanel_error_log(_("Some errors occurred while "
+					   "sending queued messages."));
+		else {
+			gchar *tmp = g_strdup_printf(_("Some errors occurred "
+					"while sending queued messages:\n%s"), errstr);
+			g_free(errstr);
+			alertpanel_error_log(tmp);
+			g_free(tmp);
 		}
 	}
 }
@@ -2126,7 +2182,7 @@ void compose_mail_cb(gpointer data, guint action, GtkWidget *widget)
 	if (item) {
 		ac = account_find_from_item(item);
 		if (ac && ac->protocol != A_NNTP) {
-			compose_new_with_folderitem(ac, item);		/* CLAWS */
+			compose_new_with_folderitem(ac, item, NULL);		/* CLAWS */
 			return;
 		}
 	}
@@ -2135,7 +2191,7 @@ void compose_mail_cb(gpointer data, guint action, GtkWidget *widget)
 	 * CLAWS - use current account
 	 */
 	if (cur_account && (cur_account->protocol != A_NNTP)) {
-		compose_new_with_folderitem(cur_account, item);
+		compose_new_with_folderitem(cur_account, item, NULL);
 		return;
 	}
 
@@ -2146,7 +2202,7 @@ void compose_mail_cb(gpointer data, guint action, GtkWidget *widget)
 	for (cur = list ; cur != NULL ; cur = g_list_next(cur)) {
 		ac = (PrefsAccount *) cur->data;
 		if (ac->protocol != A_NNTP) {
-			compose_new_with_folderitem(ac, item);
+			compose_new_with_folderitem(ac, item, NULL);
 			return;
 		}
 	}
@@ -2163,7 +2219,7 @@ void compose_news_cb(gpointer data, guint action, GtkWidget *widget)
 		ac = mainwin->summaryview->folder_item->folder->account;
 		if (ac && ac->protocol == A_NNTP) {
 			compose_new_with_folderitem(ac,
-				    mainwin->summaryview->folder_item);
+				    mainwin->summaryview->folder_item, NULL);
 			return;
 		}
 	}
@@ -2173,7 +2229,7 @@ void compose_news_cb(gpointer data, guint action, GtkWidget *widget)
 		ac = (PrefsAccount *) cur->data;
 		if (ac->protocol == A_NNTP) {
 			compose_new_with_folderitem(ac,
-				    mainwin->summaryview->folder_item);
+				    mainwin->summaryview->folder_item, NULL);
 			return;
 		}
 	}
