@@ -468,9 +468,9 @@ void undo_redo(UndoMain *undostruct)
 		/* "pull" another data structure from the list */
 		redoinfo = (UndoInfo *)undostruct->redo->data;
 		g_return_if_fail(redoinfo != NULL);
-		undostruct->undo = g_list_prepend(undostruct->redo, redoinfo);
-		undostruct->redo = g_list_remove(undostruct->undo, redoinfo);
-		g_return_if_fail(redoinfo->action == UNDO_ACTION_REPLACE_DELETE);
+		undostruct->undo = g_list_prepend(undostruct->undo, redoinfo);
+		undostruct->redo = g_list_remove(undostruct->redo, redoinfo);
+		g_return_if_fail(redoinfo->action == UNDO_ACTION_REPLACE_INSERT);
 		gtk_text_buffer_insert(buffer, &start_iter, redoinfo->text, -1);
 		break;
 	case UNDO_ACTION_REPLACE_INSERT:
@@ -537,7 +537,8 @@ void undo_insert_text_cb(GtkTextBuffer *textbuf, GtkTextIter *iter,
 	pos = gtk_text_iter_get_offset(iter);
 	if (undostruct->wrap && undostruct->undo) {
 		UndoInfo *last_undo = undostruct->undo->data;
-		if (last_undo && last_undo->action == UNDO_ACTION_INSERT
+		if (last_undo && (last_undo->action == UNDO_ACTION_INSERT
+				  || last_undo->action == UNDO_ACTION_REPLACE_INSERT)
 		&&  last_undo->start_pos < pos && last_undo->end_pos > pos) {
 			GtkTextIter start,end;
 			last_undo->end_pos += g_utf8_strlen(new_text, -1);
@@ -549,7 +550,6 @@ void undo_insert_text_cb(GtkTextBuffer *textbuf, GtkTextIter *iter,
 			return;
 		} else debug_print("add:last: %d, %d-%d (%d)\n", last_undo->action,
 			last_undo->start_pos, last_undo->end_pos, pos);
-		
 	} 
 	Xstrndup_a(text_to_insert, new_text, new_text_length, return);
 	debug_print("add:undo add %d-%ld\n", pos, pos + g_utf8_strlen(text_to_insert, -1));
@@ -573,7 +573,8 @@ void undo_delete_text_cb(GtkTextBuffer *textbuf, GtkTextIter *start,
 
 	if (undostruct->wrap && undostruct->undo) {
 		UndoInfo *last_undo = undostruct->undo->data;
-		if (last_undo && last_undo->action == UNDO_ACTION_INSERT
+		if (last_undo && (last_undo->action == UNDO_ACTION_INSERT
+				  || last_undo->action == UNDO_ACTION_REPLACE_INSERT)
 		&&  last_undo->start_pos < start_pos && last_undo->end_pos > end_pos) {
 			GtkTextIter start,end;
 			last_undo->end_pos -= g_utf8_strlen(text_to_delete, -1);
@@ -593,7 +594,12 @@ void undo_delete_text_cb(GtkTextBuffer *textbuf, GtkTextIter *start,
 	g_free(text_to_delete);
 }
 
-void undo_paste_clipboard_cb(GtkTextView *textview, UndoMain *undostruct)
+void undo_paste_clipboard(GtkTextView *textview, UndoMain *undostruct)
+{
+	undo_paste_clipboard_cb(textview, undostruct);
+}
+
+static void undo_paste_clipboard_cb(GtkTextView *textview, UndoMain *undostruct)
 {
 	if (prefs_common.undolevels > 0)
 		if (undo_get_selection(textview, NULL, NULL))
