@@ -49,6 +49,7 @@
 
 #include "utils.h"
 #include "gtk/menu.h"
+#include "plugin.h"
 #include "account.h"
 #include "prefs_account.h"
 #include "mainwindow.h"
@@ -92,6 +93,11 @@ typedef struct
 	GtkWidget *mailbox_name;
 	
 	GtkWidget *smtp_server;
+	GtkWidget *smtp_auth;
+	GtkWidget *smtp_username;
+	GtkWidget *smtp_password;
+	GtkWidget *smtp_username_label;
+	GtkWidget *smtp_password_label;
 
 	GtkWidget *recv_type;
 	GtkWidget *recv_label;
@@ -102,7 +108,7 @@ typedef struct
 	GtkWidget *recv_password_label;
 	GtkWidget *recv_imap_label;
 	GtkWidget *recv_imap_subdir;
-
+	GtkWidget *no_imap_warning;
 #ifdef USE_OPENSSL
 	GtkWidget *smtp_use_ssl;
 	GtkWidget *recv_use_ssl;
@@ -114,6 +120,277 @@ typedef struct
 
 } WizardWindow;
 
+typedef struct _AccountTemplate {
+	gchar *name;
+	gchar *domain;
+	gchar *email;
+	gchar *organization;
+	gchar *smtpserver;
+	gboolean smtpauth;
+	gchar *smtpuser;
+	gchar *smtppass;
+	RecvProtocol recvtype;
+	gchar *recvserver;
+	gchar *recvuser;
+	gchar *recvpass;
+	gchar *imapdir;
+	gchar *mboxfile;
+	gchar *mailbox;
+	gboolean smtpssl;
+	gboolean recvssl;
+} AccountTemplate;
+
+static AccountTemplate tmpl;
+
+static PrefParam template_params[] = {
+	{"name", "$USERNAME",
+	 &tmpl.name, P_STRING, NULL, NULL, NULL},
+	{"domain", "$DEFAULTDOMAIN",
+	 &tmpl.domain, P_STRING, NULL, NULL, NULL},
+	{"email", "$NAME_MAIL@$DOMAIN",
+	 &tmpl.email, P_STRING, NULL, NULL, NULL},
+	{"organization", "",
+	 &tmpl.organization, P_STRING, NULL, NULL, NULL},
+	{"smtpserver", "smtp.$DOMAIN",
+	 &tmpl.smtpserver, P_STRING, NULL, NULL, NULL},
+	{"smtpauth", "FALSE",
+	 &tmpl.smtpauth, P_BOOL, NULL, NULL, NULL},
+	{"smtpuser", "",
+	 &tmpl.smtpuser, P_STRING, NULL, NULL, NULL},
+	{"smtppass", "",
+	 &tmpl.smtppass, P_STRING, NULL, NULL, NULL},
+	{"recvtype", A_POP3,
+	 &tmpl.recvtype, P_INT, NULL, NULL, NULL},
+	{"recvserver", "pop.$DOMAIN",
+	 &tmpl.recvserver, P_STRING, NULL, NULL, NULL},
+	{"recvuser", "$LOGIN",
+	 &tmpl.recvuser, P_STRING, NULL, NULL, NULL},
+	{"recvpass", "",
+	 &tmpl.recvpass, P_STRING, NULL, NULL, NULL},
+	{"imapdir", "",
+	 &tmpl.imapdir, P_STRING, NULL, NULL, NULL},
+	{"mboxfile", "/var/mail/$LOGIN",
+	 &tmpl.mboxfile, P_STRING, NULL, NULL, NULL},
+	{"mailbox", "Mail",
+	 &tmpl.mailbox, P_STRING, NULL, NULL, NULL},
+	{"smtpssl", "FALSE",
+	 &tmpl.smtpssl, P_BOOL, NULL, NULL, NULL},
+	{"recvssl", "FALSE",
+	 &tmpl.recvssl, P_BOOL, NULL, NULL, NULL},
+	{NULL, NULL, NULL, P_INT, NULL, NULL, NULL}
+};
+
+
+static gchar *accountrc_tmpl =
+	"[AccountTemplate]\n"
+	"#you can use $DEFAULTDOMAIN here\n"
+	"#domain must be defined before the variables that use it\n"
+	"#by default, domain is extracted from the hostname\n"
+	"#domain=\n"
+	"\n"
+	"#you can use $USERNAME for name (this is the default)\n"
+	"#name=\n"
+	"\n"
+	"#you can use $LOGIN, $NAME_MAIL and $DOMAIN here \n"
+	"#$NAME_MAIL is the name without uppercase and with dots instead\n"
+	"#of spaces\n"
+	"#the default is $NAME_MAIL@$DOMAIN\n"
+	"#email=\n"
+	"\n"
+	"#you can use $DOMAIN here\n"
+	"#the default organization is empty\n"
+	"#organization=\n"
+	"\n"
+	"#you can use $DOMAIN here \n"
+	"#the default is stmp.$DOMAIN\n"
+	"#smtpserver=\n"
+	"\n"
+	"#Whether to use smtp authentication\n"
+	"#the default is 0 (no)\n"
+	"#smtpauth=\n"
+	"\n"
+	"#SMTP username\n"
+	"#you can use $LOGIN, $NAME_MAIL, $DOMAIN or $EMAIL here\n"
+	"#the default is empty (same as reception username)\n"
+	"#smtpuser=\n"
+	"\n"
+	"#SMTP password\n"
+	"#the default is empty (same as reception password)\n"
+	"#smtppass=\n"
+	"\n"
+	"#recvtype can be:\n"
+	"#0 for pop3\n"
+	"#3  for imap\n"
+	"#5  for a local mbox file\n"
+	"#recvtype=\n"
+	"\n"
+	"#you can use $DOMAIN here \n"
+	"#the default is {pop,imap}.$DOMAIN\n"
+	"#recvserver=\n"
+	"\n"
+	"#you can use $LOGIN, $NAME_MAIL, $DOMAIN or $EMAIL here\n"
+	"#default is $LOGIN\n"
+	"#recvuser=\n"
+	"\n"
+	"#default is empty\n"
+	"#recvpass=\n"
+	"\n"
+	"#imap dir if imap (relative to the home on the server\n"
+	"#default is empty\n"
+	"#imapdir=\n"
+	"\n"
+	"#mbox file if local\n"
+	"#you can use $LOGIN here\n"
+	"#default is /var/mail/$LOGIN\n"
+	"#mboxfile=\n"
+	"\n"
+	"#mailbox name if pop3 or local\n"
+	"#relative path from the user's home\n"
+	"#default is \"Mail\"\n"
+	"#mailbox=\n"
+	"\n"
+	"#whether to use ssl on STMP connections\n"
+	"#default is 0\n"
+	"#smtpssl=\n"
+	"\n"
+	"#whether to use ssl on pop or imap connections\n"
+	"#default is 0\n"
+	"#recvssl=\n";
+
+static gchar *wizard_get_default_domain_name(void)
+{
+	static gchar *domain_name = NULL;
+	
+	if (domain_name == NULL) {
+		domain_name = g_strdup(get_domain_name());
+		if (strchr(domain_name, '.') != strrchr(domain_name, '.')
+		&& strlen(strchr(domain_name, '.')) > 6) {
+			gchar *tmp = g_strdup(strchr(domain_name, '.')+1);
+			g_free(domain_name);
+			domain_name = tmp;
+		}
+	}
+	return domain_name;
+}
+
+static gchar *get_name_for_mail(void)
+{
+	gchar *name = g_strdup(tmpl.name);
+	if (name == NULL)
+		return NULL;
+	g_strdown(name);
+	while(strstr(name, " "))
+		*strstr(name, " ")='.';
+	
+	return name;
+}
+
+#define PARSE_DEFAULT(str) {	\
+	gchar *tmp = NULL, *new = NULL;	\
+	if (str != NULL) {	\
+		tmp = g_strdup(str);	\
+		if (strstr(str, "$USERNAME")) {	\
+			tmp = g_strdup(str);	\
+			*strstr(tmp, "$USERNAME") = '\0';	\
+			new = g_strconcat(tmp, g_get_real_name(), 	\
+				strstr(str, "$USERNAME")+strlen("$USERNAME"), 	\
+				NULL);	\
+			g_free(tmp);	\
+			g_free(str);	\
+			str = new;	\
+			new = NULL;	\
+		}	\
+		if (strstr(str, "$LOGIN")) {	\
+			tmp = g_strdup(str);	\
+			*strstr(tmp, "$LOGIN") = '\0';	\
+			new = g_strconcat(tmp, g_get_user_name(), 	\
+				strstr(str, "$LOGIN")+strlen("$LOGIN"), 	\
+				NULL);	\
+			g_free(tmp);	\
+			g_free(str);	\
+			str = new;	\
+			new = NULL;	\
+		}	\
+		if (strstr(str, "$EMAIL")) {	\
+			tmp = g_strdup(str);	\
+			*strstr(tmp, "$EMAIL") = '\0';	\
+			new = g_strconcat(tmp, tmpl.email, 	\
+				strstr(str, "$EMAIL")+strlen("$EMAIL"), 	\
+				NULL);	\
+			g_free(tmp);	\
+			g_free(str);	\
+			str = new;	\
+			new = NULL;	\
+		}	\
+		if (strstr(str, "$NAME_MAIL")) {	\
+			tmp = g_strdup(str);	\
+			*strstr(tmp, "$NAME_MAIL") = '\0';	\
+			new = g_strconcat(tmp, get_name_for_mail(), 	\
+				strstr(str, "$NAME_MAIL")+strlen("$NAME_MAIL"), 	\
+				NULL);	\
+			g_free(tmp);	\
+			g_free(str);	\
+			str = new;	\
+			new = NULL;	\
+		}	\
+		if (strstr(str, "$DEFAULTDOMAIN")) {	\
+			tmp = g_strdup(str);	\
+			*strstr(tmp, "$DEFAULTDOMAIN") = '\0';	\
+			new = g_strconcat(tmp, wizard_get_default_domain_name(), 	\
+				strstr(str, "$DEFAULTDOMAIN")+strlen("$DEFAULTDOMAIN"), 	\
+				NULL);	\
+			g_free(tmp);	\
+			g_free(str);	\
+			str = new;	\
+			new = NULL;	\
+		}	\
+		if (strstr(str, "$DOMAIN")) {	\
+			tmp = g_strdup(str);	\
+			*strstr(tmp, "$DOMAIN") = '\0';	\
+			new = g_strconcat(tmp, tmpl.domain, 	\
+				strstr(str, "$DOMAIN")+strlen("$DOMAIN"), 	\
+				NULL);	\
+			g_free(tmp);	\
+			g_free(str);	\
+			str = new;	\
+			new = NULL;	\
+		}	\
+	}	\
+}
+static void wizard_read_defaults(void)
+{
+	gchar *rcpath;
+
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "accountrc.tmpl", NULL);
+	if (!is_file_exist(rcpath)) {
+		str_write_to_file(accountrc_tmpl, rcpath);
+	}
+
+	prefs_read_config(template_params, "AccountTemplate", rcpath, NULL);
+
+	PARSE_DEFAULT(tmpl.domain);
+	PARSE_DEFAULT(tmpl.name);
+	PARSE_DEFAULT(tmpl.email);
+	PARSE_DEFAULT(tmpl.organization);
+	PARSE_DEFAULT(tmpl.smtpserver);
+	PARSE_DEFAULT(tmpl.smtpuser);
+	PARSE_DEFAULT(tmpl.smtppass);
+	PARSE_DEFAULT(tmpl.recvserver);
+	PARSE_DEFAULT(tmpl.recvuser);
+	PARSE_DEFAULT(tmpl.recvpass);
+	PARSE_DEFAULT(tmpl.imapdir);
+	PARSE_DEFAULT(tmpl.mboxfile);
+	PARSE_DEFAULT(tmpl.mailbox);
+/*
+	printf("defaults:"
+	"%s, %s, %s, %s, %s, %d, %s, %s, %s, %s, %s, %s, %d, %d\n",
+	tmpl.name,tmpl.domain,tmpl.email,tmpl.organization,tmpl.smtpserver,
+	tmpl.recvtype,tmpl.recvserver,tmpl.recvuser,tmpl.recvpass,
+	tmpl.imapdir,tmpl.mboxfile,tmpl.mailbox,tmpl.smtpssl,tmpl.recvssl);
+*/
+	g_free(rcpath);
+}
+
 static void initialize_fonts(WizardWindow *wizard)
 {
 	GtkWidget *widget = wizard->email;
@@ -151,6 +428,19 @@ static void initialize_fonts(WizardWindow *wizard)
 }
 
 #define XFACE "+}Axz@~a,-Yx?0Ysa|q}CLRH=89Y]\"')DSX^<6p\"d)'81yx5%G#u^o*7JG&[aPU0h1Ux.vb2yIjH83{5`/bVo|~nn/i83vE^E)qk-4W)_E.4Y=D*qvf/,Ci_=P<iY<M6"
+#define FACE "iVBORw0KGgoAAAANSUhEUgAAADAAAAAwBAMAAAClLOS0AAAAJ1BMVEUTGBYnS3BCUE9KVC9acyRy\n\
+ kC6LjITZdGiumnF/p7yrq6jJzc/5+vf7GI+IAAACZ0lEQVQ4y5XUvW/UMBQA8LRl6VZ3QLqNNhVS\n\
+ YSnKFfExEg9lbJNK3QqVoiPHzgpFnAIs5Do9ORNDVOwuFVIgduZWOfuP4vlyybVVkeCdTpHe7z1/\n\
+ RHYc85dw/gEUxi2gS5ZlDKqbUDcpzarroMEOpDU2VtcAmFKlxn+t2VWQUNa1BmAlGrsCqWKMwetv\n\
+ MMbnFaixFn9rh47DFCuzDs6hxPyrxXuOA7WqWQcp2Fhx7gTOcga6bGHSS6FHCHFouLcMJptuaQbp\n\
+ +kff90P6Yn0TMpXN4DxNVz+FQZ8Gob8DGWSsBUIG23seDXwaBBnU0EJKVhNvqx/6aHTHrmMGhNw9\n\
+ XlqkgbdEX/gh1PUcVj84G4HnuHTXDQ+6Dk3IyqFL/UfEpXvj7RChaoEsUX9rYeGE7o83wp0WcCjS\n\
+ 9/01AifhLrj0oINeuuJ53kIP+uF+7uL03eQpGWx5yzDu05fM3Z53AJDkvdODwvOfmbH3uOuwcLz+\n\
+ UBWLR8/N+MHnDsoeG4zecDl5Mhqa74NR90p+VEYmo+ioSEan8WnytANhDB8kX06TJFZJcowvfrZz\n\
+ XIMS2vxMIlHJfHMTRLOqC7TovlJFEVVGF7yafFVTkGgij+I851hZCHP5Tk8BWXObjuxhl2fm8pdu\n\
+ O0wluZDKHgJ91nVMKhuKN6cZQf9uQAs85lrjGDYmwmqzDynwClRKTCF/OwfDub0dQyzHxVUt6DzK\n\
+ eY5NseIxb8abwoVSMpZDhJyL9kJamGAxplC7izkHyaXM5/nZHFiNeRHfBFNwLjhOr+fAmw1G3OYl\n\
+ bwoijGGzD40pdeu3ROv/+Pr8AWPP4vVXbP0VAAAAAElFTkSuQmCC"
 
 static void write_welcome_email(WizardWindow *wizard)
 {
@@ -166,19 +456,21 @@ static void write_welcome_email(WizardWindow *wizard)
 	
 	get_rfc822_date(buf_date, sizeof(buf_date));
 
-	subj = g_strdup_printf(_("Welcome to Sylpheed-Claws"));
+	subj = g_strdup_printf(_("Welcome to Sylpheed-Claws "));
 
 	head = g_strdup_printf(
-		"From: %s <sylpheed-claws-users@lists.sf.net>\n"
+		"From: %s <%s>\n"
 		"To: %s <%s>\n"
 		"Date: %s\n"
 		"Subject: %s\n"
 		"X-Face: %s\n"
+		"Face: %s\n"
 		"Content-Type: text/plain; charset=UTF-8\n",
 		_("Sylpheed-Claws Team"),
+		USERS_ML_ADDR,
 		gtk_entry_get_text(GTK_ENTRY(wizard->full_name)),
 		gtk_entry_get_text(GTK_ENTRY(wizard->email)),
-		buf_date, subj, XFACE);
+		buf_date, subj, XFACE, FACE);
 	body = g_strdup_printf(
 		_("\n"
 		"Welcome to Sylpheed-Claws\n"
@@ -188,6 +480,12 @@ static void write_welcome_email(WizardWindow *wizard)
 		"mail by clicking the 'Get Mail' button at the left of the\n"
 		"toolbar.\n"
 		"\n"
+		"Sylpheed-Claws has lots of extra features accessible via plugins,\n"
+		"like anti-spam filtering and learning (via the Bogofilter or\n"
+		"SpamAssassin plugins), privacy protection (via PGP/Mime), an RSS\n"
+		"aggregator, a calendar, and much more. You can load them from\n"
+		"the menu entry '/Configuration/Plugins'.\n"
+		"\n"
 		"You can change your Account Preferences by using the menu\n"
 		"entry '/Configuration/Preferences for current account'\n"
 		"and change the general Preferences by using\n"
@@ -307,7 +605,7 @@ static gboolean wizard_write_config(WizardWindow *wizard)
 			return FALSE;
 		}
 	}
-
+	
 	if (!strlen(gtk_entry_get_text(GTK_ENTRY(wizard->smtp_server)))) {
 		alertpanel_error(_("Please enter your SMTP server."));
 		g_free(prefs_account);
@@ -315,7 +613,18 @@ static gboolean wizard_write_config(WizardWindow *wizard)
 			GTK_NOTEBOOK(wizard->notebook), 
 			SMTP_PAGE);
 		return FALSE;
+	}
 
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wizard->smtp_auth))) {
+		if (prefs_account->protocol == A_LOCAL
+		&&  !strlen(gtk_entry_get_text(GTK_ENTRY(wizard->smtp_username)))) {
+			alertpanel_error(_("Please enter your SMTP username."));
+			g_free(prefs_account);
+			gtk_notebook_set_current_page (
+				GTK_NOTEBOOK(wizard->notebook), 
+				SMTP_PAGE);
+			return FALSE;		
+		} /* if it's not local we'll use the reception server */
 	}
 
 	if (prefs_account->protocol != A_LOCAL)
@@ -347,6 +656,14 @@ static gboolean wizard_write_config(WizardWindow *wizard)
 	prefs_account->passwd = g_strdup(
 				gtk_entry_get_text(GTK_ENTRY(wizard->recv_password)));
 
+	prefs_account->smtp_userid = g_strdup(
+				gtk_entry_get_text(GTK_ENTRY(wizard->smtp_username)));
+	prefs_account->smtp_passwd = g_strdup(
+				gtk_entry_get_text(GTK_ENTRY(wizard->smtp_password)));
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wizard->smtp_auth))) {
+		prefs_account->use_smtp_auth = TRUE;
+	}
+
 #ifdef USE_OPENSSL			
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wizard->smtp_use_ssl)))
 		prefs_account->ssl_smtp = SSL_TUNNEL;
@@ -375,6 +692,9 @@ static gboolean wizard_write_config(WizardWindow *wizard)
 	if (wizard->create_mailbox && prefs_account->protocol != A_IMAP4)
 		write_welcome_email(wizard);
 	
+#ifndef G_OS_WIN32 
+	plugin_load_standard_plugins();
+#endif
 	return TRUE;
 }
 
@@ -392,7 +712,7 @@ static GtkWidget* create_page (WizardWindow *wizard, const char * title)
 	/* create the titlebar */
 	hbox = gtk_hbox_new (FALSE, 12);
 	image = stock_pixmap_widget(wizard->window, 
-			  	STOCK_PIXMAP_SYLPHEED_ICON);
+			  	STOCK_PIXMAP_SYLPHEED_CLAWS_ICON);
 	gtk_box_pack_start (GTK_BOX(hbox), image, FALSE, FALSE, 0);
      	title_string = g_strconcat ("<span size=\"xx-large\" weight=\"ultrabold\">", title ? title : "", "</span>", NULL);
 	w = gtk_label_new (title_string);
@@ -425,53 +745,47 @@ static GtkWidget* create_page (WizardWindow *wizard, const char * title)
 			 1,2,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      \
 }
 
-static gchar *get_default_email_addr(void)
-{
-	gchar *domain_name = g_strdup(get_domain_name());
-	gchar *result;
-	if (strchr(domain_name, '.') != strrchr(domain_name, '.')
-	&& strlen(strchr(domain_name, '.')) > 6) {
-		gchar *tmp = g_strdup(strchr(domain_name, '.')+1);
-		g_free(domain_name);
-		domain_name = tmp;
-	}
-	result = g_strdup_printf("%s@%s",
-				g_get_user_name(),
-				domain_name);
-	g_free(domain_name);
-	return result;
-}
-
 static gchar *get_default_server(WizardWindow * wizard, const gchar *type)
 {
-	gchar *domain_name = NULL;
-	gchar *tmp = gtk_editable_get_chars(
-			GTK_EDITABLE(wizard->email), 0, -1);
-	gchar *result;
-	
-	if (strstr(tmp, "@")) {
-		domain_name = g_strdup(strstr(tmp,"@")+1);
+	if (!strcmp(type, "smtp")) {
+		if (!tmpl.smtpserver || !strlen(tmpl.smtpserver))
+			return g_strconcat(type, ".", tmpl.domain, NULL);
+		else 
+			return g_strdup(tmpl.smtpserver);
 	} else {
-		domain_name = g_strdup(get_domain_name());
+		if (!tmpl.recvserver || !strlen(tmpl.recvserver))
+			return g_strconcat(type, ".", tmpl.domain, NULL);
+		else 
+			return g_strdup(tmpl.recvserver);
 	}
-	
-	g_free(tmp);
-
-	result = g_strdup_printf("%s.%s",
-				type, domain_name);
-	g_free(domain_name);
-	return result;
 }
 
 static gchar *get_default_account(WizardWindow * wizard)
 {
-	gchar *result = gtk_editable_get_chars(
-			GTK_EDITABLE(wizard->email), 0, -1);
+	gchar *result = NULL;
 	
-	if (strstr(result, "@")) {
-		*(strstr(result,"@")) = '\0';
-	} 
+	if (!tmpl.recvuser || !strlen(tmpl.recvuser)) {
+		result = gtk_editable_get_chars(
+				GTK_EDITABLE(wizard->email), 0, -1);
 
+		if (strstr(result, "@")) {
+			*(strstr(result,"@")) = '\0';
+		} 
+	} else {
+		result = g_strdup(tmpl.recvuser);
+	}
+	return result;
+}
+
+static gchar *get_default_smtp_account(WizardWindow * wizard)
+{
+	gchar *result = NULL;
+	
+	if (!tmpl.smtpuser || !strlen(tmpl.smtpuser)) {
+		return g_strdup("");
+	} else {
+		result = g_strdup(tmpl.smtpuser);
+	}
 	return result;
 }
 
@@ -500,9 +814,7 @@ static void wizard_email_changed(GtkWidget *widget, gpointer data)
 		gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), text);
 		g_free(text);
 	} else if (protocol == A_LOCAL) {
-		gchar *mbox = g_strdup_printf("/var/mail/%s", g_get_user_name());
-		gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), mbox);
-		g_free(mbox);
+		gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), tmpl.mboxfile?tmpl.mboxfile:"");
 	}
 	
 }
@@ -510,27 +822,25 @@ static void wizard_email_changed(GtkWidget *widget, gpointer data)
 static GtkWidget* user_page (WizardWindow * wizard)
 {
 	GtkWidget *table = gtk_table_new(3,2, FALSE);
-	gchar *text;
 	gint i = 0;
 	
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
 	wizard->full_name = gtk_entry_new();
-	gtk_entry_set_text(GTK_ENTRY(wizard->full_name), g_get_real_name());
+	gtk_entry_set_text(GTK_ENTRY(wizard->full_name), tmpl.name?tmpl.name:"");
 	GTK_TABLE_ADD_ROW_AT(table, _("<span weight=\"bold\">Your name:</span>"), 
 			     wizard->full_name, i); i++;
 	
 	wizard->email = gtk_entry_new();
-	text = get_default_email_addr();
-	gtk_entry_set_text(GTK_ENTRY(wizard->email), text);
-	g_free(text);
+	gtk_entry_set_text(GTK_ENTRY(wizard->email), tmpl.email?tmpl.email:"");
 	GTK_TABLE_ADD_ROW_AT(table, _("<span weight=\"bold\">Your email address:</span>"), 
 			     wizard->email, i); i++;
 	
 	wizard->organization = gtk_entry_new();
 	GTK_TABLE_ADD_ROW_AT(table, _("Your organization:"),
 			     wizard->organization, i); i++;
+	gtk_entry_set_text(GTK_ENTRY(wizard->organization), tmpl.organization?tmpl.organization:"");
 	
 	g_signal_connect(G_OBJECT(wizard->email), "changed",
 			 G_CALLBACK(wizard_email_changed),
@@ -547,16 +857,27 @@ static GtkWidget* mailbox_page (WizardWindow * wizard)
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
 	wizard->mailbox_name = gtk_entry_new();
-	gtk_entry_set_text(GTK_ENTRY(wizard->mailbox_name), "Mail");
+	gtk_entry_set_text(GTK_ENTRY(wizard->mailbox_name), tmpl.mailbox?tmpl.mailbox:"");
 	GTK_TABLE_ADD_ROW_AT(table, _("<span weight=\"bold\">Mailbox name:</span>"), 
 			     wizard->mailbox_name, i); i++;
 	
 	return table;
 }
 
+static void smtp_auth_changed (GtkWidget *btn, gpointer data)
+{
+	WizardWindow *wizard = (WizardWindow *)data;
+	gboolean do_auth = gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(wizard->smtp_auth));
+	gtk_widget_set_sensitive(wizard->smtp_username, do_auth);
+	gtk_widget_set_sensitive(wizard->smtp_username_label, do_auth);
+	gtk_widget_set_sensitive(wizard->smtp_password, do_auth);
+	gtk_widget_set_sensitive(wizard->smtp_password_label, do_auth);
+}
+
 static GtkWidget* smtp_page (WizardWindow * wizard)
 {
-	GtkWidget *table = gtk_table_new(1,2, FALSE);
+	GtkWidget *table = gtk_table_new(1,4, FALSE);
 	gchar *text;
 	gint i = 0;
 	
@@ -569,16 +890,51 @@ static GtkWidget* smtp_page (WizardWindow * wizard)
 	g_free(text);
 	GTK_TABLE_ADD_ROW_AT(table, _("<span weight=\"bold\">SMTP server address:</span>"), 
 			     wizard->smtp_server, i); i++;
+	wizard->smtp_auth = gtk_check_button_new_with_label(
+					_("Use authentication"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wizard->smtp_auth),
+			tmpl.smtpauth);
+	g_signal_connect(G_OBJECT(wizard->smtp_auth), "toggled",
+			 G_CALLBACK(smtp_auth_changed),
+			 wizard);
+	gtk_table_attach(GTK_TABLE(table), wizard->smtp_auth,      
+			 0,2,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0); i++;
+
+	text = get_default_smtp_account(wizard);
+
+	wizard->smtp_username = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(wizard->smtp_username), text);
+	g_free(text);
+	wizard->smtp_username_label = gtk_label_new(_("SMTP username:\n"
+					"<span size=\"small\">(empty to use the same as reception)</span>"));
+	gtk_label_set_use_markup(GTK_LABEL(wizard->smtp_username_label), TRUE);
+	gtk_table_attach(GTK_TABLE(table), wizard->smtp_username_label, 			      
+			 0,1,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      
+	if (GTK_IS_MISC(wizard->smtp_username_label))						      
+		gtk_misc_set_alignment(GTK_MISC(wizard->smtp_username_label), 1, 0.5);	      
+	gtk_table_attach(GTK_TABLE(table), wizard->smtp_username,	      
+			 1,2,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      
+	i++;
+	wizard->smtp_password = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(wizard->smtp_password), tmpl.smtppass?tmpl.smtppass:""); 
+	gtk_entry_set_visibility(GTK_ENTRY(wizard->smtp_password), FALSE);
+	wizard->smtp_password_label = gtk_label_new(_("SMTP password:\n"
+					"<span size=\"small\">(empty to use the same as reception)</span>"));
+	gtk_label_set_use_markup(GTK_LABEL(wizard->smtp_password_label), TRUE);
+	gtk_table_attach(GTK_TABLE(table), wizard->smtp_password_label, 			      
+			 0,1,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      
+	if (GTK_IS_MISC(wizard->smtp_password_label))						      
+		gtk_misc_set_alignment(GTK_MISC(wizard->smtp_password_label), 1, 0.5);	      
+	gtk_table_attach(GTK_TABLE(table), wizard->smtp_password,	      
+			 1,2,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      
+	i++;
+	smtp_auth_changed(NULL, wizard);
 	return table;
 }
 
-static void wizard_protocol_changed(GtkMenuItem *menuitem, gpointer data)
+static void wizard_protocol_change(WizardWindow *wizard, RecvProtocol protocol)
 {
-	WizardWindow *wizard = (WizardWindow *)data;
-	RecvProtocol protocol;
 	gchar *text;
-	protocol = GPOINTER_TO_INT
-		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 	
 	if (protocol == A_POP3) {
 		text = get_default_server(wizard, "pop");
@@ -589,10 +945,13 @@ static void wizard_protocol_changed(GtkMenuItem *menuitem, gpointer data)
 		gtk_widget_show(wizard->recv_password);
 		gtk_widget_show(wizard->recv_username_label);
 		gtk_widget_show(wizard->recv_password_label);
+		gtk_widget_hide(wizard->no_imap_warning);
 		gtk_label_set_text(GTK_LABEL(wizard->recv_label), _("<span weight=\"bold\">Server address:</span>"));
 		gtk_label_set_use_markup(GTK_LABEL(wizard->recv_label), TRUE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG(wizard->window), GO_FORWARD, TRUE);
 		g_free(text);
 	} else if (protocol == A_IMAP4) {
+#ifdef HAVE_LIBETPAN
 		text = get_default_server(wizard, "imap");
 		gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), text);
 		gtk_widget_show(wizard->recv_imap_label);
@@ -601,24 +960,46 @@ static void wizard_protocol_changed(GtkMenuItem *menuitem, gpointer data)
 		gtk_widget_show(wizard->recv_password);
 		gtk_widget_show(wizard->recv_username_label);
 		gtk_widget_show(wizard->recv_password_label);
+		gtk_widget_hide(wizard->no_imap_warning);
 		gtk_label_set_text(GTK_LABEL(wizard->recv_label), _("<span weight=\"bold\">Server address:</span>"));
 		gtk_label_set_use_markup(GTK_LABEL(wizard->recv_label), TRUE);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG(wizard->window), GO_FORWARD, TRUE);
 		g_free(text);
-	} else if (protocol == A_LOCAL) {
-		gchar *mbox = g_strdup_printf("/var/mail/%s", g_get_user_name());
-		gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), mbox);
-		g_free(mbox);
-		gtk_label_set_text(GTK_LABEL(wizard->recv_label), _("<span weight=\"bold\">Local mailbox:</span>"));
-		gtk_label_set_use_markup(GTK_LABEL(wizard->recv_label), TRUE);
+#else
 		gtk_widget_hide(wizard->recv_imap_label);
 		gtk_widget_hide(wizard->recv_imap_subdir);
 		gtk_widget_hide(wizard->recv_username);
 		gtk_widget_hide(wizard->recv_password);
 		gtk_widget_hide(wizard->recv_username_label);
 		gtk_widget_hide(wizard->recv_password_label);
+		gtk_widget_show(wizard->no_imap_warning);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG(wizard->window), GO_FORWARD, FALSE);
+#endif
+	} else if (protocol == A_LOCAL) {
+		gtk_entry_set_text(GTK_ENTRY(wizard->recv_server), tmpl.mboxfile?tmpl.mboxfile:"");
+		gtk_label_set_text(GTK_LABEL(wizard->recv_label), _("<span weight=\"bold\">Local mailbox:</span>"));
+		gtk_label_set_use_markup(GTK_LABEL(wizard->recv_label), TRUE);
+		gtk_widget_hide(wizard->no_imap_warning);
+		gtk_widget_hide(wizard->recv_imap_label);
+		gtk_widget_hide(wizard->recv_imap_subdir);
+		gtk_widget_hide(wizard->recv_username);
+		gtk_widget_hide(wizard->recv_password);
+		gtk_widget_hide(wizard->recv_username_label);
+		gtk_widget_hide(wizard->recv_password_label);
+		gtk_dialog_set_response_sensitive (GTK_DIALOG(wizard->window), GO_FORWARD, TRUE);
 	}
 }
 
+static void wizard_protocol_changed(GtkMenuItem *menuitem, gpointer data)
+{
+	WizardWindow *wizard = (WizardWindow *)data;
+	RecvProtocol protocol;
+	protocol = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
+
+	wizard_protocol_change(wizard, protocol);	
+}
+
 static GtkWidget* recv_page (WizardWindow * wizard)
 {
 	GtkWidget *table = gtk_table_new(5,2, FALSE);
@@ -626,7 +1007,8 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	GtkWidget *menuitem;
 	gchar *text;
 	gint i = 0;
-	
+	gint index = 0;
+
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
@@ -636,18 +1018,32 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	g_signal_connect(G_OBJECT(menuitem), "activate",
 			 G_CALLBACK(wizard_protocol_changed),
 			 wizard);
-#ifdef HAVE_LIBETPAN
+
 	MENUITEM_ADD (menu, menuitem, _("IMAP"), A_IMAP4);
 	g_signal_connect(G_OBJECT(menuitem), "activate",
 			 G_CALLBACK(wizard_protocol_changed),
 			 wizard);
-#endif
+
 	MENUITEM_ADD (menu, menuitem, _("Local mbox file"), A_LOCAL);
 	g_signal_connect(G_OBJECT(menuitem), "activate",
 			 G_CALLBACK(wizard_protocol_changed),
 			 wizard);
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (wizard->recv_type), menu);
+	switch(tmpl.recvtype) {
+	case A_POP3: 
+		index = 0;
+		break;
+	case A_IMAP4:
+		index = 1;
+		break;
+	case A_LOCAL:
+		index = 2;
+		break;
+	default:
+		index = 0;
+	}
+	gtk_option_menu_set_history(GTK_OPTION_MENU (wizard->recv_type), index);
 	GTK_TABLE_ADD_ROW_AT(table, _("<span weight=\"bold\">Server type:</span>"), 
 			     wizard->recv_type, i); i++;
 
@@ -682,6 +1078,7 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	g_free(text);
 
 	wizard->recv_password = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(wizard->recv_password), tmpl.recvpass?tmpl.recvpass:"");
 	wizard->recv_password_label = gtk_label_new(_("Password:"));
 	gtk_table_attach(GTK_TABLE(table), wizard->recv_password_label, 			      
 			 0,1,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      
@@ -693,6 +1090,7 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 	i++;
 	
 	wizard->recv_imap_subdir = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(wizard->recv_imap_subdir), tmpl.imapdir?tmpl.imapdir:"");
 	wizard->recv_imap_label = gtk_label_new(_("IMAP server directory:"));
 	
 	gtk_table_attach(GTK_TABLE(table), wizard->recv_imap_label, 			      
@@ -704,6 +1102,13 @@ static GtkWidget* recv_page (WizardWindow * wizard)
 
 	i++;
 	
+	wizard->no_imap_warning = gtk_label_new(_(
+			  "<span weight=\"bold\">Warning: this version of Sylpheed-Claws\n"
+			  "has been built without IMAP support.</span>"));
+	gtk_label_set_use_markup(GTK_LABEL(wizard->no_imap_warning), TRUE);
+	gtk_table_attach(GTK_TABLE(table), wizard->no_imap_warning, 			      
+			 0,2,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);	      
+	
 	return table;
 }
 
@@ -718,11 +1123,15 @@ static GtkWidget* ssl_page (WizardWindow * wizard)
 
 	wizard->smtp_use_ssl = gtk_check_button_new_with_label(
 					_("Use SSL to connect to SMTP server"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wizard->smtp_use_ssl),
+			tmpl.smtpssl);
 	gtk_table_attach(GTK_TABLE(table), wizard->smtp_use_ssl,      
 			 0,1,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0); i++;
 	
 	wizard->recv_use_ssl = gtk_check_button_new_with_label(
 					_("Use SSL to connect to receiving server"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wizard->recv_use_ssl),
+			tmpl.recvssl);
 	gtk_table_attach(GTK_TABLE(table), wizard->recv_use_ssl,      
 			 0,1,i,i+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 	
@@ -834,6 +1243,8 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	
 	gtk_widget_hide(mainwin->window);
 	
+	wizard_read_defaults();
+	
 	wizard->window = gtk_dialog_new_with_buttons (_("Sylpheed-Claws Setup Wizard"),
 			NULL, 0, 
 			GTK_STOCK_GO_BACK, GO_BACK,
@@ -870,7 +1281,7 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	
 	wizard->pages = g_slist_append(wizard->pages, page);
 	widget = stock_pixmap_widget(wizard->window, 
-			  	STOCK_PIXMAP_SYLPHEED_LOGO);
+			  	STOCK_PIXMAP_SYLPHEED_CLAWS_LOGO);
 
 	gtk_box_pack_start (GTK_BOX(page), widget, FALSE, FALSE, 0);
 	
@@ -893,16 +1304,7 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	
 	wizard->pages = g_slist_append(wizard->pages, widget);
 
-/*smtp page: 2 */
-	i++;
-	SMTP_PAGE = i;
-	widget = create_page (wizard, _("Sending mail"));
-	gtk_box_pack_start (GTK_BOX(widget), smtp_page(wizard), FALSE, FALSE, 0);
-	PACK_WARNING(_("Bold fields must be completed"));
-	
-	wizard->pages = g_slist_append(wizard->pages, widget);
-
-/* recv+auth page: 3 */
+/* recv+auth page: 2 */
 	i++;
 	RECV_PAGE = i;
 	widget = create_page (wizard, _("Receiving mail"));
@@ -911,6 +1313,15 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	
 	wizard->pages = g_slist_append(wizard->pages, widget);
 
+/*smtp page: 3 */
+	i++;
+	SMTP_PAGE = i;
+	widget = create_page (wizard, _("Sending mail"));
+	gtk_box_pack_start (GTK_BOX(widget), smtp_page(wizard), FALSE, FALSE, 0);
+	PACK_WARNING(_("Bold fields must be completed"));
+	
+	wizard->pages = g_slist_append(wizard->pages, widget);
+
 /* mailbox page: 4 */
 	if (create_mailbox) {
 		i++;
@@ -939,7 +1350,7 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	
 	wizard->pages = g_slist_append(wizard->pages, page);
 	widget = stock_pixmap_widget(wizard->window, 
-			  	STOCK_PIXMAP_SYLPHEED_LOGO);
+			  	STOCK_PIXMAP_SYLPHEED_CLAWS_LOGO);
 
 	gtk_box_pack_start (GTK_BOX(page), widget, FALSE, FALSE, 0);
 	
@@ -962,6 +1373,8 @@ gboolean run_wizard(MainWindow *mainwin, gboolean create_mailbox) {
 	gtk_widget_hide(wizard->recv_imap_label);
 	gtk_widget_hide(wizard->recv_imap_subdir);
 
+	wizard_protocol_change(wizard, tmpl.recvtype);
+
 	while (!wizard->finished)
 		gtk_main_iteration();
 
